@@ -57,6 +57,7 @@
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplay-foreach.h"
 
+#include "dialogs/dialogs.h"
 #include "dialogs/file-save-dialog.h"
 
 #include "actions.h"
@@ -65,9 +66,6 @@
 #include "gimp-intl.h"
 
 
-#define REVERT_DATA_KEY "revert-confirm-dialog"
-
-
 /*  local function prototypes  */
 
 static void        file_open_dialog_show        (Gimp         *gimp,
@@ -89,13 +87,9 @@ static GtkWidget * file_export_dialog_show      (Gimp         *gimp,
 static void        file_save_dialog_response    (GtkWidget    *dialog,
                                                  gint          response_id,
                                                  gpointer      data);
-static void        file_save_dialog_destroyed   (GtkWidget    *dialog,
-                                                 GimpImage    *image);
 static void        file_export_dialog_response  (GtkWidget    *dialog,
                                                  gint          response_id,
                                                  gpointer      data);
-static void        file_export_dialog_destroyed (GtkWidget    *dialog,
-                                                 GimpImage    *image);
 static void        file_new_template_callback   (GtkWidget    *widget,
                                                  const gchar  *name,
                                                  gpointer      data);
@@ -423,29 +417,29 @@ file_revert_cmd_callback (GtkAction *action,
   if (! file)
     file = gimp_image_get_imported_file (image);
 
-  dialog = g_object_get_data (G_OBJECT (image), REVERT_DATA_KEY);
-
   if (! file)
     {
       gimp_message_literal (image->gimp,
                             G_OBJECT (display), GIMP_MESSAGE_ERROR,
                             _("Revert failed. "
                               "No file name associated with this image."));
+      return;
     }
-  else if (dialog)
-    {
-      gtk_window_present (GTK_WINDOW (dialog));
-    }
-  else
+
+#define REVERT_DIALOG_KEY "gimp-revert-confirm-dialog"
+
+  dialog = dialogs_get_dialog (G_OBJECT (image), REVERT_DIALOG_KEY);
+
+  if (! dialog)
     {
       dialog =
-        gimp_message_dialog_new (_("Revert Image"), "document-revert",
+        gimp_message_dialog_new (_("Revert Image"), GIMP_ICON_DOCUMENT_REVERT,
                                  GTK_WIDGET (gimp_display_get_shell (display)),
                                  0,
                                  gimp_standard_help_func, GIMP_HELP_FILE_REVERT,
 
-                                 GTK_STOCK_CANCEL,          GTK_RESPONSE_CANCEL,
-                                 GTK_STOCK_REVERT_TO_SAVED, GTK_RESPONSE_OK,
+                                 _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                 _("_Revert"), GTK_RESPONSE_OK,
 
                                  NULL);
 
@@ -472,10 +466,10 @@ file_revert_cmd_callback (GtkAction *action,
                                    "on disk, you will lose all changes, "
                                    "including all undo information."));
 
-      g_object_set_data (G_OBJECT (image), REVERT_DATA_KEY, dialog);
-
-      gtk_widget_show (dialog);
+      dialogs_attach_dialog (G_OBJECT (image), REVERT_DIALOG_KEY, dialog);
     }
+
+  gtk_window_present (GTK_WINDOW (dialog));
 }
 
 void
@@ -593,9 +587,6 @@ file_open_dialog_show (Gimp        *gimp,
                                            NULL /*ui_manager*/,
                                            "gimp-file-open-dialog", -1, FALSE);
 
-  if (file)
-    g_object_ref (file);
-
   if (dialog)
     {
       if (! file && image)
@@ -606,10 +597,16 @@ file_open_dialog_show (Gimp        *gimp,
                                   GIMP_FILE_OPEN_LAST_FILE_KEY);
 
       if (file)
-        gtk_file_chooser_set_file (GTK_FILE_CHOOSER (dialog), file, NULL);
+        {
+          gtk_file_chooser_set_file (GTK_FILE_CHOOSER (dialog), file, NULL);
+        }
       else if (gimp->default_folder)
-        gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (dialog),
-                                                  gimp->default_folder, NULL);
+        {
+          gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (dialog),
+                                                    gimp->default_folder, NULL);
+        }
+
+      gtk_window_set_title (GTK_WINDOW (dialog), title);
 
       gimp_open_dialog_set_image (GIMP_OPEN_DIALOG (dialog),
                                   image, open_as_layers);
@@ -632,7 +629,9 @@ file_save_dialog_show (Gimp        *gimp,
 {
   GtkWidget *dialog;
 
-  dialog = g_object_get_data (G_OBJECT (image), "gimp-file-save-dialog");
+#define SAVE_DIALOG_KEY "gimp-file-save-dialog"
+
+  dialog = dialogs_get_dialog (G_OBJECT (image), SAVE_DIALOG_KEY);
 
   if (! dialog)
     {
@@ -648,15 +647,14 @@ file_save_dialog_show (Gimp        *gimp,
           gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                         GTK_WINDOW (gtk_widget_get_toplevel (parent)));
 
-          g_object_set_data_full (G_OBJECT (image),
-                                  "gimp-file-save-dialog", dialog,
-                                  (GDestroyNotify) gtk_widget_destroy);
+          dialogs_attach_dialog (G_OBJECT (image), SAVE_DIALOG_KEY, dialog);
+          g_signal_connect_object (image, "disconnect",
+                                   G_CALLBACK (gtk_widget_destroy),
+                                   dialog, G_CONNECT_SWAPPED);
+
           g_signal_connect (dialog, "response",
                             G_CALLBACK (file_save_dialog_response),
                             image);
-          g_signal_connect (dialog, "destroy",
-                            G_CALLBACK (file_save_dialog_destroyed),
-                            image);
         }
     }
 
@@ -665,7 +663,7 @@ file_save_dialog_show (Gimp        *gimp,
       gtk_window_set_title (GTK_WINDOW (dialog), title);
 
       gimp_save_dialog_set_image (GIMP_SAVE_DIALOG (dialog),
-                                  gimp, image, save_a_copy,
+                                  image, save_a_copy,
                                   close_after_saving, GIMP_OBJECT (display));
 
       gtk_window_present (GTK_WINDOW (dialog));
@@ -707,14 +705,6 @@ file_save_dialog_response (GtkWidget *dialog,
     }
 }
 
-static void
-file_save_dialog_destroyed (GtkWidget *dialog,
-                            GimpImage *image)
-{
-  if (GIMP_FILE_DIALOG (dialog)->image == image)
-    g_object_set_data (G_OBJECT (image), "gimp-file-save-dialog", NULL);
-}
-
 static GtkWidget *
 file_export_dialog_show (Gimp      *gimp,
                          GimpImage *image,
@@ -722,7 +712,9 @@ file_export_dialog_show (Gimp      *gimp,
 {
   GtkWidget *dialog;
 
-  dialog = g_object_get_data (G_OBJECT (image), "gimp-file-export-dialog");
+#define EXPORT_DIALOG_KEY "gimp-file-export-dialog"
+
+  dialog = dialogs_get_dialog (G_OBJECT (image), EXPORT_DIALOG_KEY);
 
   if (! dialog)
     {
@@ -738,22 +730,20 @@ file_export_dialog_show (Gimp      *gimp,
           gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                         GTK_WINDOW (gtk_widget_get_toplevel (parent)));
 
-          g_object_set_data_full (G_OBJECT (image),
-                                  "gimp-file-export-dialog", dialog,
-                                  (GDestroyNotify) gtk_widget_destroy);
+          dialogs_attach_dialog (G_OBJECT (image), EXPORT_DIALOG_KEY, dialog);
+          g_signal_connect_object (image, "disconnect",
+                                   G_CALLBACK (gtk_widget_destroy),
+                                   dialog, G_CONNECT_SWAPPED);
+
           g_signal_connect (dialog, "response",
                             G_CALLBACK (file_export_dialog_response),
                             image);
-          g_signal_connect (dialog, "destroy",
-                            G_CALLBACK (file_export_dialog_destroyed),
-                            image);
         }
     }
 
   if (dialog)
     {
-      gimp_export_dialog_set_image (GIMP_EXPORT_DIALOG (dialog),
-                                    gimp, image);
+      gimp_export_dialog_set_image (GIMP_EXPORT_DIALOG (dialog), image);
 
       gtk_window_present (GTK_WINDOW (dialog));
     }
@@ -796,14 +786,6 @@ file_export_dialog_response (GtkWidget *dialog,
     }
 }
 
-static void
-file_export_dialog_destroyed (GtkWidget *dialog,
-                              GimpImage *image)
-{
-  if (GIMP_FILE_DIALOG (dialog)->image == image)
-    g_object_set_data (G_OBJECT (image), "gimp-file-export-dialog", NULL);
-}
-
 static void
 file_new_template_callback (GtkWidget   *widget,
                             const gchar *name,
@@ -832,8 +814,6 @@ file_revert_confirm_response (GtkWidget   *dialog,
 
   gtk_widget_destroy (dialog);
 
-  g_object_set_data (G_OBJECT (old_image), REVERT_DATA_KEY, NULL);
-
   if (response_id == GTK_RESPONSE_OK)
     {
       Gimp              *gimp = old_image->gimp;
