@@ -97,447 +97,507 @@ static const GimpActionEntry filters_menu_actions[] =
 
 static const GimpStringActionEntry filters_actions[] =
 {
-  { "filters-alien-map", GIMP_STOCK_GEGL,
+  { "filters-alien-map", GIMP_ICON_GEGL,
     NC_("filters-action", "_Alien Map..."), NULL, NULL,
     "gegl:alien-map",
-    NULL /* FIXME GIMP_HELP_FILTER_ALIEN_MAP */ },
+    GIMP_HELP_FILTER_ALIEN_MAP },
 
-  { "filters-antialias", GIMP_STOCK_GEGL,
+  { "filters-antialias", GIMP_ICON_GEGL,
     NC_("filters-action", "_Antialias..."), NULL, NULL,
     "gegl:antialias",
-    NULL /* FIXME GIMP_HELP_FILTER_ANTIALIAS */ },
+    GIMP_HELP_FILTER_ANTIALIAS },
 
-  { "filters-apply-canvas", GIMP_STOCK_GEGL,
+  { "filters-apply-canvas", GIMP_ICON_GEGL,
     NC_("filters-action", "_Apply Canvas..."), NULL, NULL,
     "gegl:texturize-canvas",
-    NULL /* FIXME GIMP_HELP_FILTER_APPLY_CANVAS */ },
+    GIMP_HELP_FILTER_APPLY_CANVAS },
 
-  { "filters-apply-lens", GIMP_STOCK_GEGL,
+  { "filters-apply-lens", GIMP_ICON_GEGL,
     NC_("filters-action", "Apply _Lens..."), NULL, NULL,
     "gegl:apply-lens",
-    NULL /* FIXME GIMP_HELP_FILTER_APPLY_LENS */ },
+    GIMP_HELP_FILTER_APPLY_LENS },
 
-  { "filters-bump-map", GIMP_STOCK_GEGL,
+  { "filters-bump-map", GIMP_ICON_GEGL,
     NC_("filters-action", "_Bump Map..."), NULL, NULL,
     "gegl:bump-map",
-    NULL /* FIXME GIMP_HELP_FILTER_BUMP_MAP */ },
+    GIMP_HELP_FILTER_BUMP_MAP },
 
-  { "filters-c2g", GIMP_STOCK_GEGL,
+  { "filters-c2g", GIMP_ICON_GEGL,
     NC_("filters-action", "_Color to Gray..."), NULL, NULL,
     "gegl:c2g",
-    NULL /* FIXME GIMP_HELP_FILTER_C2G */ },
+    GIMP_HELP_FILTER_C2G },
 
-  { "filters-cartoon", GIMP_STOCK_GEGL,
+  { "filters-cartoon", GIMP_ICON_GEGL,
     NC_("filters-action", "Ca_rtoon..."), NULL, NULL,
     "gegl:cartoon",
-    NULL /* FIXME GIMP_HELP_FILTER_CARTOON */ },
+    GIMP_HELP_FILTER_CARTOON },
 
-  { "filters-channel-mixer", GIMP_STOCK_GEGL,
+  { "filters-channel-mixer", GIMP_ICON_GEGL,
     NC_("filters-action", "_Channel Mixer..."), NULL, NULL,
     "gegl:channel-mixer",
-    NULL /* FIXME GIMP_HELP_FILTER_CHANNEL_MIXER */ },
+    GIMP_HELP_FILTER_CHANNEL_MIXER },
 
-  { "filters-checkerboard", GIMP_STOCK_GEGL,
+  { "filters-checkerboard", GIMP_ICON_GEGL,
     NC_("filters-action", "_Checkerboard..."), NULL, NULL,
     "gegl:checkerboard",
-    NULL /* FIXME GIMP_HELP_FILTER_CHECKERBOARD */ },
+    GIMP_HELP_FILTER_CHECKERBOARD },
 
-  { "filters-color-enhance", GIMP_STOCK_GEGL,
+  { "filters-color-enhance", GIMP_ICON_GEGL,
     NC_("filters-action", "_Color Enhance..."), NULL, NULL,
     "gegl:color-enhance",
-    NULL /* FIXME GIMP_HELP_FILTER_COLOR_ENHANCE */ },
+    GIMP_HELP_FILTER_COLOR_ENHANCE },
 
-  { "filters-color-exchange", GIMP_STOCK_GEGL,
+  { "filters-color-exchange", GIMP_ICON_GEGL,
     NC_("filters-action", "_Color Exchange..."), NULL, NULL,
     "gegl:color-exchange",
-    NULL /* FIXME GIMP_HELP_FILTER_COLOR_EXCHANGE */ },
+    GIMP_HELP_FILTER_COLOR_EXCHANGE },
 
-  { "filters-color-reduction", GIMP_STOCK_GEGL,
-    NC_("filters-action", "Color _Reduction..."), NULL, NULL,
-    "gegl:color-reduction",
-    NULL /* FIXME GIMP_HELP_FILTER_COLOR_TEMPERATURE */ },
+  { "filters-dither", GIMP_ICON_GEGL,
+    NC_("filters-action", "Dithe_r..."), NULL, NULL,
+    "gegl:dither",
+    GIMP_HELP_FILTER_DITHER },
 
-  { "filters-color-rotate", GIMP_STOCK_GEGL,
+  { "filters-color-rotate", GIMP_ICON_GEGL,
     NC_("filters-action", "_Rotate Colors..."), NULL, NULL,
     "gegl:color-rotate",
-    NULL /* FIXME GIMP_HELP_FILTER_COLOR_ROTATE */ },
+    GIMP_HELP_FILTER_COLOR_ROTATE },
 
-  { "filters-color-temperature", GIMP_STOCK_GEGL,
+  { "filters-color-temperature", GIMP_ICON_GEGL,
     NC_("filters-action", "Color T_emperature..."), NULL, NULL,
     "gegl:color-temperature",
-    NULL /* FIXME GIMP_HELP_FILTER_COLOR_TEMPERATURE */ },
+    GIMP_HELP_FILTER_COLOR_TEMPERATURE },
 
-  { "filters-color-to-alpha", GIMP_STOCK_GEGL,
+  { "filters-color-to-alpha", GIMP_ICON_GEGL,
     NC_("filters-action", "Color to _Alpha..."), NULL, NULL,
     "gegl:color-to-alpha",
-    NULL /* FIXME GIMP_HELP_FILTER_COLOR_TO_ALPHA */ },
+    GIMP_HELP_FILTER_COLOR_TO_ALPHA },
 
-  { "filters-convolution-matrix", GIMP_STOCK_GEGL,
+  { "filters-component-extract", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Extract Component..."), NULL, NULL,
+    "gegl:component-extract",
+    GIMP_HELP_FILTER_COMPONENT_EXTRACT },
+
+  { "filters-convolution-matrix", GIMP_ICON_GEGL,
     NC_("filters-action", "_Convolution Matrix..."), NULL, NULL,
     "gegl:convolution-matrix",
-    NULL /* FIXME GIMP_HELP_FILTER_CONVOLUTION_MATRIX */ },
+    GIMP_HELP_FILTER_CONVOLUTION_MATRIX },
 
-  { "filters-cubism", GIMP_STOCK_GEGL,
+  { "filters-cubism", GIMP_ICON_GEGL,
     NC_("filters-action", "_Cubism..."), NULL, NULL,
     "gegl:cubism",
-    NULL /* FIXME GIMP_HELP_FILTER_CUBISM */ },
+    GIMP_HELP_FILTER_CUBISM },
 
-  { "filters-deinterlace", GIMP_STOCK_GEGL,
+  { "filters-deinterlace", GIMP_ICON_GEGL,
     NC_("filters-action", "_Deinterlace..."), NULL, NULL,
     "gegl:deinterlace",
-    NULL /* FIXME GIMP_HELP_FILTER_DEINTERLACE */ },
+    GIMP_HELP_FILTER_DEINTERLACE },
 
-  { "filters-desaturate", GIMP_STOCK_TOOL_DESATURATE,
+  { "filters-desaturate", GIMP_ICON_TOOL_DESATURATE,
     NC_("filters-action", "_Desaturate..."), NULL, NULL,
     "gimp:desaturate",
-    GIMP_HELP_TOOL_DESATURATE },
+    GIMP_HELP_FILTER_DESATURATE },
 
-  { "filters-difference-of-gaussians", GIMP_STOCK_GEGL,
+  { "filters-difference-of-gaussians", GIMP_ICON_GEGL,
     NC_("filters-action", "Difference of Gaussians..."), NULL, NULL,
     "gegl:difference-of-gaussians",
-    NULL /* FIXME GIMP_HELP_FILTER_DIFFERENCE_OF_GAUSSIANS */ },
+    GIMP_HELP_FILTER_DIFFERENCE_OF_GAUSSIANS },
 
-  { "filters-diffraction-patterns", GIMP_STOCK_GEGL,
+  { "filters-diffraction-patterns", GIMP_ICON_GEGL,
     NC_("filters-action", "Diffraction Patterns..."), NULL, NULL,
     "gegl:diffraction-patterns",
-    NULL /* FIXME GIMP_HELP_FILTER_DIFFRACTION_PATTERNS */ },
+    GIMP_HELP_FILTER_DIFFRACTION_PATTERNS },
 
-  { "filters-displace", GIMP_STOCK_GEGL,
+  { "filters-displace", GIMP_ICON_GEGL,
     NC_("filters-action", "Displace..."), NULL, NULL,
     "gegl:displace",
-    NULL /* FIXME GIMP_HELP_FILTER_DISPLACE */ },
+    GIMP_HELP_FILTER_DISPLACE },
 
-  { "filters-distance-map", GIMP_STOCK_GEGL,
+  { "filters-distance-map", GIMP_ICON_GEGL,
     NC_("filters-action", "Distance Map..."), NULL, NULL,
     "gegl:distance-transform",
-    NULL /* FIXME GIMP_HELP_FILTER_DISTANCE_MAP */ },
+    GIMP_HELP_FILTER_DISTANCE_MAP },
 
-  { "filters-dropshadow", GIMP_STOCK_GEGL,
+  { "filters-dropshadow", GIMP_ICON_GEGL,
     NC_("filters-action", "_Drop Shadow..."), NULL, NULL,
     "gegl:dropshadow",
-    NULL /* FIXME GIMP_HELP_FILTER_DROPSHADOW */ },
+    GIMP_HELP_FILTER_DROPSHADOW },
 
-  { "filters-edge", GIMP_STOCK_GEGL,
+  { "filters-edge", GIMP_ICON_GEGL,
     NC_("filters-action", "_Edge..."), NULL, NULL,
     "gegl:edge",
-    NULL /* FIXME GIMP_HELP_FILTER_EDGE_LAPLACE */ },
+    GIMP_HELP_FILTER_EDGE },
 
-  { "filters-edge-laplace", GIMP_STOCK_GEGL,
+  { "filters-edge-laplace", GIMP_ICON_GEGL,
     NC_("filters-action", "_Laplace"), NULL, NULL,
     "gegl:edge-laplace",
-    NULL /* FIXME GIMP_HELP_FILTER_EDGE_LAPLACE */ },
+    GIMP_HELP_FILTER_EDGE_LAPLACE },
 
-  { "filters-edge-sobel", GIMP_STOCK_GEGL,
+  { "filters-edge-neon", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Neon..."), NULL, NULL,
+    "gegl:edge-neon",
+    GIMP_HELP_FILTER_EDGE_NEON },
+
+  { "filters-edge-sobel", GIMP_ICON_GEGL,
     NC_("filters-action", "_Sobel..."), NULL, NULL,
     "gegl:edge-sobel",
-    NULL /* FIXME GIMP_HELP_FILTER_EDGE_SOBEL */ },
+    GIMP_HELP_FILTER_EDGE_SOBEL },
 
-  { "filters-emboss", GIMP_STOCK_GEGL,
+  { "filters-emboss", GIMP_ICON_GEGL,
     NC_("filters-action", "_Emboss..."), NULL, NULL,
     "gegl:emboss",
-    NULL /* FIXME GIMP_HELP_FILTER_EMBOSS */ },
+    GIMP_HELP_FILTER_EMBOSS },
 
-  { "filters-engrave", GIMP_STOCK_GEGL,
+  { "filters-engrave", GIMP_ICON_GEGL,
     NC_("filters-action", "En_grave..."), NULL, NULL,
     "gegl:engrave",
-    NULL /* FIXME GIMP_HELP_FILTER_ENGRAVE */ },
+    GIMP_HELP_FILTER_ENGRAVE },
 
-  { "filters-exposure", GIMP_STOCK_GEGL,
+  { "filters-exposure", GIMP_ICON_GEGL,
     NC_("filters-action", "E_xposure..."), NULL, NULL,
     "gegl:exposure",
-    NULL /* FIXME GIMP_HELP_FILTER_EXPOSURE */ },
+    GIMP_HELP_FILTER_EXPOSURE },
 
-  { "filters-fractal-trace", GIMP_STOCK_GEGL,
+  { "filters-fattal-2002", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Fattal et al. 2002..."), NULL, NULL,
+    "gegl:fattal02",
+    GIMP_HELP_FILTER_FATTAL_2002 },
+
+  { "filters-fractal-trace", GIMP_ICON_GEGL,
     NC_("filters-action", "_Fractal Trace..."), NULL, NULL,
     "gegl:fractal-trace",
-    NULL /* FIXME GIMP_HELP_FILTER_FRACTAL_TRACE */ },
+    GIMP_HELP_FILTER_FRACTAL_TRACE },
 
-  { "filters-gaussian-blur", GIMP_STOCK_GEGL,
+  { "filters-gaussian-blur", GIMP_ICON_GEGL,
     NC_("filters-action", "_Gaussian Blur..."), NULL, NULL,
     "gegl:gaussian-blur",
-    NULL /* FIXME GIMP_HELP_FILTER_GAUSSIAN_BLUR */ },
+    GIMP_HELP_FILTER_GAUSSIAN_BLUR },
 
-  { "filters-gaussian-blur-selective", GIMP_STOCK_GEGL,
+  { "filters-gaussian-blur-selective", GIMP_ICON_GEGL,
     NC_("filters-action", "_Selective Gaussian Blur..."), NULL, NULL,
     "gegl:gaussian-blur-selective",
-    NULL /* FIXME GIMP_HELP_FILTER_GAUSSIAN_BLUR_SELECTIVE */ },
+    GIMP_HELP_FILTER_GAUSSIAN_BLUR_SELECTIVE },
 
-  { "filters-grid", GIMP_STOCK_GRID,
+  { "filters-gegl-graph", GIMP_ICON_GEGL,
+    NC_("filters-action", "_GEGL graph..."), NULL, NULL,
+    "gegl:gegl",
+    GIMP_HELP_FILTER_GEGL_GRAPH },
+
+  { "filters-grid", GIMP_ICON_GRID,
     NC_("filters-action", "_Grid..."), NULL, NULL,
     "gegl:grid",
-    NULL /* FIXME GIMP_HELP_FILTER_GAUSSIAN_GRID */ },
+    GIMP_HELP_FILTER_GAUSSIAN_GRID },
 
-  { "filters-high-pass", GIMP_STOCK_GEGL,
+  { "filters-high-pass", GIMP_ICON_GEGL,
     NC_("filters-action", "_High Pass..."), NULL, NULL,
     "gegl:high-pass",
-    NULL /* FIXME GIMP_HELP_FILTER_HIGH_PASS */ },
+    GIMP_HELP_FILTER_HIGH_PASS },
 
-  { "filters-illusion", GIMP_STOCK_GEGL,
+  { "filters-illusion", GIMP_ICON_GEGL,
     NC_("filters-action", "_Illusion..."), NULL, NULL,
     "gegl:illusion",
-    NULL /* FIXME GIMP_HELP_FILTER_ILLUSION */ },
+    GIMP_HELP_FILTER_ILLUSION },
 
-  { "filters-kaleidoscope", GIMP_STOCK_GEGL,
+  { "filters-image-gradient", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Image Gradient..."), NULL, NULL,
+    "gegl:image-gradient",
+    GIMP_HELP_FILTER_IMAGE_GRADIENT },
+
+  { "filters-kaleidoscope", GIMP_ICON_GEGL,
     NC_("filters-action", "_Kaleidoscope..."), NULL, NULL,
     "gegl:mirrors",
-    NULL /* FIXME GIMP_HELP_FILTER_KALEIDOSCOPE */ },
+    GIMP_HELP_FILTER_KALEIDOSCOPE },
 
-  { "filters-lens-distortion", GIMP_STOCK_GEGL,
+  { "filters-lens-distortion", GIMP_ICON_GEGL,
     NC_("filters-action", "Lens Distortion..."), NULL, NULL,
     "gegl:lens-distortion",
-    NULL /* FIXME GIMP_HELP_FILTER_LENS_DISTORTION */ },
+    GIMP_HELP_FILTER_LENS_DISTORTION },
 
-  { "filters-lens-flare", GIMP_STOCK_GEGL,
+  { "filters-lens-flare", GIMP_ICON_GEGL,
     NC_("filters-action", "Lens Flare..."), NULL, NULL,
     "gegl:lens-flare",
-    NULL /* FIXME GIMP_HELP_FILTER_LENS_FLARE */ },
+    GIMP_HELP_FILTER_LENS_FLARE },
 
-  { "filters-maze", GIMP_STOCK_GEGL,
+  { "filters-mantiuk-2006", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Mantiuk 2006..."), NULL, NULL,
+    "gegl:mantiuk06",
+    GIMP_HELP_FILTER_MANTIUK_2006 },
+
+  { "filters-maze", GIMP_ICON_GEGL,
     NC_("filters-action", "_Maze..."), NULL, NULL,
     "gegl:maze",
-    NULL /* FIXME GIMP_HELP_FILTER_MAZE */ },
+    GIMP_HELP_FILTER_MAZE },
 
-  { "filters-mono-mixer", GIMP_STOCK_GEGL,
+  { "filters-mono-mixer", GIMP_ICON_GEGL,
     NC_("filters-action", "_Mono Mixer..."), NULL, NULL,
     "gegl:mono-mixer",
-    NULL /* FIXME GIMP_HELP_FILTER_MONO_MIXER */ },
+    GIMP_HELP_FILTER_MONO_MIXER },
 
-  { "filters-mosaic", GIMP_STOCK_GEGL,
+  { "filters-mosaic", GIMP_ICON_GEGL,
     NC_("filters-action", "_Mosaic..."), NULL, NULL,
     "gegl:mosaic",
-    NULL /* FIXME GIMP_HELP_FILTER_MOSAIC */ },
+    GIMP_HELP_FILTER_MOSAIC },
 
-  { "filters-motion-blur-circular", GIMP_STOCK_GEGL,
+  { "filters-motion-blur-circular", GIMP_ICON_GEGL,
     NC_("filters-action", "_Circular Motion Blur..."), NULL, NULL,
     "gegl:motion-blur-circular",
-    NULL /* FIXME GIMP_HELP_FILTER_MOTION_BLUR_CIRCULAR */ },
+    GIMP_HELP_FILTER_MOTION_BLUR_CIRCULAR },
 
-  { "filters-motion-blur-linear", GIMP_STOCK_GEGL,
+  { "filters-motion-blur-linear", GIMP_ICON_GEGL,
     NC_("filters-action", "_Linear Motion Blur..."), NULL, NULL,
     "gegl:motion-blur-linear",
-    NULL /* FIXME GIMP_HELP_FILTER_MOTION_BLUR_LINEAR */ },
+    GIMP_HELP_FILTER_MOTION_BLUR_LINEAR },
 
-  { "filters-motion-blur-zoom", GIMP_STOCK_GEGL,
+  { "filters-motion-blur-zoom", GIMP_ICON_GEGL,
     NC_("filters-action", "_Zoom Motion Blur..."), NULL, NULL,
     "gegl:motion-blur-zoom",
-    NULL /* FIXME GIMP_HELP_FILTER_MOTION_BLUR_ZOOM */ },
+    GIMP_HELP_FILTER_MOTION_BLUR_ZOOM },
 
-  { "filters-noise-cell", GIMP_STOCK_GEGL,
+  { "filters-noise-cell", GIMP_ICON_GEGL,
     NC_("filters-action", "_Cell Noise..."), NULL, NULL,
     "gegl:cell-noise",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_CELL */ },
+    GIMP_HELP_FILTER_NOISE_CELL },
 
-  { "filters-noise-cie-lch", GIMP_STOCK_GEGL,
+  { "filters-noise-cie-lch", GIMP_ICON_GEGL,
     NC_("filters-action", "CIE lch Noise..."), NULL, NULL,
     "gegl:noise-cie-lch",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_CIE_LCH */ },
+    GIMP_HELP_FILTER_NOISE_CIE_LCH },
 
-  { "filters-noise-hsv", GIMP_STOCK_GEGL,
+  { "filters-noise-hsv", GIMP_ICON_GEGL,
     NC_("filters-action", "HSV Noise..."), NULL, NULL,
     "gegl:noise-hsv",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_HSV */ },
+    GIMP_HELP_FILTER_NOISE_HSV },
 
-  { "filters-noise-hurl", GIMP_STOCK_GEGL,
+  { "filters-noise-hurl", GIMP_ICON_GEGL,
     NC_("filters-action", "_Hurl..."), NULL, NULL,
     "gegl:noise-hurl",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_HURL */ },
+    GIMP_HELP_FILTER_NOISE_HURL },
 
-  { "filters-noise-perlin", GIMP_STOCK_GEGL,
+  { "filters-noise-perlin", GIMP_ICON_GEGL,
     NC_("filters-action", "_Perlin Noise..."), NULL, NULL,
     "gegl:perlin-noise",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_PERLIIN */ },
+    GIMP_HELP_FILTER_NOISE_PERLIN },
 
-  { "filters-noise-pick", GIMP_STOCK_GEGL,
+  { "filters-noise-pick", GIMP_ICON_GEGL,
     NC_("filters-action", "_Pick..."), NULL, NULL,
     "gegl:noise-pick",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_PICK */ },
+    GIMP_HELP_FILTER_NOISE_PICK },
 
-  { "filters-noise-rgb", GIMP_STOCK_GEGL,
+  { "filters-noise-rgb", GIMP_ICON_GEGL,
     NC_("filters-action", "_RGB Noise..."), NULL, NULL,
     "gegl:noise-rgb",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_RGB */ },
+    GIMP_HELP_FILTER_NOISE_RGB },
 
-  { "filters-noise-reduction", GIMP_STOCK_GEGL,
+  { "filters-noise-reduction", GIMP_ICON_GEGL,
     NC_("filters-action", "Noise R_eduction..."), NULL, NULL,
     "gegl:noise-reduction",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_REDUCTION */ },
+    GIMP_HELP_FILTER_NOISE_REDUCTION },
 
-  { "filters-noise-simplex", GIMP_STOCK_GEGL,
+  { "filters-noise-simplex", GIMP_ICON_GEGL,
     NC_("filters-action", "_Simplex Noise..."), NULL, NULL,
     "gegl:simplex-noise",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_SIMPLEX */ },
+    GIMP_HELP_FILTER_NOISE_SIMPLEX },
 
-  { "filters-noise-slur", GIMP_STOCK_GEGL,
+  { "filters-noise-slur", GIMP_ICON_GEGL,
     NC_("filters-action", "_Slur..."), NULL, NULL,
     "gegl:noise-slur",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_SLUR */ },
+    GIMP_HELP_FILTER_NOISE_SLUR },
 
-  { "filters-noise-solid", GIMP_STOCK_GEGL,
+  { "filters-noise-solid", GIMP_ICON_GEGL,
     NC_("filters-action", "_Solid Noise..."), NULL, NULL,
     "gegl:noise-solid",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_SOLID */ },
+    GIMP_HELP_FILTER_NOISE_SOLID },
 
-  { "filters-noise-spread", GIMP_STOCK_GEGL,
+  { "filters-noise-spread", GIMP_ICON_GEGL,
     NC_("filters-action", "Sp_read..."), NULL, NULL,
     "gegl:noise-spread",
-    NULL /* FIXME GIMP_HELP_FILTER_NOISE_SPREAD */ },
+    GIMP_HELP_FILTER_NOISE_SPREAD },
 
-  { "filters-oilify", GIMP_STOCK_GEGL,
+  { "filters-oilify", GIMP_ICON_GEGL,
     NC_("filters-action", "Oili_fy..."), NULL, NULL,
     "gegl:oilify",
-    NULL /* FIXME GIMP_HELP_FILTER_OILIFY */ },
+    GIMP_HELP_FILTER_OILIFY },
 
-  { "filters-panorama-projection", GIMP_STOCK_GEGL,
+  { "filters-panorama-projection", GIMP_ICON_GEGL,
     NC_("filters-action", "_Panorama Projection..."), NULL, NULL,
     "gegl:panorama-projection",
-    NULL /* FIXME GIMP_HELP_FILTER_PANORAMA_PROJECTION */ },
+    GIMP_HELP_FILTER_PANORAMA_PROJECTION },
 
-  { "filters-photocopy", GIMP_STOCK_GEGL,
+  { "filters-photocopy", GIMP_ICON_GEGL,
     NC_("filters-action", "_Photocopy..."), NULL, NULL,
     "gegl:photocopy",
-    NULL /* FIXME GIMP_HELP_FILTER_PHOTOCOPY */ },
+    GIMP_HELP_FILTER_PHOTOCOPY },
 
-  { "filters-pixelize", GIMP_STOCK_GEGL,
+  { "filters-pixelize", GIMP_ICON_GEGL,
     NC_("filters-action", "_Pixelize..."), NULL, NULL,
     "gegl:pixelize",
-    NULL /* FIXME GIMP_HELP_FILTER_PIXELIZE */ },
+    GIMP_HELP_FILTER_PIXELIZE },
 
-  { "filters-plasma", GIMP_STOCK_GEGL,
+  { "filters-plasma", GIMP_ICON_GEGL,
     NC_("filters-action", "_Plasma..."), NULL, NULL,
     "gegl:plasma",
-    NULL /* FIXME GIMP_HELP_FILTER_PLASMA */ },
+    GIMP_HELP_FILTER_PLASMA },
 
-  { "filters-polar-coordinates", GIMP_STOCK_GEGL,
+  { "filters-polar-coordinates", GIMP_ICON_GEGL,
     NC_("filters-action", "P_olar Coordinates..."), NULL, NULL,
     "gegl:polar-coordinates",
-    NULL /* FIXME GIMP_HELP_FILTER_POLAR_COORDINATES */ },
+    GIMP_HELP_FILTER_POLAR_COORDINATES },
 
-  { "filters-posterize", GIMP_STOCK_TOOL_POSTERIZE,
+  { "filters-posterize", GIMP_ICON_TOOL_POSTERIZE,
     NC_("filters-action", "_Posterize..."), NULL, NULL,
     "gimp:posterize",
-    GIMP_HELP_TOOL_POSTERIZE },
+    GIMP_HELP_FILTER_POSTERIZE },
 
-  { "filters-red-eye-removal", GIMP_STOCK_GEGL,
+  { "filters-red-eye-removal", GIMP_ICON_GEGL,
     NC_("filters-action", "_Red Eye Removal..."), NULL, NULL,
     "gegl:red-eye-removal",
-    NULL /* FIXME GIMP_HELP_FILTER_RED_EYE_REMOVAL */ },
+    GIMP_HELP_FILTER_RED_EYE_REMOVAL },
 
-  { "filters-ripple", GIMP_STOCK_GEGL,
+  { "filters-reinhard-2005", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Reinhard 2005..."), NULL, NULL,
+    "gegl:reinhard05",
+    GIMP_HELP_FILTER_REINHARD_2005 },
+
+  { "filters-rgb-clip", GIMP_ICON_GEGL,
+    NC_("filters-action", "RGB _Clip..."), NULL, NULL,
+    "gegl:rgb-clip",
+    GIMP_HELP_FILTER_RGB_CLIP },
+
+  { "filters-ripple", GIMP_ICON_GEGL,
     NC_("filters-action", "_Ripple..."), NULL, NULL,
     "gegl:ripple",
-    NULL /* FIXME GIMP_HELP_FILTER_RIPPLE */ },
+    GIMP_HELP_FILTER_RIPPLE },
 
-  { "filters-saturation", GIMP_STOCK_GEGL,
+  { "filters-saturation", GIMP_ICON_GEGL,
     NC_("filters-action", "Sat_uration..."), NULL, NULL,
     "gegl:saturation",
-    NULL /* FIXME GIMP_HELP_FILTER_SATURATION */ },
+    GIMP_HELP_FILTER_SATURATION },
 
-  { "filters-semi-flatten", GIMP_STOCK_GEGL,
+  { "filters-semi-flatten", GIMP_ICON_GEGL,
     NC_("filters-action", "_Semi-Flatten..."), NULL, NULL,
     "gimp:semi-flatten",
-    NULL /* FIXME GIMP_HELP_FILTER_SEMI_FLATTEN */ },
+    GIMP_HELP_FILTER_SEMI_FLATTEN },
 
-  { "filters-sepia", GIMP_STOCK_GEGL,
+  { "filters-sepia", GIMP_ICON_GEGL,
     NC_("filters-action", "_Sepia..."), NULL, NULL,
     "gegl:sepia",
-    NULL /* FIXME GIMP_HELP_FILTER_SEPIA */ },
+    GIMP_HELP_FILTER_SEPIA },
 
-  { "filters-shift", GIMP_STOCK_GEGL,
+  { "filters-shift", GIMP_ICON_GEGL,
     NC_("filters-action", "_Shift..."), NULL, NULL,
     "gegl:shift",
-    NULL /* FIXME GIMP_HELP_FILTER_SHIFT */ },
+    GIMP_HELP_FILTER_SHIFT },
 
-  { "filters-sinus", GIMP_STOCK_GEGL,
+  { "filters-sinus", GIMP_ICON_GEGL,
     NC_("filters-action", "_Sinus..."), NULL, NULL,
     "gegl:sinus",
-    NULL /* FIXME GIMP_HELP_FILTER_SINUS */ },
+    GIMP_HELP_FILTER_SINUS },
 
-  { "filters-softglow", GIMP_STOCK_GEGL,
+  { "filters-slic", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Simple Linear Iterative Clustering..."), NULL, NULL,
+    "gegl:slic",
+    GIMP_HELP_FILTER_SLIC },
+
+  { "filters-snn-mean", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Symmetric Nearest Neighbor..."), NULL, NULL,
+    "gegl:snn-mean",
+    GIMP_HELP_FILTER_SNN_MEAN },
+
+  { "filters-softglow", GIMP_ICON_GEGL,
     NC_("filters-action", "_Softglow..."), NULL, NULL,
     "gegl:softglow",
-    NULL /* FIXME GIMP_HELP_FILTER_SOFTGLOW */ },
+    GIMP_HELP_FILTER_SOFTGLOW },
 
-  { "filters-stretch-contrast", GIMP_STOCK_GEGL,
+  { "filters-stretch-contrast", GIMP_ICON_GEGL,
     NC_("filters-action", "_Stretch Contrast..."), NULL, NULL,
     "gegl:stretch-contrast",
-    NULL /* FIXME GIMP_HELP_FILTER_STRETCH_CONTRAST */ },
+    GIMP_HELP_FILTER_STRETCH_CONTRAST },
 
-  { "filters-stretch-contrast-hsv", GIMP_STOCK_GEGL,
+  { "filters-stretch-contrast-hsv", GIMP_ICON_GEGL,
     NC_("filters-action", "_Stretch Contrast HSV..."), NULL, NULL,
     "gegl:stretch-contrast-hsv",
-    NULL /* FIXME GIMP_HELP_FILTER_STRETCH_CONTRAST_HSV */ },
+    GIMP_HELP_FILTER_STRETCH_CONTRAST_HSV },
 
-  { "filters-supernova", GIMP_STOCK_GEGL,
+  { "filters-stress", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Stress..."), NULL, NULL,
+    "gegl:stress",
+    GIMP_HELP_FILTER_STRESS },
+
+  { "filters-supernova", GIMP_ICON_GEGL,
     NC_("filters-action", "Super_nova..."), NULL, NULL,
     "gegl:supernova",
-    NULL /* FIXME GIMP_HELP_FILTER_SUPERNOVA */ },
+    GIMP_HELP_FILTER_SUPERNOVA },
 
-  { "filters-threshold-alpha", GIMP_STOCK_GEGL,
+  { "filters-threshold-alpha", GIMP_ICON_GEGL,
     NC_("filters-action", "_Threshold Alpha..."), NULL, NULL,
     "gimp:threshold-alpha",
-    NULL /* FIXME GIMP_HELP_FILTER_THRESHOLD_ALPHA */ },
+    GIMP_HELP_FILTER_THRESHOLD_ALPHA },
 
-  { "filters-tile-glass", GIMP_STOCK_GEGL,
+  { "filters-tile-glass", GIMP_ICON_GEGL,
     NC_("filters-action", "_Glass Tile..."), NULL, NULL,
     "gegl:tile-glass",
-    NULL /* FIXME GIMP_HELP_FILTER_TILE_GLASS */ },
+    GIMP_HELP_FILTER_TILE_GLASS },
 
-  { "filters-tile-paper", GIMP_STOCK_GEGL,
+  { "filters-tile-paper", GIMP_ICON_GEGL,
     NC_("filters-action", "_Paper Tile..."), NULL, NULL,
     "gegl:tile-paper",
-    NULL /* FIXME GIMP_HELP_FILTER_TILE_GLASS */ },
+    GIMP_HELP_FILTER_TILE_PAPER },
 
-  { "filters-tile-seamless", GIMP_STOCK_GEGL,
+  { "filters-tile-seamless", GIMP_ICON_GEGL,
     NC_("filters-action", "_Tile Seamless..."), NULL, NULL,
     "gegl:tile-seamless",
-    NULL /* FIXME GIMP_HELP_FILTER_TILE_SEAMLESS */ },
+    GIMP_HELP_FILTER_TILE_SEAMLESS },
 
-  { "filters-unsharp-mask", GIMP_STOCK_GEGL,
+  { "filters-unsharp-mask", GIMP_ICON_GEGL,
     NC_("filters-action", "_Unsharp Mask..."), NULL, NULL,
     "gegl:unsharp-mask",
-    NULL /* FIXME GIMP_HELP_FILTER_UNSHARP_MASK */ },
+    GIMP_HELP_FILTER_UNSHARP_MASK },
 
-  { "filters-value-propagate", GIMP_STOCK_GEGL,
+  { "filters-value-propagate", GIMP_ICON_GEGL,
     NC_("filters-action", "_Value Propagate..."), NULL, NULL,
     "gegl:value-propagate",
-    NULL /* FIXME GIMP_HELP_FILTER_VALUE_PROPAGATE */ },
+    GIMP_HELP_FILTER_VALUE_PROPAGATE },
 
-  { "filters-video-degradation", GIMP_STOCK_GEGL,
+  { "filters-video-degradation", GIMP_ICON_GEGL,
     NC_("filters-action", "Vi_deo Degradation..."), NULL, NULL,
     "gegl:video-degradation",
-    NULL /* FIXME GIMP_HELP_FILTER_VIDEO_DEGRADATION */ },
+    GIMP_HELP_FILTER_VIDEO_DEGRADATION },
 
-  { "filters-vignette", GIMP_STOCK_GEGL,
+  { "filters-vignette", GIMP_ICON_GEGL,
     NC_("filters-action", "_Vignette..."), NULL, NULL,
     "gegl:vignette",
-    NULL /* FIXME GIMP_HELP_FILTER_VIGNETTE */ },
+    GIMP_HELP_FILTER_VIGNETTE },
 
-  { "filters-waves", GIMP_STOCK_GEGL,
+  { "filters-waterpixels", GIMP_ICON_GEGL,
+    NC_("filters-action", "_Waterpixels..."), NULL, NULL,
+    "gegl:waterpixels",
+    GIMP_HELP_FILTER_WATERPIXELS },
+
+  { "filters-waves", GIMP_ICON_GEGL,
     NC_("filters-action", "_Waves..."), NULL, NULL,
     "gegl:waves",
-    NULL /* FIXME GIMP_HELP_FILTER_WAVES */ },
+    GIMP_HELP_FILTER_WAVES },
 
-  { "filters-whirl-pinch", GIMP_STOCK_GEGL,
+  { "filters-whirl-pinch", GIMP_ICON_GEGL,
     NC_("filters-action", "W_hirl and Pinch..."), NULL, NULL,
     "gegl:whirl-pinch",
-    NULL /* FIXME GIMP_HELP_FILTER_WHIRL_PINCH */ },
+    GIMP_HELP_FILTER_WHIRL_PINCH },
 
-  { "filters-wind", GIMP_STOCK_GEGL,
+  { "filters-wind", GIMP_ICON_GEGL,
     NC_("filters-action", "W_ind..."), NULL, NULL,
     "gegl:wind",
-    NULL /* FIXME GIMP_HELP_FILTER_WIND */ }
+    GIMP_HELP_FILTER_WIND }
 };
 
 static const GimpEnumActionEntry filters_repeat_actions[] =
 {
-  { "filters-repeat", "system-run",
+  { "filters-repeat", GIMP_ICON_SYSTEM_RUN,
     NC_("filters-action", "Re_peat Last"), "<primary>F",
     NC_("filters-action",
         "Rerun the last used filter using the same settings"),
     GIMP_RUN_WITH_LAST_VALS, FALSE,
     GIMP_HELP_FILTER_REPEAT },
 
-  { "filters-reshow", GIMP_STOCK_RESHOW_FILTER,
+  { "filters-reshow", GIMP_ICON_DIALOG_RESHOW_FILTER,
     NC_("filters-action", "R_e-Show Last"), "<primary><shift>F",
     NC_("filters-action", "Show the last used filter dialog again"),
     GIMP_RUN_INTERACTIVE, FALSE,
@@ -584,7 +644,7 @@ filters_actions_setup (GimpActionGroup *group)
 
   for (i = 0; i < n_entries; i++)
     {
-      entries[i].name        = g_strdup_printf ("filter-recent-%02d", i + 1);
+      entries[i].name        = g_strdup_printf ("filters-recent-%02d", i + 1);
       entries[i].icon_name   = NULL;
       entries[i].label       = "";
       entries[i].accelerator = "";
@@ -644,7 +704,7 @@ filters_actions_update (GimpActionGroup *group,
           if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
             writable = FALSE;
         }
-    }
+   }
 
 #define SET_SENSITIVE(action,condition) \
         gimp_action_group_set_action_sensitive (group, action, (condition) != 0)
@@ -660,10 +720,11 @@ filters_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("filters-checkerboard",            writable);
   SET_SENSITIVE ("filters-color-enhance",           writable && !gray);
   SET_SENSITIVE ("filters-color-exchange",          writable);
-  SET_SENSITIVE ("filters-color-reduction",         writable);
+  SET_SENSITIVE ("filters-dither",                  writable);
   SET_SENSITIVE ("filters-color-rotate",            writable);
   SET_SENSITIVE ("filters-color-temperature",       writable && !gray);
   SET_SENSITIVE ("filters-color-to-alpha",          writable && !gray && alpha);
+  SET_SENSITIVE ("filters-component-extract",       writable);
   SET_SENSITIVE ("filters-convolution-matrix",      writable);
   SET_SENSITIVE ("filters-cubism",                  writable);
   SET_SENSITIVE ("filters-deinterlace",             writable);
@@ -675,19 +736,24 @@ filters_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("filters-dropshadow",              writable && alpha);
   SET_SENSITIVE ("filters-edge",                    writable);
   SET_SENSITIVE ("filters-edge-laplace",            writable);
+  SET_SENSITIVE ("filters-edge-neon",               writable);
   SET_SENSITIVE ("filters-edge-sobel",              writable);
   SET_SENSITIVE ("filters-emboss",                  writable);
   SET_SENSITIVE ("filters-engrave",                 writable);
   SET_SENSITIVE ("filters-exposure",                writable);
+  SET_SENSITIVE ("filters-fattal-2002",             writable);
   SET_SENSITIVE ("filters-fractal-trace",           writable);
   SET_SENSITIVE ("filters-gaussian-blur",           writable);
   SET_SENSITIVE ("filters-gaussian-blur-selective", writable);
+  SET_SENSITIVE ("filters-gegl-graph",              writable);
   SET_SENSITIVE ("filters-grid",                    writable);
   SET_SENSITIVE ("filters-high-pass",               writable);
   SET_SENSITIVE ("filters-illusion",                writable);
+  SET_SENSITIVE ("filters-image-gradient",          writable);
   SET_SENSITIVE ("filters-kaleidoscope",            writable);
   SET_SENSITIVE ("filters-lens-distortion",         writable);
   SET_SENSITIVE ("filters-lens-flare",              writable);
+  SET_SENSITIVE ("filters-mantiuk-2006",            writable);
   SET_SENSITIVE ("filters-maze",                    writable);
   SET_SENSITIVE ("filters-mono-mixer",              writable && !gray);
   SET_SENSITIVE ("filters-mosaic",                  writable);
@@ -714,15 +780,20 @@ filters_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("filters-polar-coordinates",       writable);
   SET_SENSITIVE ("filters-posterize",               writable);
   SET_SENSITIVE ("filters-red-eye-removal",         writable && !gray);
+  SET_SENSITIVE ("filters-reinhard-2005",           writable);
+  SET_SENSITIVE ("filters-rgb-clip",                writable);
   SET_SENSITIVE ("filters-ripple",                  writable);
   SET_SENSITIVE ("filters-saturation",              writable && !gray);
   SET_SENSITIVE ("filters-semi-flatten",            writable && alpha);
   SET_SENSITIVE ("filters-sepia",                   writable && !gray);
   SET_SENSITIVE ("filters-shift",                   writable);
   SET_SENSITIVE ("filters-sinus",                   writable);
+  SET_SENSITIVE ("filters-slic",                    writable);
+  SET_SENSITIVE ("filters-snn-mean",                writable);
   SET_SENSITIVE ("filters-softglow",                writable);
   SET_SENSITIVE ("filters-stretch-contrast",        writable);
   SET_SENSITIVE ("filters-stretch-contrast-hsv",    writable);
+  SET_SENSITIVE ("filters-stress",                  writable);
   SET_SENSITIVE ("filters-supernova",               writable);
   SET_SENSITIVE ("filters-threshold-alpha",         writable && alpha);
   SET_SENSITIVE ("filters-tile-glass",              writable);
@@ -732,6 +803,7 @@ filters_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("filters-value-propagate",         writable);
   SET_SENSITIVE ("filters-video-degradation",       writable);
   SET_SENSITIVE ("filters-vignette",                writable);
+  SET_SENSITIVE ("filters-waterpixels",             writable);
   SET_SENSITIVE ("filters-waves",                   writable);
   SET_SENSITIVE ("filters-whirl-pinch",             writable);
   SET_SENSITIVE ("filters-wind",                    writable);
@@ -751,11 +823,11 @@ filters_actions_update (GimpActionGroup *group,
       {
         gimp_action_group_set_action_sensitive (group, "filters-repeat", FALSE);
         gimp_action_group_set_action_sensitive (group, "filters-reshow", FALSE);
-      }
+     }
 
     for (i = 0; i < gimp_filter_history_length (group->gimp); i++)
       {
-        gchar    *name = g_strdup_printf ("filter-recent-%02d", i + 1);
+        gchar    *name = g_strdup_printf ("filters-recent-%02d", i + 1);
         gboolean  sensitive;
 
         proc = gimp_filter_history_nth (group->gimp, i);
@@ -766,7 +838,7 @@ filters_actions_update (GimpActionGroup *group,
 
         g_free (name);
       }
-  }
+ }
 }
 
 static GimpActionGroup *
@@ -846,7 +918,7 @@ filters_actions_history_changed (Gimp            *gimp,
                                               sensitive);
       gimp_action_group_set_action_sensitive (group, "filters-reshow",
                                               sensitive);
-    }
+   }
   else
     {
       gimp_action_group_set_action_label (group, "filters-repeat",
@@ -866,7 +938,7 @@ filters_actions_history_changed (Gimp            *gimp,
       gchar       *name;
       gboolean     sensitive = FALSE;
 
-      name = g_strdup_printf ("filter-recent-%02d", i + 1);
+      name = g_strdup_printf ("filters-recent-%02d", i + 1);
       action = gtk_action_group_get_action (GTK_ACTION_GROUP (group), name);
       g_free (name);
 
@@ -904,7 +976,7 @@ filters_actions_history_changed (Gimp            *gimp,
   for (; i < gimp_filter_history_size (gimp); i++)
     {
       GtkAction *action;
-      gchar     *name = g_strdup_printf ("filter-recent-%02d", i + 1);
+      gchar     *name = g_strdup_printf ("filters-recent-%02d", i + 1);
 
       action = gtk_action_group_get_action (GTK_ACTION_GROUP (group), name);
       g_free (name);
