@@ -47,9 +47,10 @@ enum
 {
   PROP_0,
   PROP_MOVE_TOOL_CHANGES_ACTIVE,
-  PROP_IMAGE_MAP_TOOL_MAX_RECENT,
+  PROP_FILTER_TOOL_MAX_RECENT,
   PROP_TRUST_DIRTY_FLAG,
   PROP_SAVE_DEVICE_STATUS,
+  PROP_DEVICES_SHARE_TOOL,
   PROP_SAVE_SESSION_INFO,
   PROP_RESTORE_SESSION,
   PROP_RESTORE_MONITOR,
@@ -91,6 +92,7 @@ enum
 
   /* ignored, only for backward compatibility: */
   PROP_CURSOR_FORMAT,
+  PROP_IMAGE_MAP_TOOL_MAX_RECENT,
   PROP_INFO_WINDOW_PER_DISPLAY,
   PROP_MENU_MNEMONICS,
   PROP_SHOW_TOOL_TIPS,
@@ -127,186 +129,281 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
   object_class->set_property = gimp_gui_config_set_property;
   object_class->get_property = gimp_gui_config_get_property;
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_MOVE_TOOL_CHANGES_ACTIVE,
-                                    "move-tool-changes-active",
-                                    MOVE_TOOL_CHANGES_ACTIVE_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_INT (object_class, PROP_IMAGE_MAP_TOOL_MAX_RECENT,
-                                "image-map-tool-max-recent",
-                                IMAGE_MAP_TOOL_MAX_RECENT_BLURB,
-                                0, 255, 10,
-                                GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TRUST_DIRTY_FLAG,
-                                    "trust-dirty-flag",
-                                    TRUST_DIRTY_FLAG_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SAVE_DEVICE_STATUS,
-                                    "save-device-status",
-                                    SAVE_DEVICE_STATUS_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SAVE_SESSION_INFO,
-                                    "save-session-info",
-                                    SAVE_SESSION_INFO_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_RESTORE_SESSION,
-                                    "restore-session", RESTORE_SESSION_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_RESTORE_MONITOR,
-                                    "restore-monitor", RESTORE_MONITOR_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SAVE_TOOL_OPTIONS,
-                                    "save-tool-options",
-                                    SAVE_TOOL_OPTIONS_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SHOW_TOOLTIPS,
-                                    "show-tooltips", SHOW_TOOLTIPS_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_RESTART);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TEAROFF_MENUS,
-                                    "tearoff-menus", TEAROFF_MENUS_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_CAN_CHANGE_ACCELS,
-                                    "can-change-accels", CAN_CHANGE_ACCELS_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SAVE_ACCELS,
-                                    "save-accels", SAVE_ACCELS_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_RESTORE_ACCELS,
-                                    "restore-accels", RESTORE_ACCELS_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_INT (object_class, PROP_LAST_OPENED_SIZE,
-                                "last-opened-size", LAST_OPENED_SIZE_BLURB,
-                                0, 1024, 10,
-                                GIMP_PARAM_STATIC_STRINGS |
-                                GIMP_CONFIG_PARAM_RESTART);
-  GIMP_CONFIG_INSTALL_PROP_MEMSIZE (object_class, PROP_MAX_NEW_IMAGE_SIZE,
-                                    "max-new-image-size",
-                                    MAX_NEW_IMAGE_SIZE_BLURB,
-                                    0, GIMP_MAX_MEMSIZE, 1 << 27, /* 128MB */
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TOOLBOX_COLOR_AREA,
-                                    "toolbox-color-area",
-                                    TOOLBOX_COLOR_AREA_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TOOLBOX_FOO_AREA,
-                                    "toolbox-foo-area",
-                                    TOOLBOX_FOO_AREA_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TOOLBOX_IMAGE_AREA,
-                                    "toolbox-image-area",
-                                    TOOLBOX_IMAGE_AREA_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TOOLBOX_WILBER,
-                                    "toolbox-wilber",
-                                    TOOLBOX_WILBER_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  path = gimp_config_build_data_path ("themes");
-  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_THEME_PATH,
-                                 "theme-path", THEME_PATH_BLURB,
-                                 GIMP_CONFIG_PATH_DIR_LIST, path,
-                                 GIMP_PARAM_STATIC_STRINGS |
-                                 GIMP_CONFIG_PARAM_RESTART);
-  g_free (path);
-  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_THEME,
-                                   "theme", THEME_BLURB,
-                                   GIMP_CONFIG_DEFAULT_THEME,
-                                   GIMP_PARAM_STATIC_STRINGS);
-  path = gimp_config_build_data_path ("icons");
-  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_ICON_THEME_PATH,
-                                 "icon-theme-path", ICON_THEME_PATH_BLURB,
-                                 GIMP_CONFIG_PATH_DIR_LIST, path,
-                                 GIMP_PARAM_STATIC_STRINGS |
-                                 GIMP_CONFIG_PARAM_RESTART);
-  g_free (path);
-  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_ICON_THEME,
-                                   "icon-theme", ICON_THEME_BLURB,
-                                   GIMP_CONFIG_DEFAULT_ICON_THEME,
-                                   GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_HELP,
-                                    "use-help", USE_HELP_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SHOW_HELP_BUTTON,
-                                    "show-help-button", SHOW_HELP_BUTTON_BLURB,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_HELP_LOCALES,
-                                   "help-locales", HELP_LOCALES_BLURB,
-                                   "",
-                                   GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_HELP_BROWSER,
-                                 "help-browser", HELP_BROWSER_BLURB,
-                                 GIMP_TYPE_HELP_BROWSER_TYPE,
-                                 DEFAULT_HELP_BROWSER,
-                                 GIMP_PARAM_STATIC_STRINGS);
-  /* As a default, we hide unavailable actions. */
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SEARCH_SHOW_UNAVAILABLE_ACTIONS,
-                                    "search-show-unavailable-actions", SEARCH_SHOW_UNAVAILABLE_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_INT (object_class, PROP_ACTION_HISTORY_SIZE,
-                                "action-history-size", ACTION_HISTORY_SIZE_BLURB,
-                                0, 1000, 100,
-                                GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USER_MANUAL_ONLINE,
-                                    "user-manual-online",
-                                    USER_MANUAL_ONLINE_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_USER_MANUAL_ONLINE_URI,
-                                   "user-manual-online-uri",
-                                   USER_MANUAL_ONLINE_URI_BLURB,
-                                   DEFAULT_USER_MANUAL_ONLINE_URI,
-                                   GIMP_PARAM_STATIC_STRINGS);
-  GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_DOCK_WINDOW_HINT,
-                                 "dock-window-hint",
-                                 DOCK_WINDOW_HINT_BLURB,
-                                 GIMP_TYPE_WINDOW_HINT,
-                                 GIMP_WINDOW_HINT_UTILITY,
-                                 GIMP_PARAM_STATIC_STRINGS |
-                                 GIMP_CONFIG_PARAM_RESTART);
-  GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_CURSOR_HANDEDNESS,
-                                 "cursor-handedness", CURSOR_HANDEDNESS_BLURB,
-                                 GIMP_TYPE_HANDEDNESS,
-                                 GIMP_HANDEDNESS_RIGHT,
-                                 GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_MOVE_TOOL_CHANGES_ACTIVE,
+                            "move-tool-changes-active",
+                            "Move tool changes active layer",
+                            MOVE_TOOL_CHANGES_ACTIVE_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_PLAYGROUND_NPD_TOOL,
-                                    "playground-npd-tool",
-                                    PLAYGROUND_NPD_TOOL_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_RESTART);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class,
-                                    PROP_PLAYGROUND_HANDLE_TRANSFORM_TOOL,
-                                    "playground-handle-transform-tool",
-                                    PLAYGROUND_HANDLE_TRANSFORM_TOOL_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_RESTART);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class,
-                                    PROP_PLAYGROUND_SEAMLESS_CLONE_TOOL,
-                                    "playground-seamless-clone-tool",
-                                    PLAYGROUND_SEAMLESS_CLONE_TOOL_BLURB,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_RESTART);
+  GIMP_CONFIG_PROP_INT (object_class, PROP_FILTER_TOOL_MAX_RECENT,
+                        "filter-tool-max-recent",
+                        "Max recent settings to keep in filters",
+                        FILTER_TOOL_MAX_RECENT_BLURB,
+                        0, 255, 10,
+                        GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TRUST_DIRTY_FLAG,
+                            "trust-dirty-flag",
+                            "Trust dirty flag",
+                            TRUST_DIRTY_FLAG_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SAVE_DEVICE_STATUS,
+                            "save-device-status",
+                            "Save device status",
+                            SAVE_DEVICE_STATUS_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_DEVICES_SHARE_TOOL,
+                            "devices-share-tool",
+                            "Devics share tool",
+                            DEVICES_SHARE_TOOL_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SAVE_SESSION_INFO,
+                            "save-session-info",
+                            "Save session",
+                            SAVE_SESSION_INFO_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_RESTORE_SESSION,
+                            "restore-session",
+                            "Restore session",
+                            RESTORE_SESSION_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_RESTORE_MONITOR,
+                            "restore-monitor",
+                            "Restore monitor",
+                            RESTORE_MONITOR_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SAVE_TOOL_OPTIONS,
+                            "save-tool-options",
+                            "Save tool options",
+                            SAVE_TOOL_OPTIONS_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_TOOLTIPS,
+                            "show-tooltips",
+                            "Show tooltips",
+                            SHOW_TOOLTIPS_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_RESTART);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TEAROFF_MENUS,
+                            "tearoff-menus",
+                            "Tearoff menus",
+                            TEAROFF_MENUS_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_CAN_CHANGE_ACCELS,
+                            "can-change-accels",
+                            "Can change accelerators",
+                            CAN_CHANGE_ACCELS_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SAVE_ACCELS,
+                            "save-accels",
+                            "Save accelerators",
+                            SAVE_ACCELS_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_RESTORE_ACCELS,
+                            "restore-accels",
+                            "Restore acclerator",
+                            RESTORE_ACCELS_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_INT (object_class, PROP_LAST_OPENED_SIZE,
+                        "last-opened-size",
+                        "Size of recently used menu",
+                        LAST_OPENED_SIZE_BLURB,
+                        0, 1024, 10,
+                        GIMP_PARAM_STATIC_STRINGS |
+                        GIMP_CONFIG_PARAM_RESTART);
+
+  GIMP_CONFIG_PROP_MEMSIZE (object_class, PROP_MAX_NEW_IMAGE_SIZE,
+                            "max-new-image-size",
+                            "Maximum new image size",
+                            MAX_NEW_IMAGE_SIZE_BLURB,
+                            0, GIMP_MAX_MEMSIZE, 1 << 27, /* 128MB */
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TOOLBOX_COLOR_AREA,
+                            "toolbox-color-area",
+                            "Show toolbox color area",
+                            TOOLBOX_COLOR_AREA_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TOOLBOX_FOO_AREA,
+                            "toolbox-foo-area",
+                            "Show toolbox foo area",
+                            TOOLBOX_FOO_AREA_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TOOLBOX_IMAGE_AREA,
+                            "toolbox-image-area",
+                            "Show toolbox image area",
+                            TOOLBOX_IMAGE_AREA_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TOOLBOX_WILBER,
+                            "toolbox-wilber",
+                            "Show toolbox wilber",
+                            TOOLBOX_WILBER_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  path = gimp_config_build_data_path ("themes");
+  GIMP_CONFIG_PROP_PATH (object_class, PROP_THEME_PATH,
+                         "theme-path",
+                         "Theme path",
+                         THEME_PATH_BLURB,
+                         GIMP_CONFIG_PATH_DIR_LIST, path,
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_RESTART);
+  g_free (path);
+
+  GIMP_CONFIG_PROP_STRING (object_class, PROP_THEME,
+                           "theme",
+                           "Theme",
+                           THEME_BLURB,
+                           GIMP_CONFIG_DEFAULT_THEME,
+                           GIMP_PARAM_STATIC_STRINGS);
+
+  path = gimp_config_build_data_path ("icons");
+  GIMP_CONFIG_PROP_PATH (object_class, PROP_ICON_THEME_PATH,
+                         "icon-theme-path",
+                         "Icon theme path",
+                         ICON_THEME_PATH_BLURB,
+                         GIMP_CONFIG_PATH_DIR_LIST, path,
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_RESTART);
+  g_free (path);
+
+  GIMP_CONFIG_PROP_STRING (object_class, PROP_ICON_THEME,
+                           "icon-theme",
+                           "Icon theme",
+                           ICON_THEME_BLURB,
+                           GIMP_CONFIG_DEFAULT_ICON_THEME,
+                           GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_HELP,
+                            "use-help",
+                            "Use help",
+                            USE_HELP_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_HELP_BUTTON,
+                            "show-help-button",
+                            "Show help button",
+                            SHOW_HELP_BUTTON_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_STRING (object_class, PROP_HELP_LOCALES,
+                           "help-locales",
+                           "Help locales",
+                           HELP_LOCALES_BLURB,
+                           "",
+                           GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_HELP_BROWSER,
+                         "help-browser",
+                         "Help browser",
+                         HELP_BROWSER_BLURB,
+                         GIMP_TYPE_HELP_BROWSER_TYPE,
+                         DEFAULT_HELP_BROWSER,
+                         GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USER_MANUAL_ONLINE,
+                            "user-manual-online",
+                            "User manual online",
+                            USER_MANUAL_ONLINE_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_STRING (object_class, PROP_USER_MANUAL_ONLINE_URI,
+                           "user-manual-online-uri",
+                           "User manual online URI",
+                           USER_MANUAL_ONLINE_URI_BLURB,
+                           DEFAULT_USER_MANUAL_ONLINE_URI,
+                           GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SEARCH_SHOW_UNAVAILABLE_ACTIONS,
+                            "search-show-unavailable-actions",
+                            "Show unavailable actions",
+                            SEARCH_SHOW_UNAVAILABLE_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_INT (object_class, PROP_ACTION_HISTORY_SIZE,
+                        "action-history-size",
+                        "Action history size",
+                        ACTION_HISTORY_SIZE_BLURB,
+                        0, 1000, 100,
+                        GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_DOCK_WINDOW_HINT,
+                         "dock-window-hint",
+                         "Dock window hint",
+                         DOCK_WINDOW_HINT_BLURB,
+                         GIMP_TYPE_WINDOW_HINT,
+                         GIMP_WINDOW_HINT_UTILITY,
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_RESTART);
+
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_CURSOR_HANDEDNESS,
+                         "cursor-handedness",
+                         "Cursor handedness",
+                         CURSOR_HANDEDNESS_BLURB,
+                         GIMP_TYPE_HANDEDNESS,
+                         GIMP_HANDEDNESS_RIGHT,
+                         GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_PLAYGROUND_NPD_TOOL,
+                            "playground-npd-tool",
+                            "Playground N-Point Deformation tool",
+                            PLAYGROUND_NPD_TOOL_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_RESTART);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class,
+                            PROP_PLAYGROUND_HANDLE_TRANSFORM_TOOL,
+                            "playground-handle-transform-tool",
+                            "Playground Handle Transform tool",
+                            PLAYGROUND_HANDLE_TRANSFORM_TOOL_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_RESTART);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class,
+                            PROP_PLAYGROUND_SEAMLESS_CLONE_TOOL,
+                            "playground-seamless-clone-tool",
+                            "Playground Seamless Clone tool",
+                            PLAYGROUND_SEAMLESS_CLONE_TOOL_BLURB,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_RESTART);
 
   g_object_class_install_property (object_class, PROP_HIDE_DOCKS,
                                    g_param_spec_boolean ("hide-docks",
@@ -339,50 +436,71 @@ gimp_gui_config_class_init (GimpGuiConfigClass *klass)
                                                      GIMP_PARAM_STATIC_STRINGS));
 
   /*  only for backward compatibility:  */
-  GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_CURSOR_FORMAT,
-                                 "cursor-format", CURSOR_FORMAT_BLURB,
-                                 GIMP_TYPE_CURSOR_FORMAT,
-                                 GIMP_CURSOR_FORMAT_PIXBUF,
-                                 GIMP_PARAM_STATIC_STRINGS |
-                                 GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_INFO_WINDOW_PER_DISPLAY,
-                                    "info-window-per-display",
-                                    NULL,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_MENU_MNEMONICS,
-                                    "menu-mnemonics", NULL,
-                                    TRUE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SHOW_TOOL_TIPS,
-                                    "show-tool-tips", NULL,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_SHOW_TIPS,
-                                    "show-tips", NULL,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_ENUM (object_class, PROP_TOOLBOX_WINDOW_HINT,
-                                 "toolbox-window-hint", NULL,
-                                 GIMP_TYPE_WINDOW_HINT,
-                                 GIMP_WINDOW_HINT_UTILITY,
-                                 GIMP_PARAM_STATIC_STRINGS |
-                                 GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_TRANSIENT_DOCKS,
-                                    "transient-docks", NULL,
-                                    FALSE,
-                                    GIMP_PARAM_STATIC_STRINGS |
-                                    GIMP_CONFIG_PARAM_IGNORE);
-  GIMP_CONFIG_INSTALL_PROP_PATH (object_class, PROP_WEB_BROWSER,
-                                 "web-browser", NULL,
-                                 GIMP_CONFIG_PATH_FILE,
-                                 "not used any longer",
-                                 GIMP_PARAM_STATIC_STRINGS |
-                                 GIMP_CONFIG_PARAM_IGNORE);
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_CURSOR_FORMAT,
+                         "cursor-format",
+                         NULL, NULL,
+                         GIMP_TYPE_CURSOR_FORMAT,
+                         GIMP_CURSOR_FORMAT_PIXBUF,
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_INT (object_class, PROP_IMAGE_MAP_TOOL_MAX_RECENT,
+                        "image-map-tool-max-recent",
+                        NULL, NULL,
+                        0, 255, 10,
+                        GIMP_PARAM_STATIC_STRINGS |
+                        GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_INFO_WINDOW_PER_DISPLAY,
+                            "info-window-per-display",
+                            NULL, NULL,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_MENU_MNEMONICS,
+                            "menu-mnemonics",
+                            NULL, NULL,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_TOOL_TIPS,
+                            "show-tool-tips",
+                            NULL, NULL,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SHOW_TIPS,
+                            "show-tips",
+                            NULL, NULL,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_TOOLBOX_WINDOW_HINT,
+                         "toolbox-window-hint",
+                         NULL, NULL,
+                         GIMP_TYPE_WINDOW_HINT,
+                         GIMP_WINDOW_HINT_UTILITY,
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_TRANSIENT_DOCKS,
+                            "transient-docks",
+                            NULL, NULL,
+                            FALSE,
+                            GIMP_PARAM_STATIC_STRINGS |
+                            GIMP_CONFIG_PARAM_IGNORE);
+
+  GIMP_CONFIG_PROP_PATH (object_class, PROP_WEB_BROWSER,
+                         "web-browser",
+                         NULL, NULL,
+                         GIMP_CONFIG_PATH_FILE,
+                         "not used any longer",
+                         GIMP_PARAM_STATIC_STRINGS |
+                         GIMP_CONFIG_PARAM_IGNORE);
 }
 
 static void
@@ -400,7 +518,6 @@ gimp_gui_config_finalize (GObject *object)
   g_free (gui_config->icon_theme_path);
   g_free (gui_config->icon_theme);
   g_free (gui_config->help_locales);
-  g_free (gui_config->web_browser);
   g_free (gui_config->user_manual_online_uri);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -419,8 +536,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_MOVE_TOOL_CHANGES_ACTIVE:
       gui_config->move_tool_changes_active = g_value_get_boolean (value);
       break;
+    case PROP_FILTER_TOOL_MAX_RECENT:
     case PROP_IMAGE_MAP_TOOL_MAX_RECENT:
-      gui_config->image_map_tool_max_recent = g_value_get_int (value);
+      gui_config->filter_tool_max_recent = g_value_get_int (value);
       break;
     case PROP_TRUST_DIRTY_FLAG:
       gui_config->trust_dirty_flag = g_value_get_boolean (value);
@@ -428,6 +546,9 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_SAVE_DEVICE_STATUS:
       gui_config->save_device_status = g_value_get_boolean (value);
       break;
+    case PROP_DEVICES_SHARE_TOOL:
+      gui_config->devices_share_tool = g_value_get_boolean (value);
+      break;
     case PROP_SAVE_SESSION_INFO:
       gui_config->save_session_info = g_value_get_boolean (value);
       break;
@@ -502,12 +623,6 @@ gimp_gui_config_set_property (GObject      *object,
     case PROP_HELP_BROWSER:
       gui_config->help_browser = g_value_get_enum (value);
       break;
-    case PROP_SEARCH_SHOW_UNAVAILABLE_ACTIONS:
-      gui_config->search_show_unavailable = g_value_get_boolean (value);
-      break;
-    case PROP_ACTION_HISTORY_SIZE:
-      gui_config->action_history_size = g_value_get_int (value);
-      break;
     case PROP_USER_MANUAL_ONLINE:
       gui_config->user_manual_online = g_value_get_boolean (value);
       break;
@@ -515,6 +630,12 @@ gimp_gui_config_set_property (GObject      *object,
       g_free (gui_config->user_manual_online_uri);
       gui_config->user_manual_online_uri = g_value_dup_string (value);
       break;
+    case PROP_SEARCH_SHOW_UNAVAILABLE_ACTIONS:
+      gui_config->search_show_unavailable = g_value_get_boolean (value);
+      break;
+    case PROP_ACTION_HISTORY_SIZE:
+      gui_config->action_history_size = g_value_get_int (value);
+      break;
     case PROP_DOCK_WINDOW_HINT:
       gui_config->dock_window_hint = g_value_get_enum (value);
       break;
@@ -575,8 +696,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_MOVE_TOOL_CHANGES_ACTIVE:
       g_value_set_boolean (value, gui_config->move_tool_changes_active);
       break;
+    case PROP_FILTER_TOOL_MAX_RECENT:
     case PROP_IMAGE_MAP_TOOL_MAX_RECENT:
-      g_value_set_int (value, gui_config->image_map_tool_max_recent);
+      g_value_set_int (value, gui_config->filter_tool_max_recent);
       break;
     case PROP_TRUST_DIRTY_FLAG:
       g_value_set_boolean (value, gui_config->trust_dirty_flag);
@@ -584,6 +706,9 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_SAVE_DEVICE_STATUS:
       g_value_set_boolean (value, gui_config->save_device_status);
       break;
+    case PROP_DEVICES_SHARE_TOOL:
+      g_value_set_boolean (value, gui_config->devices_share_tool);
+      break;
     case PROP_SAVE_SESSION_INFO:
       g_value_set_boolean (value, gui_config->save_session_info);
       break;
@@ -653,18 +778,18 @@ gimp_gui_config_get_property (GObject    *object,
     case PROP_HELP_BROWSER:
       g_value_set_enum (value, gui_config->help_browser);
       break;
-    case PROP_SEARCH_SHOW_UNAVAILABLE_ACTIONS:
-      g_value_set_boolean (value, gui_config->search_show_unavailable);
-      break;
-    case PROP_ACTION_HISTORY_SIZE:
-      g_value_set_int (value, gui_config->action_history_size);
-      break;
     case PROP_USER_MANUAL_ONLINE:
       g_value_set_boolean (value, gui_config->user_manual_online);
       break;
     case PROP_USER_MANUAL_ONLINE_URI:
       g_value_set_string (value, gui_config->user_manual_online_uri);
       break;
+    case PROP_SEARCH_SHOW_UNAVAILABLE_ACTIONS:
+      g_value_set_boolean (value, gui_config->search_show_unavailable);
+      break;
+    case PROP_ACTION_HISTORY_SIZE:
+      g_value_set_int (value, gui_config->action_history_size);
+      break;
     case PROP_DOCK_WINDOW_HINT:
       g_value_set_enum (value, gui_config->dock_window_hint);
       break;
