@@ -42,9 +42,10 @@ struct _GimpGuiConfig
   GimpDisplayConfig    parent_instance;
 
   gboolean             move_tool_changes_active;
-  gint                 image_map_tool_max_recent;
+  gint                 filter_tool_max_recent;
   gboolean             trust_dirty_flag;
   gboolean             save_device_status;
+  gboolean             devices_share_tool;
   gboolean             save_session_info;
   gboolean             restore_session;
   gboolean             restore_monitor;
@@ -68,14 +69,10 @@ struct _GimpGuiConfig
   gboolean             show_help_button;
   gchar               *help_locales;
   GimpHelpBrowserType  help_browser;
-
-  /* action search preferences */
-  gboolean             search_show_unavailable;
-  gint                 action_history_size;
-
-  gchar               *web_browser;
   gboolean             user_manual_online;
   gchar               *user_manual_online_uri;
+  gboolean             search_show_unavailable;
+  gint                 action_history_size;
   GimpWindowHint       dock_window_hint;
   GimpHandedness       cursor_handedness;
 
