@@ -42,9 +42,6 @@ _("Specifies how the area around the image should be drawn.")
 #define COLOR_PROFILE_POLICY_BLURB \
 _("How to handle embedded color profiles when opening a file.")
 
-#define CURSOR_FORMAT_BLURB \
-_("Sets the pixel format to use for mouse pointers.")
-
 #define CURSOR_MODE_BLURB \
 _("Sets the type of mouse pointers to use.")
 
@@ -175,8 +172,8 @@ _("Sets the browser used by the help system.")
 "colon-separated list of language identifiers with decreasing priority. " \
 "If empty, the language is taken from the user's locale setting."
 
-#define IMAGE_MAP_TOOL_MAX_RECENT_BLURB \
-"How many recent settings to keep around in color correction tools"
+#define FILTER_TOOL_MAX_RECENT_BLURB \
+"How many recent settings to keep around in filter tools"
 
 #define IMAGE_STATUS_FORMAT_BLURB \
 _("Sets the text to appear in image window status bars.")
@@ -184,6 +181,17 @@ _("Sets the text to appear in image window status bars.")
 #define IMAGE_TITLE_FORMAT_BLURB \
 _("Sets the text to appear in image window titles.")
 
+#define IMPORT_PROMOTE_FLOAT_BLURB \
+_("Promote imported images to floating point precision. Does not apply " \
+  "to indexed images.")
+
+#define IMPORT_PROMOTE_DITHER_BLURB \
+_("When promoting imported images to floating point precision, also add " \
+  "minimal noise in order do distribute color values a bit.")
+
+#define IMPORT_ADD_ALPHA_BLURB \
+_("Add an alpha channel to all layers of imported images.")
+
 #define INITIAL_ZOOM_TO_FIT_BLURB \
 _("When enabled, this will ensure that the full image is visible after a " \
   "file is opened, otherwise it will be displayed with a scale of 1:1.")
@@ -251,13 +259,6 @@ _("Sets how many processors GIMP should try to use simultaneously.")
 
 #define PATTERN_PATH_WRITABLE_BLURB ""
 
-#define PERFECT_MOUSE_BLURB \
-_("When enabled, the X server is queried for the mouse's current position " \
-  "on each motion event, rather than relying on the position hint.  This " \
-  "means painting with large brushes should be more accurate, but it may " \
-  "be slower.  Conversely, on some X servers enabling this option results " \
-  "in faster painting.")
-
 #define FILTER_HISTORY_SIZE_BLURB \
 "How many recently used filters and plug-ins to keep on the Filters menu."
 
@@ -301,6 +302,11 @@ _("When enabled, GIMP will try to restore windows on the monitor they " \
 _("Remember the current tool, pattern, color, and brush across GIMP " \
   "sessions.")
 
+#define DEVICES_SHARE_TOOL_BLURB \
+_("When enabled, the same tool and tool options will be used for all " \
+  "input devices. No tool switching will occur when the input device " \
+  "changes.")
+
 #define SAVE_DOCUMENT_HISTORY_BLURB \
 _("Keep a permanent record of all opened and saved files in the Recent " \
   "Documents list.")
@@ -430,6 +436,141 @@ _("Sets the folder for temporary storage. Files will appear here " \
 #define ICON_THEME_PATH_BLURB \
 "Sets the icon theme search path."
 
+#define IMAGE_CONVERT_PROFILE_INTENT_BLURB \
+_("Sets the default rendering intent for the 'Convert to Color Profile' dialog.")
+
+#define IMAGE_CONVERT_PROFILE_BPC_BLURB \
+_("Sets the default 'Black Point Compensation' state for the " \
+  "'Convert to Color Profile' dialog.")
+
+#define IMAGE_CONVERT_PRECISION_LAYER_DITHER_METHOD_BLURB \
+_("Sets the default layer dithering method for the 'Convert Precision' dialog.")
+
+#define IMAGE_CONVERT_PRECISION_TEXT_LAYER_DITHER_METHOD_BLURB \
+_("Sets the default text layer dithering method for the 'Convert Precision' dialog.")
+
+#define IMAGE_CONVERT_PRECISION_CHANNEL_DITHER_METHOD_BLURB \
+_("Sets the default channel dithering method for the 'Convert Precision' dialog.")
+
+#define IMAGE_CONVERT_INDEXED_PALETTE_TYPE_BLURB \
+_("Sets the default palette type for the 'Convert to Indexed' dialog.")
+
+#define IMAGE_CONVERT_INDEXED_MAX_COLORS_BLURB \
+_("Sets the default maximum number of colors for the 'Convert to Indexed' dialog.")
+
+#define IMAGE_CONVERT_INDEXED_REMOVE_DUPLICATES_BLURB \
+_("Sets the default 'Remove duplicate colors' state for the 'Convert to Indexed' dialog.")
+
+#define IMAGE_CONVERT_INDEXED_DITHER_TYPE_BLURB \
+_("Sets the default dithering type for the 'Convert to Indexed' dialog.")
+
+#define IMAGE_CONVERT_INDEXED_DITHER_ALPHA_BLURB \
+_("Sets the default 'Dither alpha' state for the 'Convert to Indexed' dialog.")
+
+#define IMAGE_CONVERT_INDEXED_DITHER_TEXT_LAYERS_BLURB \
+_("Sets the default 'Dither text layers' state for the 'Convert to Indexed' dialog.")
+
+#define IMAGE_RESIZE_FILL_TYPE_BLURB \
+_("Sets the default fill type for the 'Canvas Size' dialog.")
+
+#define IMAGE_RESIZE_LAYER_SET_BLURB \
+_("Sets the default set of layers to resize for the 'Canvas Size' dialog.")
+
+#define IMAGE_RESIZE_RESIZE_TEXT_LAYERS_BLURB \
+_("Sets the default 'Resize text layers' state for the 'Canvas Size' dialog.")
+
+#define LAYER_NEW_NAME_BLURB \
+_("Sets the default layer name for the 'New Layer' dialog.")
+
+#define LAYER_NEW_MODE_BLURB \
+_("Sets the default mode for the 'New Layer' dialog.")
+
+#define LAYER_NEW_BLEND_SPACE_BLURB \
+_("Sets the default blend space for the 'New Layer' dialog.")
+
+#define LAYER_NEW_COMPOSITE_SPACE_BLURB \
+_("Sets the default composite space for the 'New Layer' dialog.")
+
+#define LAYER_NEW_COMPOSITE_MODE_BLURB \
+_("Sets the default composite mode for the 'New Layer' dialog.")
+
+#define LAYER_NEW_OPACITY_BLURB \
+_("Sets the default opacity for the 'New Layer' dialog.")
+
+#define LAYER_NEW_FILL_TYPE_BLURB \
+_("Sets the default fill type for the 'New Layer' dialog.")
+
+#define LAYER_RESIZE_FILL_TYPE_BLURB \
+_("Sets the default fill type for the 'Layer Boundary Size' dialog.")
+
+#define LAYER_ADD_MASK_TYPE_BLURB \
+_("Sets the default mask for the 'Add Layer Mask' dialog.")
+
+#define LAYER_ADD_MASK_INVERT_BLURB \
+_("Sets the default 'invert mask' state for the 'Add Layer Mask' dialog.")
+
+#define LAYER_MERGE_TYPE_BLURB \
+_("Sets the default merge type for the 'Merge Visible Layers' dialog.")
+
+#define LAYER_MERGE_ACTIVE_GROUP_ONLY_BLURB \
+_("Sets the default 'Active group only' for the 'Merge Visible Layers' dialog.")
+
+#define LAYER_MERGE_DISCARD_INVISIBLE_BLURB \
+_("Sets the default 'Discard invisible' for the 'Merge Visible Layers' dialog.")
+
+#define CHANNEL_NEW_NAME_BLURB \
+_("Sets the default channel name for the 'New Channel' dialog.")
+
+#define CHANNEL_NEW_COLOR_BLURB \
+_("Sets the default color and opacity for the 'New Channel' dialog.")
+
+#define VECTORS_NEW_NAME_BLURB \
+_("Sets the default path name for the 'New Path' dialog.")
+
+#define VECTORS_EXPORT_PATH_BLURB \
+_("Sets the default folder path for the 'Export Path' dialog.")
+
+#define VECTORS_EXPORT_ACTIVE_ONLY_BLURB \
+_("Sets the default 'Export the active path' state for the 'Export Path' dialog.")
+
+#define VECTORS_IMPORT_PATH_BLURB \
+_("Sets the default folder path for the 'Import Path' dialog.")
+
+#define VECTORS_IMPORT_MERGE_BLURB \
+_("Sets the default 'Merge imported paths' state for the 'Import Path' dialog.")
+
+#define VECTORS_IMPORT_SCALE_BLURB \
+_("Sets the default 'Scale imported paths to fit size' state for the 'Import Path' dialog.")
+
+#define SELECTION_FEATHER_RADIUS_BLURB \
+_("Sets the default feather radius for the 'Feather Selection' dialog.")
+
+#define SELECTION_GROW_RADIUS_BLURB \
+_("Sets the default grow radius for the 'Grow Selection' dialog.")
+
+#define SELECTION_SHRINK_RADIUS_BLURB \
+_("Sets the default shrink radius for the 'Shrink Selection' dialog.")
+
+#define SELECTION_SHRINK_EDGE_LOCK_BLURB \
+_("Sets the default 'Selected areas continue outside the image' setting " \
+  "for the 'Shrink Selection' dialog.")
+
+#define SELECTION_BORDER_RADIUS_BLURB \
+_("Sets the default border radius for the 'Border Selection' dialog.")
+
+#define SELECTION_BORDER_EDGE_LOCK_BLURB \
+_("Sets the default 'Selected areas continue outside the image' setting " \
+  "for the 'Border Selection' dialog.")
+
+#define SELECTION_BORDER_STYLE_BLURB \
+_("Sets the default border style for the 'Border Selection' dialog.")
+
+#define FILL_OPTIONS_BLURB \
+"The default fill options for the fill dialogs."
+
+#define STROKE_OPTIONS_BLURB \
+"The default stroke options for the stroke dialogs."
+
 #define THUMBNAIL_SIZE_BLURB \
 _("Sets the size of the thumbnail shown in the Open dialog.")
 
@@ -504,4 +645,5 @@ _("When enabled, uses OpenCL for some operations.")
 #define ACTION_HISTORY_SIZE_BLURB \
 "The maximum number of actions saved in history."
 
+
 #endif  /* __GIMP_RC_BLURBS_H__ */
