@@ -39,12 +39,12 @@
 #include "gimpfilloptions.h"
 #include "gimpdrawableundo.h"
 #include "gimpimage.h"
+#include "gimpimage-new.h"
 #include "gimpimage-undo.h"
 #include "gimplayer.h"
-#include "gimplayer-floating-sel.h"
+#include "gimplayer-floating-selection.h"
 #include "gimplayer-new.h"
 #include "gimplist.h"
-#include "gimppattern.h"
 #include "gimppickable.h"
 #include "gimpselection.h"
 #include "gimptempbuf.h"
@@ -63,63 +63,101 @@ static GimpBuffer * gimp_edit_extract (GimpImage     *image,
 
 /*  public functions  */
 
-const GimpBuffer *
+GimpObject *
 gimp_edit_cut (GimpImage     *image,
                GimpDrawable  *drawable,
                GimpContext   *context,
                GError       **error)
 {
-  GimpBuffer *buffer;
-
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
   g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (drawable)), NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  buffer = gimp_edit_extract (image, GIMP_PICKABLE (drawable),
-                              context, TRUE, error);
-
-  if (buffer)
+  if (GIMP_IS_LAYER (drawable) &&
+      gimp_channel_is_empty (gimp_image_get_mask (image)))
     {
-      gimp_set_global_buffer (image->gimp, buffer);
-      g_object_unref (buffer);
+      GimpImage *clip_image;
 
-      return image->gimp->global_buffer;
+      clip_image = gimp_image_new_from_drawable (image->gimp, drawable);
+      gimp_container_remove (image->gimp->images, GIMP_OBJECT (clip_image));
+      gimp_set_clipboard_image (image->gimp, clip_image);
+      g_object_unref (clip_image);
+
+      gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_CUT,
+                                   C_("undo-type", "Cut Layer"));
+
+      gimp_image_remove_layer (image, GIMP_LAYER (drawable),
+                               TRUE, NULL);
+
+      gimp_image_undo_group_end (image);
+
+      return GIMP_OBJECT (gimp_get_clipboard_image (image->gimp));
+    }
+  else
+    {
+      GimpBuffer *buffer;
+
+      buffer = gimp_edit_extract (image, GIMP_PICKABLE (drawable),
+                                  context, TRUE, error);
+
+      if (buffer)
+        {
+          gimp_set_clipboard_buffer (image->gimp, buffer);
+          g_object_unref (buffer);
+
+          return GIMP_OBJECT (gimp_get_clipboard_buffer (image->gimp));
+        }
     }
 
   return NULL;
 }
 
-const GimpBuffer *
+GimpObject *
 gimp_edit_copy (GimpImage     *image,
                 GimpDrawable  *drawable,
                 GimpContext   *context,
                 GError       **error)
 {
-  GimpBuffer *buffer;
-
   g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
   g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (drawable)), NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  buffer = gimp_edit_extract (image, GIMP_PICKABLE (drawable),
-                              context, FALSE, error);
-
-  if (buffer)
+  if (GIMP_IS_LAYER (drawable) &&
+      gimp_channel_is_empty (gimp_image_get_mask (image)))
     {
-      gimp_set_global_buffer (image->gimp, buffer);
-      g_object_unref (buffer);
+      GimpImage *clip_image;
 
-      return image->gimp->global_buffer;
+      clip_image = gimp_image_new_from_drawable (image->gimp, drawable);
+      gimp_container_remove (image->gimp->images, GIMP_OBJECT (clip_image));
+      gimp_set_clipboard_image (image->gimp, clip_image);
+      g_object_unref (clip_image);
+
+      return GIMP_OBJECT (gimp_get_clipboard_image (image->gimp));
+    }
+  else
+    {
+      GimpBuffer *buffer;
+
+      buffer = gimp_edit_extract (image, GIMP_PICKABLE (drawable),
+                                  context, FALSE, error);
+
+      if (buffer)
+        {
+          gimp_set_clipboard_buffer (image->gimp, buffer);
+          g_object_unref (buffer);
+
+          return GIMP_OBJECT (gimp_get_clipboard_buffer (image->gimp));
+        }
     }
 
   return NULL;
 }
 
-const GimpBuffer *
+GimpBuffer *
 gimp_edit_copy_visible (GimpImage    *image,
                         GimpContext  *context,
                         GError      **error)
@@ -135,63 +173,44 @@ gimp_edit_copy_visible (GimpImage    *image,
 
   if (buffer)
     {
-      gimp_set_global_buffer (image->gimp, buffer);
+      gimp_set_clipboard_buffer (image->gimp, buffer);
       g_object_unref (buffer);
 
-      return image->gimp->global_buffer;
+      return gimp_get_clipboard_buffer (image->gimp);
     }
 
   return NULL;
 }
 
-GimpLayer *
-gimp_edit_paste (GimpImage    *image,
-                 GimpDrawable *drawable,
-                 GimpBuffer   *paste,
-                 gboolean      paste_into,
-                 gint          viewport_x,
-                 gint          viewport_y,
-                 gint          viewport_width,
-                 gint          viewport_height)
+void
+gimp_edit_get_paste_offset (GimpImage    *image,
+                            GimpDrawable *drawable,
+                            GimpObject   *paste,
+                            gint          viewport_x,
+                            gint          viewport_y,
+                            gint          viewport_width,
+                            gint          viewport_height,
+                            gint         *offset_x,
+                            gint         *offset_y)
 {
-  GimpLayer  *layer;
-  const Babl *format;
-  gint        image_width;
-  gint        image_height;
-  gint        width;
-  gint        height;
-  gint        offset_x;
-  gint        offset_y;
-  gboolean    clamp_to_image = TRUE;
+  gint     image_width;
+  gint     image_height;
+  gint     width;
+  gint     height;
+  gboolean clamp_to_image = TRUE;
 
-  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
-  g_return_val_if_fail (drawable == NULL || GIMP_IS_DRAWABLE (drawable), NULL);
-  g_return_val_if_fail (drawable == NULL ||
-                        gimp_item_is_attached (GIMP_ITEM (drawable)), NULL);
-  g_return_val_if_fail (GIMP_IS_BUFFER (paste), NULL);
-
-  /*  Make a new layer: if drawable == NULL,
-   *  user is pasting into an empty image.
-   */
-
-  if (drawable)
-    format = gimp_drawable_get_format_with_alpha (drawable);
-  else
-    format = gimp_image_get_layer_format (image, TRUE);
-
-  layer = gimp_layer_new_from_buffer (paste, image,
-                                      format,
-                                      _("Pasted Layer"),
-                                      GIMP_OPACITY_OPAQUE, GIMP_NORMAL_MODE);
-
-  if (! layer)
-    return NULL;
+  g_return_if_fail (GIMP_IS_IMAGE (image));
+  g_return_if_fail (drawable == NULL || GIMP_IS_DRAWABLE (drawable));
+  g_return_if_fail (drawable == NULL ||
+                    gimp_item_is_attached (GIMP_ITEM (drawable)));
+  g_return_if_fail (GIMP_IS_VIEWABLE (paste));
+  g_return_if_fail (offset_x != NULL);
+  g_return_if_fail (offset_y != NULL);
 
   image_width  = gimp_image_get_width  (image);
   image_height = gimp_image_get_height (image);
 
-  width  = gimp_item_get_width  (GIMP_ITEM (layer));
-  height = gimp_item_get_height (GIMP_ITEM (layer));
+  gimp_viewable_get_size (GIMP_VIEWABLE (paste), &width, &height);
 
   if (viewport_width  == image_width &&
       viewport_height == image_height)
@@ -208,41 +227,61 @@ gimp_edit_paste (GimpImage    *image,
     {
       /*  if pasting to a drawable  */
 
-      gint     off_x, off_y;
-      gint     x1, y1, x2, y2;
-      gint     paste_x, paste_y;
-      gint     paste_width, paste_height;
-      gboolean have_mask;
+      GimpContainer *children;
+      gint           off_x, off_y;
+      gint           target_x, target_y;
+      gint           target_width, target_height;
+      gint           paste_x, paste_y;
+      gint           paste_width, paste_height;
+      gboolean       have_mask;
+
+      have_mask = ! gimp_channel_is_empty (gimp_image_get_mask (image));
 
       gimp_item_get_offset (GIMP_ITEM (drawable), &off_x, &off_y);
-      have_mask = gimp_item_mask_bounds (GIMP_ITEM (drawable),
-                                         &x1, &y1, &x2, &y2);
 
-      if (! have_mask         && /* if we have no mask */
-          viewport_width  > 0 && /* and we have a viewport */
+      children = gimp_viewable_get_children (GIMP_VIEWABLE (drawable));
+
+      if (children && gimp_container_get_n_children (children) == 0)
+        {
+          /* treat empty layer groups as image-sized, use the selection
+           * as target
+           */
+          gimp_item_bounds (GIMP_ITEM (gimp_image_get_mask (image)),
+                            &target_x, &target_y,
+                            &target_width, &target_height);
+        }
+      else
+        {
+          gimp_item_mask_intersect (GIMP_ITEM (drawable),
+                                    &target_x, &target_y,
+                                    &target_width, &target_height);
+        }
+
+      if (! have_mask         &&    /* if we have no mask */
+          viewport_width  > 0 &&    /* and we have a viewport */
           viewport_height > 0 &&
-          (width  < (x2 - x1) || /* and the paste is smaller than the target */
-           height < (y2 - y1)) &&
+          (width  < target_width || /* and the paste is smaller than the target */
+           height < target_height) &&
 
           /* and the viewport intersects with the target */
           gimp_rectangle_intersect (viewport_x, viewport_y,
                                     viewport_width, viewport_height,
-                                    off_x, off_y,
-                                    x2 - x1, y2 - y1,
+                                    off_x, off_y, /* target_x,y are 0 */
+                                    target_width, target_height,
                                     &paste_x, &paste_y,
                                     &paste_width, &paste_height))
         {
           /*  center on the viewport  */
 
-          offset_x = paste_x + (paste_width - width)  / 2;
-          offset_y = paste_y + (paste_height- height) / 2;
+          *offset_x = paste_x + (paste_width - width)  / 2;
+          *offset_y = paste_y + (paste_height- height) / 2;
         }
       else
         {
           /*  otherwise center on the target  */
 
-          offset_x = off_x + ((x1 + x2) - width)  / 2;
-          offset_y = off_y + ((y1 + y2) - height) / 2;
+          *offset_x = off_x + target_x + (target_width  - width)  / 2;
+          *offset_y = off_y + target_y + (target_height - height) / 2;
 
           /*  and keep it that way  */
           clamp_to_image = FALSE;
@@ -255,15 +294,15 @@ gimp_edit_paste (GimpImage    *image,
     {
       /*  center on the viewport  */
 
-      offset_x = viewport_x + (viewport_width  - width)  / 2;
-      offset_y = viewport_y + (viewport_height - height) / 2;
+      *offset_x = viewport_x + (viewport_width  - width)  / 2;
+      *offset_y = viewport_y + (viewport_height - height) / 2;
     }
   else
     {
       /*  otherwise center on the image  */
 
-      offset_x = (image_width  - width)  / 2;
-      offset_y = (image_height - height) / 2;
+      *offset_x = (image_width  - width)  / 2;
+      *offset_y = (image_height - height) / 2;
 
       /*  and keep it that way  */
       clamp_to_image = FALSE;
@@ -274,32 +313,166 @@ gimp_edit_paste (GimpImage    *image,
       /*  Ensure that the pasted layer is always within the image, if it
        *  fits and aligned at top left if it doesn't. (See bug #142944).
        */
-      offset_x = MIN (offset_x, image_width  - width);
-      offset_y = MIN (offset_y, image_height - height);
-      offset_x = MAX (offset_x, 0);
-      offset_y = MAX (offset_y, 0);
+      *offset_x = MIN (*offset_x, image_width  - width);
+      *offset_y = MIN (*offset_y, image_height - height);
+      *offset_x = MAX (*offset_x, 0);
+      *offset_y = MAX (*offset_y, 0);
+    }
+}
+
+GimpLayer *
+gimp_edit_paste (GimpImage     *image,
+                 GimpDrawable  *drawable,
+                 GimpObject    *paste,
+                 GimpPasteType  paste_type,
+                 gint           viewport_x,
+                 gint           viewport_y,
+                 gint           viewport_width,
+                 gint           viewport_height)
+{
+  GimpLayer  *layer = NULL;
+  const Babl *floating_format;
+  gint        offset_x;
+  gint        offset_y;
+
+  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (drawable == NULL || GIMP_IS_DRAWABLE (drawable), NULL);
+  g_return_val_if_fail (drawable == NULL ||
+                        gimp_item_is_attached (GIMP_ITEM (drawable)), NULL);
+  g_return_val_if_fail (GIMP_IS_IMAGE (paste) || GIMP_IS_BUFFER (paste), NULL);
+
+  /*  change paste type to NEW_LAYER for cases where we can't attach a
+   *  floating selection
+   */
+  if (! drawable                                            ||
+      gimp_viewable_get_children (GIMP_VIEWABLE (drawable)) ||
+      gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+    {
+      paste_type = GIMP_PASTE_TYPE_NEW_LAYER;
     }
 
+  /*  floating pastes always have the pasted-to drawable's format with
+   *  alpha; if drawable == NULL, user is pasting into an empty image
+   */
+  if (drawable)
+    floating_format = gimp_drawable_get_format_with_alpha (drawable);
+  else
+    floating_format = gimp_image_get_layer_format (image, TRUE);
+
+  if (GIMP_IS_IMAGE (paste))
+    {
+      GType layer_type;
+
+      layer = gimp_image_get_layer_iter (GIMP_IMAGE (paste))->data;
+
+      switch (paste_type)
+        {
+        case GIMP_PASTE_TYPE_FLOATING:
+        case GIMP_PASTE_TYPE_FLOATING_INTO:
+          /*  when pasting as floating selection, force creation of a
+           *  plain layer, so gimp_item_convert() will collapse a
+           *  group layer
+           */
+          layer_type = GIMP_TYPE_LAYER;
+          break;
+
+        case GIMP_PASTE_TYPE_NEW_LAYER:
+          layer_type = G_TYPE_FROM_INSTANCE (layer);
+          break;
+
+        default:
+          g_return_val_if_reached (NULL);
+        }
+
+      layer = GIMP_LAYER (gimp_item_convert (GIMP_ITEM (layer),
+                                             image, layer_type));
+
+      switch (paste_type)
+        {
+        case GIMP_PASTE_TYPE_FLOATING:
+        case GIMP_PASTE_TYPE_FLOATING_INTO:
+          /*  when pasting as floating selection, get rid of the layer mask,
+           *  and make sure the layer has the right format
+           */
+          if (gimp_layer_get_mask (layer))
+            gimp_layer_apply_mask (layer, GIMP_MASK_DISCARD, FALSE);
+
+          if (gimp_drawable_get_format (GIMP_DRAWABLE (layer)) !=
+              floating_format)
+            {
+              gimp_drawable_convert_type (GIMP_DRAWABLE (layer), image,
+                                          gimp_drawable_get_base_type (drawable),
+                                          gimp_drawable_get_precision (drawable),
+                                          TRUE,
+                                          NULL,
+                                          GEGL_DITHER_NONE, GEGL_DITHER_NONE,
+                                          FALSE, NULL);
+            }
+          break;
+
+        default:
+          break;
+        }
+    }
+  else if (GIMP_IS_BUFFER (paste))
+    {
+      layer = gimp_layer_new_from_buffer (GIMP_BUFFER (paste), image,
+                                          floating_format,
+                                          _("Pasted Layer"),
+                                          GIMP_OPACITY_OPAQUE,
+                                          GIMP_LAYER_MODE_NORMAL_LEGACY);
+    }
+
+  if (! layer)
+    return NULL;
+
+  gimp_edit_get_paste_offset (image, drawable, GIMP_OBJECT (layer),
+                              viewport_x,
+                              viewport_y,
+                              viewport_width,
+                              viewport_height,
+                              &offset_x,
+                              &offset_y);
   gimp_item_set_offset (GIMP_ITEM (layer), offset_x, offset_y);
 
-  /*  Start a group undo  */
   gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_EDIT_PASTE,
                                C_("undo-type", "Paste"));
 
-  /*  If there is a selection mask clear it--
-   *  this might not always be desired, but in general,
-   *  it seems like the correct behavior.
-   */
-  if (! gimp_channel_is_empty (gimp_image_get_mask (image)) && ! paste_into)
-    gimp_channel_clear (gimp_image_get_mask (image), NULL, TRUE);
+  switch (paste_type)
+    {
+    case GIMP_PASTE_TYPE_FLOATING:
+      /*  if there is a selection mask clear it - this might not
+       *  always be desired, but in general, it seems like the correct
+       *  behavior
+       */
+      if (! gimp_channel_is_empty (gimp_image_get_mask (image)))
+        gimp_channel_clear (gimp_image_get_mask (image), NULL, TRUE);
 
-  /*  if there's a drawable, add a new floating selection  */
-  if (drawable)
-    floating_sel_attach (layer, drawable);
-  else
-    gimp_image_add_layer (image, layer, NULL, 0, TRUE);
+      /* fall thru */
+
+    case GIMP_PASTE_TYPE_FLOATING_INTO:
+      floating_sel_attach (layer, drawable);
+      break;
+
+    case GIMP_PASTE_TYPE_NEW_LAYER:
+      {
+        GimpLayer *parent   = NULL;
+        gint       position = 0;
+
+        /* always add on top of the passed layer, where we would
+         * attach a floating selection
+         */
+        if (GIMP_IS_LAYER (drawable))
+          {
+            parent   = gimp_layer_get_parent (GIMP_LAYER (drawable));
+            position = gimp_item_get_index (GIMP_ITEM (drawable));
+          }
+
+        gimp_image_add_layer (image, layer, parent, position, TRUE);
+      }
+      break;
+    }
 
-  /*  end the group undo  */
   gimp_image_undo_group_end (image);
 
   return layer;
@@ -427,10 +600,7 @@ gimp_edit_fill (GimpImage       *image,
                 GimpFillOptions *options,
                 const gchar     *undo_desc)
 {
-  GeglBuffer  *dest_buffer;
-  GimpPattern *pattern = NULL;
-  GimpRGB      color;
-  const Babl  *format;
+  GeglBuffer  *buffer;
   gint         x, y, width, height;
 
   g_return_if_fail (GIMP_IS_IMAGE (image));
@@ -441,57 +611,24 @@ gimp_edit_fill (GimpImage       *image,
   if (! gimp_item_mask_intersect (GIMP_ITEM (drawable), &x, &y, &width, &height))
     return;  /*  nothing to do, but the fill succeeded  */
 
-  switch (gimp_fill_options_get_style (options))
-    {
-    case GIMP_FILL_STYLE_SOLID:
-      gimp_context_get_foreground (GIMP_CONTEXT (options), &color);
-      break;
-
-    case GIMP_FILL_STYLE_PATTERN:
-      pattern = gimp_context_get_pattern (GIMP_CONTEXT (options));
-      break;
-    }
-
-  if (pattern &&
-      babl_format_has_alpha (gimp_temp_buf_get_format (pattern->mask)) &&
-      ! gimp_drawable_has_alpha (drawable))
-    {
-      format = gimp_drawable_get_format_with_alpha (drawable);
-    }
-  else
-    {
-      format = gimp_drawable_get_format (drawable);
-    }
-
-  dest_buffer = gegl_buffer_new (GEGL_RECTANGLE (0, 0, width, height),
-                                 format);
-
-  if (pattern)
-    {
-      GeglBuffer *src_buffer = gimp_pattern_create_buffer (pattern);
-
-      gegl_buffer_set_pattern (dest_buffer, NULL, src_buffer, 0, 0);
-      g_object_unref (src_buffer);
-    }
-  else
-    {
-      GeglColor *gegl_color = gimp_gegl_color_new (&color);
-
-      gegl_buffer_set_color (dest_buffer, NULL, gegl_color);
-      g_object_unref (gegl_color);
-    }
+  buffer = gimp_fill_options_create_buffer (options, drawable,
+                                            GEGL_RECTANGLE (0, 0,
+                                                            width, height));
 
   if (! undo_desc)
     undo_desc = gimp_fill_options_get_undo_desc (options);
 
-  gimp_drawable_apply_buffer (drawable, dest_buffer,
+  gimp_drawable_apply_buffer (drawable, buffer,
                               GEGL_RECTANGLE (0, 0, width, height),
                               TRUE, undo_desc,
                               gimp_context_get_opacity (GIMP_CONTEXT (options)),
                               gimp_context_get_paint_mode (GIMP_CONTEXT (options)),
+                              GIMP_LAYER_COLOR_SPACE_AUTO,
+                              GIMP_LAYER_COLOR_SPACE_AUTO,
+                              GIMP_LAYER_COMPOSITE_AUTO,
                               NULL, x, y);
 
-  g_object_unref (dest_buffer);
+  g_object_unref (buffer);
 
   gimp_drawable_update (drawable, x, y, width, height);
 }
@@ -527,6 +664,9 @@ gimp_edit_fade (GimpImage   *image,
                                   gimp_object_get_name (undo),
                                   gimp_context_get_opacity (context),
                                   gimp_context_get_paint_mode (context),
+                                  GIMP_LAYER_COLOR_SPACE_AUTO,
+                                  GIMP_LAYER_COLOR_SPACE_AUTO,
+                                  GIMP_LAYER_COMPOSITE_AUTO,
                                   NULL, undo->x, undo->y);
 
       g_object_unref (buffer);
