@@ -61,6 +61,8 @@
 
 struct _GimpUserInstall
 {
+  GObject                 *gimp;
+
   gboolean                verbose;
 
   gchar                  *old_dir;
@@ -145,10 +147,12 @@ static gboolean  user_install_migrate_files      (GimpUserInstall    *install);
 /*  public functions  */
 
 GimpUserInstall *
-gimp_user_install_new (gboolean verbose)
+gimp_user_install_new (GObject  *gimp,
+                       gboolean  verbose)
 {
   GimpUserInstall *install = g_slice_new0 (GimpUserInstall);
 
+  install->gimp    = gimp;
   install->verbose = verbose;
 
   user_install_detect_old (install, gimp_directory ());
@@ -206,15 +210,15 @@ gimp_user_install_run (GimpUserInstall *install)
 
   if (install->migrate)
     user_install_log (install,
-		      _("It seems you have used GIMP %s before.  "
-			"GIMP will now migrate your user settings to '%s'."),
-		      install->migrate, dirname);
+                      _("It seems you have used GIMP %s before.  "
+                        "GIMP will now migrate your user settings to '%s'."),
+                      install->migrate, dirname);
   else
     user_install_log (install,
-		      _("It appears that you are using GIMP for the "
-			"first time.  GIMP will now create a folder "
-			"named '%s' and copy some files to it."),
-			dirname);
+                      _("It appears that you are using GIMP for the "
+                        "first time.  GIMP will now create a folder "
+                        "named '%s' and copy some files to it."),
+                      dirname);
 
   g_free (dirname);
 
@@ -278,7 +282,7 @@ user_install_detect_old (GimpUserInstall *install,
           if (migrate)
             {
 #ifdef GIMP_UNSTABLE
-	      g_printerr ("gimp-user-install: migrating from %s\n", dir);
+              g_printerr ("gimp-user-install: migrating from %s\n", dir);
 #endif
               install->old_major = 2;
               install->old_minor = i;
@@ -475,10 +479,14 @@ user_install_mkdir_with_parents (GimpUserInstall *install,
 
 /* The regexp pattern of all options changed from menurc of GIMP 2.8.
  * Add any pattern that we want to recognize for replacement in the menurc of
- * the next release*/
-#define MENURC_OVER20_UPDATE_PATTERN "\"<Actions>/file/file-export-to\""         "|" \
-                                     "\"<Actions>/file/file-export\""            "|" \
-                                     "\"<Actions>/tools/tools-value-[1-4]-.*\""
+ * the next release
+ */
+#define MENURC_OVER20_UPDATE_PATTERN \
+  "\"<Actions>/file/file-export-to\""           "|" \
+  "\"<Actions>/file/file-export\""              "|" \
+  "\"<Actions>/edit/edit-paste-as-new\""        "|" \
+  "\"<Actions>/buffers/buffers-paste-as-new\""  "|" \
+  "\"<Actions>/tools/tools-value-[1-4]-.*\""
 
 /**
  * callback to use for updating a menurc from GIMP over 2.0.
@@ -490,11 +498,11 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
                            GString          *new_value,
                            gpointer          data)
 {
-  gchar *match;
-  match = g_match_info_fetch (matched_value, 0);
+  gchar *match = g_match_info_fetch (matched_value, 0);
 
-  /* file-export-* changes to follow file-save-* patterns.
-   * Actions available since GIMP 2.8, changed for 2.10 in commit 4b14ed2. */
+  /* file-export-* changes to follow file-save-* patterns.  Actions
+   * available since GIMP 2.8, changed for 2.10 in commit 4b14ed2.
+   */
   if (g_strcmp0 (match, "\"<Actions>/file/file-export\"") == 0)
     {
       g_string_append (new_value, "\"<Actions>/file/file-export-as\"");
@@ -503,8 +511,19 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
     {
       g_string_append (new_value, "\"<Actions>/file/file-export\"");
     }
-  /* Tools settings renamed more user-friendly.
-   * Actions available since GIMP 2.4, changed for 2.10 in commit 0bdb747. */
+  /* "*-paste-as-new" renamed to "*-paste-as-new-image"
+   */
+  else if (g_strcmp0 (match, "\"<Actions>/edit/edit-paste-as-new\"") == 0)
+    {
+      g_string_append (new_value, "\"<Actions>/edit/edit-paste-as-new-image\"");
+    }
+  else if (g_strcmp0 (match, "\"<Actions>/buffers/buffers-paste-as-new\"") == 0)
+    {
+      g_string_append (new_value, "\"<Actions>/buffers/buffers-paste-as-new-image\"");
+    }
+  /* Tools settings renamed more user-friendly.  Actions available
+   * since GIMP 2.4, changed for 2.10 in commit 0bdb747.
+   */
   else if (g_str_has_prefix (match, "\"<Actions>/tools/tools-value-1-"))
     {
       g_string_append (new_value, "\"<Actions>/tools/tools-opacity-");
@@ -525,7 +544,9 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
       g_string_append (new_value, "\"<Actions>/tools/tools-angle-");
       g_string_append (new_value, match + 31);
     }
-  /* Should not happen. Just in case we match something unexpected by mistake. */
+  /* Should not happen. Just in case we match something unexpected by
+   * mistake.
+   */
   else
     {
       g_string_append (new_value, match);
@@ -535,7 +556,8 @@ user_update_menurc_over20 (const GMatchInfo *matched_value,
   return FALSE;
 }
 
-#define CONTROLLERRC_UPDATE_PATTERN "\\(map \"(scroll|cursor)-[^\"]*\\bcontrol\\b[^\"]*\""
+#define CONTROLLERRC_UPDATE_PATTERN \
+  "\\(map \"(scroll|cursor)-[^\"]*\\bcontrol\\b[^\"]*\""
 
 static gboolean
 user_update_controllerrc (const GMatchInfo *matched_value,
@@ -563,14 +585,16 @@ user_update_controllerrc (const GMatchInfo *matched_value,
   return FALSE;
 }
 
-#define GIMPRC_UPDATE_PATTERN "\\(theme [^)]*\\)"
+#define GIMPRC_UPDATE_PATTERN \
+  "\\(theme [^)]*\\)"    "|" \
+  "\\(.*-path [^)]*\\)"
 
 static gboolean
 user_update_gimprc (const GMatchInfo *matched_value,
                     GString          *new_value,
                     gpointer          data)
 {
-  /* Do not migrate themes from GIMP < 2.10. */
+  /* Do not migrate paths and themes from GIMP < 2.10. */
   return FALSE;
 }
 
@@ -754,7 +778,8 @@ user_install_migrate_files (GimpUserInstall *install)
           g_snprintf (dest, sizeof (dest), "%s%c%s",
                       gimp_directory (), G_DIR_SEPARATOR, basename);
 
-          user_install_file_copy (install, source, dest, update_pattern, update_callback);
+          user_install_file_copy (install, source, dest,
+                                  update_pattern, update_callback);
         }
       else if (g_file_test (source, G_FILE_TEST_IS_DIR))
         {
@@ -783,7 +808,7 @@ user_install_migrate_files (GimpUserInstall *install)
 
   gimp_templates_migrate (install->old_dir);
 
-  gimprc = gimp_rc_new (NULL, NULL, FALSE);
+  gimprc = gimp_rc_new (install->gimp, NULL, NULL, FALSE);
   gimp_rc_migrate (gimprc);
   gimp_rc_save (gimprc);
   g_object_unref (gimprc);
