@@ -34,6 +34,7 @@
 #include "gimpbrush-transform.h"
 #include "gimpbrushcache.h"
 #include "gimpbrushgenerated.h"
+#include "gimpbrushpipe.h"
 #include "gimpmarshal.h"
 #include "gimptagged.h"
 #include "gimptempbuf.h"
@@ -172,6 +173,8 @@ gimp_brush_init (GimpBrush *brush)
   brush->priv->x_axis.y =  0.0;
   brush->priv->y_axis.x =  0.0;
   brush->priv->y_axis.y = 15.0;
+
+  brush->priv->blur_hardness = 1.0;
 }
 
 static void
@@ -190,6 +193,17 @@ gimp_brush_finalize (GObject *object)
       gimp_temp_buf_unref (brush->priv->pixmap);
       brush->priv->pixmap = NULL;
     }
+  if (brush->priv->blured_mask)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_mask);
+      brush->priv->blured_mask = NULL;
+    }
+
+  if (brush->priv->blured_pixmap)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_pixmap);
+      brush->priv->blured_pixmap = NULL;
+    }
 
   if (brush->priv->mask_cache)
     {
@@ -314,7 +328,8 @@ gimp_brush_get_new_preview (GimpViewable *viewable,
                GimpBrushGenerated *gen_brush = GIMP_BRUSH_GENERATED (brush);
 
                mask_buf = gimp_brush_transform_mask (brush, NULL, scale,
-                                                     0.0, 0.0,
+                                                     (gimp_brush_generated_get_aspect_ratio (gen_brush) - 1.0) * 20.0 / 19.0,
+                                                     gimp_brush_generated_get_angle (gen_brush) / 360.0,
                                                      gimp_brush_generated_get_hardness (gen_brush));
             }
           else
@@ -414,6 +429,18 @@ gimp_brush_dirty (GimpData *data)
   if (brush->priv->boundary_cache)
     gimp_brush_cache_clear (brush->priv->boundary_cache);
 
+   if (brush->priv->blured_mask)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_mask);
+      brush->priv->blured_mask = NULL;
+    }
+
+  if (brush->priv->blured_pixmap)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_pixmap);
+      brush->priv->blured_pixmap = NULL;
+    }
+
   GIMP_DATA_CLASS (parent_class)->dirty (data);
 }
 
@@ -447,6 +474,18 @@ gimp_brush_real_end_use (GimpBrush *brush)
 
   g_object_unref (brush->priv->boundary_cache);
   brush->priv->boundary_cache = NULL;
+
+  if (brush->priv->blured_mask)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_mask);
+      brush->priv->blured_mask = NULL;
+    }
+
+  if (brush->priv->blured_pixmap)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_pixmap);
+      brush->priv->blured_pixmap = NULL;
+    }
 }
 
 static GimpBrush *
@@ -622,6 +661,7 @@ gimp_brush_transform_mask (GimpBrush *brush,
   const GimpTempBuf *mask;
   gint               width;
   gint               height;
+  gdouble            effective_hardness = hardness;
 
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), NULL);
   g_return_val_if_fail (scale > 0.0, NULL);
@@ -645,11 +685,33 @@ gimp_brush_transform_mask (GimpBrush *brush,
         }
       else
         {
+          /* This code makes sure that brushes using blur for hardness
+           * (all of them but generated) are blurred once and no more.
+           * It also makes hardnes dynamics not work for these brushes.
+           * This is intentional. Confoliving for each stamp is too expensive.*/
+          if (! brush->priv->blured_mask &&
+              ! GIMP_IS_BRUSH_GENERATED(brush) &&
+              ! GIMP_IS_BRUSH_PIPE(brush) && /*Cant cache pipes. Sanely anway*/
+              hardness < 1.0)
+            {
+               brush->priv->blured_mask = GIMP_BRUSH_GET_CLASS (brush)->transform_mask (brush,
+                                                                 1.0,
+                                                                 0.0,
+                                                                 0.0,
+                                                                 hardness);
+               brush->priv->blur_hardness = hardness;
+            }
+
+          if (brush->priv->blured_mask)
+            {
+              effective_hardness = 1.0; /*Hardness has already been applied*/
+            }
+
           mask = GIMP_BRUSH_GET_CLASS (brush)->transform_mask (brush,
                                                                scale,
                                                                aspect_ratio,
                                                                angle,
-                                                               hardness);
+                                                               effective_hardness);
         }
 
       if (op)
@@ -669,7 +731,12 @@ gimp_brush_transform_mask (GimpBrush *brush,
                                         NULL);
 
           gegl_node_link_many (source, op, target, NULL);
-          gegl_node_process (target);
+          gegl_node_blit (target, 1.0,
+                          GEGL_RECTANGLE (0, 0,
+                                          gegl_buffer_get_width (buffer),
+                                          gegl_buffer_get_height (buffer)),
+                          NULL, NULL, 0, GEGL_BLIT_DEFAULT);
+
 
           g_object_unref (graph);
           g_object_unref (buffer);
@@ -678,7 +745,7 @@ gimp_brush_transform_mask (GimpBrush *brush,
       gimp_brush_cache_add (brush->priv->mask_cache,
                             (gpointer) mask,
                             op, width, height,
-                            scale, aspect_ratio, angle, hardness);
+                            scale, aspect_ratio, angle, effective_hardness);
     }
 
   return mask;
@@ -695,6 +762,7 @@ gimp_brush_transform_pixmap (GimpBrush *brush,
   const GimpTempBuf *pixmap;
   gint               width;
   gint               height;
+  gdouble            effective_hardness = hardness;
 
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), NULL);
   g_return_val_if_fail (brush->priv->pixmap != NULL, NULL);
@@ -719,11 +787,28 @@ gimp_brush_transform_pixmap (GimpBrush *brush,
         }
       else
         {
+         if (! brush->priv->blured_pixmap &&
+             ! GIMP_IS_BRUSH_GENERATED(brush) &&
+             ! GIMP_IS_BRUSH_PIPE(brush) /*Cant cache pipes. Sanely anway*/
+             && hardness < 1.0)
+          {
+             brush->priv->blured_pixmap = GIMP_BRUSH_GET_CLASS (brush)->transform_pixmap (brush,
+                                                                      1.0,
+                                                                      0.0,
+                                                                      0.0,
+                                                                      hardness);
+             brush->priv->blur_hardness = hardness;
+           }
+
+          if (brush->priv->blured_pixmap) {
+            effective_hardness = 1.0; /*Hardness has already been applied*/
+          }
+
           pixmap = GIMP_BRUSH_GET_CLASS (brush)->transform_pixmap (brush,
                                                                    scale,
                                                                    aspect_ratio,
                                                                    angle,
-                                                                   hardness);
+                                                                   effective_hardness);
         }
 
       if (op)
@@ -743,7 +828,11 @@ gimp_brush_transform_pixmap (GimpBrush *brush,
                                         NULL);
 
           gegl_node_link_many (source, op, target, NULL);
-          gegl_node_process (target);
+          gegl_node_blit (target, 1.0,
+                          GEGL_RECTANGLE (0, 0,
+                                          gegl_buffer_get_width (buffer),
+                                          gegl_buffer_get_height (buffer)),
+                          NULL, NULL, 0, GEGL_BLIT_DEFAULT);
 
           g_object_unref (graph);
           g_object_unref (buffer);
@@ -752,7 +841,7 @@ gimp_brush_transform_pixmap (GimpBrush *brush,
       gimp_brush_cache_add (brush->priv->pixmap_cache,
                             (gpointer) pixmap,
                             op, width, height,
-                            scale, aspect_ratio, angle, hardness);
+                            scale, aspect_ratio, angle, effective_hardness);
     }
 
   return pixmap;
@@ -809,41 +898,99 @@ gimp_brush_transform_boundary (GimpBrush *brush,
 }
 
 GimpTempBuf *
-gimp_brush_get_mask (const GimpBrush *brush)
+gimp_brush_get_mask (GimpBrush *brush)
 {
   g_return_val_if_fail (brush != NULL, NULL);
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), NULL);
 
+  if (brush->priv->blured_mask)
+    {
+      return brush->priv->blured_mask;
+    }
   return brush->priv->mask;
 }
 
 GimpTempBuf *
-gimp_brush_get_pixmap (const GimpBrush *brush)
+gimp_brush_get_pixmap (GimpBrush *brush)
 {
   g_return_val_if_fail (brush != NULL, NULL);
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), NULL);
 
+  if(brush->priv->blured_pixmap)
+    {
+      return brush->priv->blured_pixmap;
+    }
   return brush->priv->pixmap;
 }
 
-gint
-gimp_brush_get_width (const GimpBrush *brush)
+void
+gimp_brush_flush_blur_caches (GimpBrush *brush)
+{
+  if (brush->priv->blured_mask)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_mask);
+      brush->priv->blured_mask = NULL;
+    }
+
+  if (brush->priv->blured_pixmap)
+    {
+      gimp_temp_buf_unref (brush->priv->blured_pixmap);
+      brush->priv->blured_pixmap = NULL;
+    }
+
+  if (brush->priv->mask_cache)
+    gimp_brush_cache_clear (brush->priv->mask_cache);
+
+  if (brush->priv->pixmap_cache)
+    gimp_brush_cache_clear (brush->priv->pixmap_cache);
+
+  if (brush->priv->boundary_cache)
+    gimp_brush_cache_clear (brush->priv->boundary_cache);
+
+}
+
+gdouble
+gimp_brush_get_blur_hardness (GimpBrush *brush)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), 0);
 
+  return brush->priv->blur_hardness;
+}
+
+gint
+gimp_brush_get_width (GimpBrush *brush)
+{
+  g_return_val_if_fail (GIMP_IS_BRUSH (brush), 0);
+  if (brush->priv->blured_mask)
+    {
+      return gimp_temp_buf_get_width (brush->priv->blured_mask);
+    }
+  if (brush->priv->blured_pixmap)
+    {
+      return gimp_temp_buf_get_width (brush->priv->blured_pixmap);
+    }
   return gimp_temp_buf_get_width (brush->priv->mask);
 }
 
 gint
-gimp_brush_get_height (const GimpBrush *brush)
+gimp_brush_get_height (GimpBrush *brush)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), 0);
 
+  if (brush->priv->blured_mask)
+  {
+    return gimp_temp_buf_get_height (brush->priv->blured_mask);
+  }
+
+  if (brush->priv->blured_pixmap)
+  {
+    return gimp_temp_buf_get_height (brush->priv->blured_pixmap);
+  }
   return gimp_temp_buf_get_height (brush->priv->mask);
 }
 
 gint
-gimp_brush_get_spacing (const GimpBrush *brush)
+gimp_brush_get_spacing (GimpBrush *brush)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), 0);
 
@@ -868,7 +1015,7 @@ gimp_brush_set_spacing (GimpBrush *brush,
 static const GimpVector2 fail = { 0.0, 0.0 };
 
 GimpVector2
-gimp_brush_get_x_axis (const GimpBrush *brush)
+gimp_brush_get_x_axis (GimpBrush *brush)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), fail);
 
@@ -876,7 +1023,7 @@ gimp_brush_get_x_axis (const GimpBrush *brush)
 }
 
 GimpVector2
-gimp_brush_get_y_axis (const GimpBrush *brush)
+gimp_brush_get_y_axis (GimpBrush *brush)
 {
   g_return_val_if_fail (GIMP_IS_BRUSH (brush), fail);
 
