@@ -90,7 +90,7 @@ static gboolean        gimp_group_layer_get_expanded (GimpViewable    *viewable)
 static void            gimp_group_layer_set_expanded (GimpViewable    *viewable,
                                                       gboolean         expanded);
 
-static gboolean  gimp_group_layer_is_position_locked (const GimpItem  *item);
+static gboolean  gimp_group_layer_is_position_locked (GimpItem        *item);
 static GimpItem      * gimp_group_layer_duplicate    (GimpItem        *item,
                                                       GType            new_type);
 static void            gimp_group_layer_convert      (GimpItem        *item,
@@ -109,6 +109,7 @@ static void            gimp_group_layer_scale        (GimpItem        *item,
                                                       GimpProgress    *progress);
 static void            gimp_group_layer_resize       (GimpItem        *item,
                                                       GimpContext     *context,
+                                                      GimpFillType     fill_type,
                                                       gint             new_width,
                                                       gint             new_height,
                                                       gint             offset_x,
@@ -132,18 +133,16 @@ static void            gimp_group_layer_transform    (GimpItem        *item,
                                                       GimpTransformResize clip_result,
                                                       GimpProgress    *progress);
 
-static gint64      gimp_group_layer_estimate_memsize (const GimpDrawable *drawable,
+static gint64      gimp_group_layer_estimate_memsize (GimpDrawable      *drawable,
                                                       GimpComponentType  component_type,
                                                       gint               width,
                                                       gint               height);
 static void            gimp_group_layer_convert_type (GimpDrawable      *drawable,
                                                       GimpImage         *dest_image,
                                                       const Babl        *new_format,
-                                                      GimpImageBaseType  new_base_type,
-                                                      GimpPrecision      new_precision,
-                                                      gint               layer_dither_type,
-                                                      gint               mask_dither_type,
-                                                      gboolean           convert_profile,
+                                                      GimpColorProfile  *dest_profile,
+                                                      GeglDitherMethod   layer_dither_type,
+                                                      GeglDitherMethod   mask_dither_type,
                                                       gboolean           push_undo,
                                                       GimpProgress      *progress);
 
@@ -209,7 +208,7 @@ gimp_group_layer_class_init (GimpGroupLayerClass *klass)
 
   gimp_object_class->get_memsize    = gimp_group_layer_get_memsize;
 
-  viewable_class->default_icon_name = "gtk-directory";
+  viewable_class->default_icon_name = "gimp-group-layer";
   viewable_class->get_size          = gimp_group_layer_get_size;
   viewable_class->get_children      = gimp_group_layer_get_children;
   viewable_class->set_expanded      = gimp_group_layer_set_expanded;
@@ -387,7 +386,13 @@ gimp_group_layer_get_size (GimpViewable *viewable,
       return TRUE;
     }
 
-  return GIMP_VIEWABLE_CLASS (parent_class)->get_size (viewable, width, height);
+  /*  return the size only if there are children...  */
+  if (gimp_item_stack_get_item_iter (GIMP_ITEM_STACK (private->children)))
+    return GIMP_VIEWABLE_CLASS (parent_class)->get_size (viewable,
+                                                         width, height);
+
+  /*  ...otherwise return "no content"  */
+  return FALSE;
 }
 
 static GimpContainer *
@@ -414,7 +419,7 @@ gimp_group_layer_set_expanded (GimpViewable *viewable,
 }
 
 static gboolean
-gimp_group_layer_is_position_locked (const GimpItem *item)
+gimp_group_layer_is_position_locked (GimpItem *item)
 {
   GimpGroupLayerPrivate *private = GET_PRIVATE (item);
   GList                 *list;
@@ -633,12 +638,13 @@ gimp_group_layer_scale (GimpItem              *item,
 }
 
 static void
-gimp_group_layer_resize (GimpItem    *item,
-                         GimpContext *context,
-                         gint         new_width,
-                         gint         new_height,
-                         gint         offset_x,
-                         gint         offset_y)
+gimp_group_layer_resize (GimpItem     *item,
+                         GimpContext  *context,
+                         GimpFillType  fill_type,
+                         gint          new_width,
+                         gint          new_height,
+                         gint          offset_x,
+                         gint          offset_y)
 {
   GimpGroupLayer        *group   = GIMP_GROUP_LAYER (item);
   GimpGroupLayerPrivate *private = GET_PRIVATE (item);
@@ -679,7 +685,7 @@ gimp_group_layer_resize (GimpItem    *item,
           gint child_offset_x = gimp_item_get_offset_x (child) - child_x;
           gint child_offset_y = gimp_item_get_offset_y (child) - child_y;
 
-          gimp_item_resize (child, context,
+          gimp_item_resize (child, context, fill_type,
                             child_width, child_height,
                             child_offset_x, child_offset_y);
         }
@@ -698,7 +704,7 @@ gimp_group_layer_resize (GimpItem    *item,
   mask = gimp_layer_get_mask (GIMP_LAYER (group));
 
   if (mask)
-    gimp_item_resize (GIMP_ITEM (mask), context,
+    gimp_item_resize (GIMP_ITEM (mask), context, GIMP_FILL_TRANSPARENT,
                       new_width, new_height, offset_x, offset_y);
 
   gimp_group_layer_resume_resize (group, TRUE);
@@ -811,10 +817,10 @@ gimp_group_layer_transform (GimpItem               *item,
 }
 
 static gint64
-gimp_group_layer_estimate_memsize (const GimpDrawable *drawable,
-                                   GimpComponentType   component_type,
-                                   gint                width,
-                                   gint                height)
+gimp_group_layer_estimate_memsize (GimpDrawable      *drawable,
+                                   GimpComponentType  component_type,
+                                   gint               width,
+                                   gint               height)
 {
   GimpGroupLayerPrivate *private = GET_PRIVATE (drawable);
   GList                 *list;
@@ -876,16 +882,14 @@ get_projection_format (GimpProjectable   *projectable,
 }
 
 static void
-gimp_group_layer_convert_type (GimpDrawable      *drawable,
-                               GimpImage         *dest_image,
-                               const Babl        *new_format /* unused */,
-                               GimpImageBaseType  new_base_type,
-                               GimpPrecision      new_precision,
-                               gint               layer_dither_type,
-                               gint               mask_dither_type,
-                               gboolean           convert_profile,
-                               gboolean           push_undo,
-                               GimpProgress      *progress)
+gimp_group_layer_convert_type (GimpDrawable     *drawable,
+                               GimpImage        *dest_image,
+                               const Babl       *new_format,
+                               GimpColorProfile *dest_profile,
+                               GeglDitherMethod  layer_dither_type,
+                               GeglDitherMethod  mask_dither_type,
+                               gboolean          push_undo,
+                               GimpProgress     *progress)
 {
   GimpGroupLayer        *group   = GIMP_GROUP_LAYER (drawable);
   GimpGroupLayerPrivate *private = GET_PRIVATE (drawable);
@@ -903,9 +907,10 @@ gimp_group_layer_convert_type (GimpDrawable      *drawable,
    *  values so the projection will create its tiles with the right
    *  depth
    */
-  private->convert_format = get_projection_format (GIMP_PROJECTABLE (drawable),
-                                                   new_base_type,
-                                                   new_precision);
+  private->convert_format =
+    get_projection_format (GIMP_PROJECTABLE (drawable),
+                           gimp_babl_format_get_base_type (new_format),
+                           gimp_babl_format_get_precision (new_format));
   gimp_projectable_structure_changed (GIMP_PROJECTABLE (drawable));
   gimp_pickable_flush (GIMP_PICKABLE (private->projection));
 
@@ -923,12 +928,15 @@ gimp_group_layer_convert_type (GimpDrawable      *drawable,
   mask = gimp_layer_get_mask (GIMP_LAYER (group));
 
   if (mask &&
-      new_precision != gimp_drawable_get_precision (GIMP_DRAWABLE (mask)))
+      gimp_babl_format_get_precision (new_format) !=
+      gimp_drawable_get_precision (GIMP_DRAWABLE (mask)))
     {
       gimp_drawable_convert_type (GIMP_DRAWABLE (mask), dest_image,
-                                  GIMP_GRAY, new_precision,
+                                  GIMP_GRAY,
+                                  gimp_babl_format_get_precision (new_format),
+                                  gimp_drawable_has_alpha (GIMP_DRAWABLE (mask)),
+                                  NULL,
                                   layer_dither_type, mask_dither_type,
-                                  convert_profile,
                                   push_undo, progress);
     }
 }
@@ -1141,13 +1149,24 @@ gimp_group_layer_update_size (GimpGroupLayer *group)
        list = g_list_next (list))
     {
       GimpItem *child = list->data;
+      gint      child_width;
+      gint      child_height;
+
+      if (! gimp_viewable_get_size (GIMP_VIEWABLE (child),
+                                    &child_width, &child_height))
+        {
+          /*  ignore children without content (empty group layers);
+           *  see bug 777017
+           */
+          continue;
+        }
 
       if (first)
         {
           x      = gimp_item_get_offset_x (child);
           y      = gimp_item_get_offset_y (child);
-          width  = gimp_item_get_width    (child);
-          height = gimp_item_get_height   (child);
+          width  = child_width;
+          height = child_height;
 
           first = FALSE;
         }
@@ -1156,8 +1175,8 @@ gimp_group_layer_update_size (GimpGroupLayer *group)
           gimp_rectangle_union (x, y, width, height,
                                 gimp_item_get_offset_x (child),
                                 gimp_item_get_offset_y (child),
-                                gimp_item_get_width    (child),
-                                gimp_item_get_height   (child),
+                                child_width,
+                                child_height,
                                 &x, &y, &width, &height);
         }
     }
