@@ -42,11 +42,12 @@
 #include "gimpimage-merge.h"
 #include "gimpimage-undo.h"
 #include "gimpitemstack.h"
-#include "gimplayer-floating-sel.h"
+#include "gimplayer-floating-selection.h"
 #include "gimplayer-new.h"
 #include "gimplayermask.h"
 #include "gimpmarshal.h"
 #include "gimpparasitelist.h"
+#include "gimppickable.h"
 #include "gimpundostack.h"
 
 #include "gimp-intl.h"
@@ -404,8 +405,8 @@ gimp_image_merge_visible_vectors (GimpImage  *image,
   else
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
-			   _("Not enough visible paths for a merge. "
-			     "There must be at least two."));
+                           _("Not enough visible paths for a merge. "
+                             "There must be at least two."));
       return NULL;
     }
 }
@@ -527,7 +528,8 @@ gimp_image_merge_layers (GimpImage     *image,
       merge_layer = gimp_layer_new (image, (x2 - x1), (y2 - y1),
                                     gimp_image_get_layer_format (image, FALSE),
                                     gimp_object_get_name (layer),
-                                    GIMP_OPACITY_OPAQUE, GIMP_NORMAL_MODE);
+                                    GIMP_OPACITY_OPAQUE,
+                                    GIMP_LAYER_MODE_NORMAL_LEGACY);
       if (! merge_layer)
         {
           g_warning ("%s: could not allocate merge layer.", G_STRFUNC);
@@ -538,6 +540,8 @@ gimp_image_merge_layers (GimpImage     *image,
 
       /*  get the background for compositing  */
       gimp_context_get_background (context, &bg);
+      gimp_pickable_srgb_to_image_color (GIMP_PICKABLE (layer),
+                                         &bg, &bg);
 
       color = gimp_gegl_color_new (&bg);
       gegl_buffer_set_color (gimp_drawable_get_buffer (GIMP_DRAWABLE (merge_layer)),
@@ -557,7 +561,8 @@ gimp_image_merge_layers (GimpImage     *image,
         gimp_layer_new (image, (x2 - x1), (y2 - y1),
                         gimp_drawable_get_format_with_alpha (GIMP_DRAWABLE (layer)),
                         "merged layer",
-                        GIMP_OPACITY_OPAQUE, GIMP_NORMAL_MODE);
+                        GIMP_OPACITY_OPAQUE,
+                        GIMP_LAYER_MODE_NORMAL_LEGACY);
 
       if (!merge_layer)
         {
@@ -593,30 +598,41 @@ gimp_image_merge_layers (GimpImage     *image,
 
   for (layers = reverse_list; layers; layers = g_slist_next (layers))
     {
-      GeglBuffer           *merge_buffer;
-      GeglBuffer           *layer_buffer;
-      GimpApplicator       *applicator;
-      GimpLayerModeEffects  mode;
+      GeglBuffer             *merge_buffer;
+      GeglBuffer             *layer_buffer;
+      GimpApplicator         *applicator;
+      GimpLayerMode           mode;
+      GimpLayerColorSpace     blend_space;
+      GimpLayerColorSpace     composite_space;
+      GimpLayerCompositeMode  composite_mode;
 
       layer = layers->data;
 
       gimp_item_get_offset (GIMP_ITEM (layer), &off_x, &off_y);
 
-      /* DISSOLVE_MODE is special since it is the only mode that does not
+      /* MODE_DISSOLVE is special since it is the only mode that does not
        *  work on the projection with the lower layer, but only locally on
        *  the layers alpha channel.
        */
-      mode = gimp_layer_get_mode (layer);
-      if (layer == bottom_layer && mode != GIMP_DISSOLVE_MODE)
-        mode = GIMP_NORMAL_MODE;
+      mode            = gimp_layer_get_mode (layer);
+      blend_space     = gimp_layer_get_blend_space (layer);
+      composite_space = gimp_layer_get_composite_space (layer);
+      composite_mode  = gimp_layer_get_composite_mode (layer);
+
+      if (layer == bottom_layer)
+        {
+          if (mode != GIMP_LAYER_MODE_DISSOLVE)
+            mode          = GIMP_LAYER_MODE_NORMAL_LEGACY;
+
+          blend_space     = GIMP_LAYER_COLOR_SPACE_AUTO;
+          composite_space = GIMP_LAYER_COLOR_SPACE_AUTO;
+          composite_mode  = GIMP_LAYER_COMPOSITE_AUTO;
+        }
 
       merge_buffer = gimp_drawable_get_buffer (GIMP_DRAWABLE (merge_layer));
       layer_buffer = gimp_drawable_get_buffer (GIMP_DRAWABLE (layer));
 
-      applicator =
-        gimp_applicator_new (NULL,
-                             gimp_drawable_get_linear (GIMP_DRAWABLE (layer)),
-                             FALSE, FALSE);
+      applicator = gimp_applicator_new (NULL, FALSE, FALSE);
 
       if (gimp_layer_get_mask (layer) &&
           gimp_layer_get_apply_mask (layer))
@@ -639,9 +655,9 @@ gimp_image_merge_layers (GimpImage     *image,
                                         - (x1 - off_x),
                                         - (y1 - off_y));
 
-      gimp_applicator_set_mode (applicator,
-                                gimp_layer_get_opacity (layer),
-                                mode);
+      gimp_applicator_set_opacity (applicator, gimp_layer_get_opacity (layer));
+      gimp_applicator_set_mode (applicator, mode,
+                                blend_space, composite_space, composite_mode);
 
       gimp_applicator_blit (applicator,
                             GEGL_RECTANGLE (0, 0,
@@ -683,10 +699,7 @@ gimp_image_merge_layers (GimpImage     *image,
                             TRUE);
     }
 
-  gimp_drawable_update (GIMP_DRAWABLE (merge_layer),
-                        0, 0,
-                        gimp_item_get_width  (GIMP_ITEM (merge_layer)),
-                        gimp_item_get_height (GIMP_ITEM (merge_layer)));
+  gimp_drawable_update (GIMP_DRAWABLE (merge_layer), 0, 0, -1, -1);
 
   return merge_layer;
 }
