@@ -56,11 +56,15 @@ struct _GimpImagePrivate
   const Babl        *babl_palette_rgb;      /*  palette's RGB Babl format    */
   const Babl        *babl_palette_rgba;     /*  palette's RGBA Babl format   */
 
+  gboolean           is_color_managed;      /*  is this image color managed  */
   GimpColorProfile  *color_profile;         /*  image's color profile        */
-  GimpColorTransform transform_to_srgb;     /*  from layer pixels to sRGB    */
-  GimpColorTransform transform_from_srgb;   /*  from sRGB to layer pixels    */
-  const Babl        *transform_layer_format;/*  layer format for transforms  */
-  const Babl        *transform_srgb_format; /*  sRGB format for transforms   */
+
+  /*  Cached color transforms: from layer to sRGB u8 and double, and back    */
+  gboolean            color_transforms_created;
+  GimpColorTransform *transform_to_srgb_u8;
+  GimpColorTransform *transform_from_srgb_u8;
+  GimpColorTransform *transform_to_srgb_double;
+  GimpColorTransform *transform_from_srgb_double;
 
   GimpMetadata      *metadata;              /*  image's metadata             */
 
