@@ -50,13 +50,14 @@ enum
   PROP_VERTICAL_SYMMETRY,
   PROP_POINT_SYMMETRY,
   PROP_DISABLE_TRANSFORMATION,
-  PROP_HORIZONTAL_POSITION,
-  PROP_VERTICAL_POSITION
+  PROP_MIRROR_POSITION_X,
+  PROP_MIRROR_POSITION_Y
 };
 
 
 /* Local function prototypes */
 
+static void       gimp_mirror_constructed             (GObject             *object);
 static void       gimp_mirror_finalize                (GObject             *object);
 static void       gimp_mirror_set_property            (GObject             *object,
                                                        guint                property_id,
@@ -95,6 +96,12 @@ static void       gimp_mirror_set_vertical_symmetry   (GimpMirror          *mirr
 static void       gimp_mirror_set_point_symmetry      (GimpMirror          *mirror,
                                                        gboolean             active);
 
+static void       gimp_mirror_image_size_changed_cb   (GimpImage           *image,
+                                                       gint                 previous_origin_x,
+                                                       gint                 previous_origin_y,
+                                                       gint                 previous_width,
+                                                       gint                 previous_height,
+                                                       GimpSymmetry        *sym);
 
 G_DEFINE_TYPE (GimpMirror, gimp_mirror, GIMP_TYPE_SYMMETRY)
 
@@ -106,7 +113,9 @@ gimp_mirror_class_init (GimpMirrorClass *klass)
 {
   GObjectClass      *object_class   = G_OBJECT_CLASS (klass);
   GimpSymmetryClass *symmetry_class = GIMP_SYMMETRY_CLASS (klass);
+  GParamSpec        *pspec;
 
+  object_class->constructed         = gimp_mirror_constructed;
   object_class->finalize            = gimp_mirror_finalize;
   object_class->set_property        = gimp_mirror_set_property;
   object_class->get_property        = gimp_mirror_get_property;
@@ -118,16 +127,16 @@ gimp_mirror_class_init (GimpMirrorClass *klass)
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_HORIZONTAL_SYMMETRY,
                             "horizontal-symmetry",
-                            _("Horizontal Mirror"),
-                            NULL,
+                            _("Horizontal Symmetry"),
+                            _("Reflect the initial stroke across a horizontal axis"),
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS |
                             GIMP_SYMMETRY_PARAM_GUI);
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_VERTICAL_SYMMETRY,
                             "vertical-symmetry",
-                            _("Vertical Mirror"),
-                            NULL,
+                            _("Vertical Symmetry"),
+                            _("Reflect the initial stroke across a vertical axis"),
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS |
                             GIMP_SYMMETRY_PARAM_GUI);
@@ -135,7 +144,7 @@ gimp_mirror_class_init (GimpMirrorClass *klass)
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_POINT_SYMMETRY,
                             "point-symmetry",
                             _("Central Symmetry"),
-                            NULL,
+                            _("Invert the initial stroke through a point"),
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS |
                             GIMP_SYMMETRY_PARAM_GUI);
@@ -143,26 +152,34 @@ gimp_mirror_class_init (GimpMirrorClass *klass)
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_DISABLE_TRANSFORMATION,
                             "disable-transformation",
                             _("Disable brush transform"),
-                            _("Disable brush transformation (faster)"),
+                            _("Disable brush reflection"),
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS |
                             GIMP_SYMMETRY_PARAM_GUI);
 
-  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_HORIZONTAL_POSITION,
-                           "horizontal-position",
+  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_MIRROR_POSITION_X,
+                           "mirror-position-x",
+                           _("Vertical axis position"),
+                           NULL,
+                           0.0, G_MAXDOUBLE, 0.0,
+                           GIMP_PARAM_STATIC_STRINGS |
+                           GIMP_SYMMETRY_PARAM_GUI);
+
+  pspec = g_object_class_find_property (object_class, "mirror-position-x");
+  gegl_param_spec_set_property_key (pspec, "unit", "pixel-coordinate");
+  gegl_param_spec_set_property_key (pspec, "axis", "x");
+
+  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_MIRROR_POSITION_Y,
+                           "mirror-position-y",
                            _("Horizontal axis position"),
                            NULL,
                            0.0, G_MAXDOUBLE, 0.0,
                            GIMP_PARAM_STATIC_STRINGS |
                            GIMP_SYMMETRY_PARAM_GUI);
 
-  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_VERTICAL_POSITION,
-                           "vertical-position",
-                           _("Vertical axis position"),
-                           NULL,
-                           0.0, G_MAXDOUBLE, 0.0,
-                           GIMP_PARAM_STATIC_STRINGS |
-                           GIMP_SYMMETRY_PARAM_GUI);
+  pspec = g_object_class_find_property (object_class, "mirror-position-y");
+  gegl_param_spec_set_property_key (pspec, "unit", "pixel-coordinate");
+  gegl_param_spec_set_property_key (pspec, "axis", "y");
 }
 
 static void
@@ -170,6 +187,16 @@ gimp_mirror_init (GimpMirror *mirror)
 {
 }
 
+static void
+gimp_mirror_constructed (GObject *object)
+{
+  GimpSymmetry *sym = GIMP_SYMMETRY (object);
+
+  g_signal_connect_object (sym->image, "size-changed-detailed",
+                           G_CALLBACK (gimp_mirror_image_size_changed_cb),
+                           sym, 0);
+}
+
 static void
 gimp_mirror_finalize (GObject *object)
 {
@@ -213,48 +240,34 @@ gimp_mirror_set_property (GObject      *object,
       gimp_mirror_set_horizontal_symmetry (mirror,
                                            g_value_get_boolean (value));
       break;
+
     case PROP_VERTICAL_SYMMETRY:
       gimp_mirror_set_vertical_symmetry (mirror,
                                          g_value_get_boolean (value));
       break;
+
     case PROP_POINT_SYMMETRY:
       gimp_mirror_set_point_symmetry (mirror,
                                       g_value_get_boolean (value));
       break;
+
     case PROP_DISABLE_TRANSFORMATION:
       mirror->disable_transformation = g_value_get_boolean (value);
       break;
-    case PROP_HORIZONTAL_POSITION:
-      if (g_value_get_double (value) > 0.0 &&
-          g_value_get_double (value) < (gdouble) gimp_image_get_height (image))
-        {
-          mirror->horizontal_position = g_value_get_double (value);
-          if (mirror->horizontal_guide)
-            {
-              g_signal_handlers_block_by_func (mirror->horizontal_guide,
-                                               gimp_mirror_guide_position_cb,
-                                               mirror);
-              gimp_image_move_guide (image, mirror->horizontal_guide,
-                                     mirror->horizontal_position,
-                                     FALSE);
-              g_signal_handlers_unblock_by_func (mirror->horizontal_guide,
-                                                 gimp_mirror_guide_position_cb,
-                                                 mirror);
-            }
-        }
-      break;
-    case PROP_VERTICAL_POSITION:
+
+    case PROP_MIRROR_POSITION_X:
       if (g_value_get_double (value) > 0.0 &&
           g_value_get_double (value) < (gdouble) gimp_image_get_width (image))
         {
-          mirror->vertical_position = g_value_get_double (value);
+          mirror->mirror_position_x = g_value_get_double (value);
+
           if (mirror->vertical_guide)
             {
               g_signal_handlers_block_by_func (mirror->vertical_guide,
                                                gimp_mirror_guide_position_cb,
                                                mirror);
               gimp_image_move_guide (image, mirror->vertical_guide,
-                                     mirror->vertical_position,
+                                     mirror->mirror_position_x,
                                      FALSE);
               g_signal_handlers_unblock_by_func (mirror->vertical_guide,
                                                  gimp_mirror_guide_position_cb,
@@ -262,6 +275,28 @@ gimp_mirror_set_property (GObject      *object,
             }
         }
       break;
+
+    case PROP_MIRROR_POSITION_Y:
+      if (g_value_get_double (value) > 0.0 &&
+          g_value_get_double (value) < (gdouble) gimp_image_get_height (image))
+        {
+          mirror->mirror_position_y = g_value_get_double (value);
+
+          if (mirror->horizontal_guide)
+            {
+              g_signal_handlers_block_by_func (mirror->horizontal_guide,
+                                               gimp_mirror_guide_position_cb,
+                                               mirror);
+              gimp_image_move_guide (image, mirror->horizontal_guide,
+                                     mirror->mirror_position_y,
+                                     FALSE);
+              g_signal_handlers_unblock_by_func (mirror->horizontal_guide,
+                                                 gimp_mirror_guide_position_cb,
+                                                 mirror);
+            }
+        }
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -290,11 +325,11 @@ gimp_mirror_get_property (GObject    *object,
     case PROP_DISABLE_TRANSFORMATION:
       g_value_set_boolean (value, mirror->disable_transformation);
       break;
-    case PROP_HORIZONTAL_POSITION:
-      g_value_set_double (value, mirror->horizontal_position);
+    case PROP_MIRROR_POSITION_X:
+      g_value_set_double (value, mirror->mirror_position_x);
       break;
-    case PROP_VERTICAL_POSITION:
-      g_value_set_double (value, mirror->vertical_position);
+    case PROP_MIRROR_POSITION_Y:
+      g_value_set_double (value, mirror->mirror_position_y);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -318,22 +353,22 @@ gimp_mirror_update_strokes (GimpSymmetry *sym,
   if (mirror->horizontal_mirror)
     {
       coords = g_memdup (origin, sizeof (GimpCoords));
-      coords->y = 2.0 * mirror->horizontal_position - origin->y;
+      coords->y = 2.0 * mirror->mirror_position_y - origin->y;
       strokes = g_list_prepend (strokes, coords);
     }
 
   if (mirror->vertical_mirror)
     {
       coords = g_memdup (origin, sizeof (GimpCoords));
-      coords->x = 2.0 * mirror->vertical_position - origin->x;
+      coords->x = 2.0 * mirror->mirror_position_x - origin->x;
       strokes = g_list_prepend (strokes, coords);
     }
 
   if (mirror->point_symmetry)
     {
       coords = g_memdup (origin, sizeof (GimpCoords));
-      coords->x = 2.0 * mirror->vertical_position - origin->x;
-      coords->y = 2.0 * mirror->horizontal_position - origin->y;
+      coords->x = 2.0 * mirror->mirror_position_x - origin->x;
+      coords->y = 2.0 * mirror->mirror_position_y - origin->y;
       strokes = g_list_prepend (strokes, coords);
     }
 
@@ -342,9 +377,10 @@ gimp_mirror_update_strokes (GimpSymmetry *sym,
   g_signal_emit_by_name (sym, "strokes-updated", sym->image);
 }
 
-static void gimp_mirror_prepare_operations (GimpMirror *mirror,
-                                            gint        paint_width,
-                                            gint        paint_height)
+static void
+gimp_mirror_prepare_operations (GimpMirror *mirror,
+                                gint        paint_width,
+                                gint        paint_height)
 {
   if (paint_width == mirror->last_paint_width &&
       paint_height == mirror->last_paint_height)
@@ -448,13 +484,13 @@ gimp_mirror_add_guide (GimpMirror          *mirror,
   if (orientation == GIMP_ORIENTATION_HORIZONTAL)
     {
       /* Mirror guide position at first activation is at canvas middle. */
-      if (mirror->horizontal_position < 1.0)
+      if (mirror->mirror_position_y < 1.0)
         position = gimp_image_get_height (image) / 2.0;
       else
-        position = mirror->horizontal_position;
+        position = mirror->mirror_position_y;
 
       g_object_set (mirror,
-                    "horizontal-position", position,
+                    "mirror-position-y", position,
                     NULL);
 
       mirror->horizontal_guide = guide;
@@ -462,13 +498,13 @@ gimp_mirror_add_guide (GimpMirror          *mirror,
   else
     {
       /* Mirror guide position at first activation is at canvas middle. */
-      if (mirror->vertical_position < 1.0)
+      if (mirror->mirror_position_x < 1.0)
         position = gimp_image_get_width (image) / 2.0;
       else
-        position = mirror->vertical_position;
+        position = mirror->mirror_position_x;
 
       g_object_set (mirror,
-                    "vertical-position", position,
+                    "mirror-position-x", position,
                     NULL);
 
       mirror->vertical_guide = guide;
@@ -503,6 +539,7 @@ gimp_mirror_remove_guide (GimpMirror          *mirror,
   g_signal_handlers_disconnect_by_func (G_OBJECT (guide),
                                         gimp_mirror_guide_position_cb,
                                         mirror);
+
   gimp_image_remove_guide (image, guide, FALSE);
   g_object_unref (guide);
 
@@ -533,7 +570,7 @@ gimp_mirror_guide_removed_cb (GObject    *object,
       mirror->horizontal_mirror   = FALSE;
       mirror->point_symmetry      = FALSE;
       g_object_set (mirror,
-                    "horizontal-position", 0.0,
+                    "mirror-position-y", 0.0,
                     NULL);
 
       if (mirror->vertical_guide &&
@@ -545,6 +582,7 @@ gimp_mirror_guide_removed_cb (GObject    *object,
           g_signal_handlers_disconnect_by_func (G_OBJECT (mirror->vertical_guide),
                                                 gimp_mirror_guide_position_cb,
                                                 mirror);
+
           gimp_image_remove_guide (symmetry->image,
                                    mirror->vertical_guide,
                                    FALSE);
@@ -559,7 +597,7 @@ gimp_mirror_guide_removed_cb (GObject    *object,
       mirror->vertical_mirror   = FALSE;
       mirror->point_symmetry    = FALSE;
       g_object_set (mirror,
-                    "vertical-position", 0.0,
+                    "mirror-position-x", 0.0,
                     NULL);
 
       if (mirror->horizontal_guide &&
@@ -571,6 +609,7 @@ gimp_mirror_guide_removed_cb (GObject    *object,
           g_signal_handlers_disconnect_by_func (G_OBJECT (mirror->horizontal_guide),
                                                 gimp_mirror_guide_position_cb,
                                                 mirror);
+
           gimp_image_remove_guide (symmetry->image,
                                    mirror->horizontal_guide,
                                    FALSE);
@@ -587,7 +626,7 @@ gimp_mirror_guide_removed_cb (GObject    *object,
   else
     {
       gimp_mirror_reset (mirror);
-      g_signal_emit_by_name (mirror, "update-ui",
+      g_signal_emit_by_name (mirror, "gui-param-changed",
                              GIMP_SYMMETRY (mirror)->image);
     }
 }
@@ -602,13 +641,13 @@ gimp_mirror_guide_position_cb (GObject    *object,
   if (guide == mirror->horizontal_guide)
     {
       g_object_set (mirror,
-                    "horizontal-position", (gdouble) gimp_guide_get_position (guide),
+                    "mirror-position-y", (gdouble) gimp_guide_get_position (guide),
                     NULL);
     }
   else if (guide == mirror->vertical_guide)
     {
       g_object_set (mirror,
-                    "vertical-position", (gdouble) gimp_guide_get_position (guide),
+                    "mirror-position-x", (gdouble) gimp_guide_get_position (guide),
                     NULL);
     }
 }
@@ -623,6 +662,7 @@ gimp_mirror_active_changed (GimpSymmetry *sym)
       if ((mirror->horizontal_mirror || mirror->point_symmetry) &&
           ! mirror->horizontal_guide)
         gimp_mirror_add_guide (mirror, GIMP_ORIENTATION_HORIZONTAL);
+
       if ((mirror->vertical_mirror || mirror->point_symmetry) &&
           ! mirror->vertical_guide)
         gimp_mirror_add_guide (mirror, GIMP_ORIENTATION_VERTICAL);
@@ -631,6 +671,7 @@ gimp_mirror_active_changed (GimpSymmetry *sym)
     {
       if (mirror->horizontal_guide)
         gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_HORIZONTAL);
+
       if (mirror->vertical_guide)
         gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_VERTICAL);
     }
@@ -651,7 +692,9 @@ gimp_mirror_set_horizontal_symmetry (GimpMirror *mirror,
         gimp_mirror_add_guide (mirror, GIMP_ORIENTATION_HORIZONTAL);
     }
   else if (! mirror->point_symmetry)
-    gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_HORIZONTAL);
+    {
+      gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_HORIZONTAL);
+    }
 
   gimp_mirror_reset (mirror);
 }
@@ -671,7 +714,9 @@ gimp_mirror_set_vertical_symmetry (GimpMirror *mirror,
         gimp_mirror_add_guide (mirror, GIMP_ORIENTATION_VERTICAL);
     }
   else if (! mirror->point_symmetry)
-    gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_VERTICAL);
+    {
+      gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_VERTICAL);
+    }
 
   gimp_mirror_reset (mirror);
 }
@@ -700,6 +745,7 @@ gimp_mirror_set_point_symmetry (GimpMirror *mirror,
       /* Remove the horizontal guide unless needed by horizontal mirror */
       if (! mirror->horizontal_mirror)
         gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_HORIZONTAL);
+
       /* Remove the vertical guide unless needed by vertical mirror */
       if (! mirror->vertical_mirror)
         gimp_mirror_remove_guide (mirror, GIMP_ORIENTATION_VERTICAL);
@@ -707,3 +753,18 @@ gimp_mirror_set_point_symmetry (GimpMirror *mirror,
 
   gimp_mirror_reset (mirror);
 }
+
+static void
+gimp_mirror_image_size_changed_cb (GimpImage    *image,
+                                   gint          previous_origin_x,
+                                   gint          previous_origin_y,
+                                   gint          previous_width,
+                                   gint          previous_height,
+                                   GimpSymmetry *sym)
+{
+  if (previous_width != gimp_image_get_width (image) ||
+      previous_height != gimp_image_get_height (image))
+    {
+      g_signal_emit_by_name (sym, "gui-param-changed", sym->image);
+    }
+}
