@@ -349,7 +349,7 @@ insert_spacers (const gchar *string)
   return g_string_free (str, FALSE);
 }
 
-static void inline
+static inline void
 mix_colors (const GdkRGBA *start,
             const GdkRGBA *end,
             GdkRGBA       *target,
