@@ -2,7 +2,7 @@
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * gimpcursorview.c
- * Copyright (C) 2005 Michael Natterer <mitch@gimp.org>
+ * Copyright (C) 2005-2016 Michael Natterer <mitch@gimp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,9 @@
 
 #include "display-types.h"
 
+#include "config/gimpcoreconfig.h"
+
+#include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-pick-color.h"
@@ -214,13 +217,13 @@ gimp_cursor_view_init (GimpCursorView *view)
   gtk_widget_show (table);
 
   view->priv->pixel_x_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->pixel_x_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->pixel_x_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
                              _("X"), 0.5, 0.5,
                              view->priv->pixel_x_label, 1, FALSE);
 
   view->priv->pixel_y_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->pixel_y_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->pixel_y_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
                              _("Y"), 0.5, 0.5,
                              view->priv->pixel_y_label, 1, FALSE);
@@ -239,13 +242,13 @@ gimp_cursor_view_init (GimpCursorView *view)
   gtk_widget_show (table);
 
   view->priv->unit_x_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->unit_x_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->unit_x_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
                              _("X"), 0.5, 0.5,
                              view->priv->unit_x_label, 1, FALSE);
 
   view->priv->unit_y_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->unit_y_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->unit_y_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
                              _("Y"), 0.5, 0.5,
                              view->priv->unit_y_label, 1, FALSE);
@@ -264,13 +267,13 @@ gimp_cursor_view_init (GimpCursorView *view)
   gtk_widget_show (table);
 
   view->priv->selection_x_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->selection_x_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->selection_x_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
                              _("X"), 0.5, 0.5,
                              view->priv->selection_x_label, 1, FALSE);
 
   view->priv->selection_y_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->selection_y_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->selection_y_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
                              _("Y"), 0.5, 0.5,
                              view->priv->selection_y_label, 1, FALSE);
@@ -286,14 +289,14 @@ gimp_cursor_view_init (GimpCursorView *view)
   gtk_widget_show (table);
 
   view->priv->selection_width_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->selection_width_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->selection_width_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
                              /* Width */
                              _("W"), 0.5, 0.5,
                              view->priv->selection_width_label, 1, FALSE);
 
   view->priv->selection_height_label = gtk_label_new (_("n/a"));
-  gtk_misc_set_alignment (GTK_MISC (view->priv->selection_height_label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (view->priv->selection_height_label), 1.0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
                              /* Height */
                              _("H"), 0.5, 0.5,
@@ -317,7 +320,7 @@ gimp_cursor_view_init (GimpCursorView *view)
 
   view->priv->color_frame_2 = gimp_color_frame_new ();
   gimp_color_frame_set_mode (GIMP_COLOR_FRAME (view->priv->color_frame_2),
-                             GIMP_COLOR_FRAME_MODE_RGB);
+                             GIMP_COLOR_FRAME_MODE_RGB_PERCENT);
   gtk_box_pack_start (GTK_BOX (view->priv->color_hbox), view->priv->color_frame_2,
                       TRUE, TRUE, 0);
   gtk_widget_show (view->priv->color_frame_2);
@@ -367,7 +370,8 @@ gimp_cursor_view_set_property (GObject      *object,
     case PROP_SAMPLE_MERGED:
       view->priv->sample_merged = g_value_get_boolean (value);
       break;
-   default:
+
+    default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
@@ -386,7 +390,8 @@ gimp_cursor_view_get_property (GObject    *object,
     case PROP_SAMPLE_MERGED:
       g_value_set_boolean (value, view->priv->sample_merged);
       break;
-   default:
+
+    default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
@@ -516,9 +521,10 @@ static void
 gimp_cursor_view_set_context (GimpDocked  *docked,
                               GimpContext *context)
 {
-  GimpCursorView *view    = GIMP_CURSOR_VIEW (docked);
-  GimpDisplay    *display = NULL;
-  GimpImage      *image   = NULL;
+  GimpCursorView  *view    = GIMP_CURSOR_VIEW (docked);
+  GimpColorConfig *config  = NULL;
+  GimpDisplay     *display = NULL;
+  GimpImage       *image   = NULL;
 
   if (context == view->priv->context)
     return;
@@ -549,10 +555,16 @@ gimp_cursor_view_set_context (GimpDocked  *docked,
                                 G_CALLBACK (gimp_cursor_view_image_changed),
                                 view);
 
+      config  = context->gimp->config->color_management;
       display = gimp_context_get_display (context);
       image   = gimp_context_get_image (context);
     }
 
+  gimp_color_frame_set_color_config (GIMP_COLOR_FRAME (view->priv->color_frame_1),
+                                     config);
+  gimp_color_frame_set_color_config (GIMP_COLOR_FRAME (view->priv->color_frame_2),
+                                     config);
+
   gimp_cursor_view_diplay_changed (view, display, view->priv->context);
   gimp_cursor_view_image_changed (view, image, view->priv->context);
 }
