@@ -29,7 +29,9 @@
 #include "config/gimpdisplayconfig.h"
 
 #include "core/gimp.h"
+#include "core/gimp-filter-history.h"
 #include "core/gimpimage.h"
+#include "core/gimpitem.h"
 
 #include "widgets/gimpcontrollers.h"
 #include "widgets/gimpcontrollerkeyboard.h"
@@ -43,12 +45,13 @@
 #include "widgets/gimpuimanager.h"
 #include "widgets/gimpwidgets-utils.h"
 
-#include "tools/gimpimagemaptool.h"
+#include "tools/gimpguidetool.h"
 #include "tools/gimpmovetool.h"
-#include "tools/gimppainttool.h"
+#include "tools/gimpsamplepointtool.h"
 #include "tools/gimptoolcontrol.h"
 #include "tools/tool_manager.h"
 
+#include "gimpcanvas.h"
 #include "gimpdisplay.h"
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-autoscroll.h"
@@ -324,11 +327,21 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
   if (gimp_display_shell_events (canvas, event, shell))
     return TRUE;
 
-  /*  ignore events on overlays, which are the canvas' children
+  /*  events on overlays have a different window, but these windows'
+   *  user_data can still be the canvas, we need to check manually if
+   *  the event's window and the canvas' window are different.
    */
-  if (gtk_widget_is_ancestor (gtk_get_event_widget (event), shell->canvas))
+  if (event->any.window != gtk_widget_get_window (canvas))
     {
-      return FALSE;
+      GtkWidget *event_widget;
+
+      gdk_window_get_user_data (event->any.window, (gpointer) &event_widget);
+
+      /*  if the event came from a different window than the canvas',
+       *  check if it came from a canvas child and bail out.
+       */
+      if (gtk_widget_get_ancestor (event_widget, GIMP_TYPE_CANVAS))
+        return FALSE;
     }
 
   display = shell->display;
@@ -634,7 +647,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
 
         active_tool = tool_manager_get_active (gimp);
 
-        state &= ~gimp_display_shell_key_to_state (bevent->button);
+        state &= ~gimp_display_shell_button_to_state (bevent->button);
 
         if (bevent->button == 1)
           {
@@ -853,6 +866,12 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                                                 y,
                                                 constrain);
               }
+            else if (shell->scaling)
+              {
+                gimp_display_shell_scale_drag (shell,
+                                               shell->scroll_last_x - x,
+                                               shell->scroll_last_y - y);
+              }
             else
               {
                 gimp_display_shell_scroll (shell,
@@ -1222,6 +1241,43 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
   return return_val;
 }
 
+void
+gimp_display_shell_canvas_grab_notify (GtkWidget        *canvas,
+                                       gboolean          was_grabbed,
+                                       GimpDisplayShell *shell)
+{
+  GimpDisplay *display;
+  GimpImage   *image;
+  Gimp        *gimp;
+
+  /*  are we in destruction?  */
+  if (! shell->display || ! gimp_display_get_shell (shell->display))
+    return;
+
+  display = shell->display;
+  gimp    = gimp_display_get_gimp (display);
+  image   = gimp_display_get_image (display);
+
+  if (! image)
+    return;
+
+  GIMP_LOG (TOOL_EVENTS, "grab_notify (display %p): was_grabbed = %s",
+            display, was_grabbed ? "TRUE" : "FALSE");
+
+  if (! was_grabbed)
+    {
+      if (! gimp_image_is_empty (image))
+        {
+          GimpTool *active_tool = tool_manager_get_active (gimp);
+
+          if (active_tool && active_tool->focus_display == display)
+            {
+              tool_manager_modifier_state_active (gimp, 0, display);
+            }
+        }
+    }
+}
+
 void
 gimp_display_shell_buffer_stroke (GimpMotionBuffer *buffer,
                                   const GimpCoords *coords,
@@ -1267,10 +1323,10 @@ gimp_display_shell_buffer_hover (GimpMotionBuffer *buffer,
 }
 
 static gboolean
-gimp_display_shell_ruler_button_press (GtkWidget        *widget,
-                                       GdkEventButton   *event,
-                                       GimpDisplayShell *shell,
-                                       gboolean          horizontal)
+gimp_display_shell_ruler_button_press (GtkWidget           *widget,
+                                       GdkEventButton      *event,
+                                       GimpDisplayShell    *shell,
+                                       GimpOrientationType  orientation)
 {
   GimpDisplay *display = shell->display;
 
@@ -1282,34 +1338,7 @@ gimp_display_shell_ruler_button_press (GtkWidget        *widget,
 
   if (event->type == GDK_BUTTON_PRESS && event->button == 1)
     {
-      GimpTool *active_tool;
-      gboolean  sample_point;
-
-      active_tool  = tool_manager_get_active (display->gimp);
-      sample_point = (event->state & gimp_get_toggle_behavior_mask ());
-
-      if (! ((sample_point && (GIMP_IS_COLOR_TOOL (active_tool) &&
-                               ! GIMP_IS_IMAGE_MAP_TOOL (active_tool) &&
-                               ! (GIMP_IS_PAINT_TOOL (active_tool) &&
-                                  ! GIMP_PAINT_TOOL (active_tool)->pick_colors)))
-
-             ||
-
-             (! sample_point && GIMP_IS_MOVE_TOOL (active_tool))))
-        {
-          GimpToolInfo *tool_info;
-
-          tool_info = gimp_get_tool_info (display->gimp,
-                                          sample_point ?
-                                          "gimp-color-picker-tool" :
-                                          "gimp-move-tool");
-
-          if (tool_info)
-            gimp_context_set_tool (gimp_get_user_context (display->gimp),
-                                   tool_info);
-        }
-
-      active_tool = tool_manager_get_active (display->gimp);
+      GimpTool *active_tool = tool_manager_get_active (display->gimp);
 
       if (active_tool)
         {
@@ -1322,12 +1351,15 @@ gimp_display_shell_ruler_button_press (GtkWidget        *widget,
             {
               if (gimp_display_shell_keyboard_grab (shell, (GdkEvent *) event))
                 {
-                  if (sample_point)
-                    gimp_color_tool_start_sample_point (active_tool, display);
-                  else if (horizontal)
-                    gimp_move_tool_start_hguide (active_tool, display);
+                  if (event->state & gimp_get_toggle_behavior_mask ())
+                    {
+                      gimp_sample_point_tool_start_new (active_tool, display);
+                    }
                   else
-                    gimp_move_tool_start_vguide (active_tool, display);
+                    {
+                      gimp_guide_tool_start_new (active_tool, display,
+                                                 orientation);
+                    }
 
                   return TRUE;
                 }
@@ -1347,7 +1379,8 @@ gimp_display_shell_hruler_button_press (GtkWidget        *widget,
                                         GdkEventButton   *event,
                                         GimpDisplayShell *shell)
 {
-  return gimp_display_shell_ruler_button_press (widget, event, shell, TRUE);
+  return gimp_display_shell_ruler_button_press (widget, event, shell,
+                                                GIMP_ORIENTATION_HORIZONTAL);
 }
 
 gboolean
@@ -1355,7 +1388,8 @@ gimp_display_shell_vruler_button_press (GtkWidget        *widget,
                                         GdkEventButton   *event,
                                         GimpDisplayShell *shell)
 {
-  return gimp_display_shell_ruler_button_press (widget, event, shell, FALSE);
+  return gimp_display_shell_ruler_button_press (widget, event, shell,
+                                                GIMP_ORIENTATION_VERTICAL);
 }
 
 
@@ -1371,17 +1405,21 @@ gimp_display_shell_key_to_state (gint key)
     case GDK_KEY_Alt_L:
     case GDK_KEY_Alt_R:
       return GDK_MOD1_MASK;
+
     case GDK_KEY_Shift_L:
     case GDK_KEY_Shift_R:
       return GDK_SHIFT_MASK;
+
     case GDK_KEY_Control_L:
     case GDK_KEY_Control_R:
       return GDK_CONTROL_MASK;
+
 #ifdef GDK_WINDOWING_QUARTZ
     case GDK_KEY_Meta_L:
     case GDK_KEY_Meta_R:
       return GDK_MOD2_MASK;
 #endif
+
     default:
       return 0;
     }
@@ -1506,12 +1544,16 @@ gimp_display_shell_start_scrolling (GimpDisplayShell *shell,
   shell->scrolling         = TRUE;
   shell->scroll_last_x     = x;
   shell->scroll_last_y     = y;
-  shell->rotating          = (state & GDK_SHIFT_MASK) ? TRUE : FALSE;
+  shell->rotating          = (state & gimp_get_extend_selection_mask ()) ? TRUE : FALSE;
   shell->rotate_drag_angle = shell->rotate_angle;
+  shell->scaling           = (state & gimp_get_toggle_behavior_mask ()) ? TRUE : FALSE;
 
   if (shell->rotating)
     gimp_display_shell_set_override_cursor (shell,
                                             (GimpCursorType) GDK_EXCHANGE);
+  if (shell->scaling)
+    gimp_display_shell_set_override_cursor (shell,
+                                            (GimpCursorType) GIMP_CURSOR_ZOOM);
   else
     gimp_display_shell_set_override_cursor (shell,
                                             (GimpCursorType) GDK_FLEUR);
@@ -1530,6 +1572,7 @@ gimp_display_shell_stop_scrolling (GimpDisplayShell *shell,
   shell->scroll_last_y     = 0;
   shell->rotating          = FALSE;
   shell->rotate_drag_angle = 0.0;
+  shell->scaling           = FALSE;
 
   gimp_display_shell_pointer_ungrab (shell, event);
 }
@@ -1814,8 +1857,45 @@ gimp_display_shell_initialize_tool (GimpDisplayShell *shell,
                 (gimp_tool_control_get_dirty_mask (active_tool->control) &
                  GIMP_DIRTY_ACTIVE_DRAWABLE)))
         {
-          /*  create a new one, deleting the current  */
-          gimp_context_tool_changed (gimp_get_user_context (gimp));
+          GimpProcedure *procedure = g_object_get_data (G_OBJECT (active_tool),
+                                                        "gimp-gegl-procedure");
+
+          if (image == gimp_item_get_image (GIMP_ITEM (active_tool->drawable)))
+            {
+              /*  When changing between drawables if the *same* image,
+               *  halt the tool so it doesn't get committed on tool
+               *  change. This is a pure "probably better this way"
+               *  decision because the user is likely changing their
+               *  mind or was simply on the wrong layer. See bug
+               *  #776370.
+               */
+              tool_manager_control_active (gimp, GIMP_TOOL_ACTION_HALT,
+                                           active_tool->display);
+            }
+
+          if (procedure)
+            {
+              /*  We can't just recreate an operation tool, we must
+               *  make sure the right stuff gets set on it, so
+               *  re-activate the procedure that created it instead of
+               *  just calling gimp_context_tool_changed(). See
+               *  GimpGeglProcedure and bug #776370.
+               */
+              GimpImageWindow *window;
+              GimpUIManager   *manager;
+
+              window  = gimp_display_shell_get_window (shell);
+              manager = gimp_image_window_get_ui_manager (window);
+
+              gimp_filter_history_add (gimp, procedure);
+              gimp_ui_manager_activate_action (manager, "filters",
+                                               "filters-reshow");
+            }
+          else
+            {
+              /*  create a new one, deleting the current  */
+              gimp_context_tool_changed (gimp_get_user_context (gimp));
+            }
 
           /*  make sure the newly created tool has the right state  */
           gimp_display_shell_update_focus (shell, TRUE, image_coords, state);
