@@ -175,7 +175,7 @@ static void   gimp_display_shell_transform_overlay (GimpDisplayShell *shell,
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpDisplayShell, gimp_display_shell,
-                         GTK_TYPE_BOX,
+                         GTK_TYPE_EVENT_BOX,
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_PROGRESS,
                                                 gimp_display_shell_progress_iface_init)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_COLOR_MANAGED,
@@ -392,6 +392,7 @@ gimp_display_shell_constructed (GObject *object)
   GimpDisplayConfig     *config;
   GimpImage             *image;
   GimpColorDisplayStack *filter;
+  GtkWidget             *main_vbox;
   GtkWidget             *upper_hbox;
   GtkWidget             *right_vbox;
   GtkWidget             *lower_hbox;
@@ -411,6 +412,8 @@ gimp_display_shell_constructed (GObject *object)
   config = shell->display->config;
   image  = gimp_display_get_image (shell->display);
 
+  gimp_display_shell_profile_init (shell);
+
   if (image)
     {
       image_width  = gimp_image_get_width  (image);
@@ -464,34 +467,47 @@ gimp_display_shell_constructed (GObject *object)
    *
    *  shell
    *     |
-   *     +-- upper_hbox
-   *     |      |
-   *     |      +-- inner_table
-   *     |      |      |
-   *     |      |      +-- origin
-   *     |      |      +-- hruler
-   *     |      |      +-- vruler
-   *     |      |      +-- canvas
-   *     |      |
-   *     |      +-- right_vbox
-   *     |             |
-   *     |             +-- zoom_on_resize_button
-   *     |             +-- vscrollbar
-   *     |
-   *     +-- lower_hbox
-   *     |      |
-   *     |      +-- quick_mask
-   *     |      +-- hscrollbar
-   *     |      +-- navbutton
-   *     |
-   *     +-- statusbar
+   *     +-- main_vbox
+   *            |
+   *            +-- upper_hbox
+   *            |      |
+   *            |      +-- inner_table
+   *            |      |      |
+   *            |      |      +-- origin
+   *            |      |      +-- hruler
+   *            |      |      +-- vruler
+   *            |      |      +-- canvas
+   *            |      |
+   *            |      +-- right_vbox
+   *            |             |
+   *            |             +-- zoom_on_resize_button
+   *            |             +-- vscrollbar
+   *            |
+   *            +-- lower_hbox
+   *            |      |
+   *            |      +-- quick_mask
+   *            |      +-- hscrollbar
+   *            |      +-- navbutton
+   *            |
+   *            +-- statusbar
+   *
+   *  Note that we separate "shell" and "main_vbox", so that we can make
+   *  "shell" a GtkEventBox, giving it its own window.  This isolates our
+   *  events from those of our ancestors, avoiding some potential slowdowns,
+   *  and making things generally smoother.  See bug #778966.
    */
 
   /*  first, set up the container hierarchy  *********************************/
 
+  /*  the root vbox  */
+
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_container_add (GTK_CONTAINER (shell), main_vbox);
+  gtk_widget_show (main_vbox);
+
   /*  a hbox for the inner_table and the vertical scrollbar  */
   upper_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_box_pack_start (GTK_BOX (shell), upper_hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (main_vbox), upper_hbox, TRUE, TRUE, 0);
   gtk_widget_show (upper_hbox);
 
   /*  the table containing origin, rulers and the canvas  */
@@ -510,7 +526,7 @@ gimp_display_shell_constructed (GObject *object)
    *  the navigation button
    */
   lower_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 1);
-  gtk_box_pack_start (GTK_BOX (shell), lower_hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (main_vbox), lower_hbox, FALSE, FALSE, 0);
   gtk_widget_show (lower_hbox);
 
   /*  create the scrollbars  *************************************************/
@@ -532,7 +548,7 @@ gimp_display_shell_constructed (GObject *object)
   /*  the menu popup button  */
   shell->origin = gtk_event_box_new ();
 
-  gtk_image = gtk_image_new_from_icon_name (GIMP_STOCK_MENU_RIGHT,
+  gtk_image = gtk_image_new_from_icon_name (GIMP_ICON_MENU_RIGHT,
                                             GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (shell->origin), gtk_image);
   gtk_widget_show (gtk_image);
@@ -590,6 +606,10 @@ gimp_display_shell_constructed (GObject *object)
   gimp_devices_add_widget (shell->display->gimp, shell->hrule);
   gimp_devices_add_widget (shell->display->gimp, shell->vrule);
 
+  g_signal_connect (shell->canvas, "grab-notify",
+                    G_CALLBACK (gimp_display_shell_canvas_grab_notify),
+                    shell);
+
   g_signal_connect (shell->canvas, "realize",
                     G_CALLBACK (gimp_display_shell_canvas_realize),
                     shell);
@@ -647,7 +667,7 @@ gimp_display_shell_constructed (GObject *object)
                                      NULL);
   gtk_widget_set_can_focus (shell->zoom_button, FALSE);
 
-  gtk_image = gtk_image_new_from_icon_name (GIMP_STOCK_ZOOM_FOLLOW_WINDOW,
+  gtk_image = gtk_image_new_from_icon_name (GIMP_ICON_ZOOM_FOLLOW_WINDOW,
                                             GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (shell->zoom_button), gtk_image);
   gtk_widget_show (gtk_image);
@@ -671,7 +691,7 @@ gimp_display_shell_constructed (GObject *object)
                                            NULL);
   gtk_widget_set_can_focus (shell->quick_mask_button, FALSE);
 
-  gtk_image = gtk_image_new_from_icon_name (GIMP_STOCK_QUICK_MASK_OFF,
+  gtk_image = gtk_image_new_from_icon_name (GIMP_ICON_QUICK_MASK_OFF,
                                             GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (shell->quick_mask_button), gtk_image);
   gtk_widget_show (gtk_image);
@@ -695,7 +715,7 @@ gimp_display_shell_constructed (GObject *object)
   /*  the navigation window button  */
   shell->nav_ebox = gtk_event_box_new ();
 
-  gtk_image = gtk_image_new_from_icon_name (GIMP_STOCK_NAVIGATION,
+  gtk_image = gtk_image_new_from_icon_name (GIMP_ICON_DIALOG_NAVIGATION,
                                             GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (shell->nav_ebox), gtk_image);
   gtk_widget_show (gtk_image);
@@ -714,7 +734,7 @@ gimp_display_shell_constructed (GObject *object)
   gimp_statusbar_set_shell (GIMP_STATUSBAR (shell->statusbar), shell);
   gimp_help_set_help_data (shell->statusbar, NULL,
                            GIMP_HELP_IMAGE_WINDOW_STATUS_BAR);
-  gtk_box_pack_end (GTK_BOX (shell), shell->statusbar, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (main_vbox), shell->statusbar, FALSE, FALSE, 0);
 
   /*  pack all the widgets  **************************************************/
 
@@ -747,10 +767,9 @@ gimp_display_shell_constructed (GObject *object)
 
   gtk_widget_show (GTK_WIDGET (shell->canvas));
 
-  /*  add display filter for color management  */
+  /*  add display filters  */
 
-  filter = gimp_display_shell_filter_new (shell,
-                                          GIMP_CORE_CONFIG (config)->color_management);
+  filter = gimp_display_shell_filter_new (shell);
 
   if (filter)
     {
@@ -782,6 +801,9 @@ gimp_display_shell_constructed (GObject *object)
 
       gimp_statusbar_empty (GIMP_STATUSBAR (shell->statusbar));
     }
+
+  /* make sure the information is up-to-date */
+  gimp_display_shell_scale_update (shell);
 }
 
 static void
@@ -818,7 +840,7 @@ gimp_display_shell_dispose (GObject *object)
       shell->checkerboard = NULL;
     }
 
-  gimp_display_shell_profile_dispose (shell);
+  gimp_display_shell_profile_finalize (shell);
 
   if (shell->filter_buffer)
     {
@@ -866,12 +888,6 @@ gimp_display_shell_dispose (GObject *object)
       shell->nav_popup = NULL;
     }
 
-  if (shell->grid_dialog)
-    {
-      gtk_widget_destroy (shell->grid_dialog);
-      shell->grid_dialog = NULL;
-    }
-
   if (shell->blink_timeout_id)
     {
       g_source_remove (shell->blink_timeout_id);
@@ -1379,6 +1395,14 @@ gimp_display_shell_get_statusbar (GimpDisplayShell *shell)
   return GIMP_STATUSBAR (shell->statusbar);
 }
 
+GimpColorConfig *
+gimp_display_shell_get_color_config (GimpDisplayShell *shell)
+{
+  g_return_val_if_fail (GIMP_IS_DISPLAY_SHELL (shell), NULL);
+
+  return shell->color_config;
+}
+
 void
 gimp_display_shell_present (GimpDisplayShell *shell)
 {
@@ -1514,23 +1538,28 @@ gimp_display_shell_fill (GimpDisplayShell *shell,
                          GimpUnit          unit,
                          gdouble           scale)
 {
-  GimpImageWindow *window;
+  GimpDisplayConfig *config;
+  GimpImageWindow   *window;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (GIMP_IS_DISPLAY (shell->display));
   g_return_if_fail (GIMP_IS_IMAGE (image));
 
+  config = shell->display->config;
   window = gimp_display_shell_get_window (shell);
 
+  shell->dot_for_dot = config->default_dot_for_dot;
+
   gimp_display_shell_set_unit (shell, unit);
   gimp_display_shell_set_initial_scale (shell, scale, NULL, NULL);
+  gimp_display_shell_scale_update (shell);
 
   /* center the image so subsequent stuff only moves it a little in
    * the center
    */
   gimp_display_shell_scroll_center_image (shell, TRUE, TRUE);
 
-  gimp_display_shell_sync_config (shell, shell->display->config);
+  gimp_display_shell_sync_config (shell, config);
 
   gimp_image_window_suspend_keep_pos (window);
   gimp_display_shell_appearance_update (shell);
