@@ -17,9 +17,22 @@
 
 #include "config.h"
 
+#include <math.h>
+
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#ifdef G_OS_WIN32
+#include <windef.h>
+#include <winbase.h>
+#include <windows.h>
+#endif
+
+#ifdef GDK_WINDOWING_QUARTZ
+#import <AppKit/AppKit.h>
+#include <gdk/gdkquartz.h>
+#endif /* !GDK_WINDOWING_QUARTZ */
+
 #include "libgimpmath/gimpmath.h"
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpwidgets/gimpwidgets.h"
@@ -49,6 +62,7 @@
 #include "widgets/gimptoolbox.h"
 #include "widgets/gimpuimanager.h"
 #include "widgets/gimpview.h"
+#include "widgets/gimpviewrenderer.h"
 #include "widgets/gimpwidgets-utils.h"
 
 #include "gimpdisplay.h"
@@ -85,7 +99,6 @@ enum
 {
   PROP_0,
   PROP_GIMP,
-  PROP_MENU_FACTORY,
   PROP_DIALOG_FACTORY,
   PROP_INITIAL_SCREEN,
   PROP_INITIAL_MONITOR
@@ -120,6 +133,8 @@ struct _GimpImageWindowPrivate
   gint               initial_monitor;
 
   gint               suspend_keep_pos;
+
+  gint               update_ui_manager_idle_id;
 };
 
 typedef struct
@@ -157,6 +172,7 @@ static void      gimp_image_window_get_property        (GObject             *obj
                                                         GValue              *value,
                                                         GParamSpec          *pspec);
 
+static void      gimp_image_window_map                 (GtkWidget           *widget);
 static gboolean  gimp_image_window_delete_event        (GtkWidget           *widget,
                                                         GdkEventAny         *event);
 static gboolean  gimp_image_window_configure_event     (GtkWidget           *widget,
@@ -170,6 +186,9 @@ static void      gimp_image_window_monitor_changed     (GimpWindow          *win
                                                         gint                 monitor);
 
 static GList *   gimp_image_window_get_docks           (GimpDockContainer   *dock_container);
+static GimpDialogFactory *
+                 gimp_image_window_dock_container_get_dialog_factory
+                                                       (GimpDockContainer   *dock_container);
 static GimpUIManager *
                  gimp_image_window_dock_container_get_ui_manager
                                                        (GimpDockContainer   *dock_container);
@@ -203,6 +222,8 @@ static void      gimp_image_window_show_tooltip        (GimpUIManager       *man
                                                         GimpImageWindow     *window);
 static void      gimp_image_window_hide_tooltip        (GimpUIManager       *manager,
                                                         GimpImageWindow     *window);
+static gboolean  gimp_image_window_update_ui_manager_idle
+                                                       (GimpImageWindow     *window);
 static void      gimp_image_window_update_ui_manager   (GimpImageWindow     *window);
 
 static void      gimp_image_window_shell_size_allocate (GimpDisplayShell    *shell,
@@ -276,6 +297,7 @@ gimp_image_window_class_init (GimpImageWindowClass *klass)
   object_class->set_property       = gimp_image_window_set_property;
   object_class->get_property       = gimp_image_window_get_property;
 
+  widget_class->map                = gimp_image_window_map;
   widget_class->delete_event       = gimp_image_window_delete_event;
   widget_class->configure_event    = gimp_image_window_configure_event;
   widget_class->window_state_event = gimp_image_window_window_state_event;
@@ -289,12 +311,6 @@ gimp_image_window_class_init (GimpImageWindowClass *klass)
                                                         GIMP_TYPE_GIMP,
                                                         GIMP_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT_ONLY));
-  g_object_class_install_property (object_class, PROP_MENU_FACTORY,
-                                   g_param_spec_object ("menu-factory",
-                                                        NULL, NULL,
-                                                        GIMP_TYPE_MENU_FACTORY,
-                                                        GIMP_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY));
 
   g_object_class_install_property (object_class, PROP_DIALOG_FACTORY,
                                    g_param_spec_object ("dialog-factory",
@@ -337,10 +353,11 @@ gimp_image_window_init (GimpImageWindow *window)
 static void
 gimp_image_window_dock_container_iface_init (GimpDockContainerInterface *iface)
 {
-  iface->get_docks      = gimp_image_window_get_docks;
-  iface->get_ui_manager = gimp_image_window_dock_container_get_ui_manager;
-  iface->add_dock       = gimp_image_window_add_dock;
-  iface->get_dock_side  = gimp_image_window_get_dock_side;
+  iface->get_docks          = gimp_image_window_get_docks;
+  iface->get_dialog_factory = gimp_image_window_dock_container_get_dialog_factory;
+  iface->get_ui_manager     = gimp_image_window_dock_container_get_ui_manager;
+  iface->add_dock           = gimp_image_window_add_dock;
+  iface->get_dock_side      = gimp_image_window_get_dock_side;
 }
 
 static void
@@ -355,12 +372,20 @@ gimp_image_window_constructed (GObject *object)
 {
   GimpImageWindow        *window  = GIMP_IMAGE_WINDOW (object);
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+  GimpMenuFactory        *menu_factory;
   GimpGuiConfig          *config;
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
   g_assert (GIMP_IS_GIMP (private->gimp));
-  g_assert (GIMP_IS_UI_MANAGER (private->menubar_manager));
+  g_assert (GIMP_IS_DIALOG_FACTORY (private->dialog_factory));
+
+  menu_factory = gimp_dialog_factory_get_menu_factory (private->dialog_factory);
+
+  private->menubar_manager = gimp_menu_factory_manager_new (menu_factory,
+                                                            "<Image>",
+                                                            window,
+                                                            FALSE);
 
   g_signal_connect_object (private->dialog_factory, "dock-window-added",
                            G_CALLBACK (gimp_image_window_update_ui_manager),
@@ -508,6 +533,12 @@ gimp_image_window_dispose (GObject *object)
       private->menubar_manager = NULL;
     }
 
+  if (private->update_ui_manager_idle_id)
+    {
+      g_source_remove (private->update_ui_manager_idle_id);
+      private->update_ui_manager_idle_id = 0;
+    }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -539,16 +570,6 @@ gimp_image_window_set_property (GObject      *object,
     case PROP_GIMP:
       private->gimp = g_value_get_object (value);
       break;
-    case PROP_MENU_FACTORY:
-      {
-        GimpMenuFactory *factory = g_value_get_object (value);
-
-        private->menubar_manager = gimp_menu_factory_manager_new (factory,
-                                                                  "<Image>",
-                                                                  window,
-                                                                  FALSE);
-      }
-      break;
     case PROP_DIALOG_FACTORY:
       private->dialog_factory = g_value_get_object (value);
       break;
@@ -589,13 +610,40 @@ gimp_image_window_get_property (GObject    *object,
       g_value_set_int (value, private->initial_monitor);
       break;
 
-    case PROP_MENU_FACTORY:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
 }
 
+static void
+gimp_image_window_map (GtkWidget *widget)
+{
+#ifdef GDK_WINDOWING_QUARTZ
+  GdkWindow *gdk_window;
+  NSWindow  *ns_window;
+#endif /* !GDK_WINDOWING_QUARTZ */
+
+  GTK_WIDGET_CLASS (parent_class)->map (widget);
+
+#ifdef GDK_WINDOWING_QUARTZ
+  gdk_window = gtk_widget_get_window (GTK_WIDGET (widget));
+  ns_window = gdk_quartz_window_get_nswindow (gdk_window);
+
+  /* Disable the new-style full screen mode. For now only the "old-style"
+   * full screen mode, via the "View" menu, is supported. In the future, and
+   * as soon as GTK+ has proper support for this, we will migrate to the
+   * new-style full screen mode.
+   */
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_7
+  ns_window.collectionBehavior |= NSWindowCollectionBehaviorFullScreenAuxiliary;
+#else
+  /* Hard code the define ... */
+  ns_window.collectionBehavior |= 1 << 8;
+#endif
+#endif /* !GDK_WINDOWING_QUARTZ */
+}
+
 static gboolean
 gimp_image_window_delete_event (GtkWidget   *widget,
                                 GdkEventAny *event)
@@ -815,14 +863,18 @@ gimp_image_window_get_docks (GimpDockContainer *dock_container)
   return all_docks;
 }
 
+static GimpDialogFactory *
+gimp_image_window_dock_container_get_dialog_factory (GimpDockContainer *dock_container)
+{
+  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (dock_container);
+
+  return private->dialog_factory;
+}
+
 static GimpUIManager *
 gimp_image_window_dock_container_get_ui_manager (GimpDockContainer *dock_container)
 {
-  GimpImageWindow *window;
-
-  g_return_val_if_fail (GIMP_IS_IMAGE_WINDOW (dock_container), FALSE);
-
-  window = GIMP_IMAGE_WINDOW (dock_container);
+  GimpImageWindow *window = GIMP_IMAGE_WINDOW (dock_container);
 
   return gimp_image_window_get_ui_manager (window);
 }
@@ -965,6 +1017,11 @@ gimp_image_window_set_aux_info (GimpSessionManaged *session_managed,
   gint                    right_docks_width     = G_MININT;
   gboolean                wait_with_right_docks = FALSE;
   gboolean                maximized             = FALSE;
+#ifdef G_OS_WIN32
+  STARTUPINFO             StartupInfo;
+
+  GetStartupInfo (&StartupInfo);
+#endif
 
   g_return_if_fail (GIMP_IS_IMAGE_WINDOW (session_managed));
 
@@ -1036,10 +1093,32 @@ gimp_image_window_set_aux_info (GimpSessionManaged *session_managed,
         }
     }
 
-  if (maximized)
+#ifdef G_OS_WIN32
+  /* On Windows, user can provide startup hints to have a program
+   * maximized/minimized on startup. This can be done through command
+   * line: `start /max gimp-2.9.exe` or with the shortcut's "run"
+   * property.
+   * When such a hint is given, we should follow it and bypass the
+   * session's information.
+   */
+  if (StartupInfo.wShowWindow == SW_SHOWMAXIMIZED)
     gtk_window_maximize (GTK_WINDOW (session_managed));
+  else if (StartupInfo.wShowWindow == SW_SHOWMINIMIZED   ||
+           StartupInfo.wShowWindow == SW_SHOWMINNOACTIVE ||
+           StartupInfo.wShowWindow == SW_MINIMIZE)
+    /* XXX Iconification does not seem to work. I see the
+     * window being iconified and immediately re-raised.
+     * I leave this piece of code for later improvement. */
+    gtk_window_iconify (GTK_WINDOW (session_managed));
   else
-    gtk_window_unmaximize (GTK_WINDOW (session_managed));
+    /* Another show property not relevant to min/max.
+     * Defaults is: SW_SHOWNORMAL
+     */
+#endif
+    if (maximized)
+      gtk_window_maximize (GTK_WINDOW (session_managed));
+    else
+      gtk_window_unmaximize (GTK_WINDOW (session_managed));
 }
 
 
@@ -1048,7 +1127,6 @@ gimp_image_window_set_aux_info (GimpSessionManaged *session_managed,
 GimpImageWindow *
 gimp_image_window_new (Gimp              *gimp,
                        GimpImage         *image,
-                       GimpMenuFactory   *menu_factory,
                        GimpDialogFactory *dialog_factory,
                        GdkScreen         *screen,
                        gint               monitor)
@@ -1058,13 +1136,11 @@ gimp_image_window_new (Gimp              *gimp,
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (image == NULL || GIMP_IS_IMAGE (image), NULL);
-  g_return_val_if_fail (GIMP_IS_MENU_FACTORY (menu_factory), NULL);
   g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (dialog_factory), NULL);
   g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
 
   window = g_object_new (GIMP_TYPE_IMAGE_WINDOW,
                          "gimp",            gimp,
-                         "menu-factory",    menu_factory,
                          "dialog-factory",  dialog_factory,
                          "initial-screen",  screen,
                          "initial-monitor", monitor,
@@ -1397,7 +1473,8 @@ gimp_image_window_shrink_wrap (GimpImageWindow *window,
   GdkRectangle      rect;
   gint              monitor;
   gint              disp_width, disp_height;
-  gint              width, height;
+  gdouble           x, y;
+  gdouble           width, height;
   gint              max_auto_width, max_auto_height;
   gint              border_width, border_height;
   gboolean          resize = FALSE;
@@ -1425,8 +1502,15 @@ gimp_image_window_shrink_wrap (GimpImageWindow *window,
                                               gtk_widget_get_window (widget));
   gdk_screen_get_monitor_workarea (screen, monitor, &rect);
 
-  width  = SCALEX (active_shell, gimp_image_get_width  (image));
-  height = SCALEY (active_shell, gimp_image_get_height (image));
+  gimp_display_shell_transform_bounds (active_shell,
+                                       0, 0,
+                                       gimp_image_get_width (image),
+                                       gimp_image_get_height (image),
+                                       &x, &y,
+                                       &width, &height);
+
+  width  = ceil (width  - x);
+  height = ceil (height - y);
 
   disp_width  = active_shell->disp_width;
   disp_height = active_shell->disp_height;
@@ -1817,13 +1901,32 @@ gimp_image_window_hide_tooltip (GimpUIManager   *manager,
   gimp_statusbar_pop (statusbar, "menu-tooltip");
 }
 
+static gboolean
+gimp_image_window_update_ui_manager_idle (GimpImageWindow *window)
+{
+  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+
+  g_assert (private->active_shell != NULL);
+
+  gimp_ui_manager_update (private->menubar_manager,
+                          private->active_shell->display);
+
+  private->update_ui_manager_idle_id = 0;
+
+  return G_SOURCE_REMOVE;
+}
+
 static void
 gimp_image_window_update_ui_manager (GimpImageWindow *window)
 {
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
 
-  gimp_ui_manager_update (private->menubar_manager,
-                          private->active_shell->display);
+  if (! private->update_ui_manager_idle_id)
+    {
+      private->update_ui_manager_idle_id =
+        g_idle_add ((GSourceFunc) gimp_image_window_update_ui_manager_idle,
+                    window);
+    }
 }
 
 static void
@@ -1939,7 +2042,7 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
   gimp_context_set_display (gimp_get_user_context (private->gimp),
                             active_display);
 
-  gimp_ui_manager_update (private->menubar_manager, active_display);
+  gimp_image_window_update_ui_manager (window);
 
   gimp_image_window_update_tab_labels (window);
 }
@@ -2018,6 +2121,12 @@ gimp_image_window_disconnect_from_active_shell (GimpImageWindow *window)
 
   if (private->menubar_manager)
     gimp_image_window_hide_tooltip (private->menubar_manager, window);
+
+  if (private->update_ui_manager_idle_id)
+    {
+      g_source_remove (private->update_ui_manager_idle_id);
+      private->update_ui_manager_idle_id = 0;
+    }
 }
 
 static void
@@ -2045,7 +2154,7 @@ gimp_image_window_image_notify (GimpDisplay      *display,
   gimp_view_set_viewable (GIMP_VIEW (view),
                           GIMP_VIEWABLE (gimp_display_get_image (display)));
 
-  gimp_ui_manager_update (private->menubar_manager, display);
+  gimp_image_window_update_ui_manager (window);
 }
 
 static void
@@ -2202,22 +2311,16 @@ static void
 gimp_image_window_shell_scaled (GimpDisplayShell *shell,
                                 GimpImageWindow  *window)
 {
-  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
-
   /* update the <Image>/View/Zoom menu */
-  gimp_ui_manager_update (private->menubar_manager,
-                          shell->display);
+  gimp_image_window_update_ui_manager (window);
 }
 
 static void
 gimp_image_window_shell_rotated (GimpDisplayShell *shell,
                                  GimpImageWindow  *window)
 {
-  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
-
   /* update the <Image>/View/Rotate menu */
-  gimp_ui_manager_update (private->menubar_manager,
-                          shell->display);
+  gimp_image_window_update_ui_manager (window);
 }
 
 static void
@@ -2260,6 +2363,8 @@ gimp_image_window_create_tab_label (GimpImageWindow  *window,
                                  GIMP_TYPE_VIEW, GIMP_TYPE_IMAGE,
                                  GIMP_VIEW_SIZE_LARGE, 0, FALSE);
   gtk_widget_set_size_request (view, GIMP_VIEW_SIZE_LARGE, -1);
+  gimp_view_renderer_set_color_config (GIMP_VIEW (view)->renderer,
+                                       gimp_display_shell_get_color_config (shell));
   gtk_box_pack_start (GTK_BOX (hbox), view, FALSE, FALSE, 0);
   gtk_widget_show (view);
 
@@ -2273,7 +2378,7 @@ gimp_image_window_create_tab_label (GimpImageWindow  *window,
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
-  gtk_image = gtk_image_new_from_icon_name (GIMP_STOCK_CLOSE,
+  gtk_image = gtk_image_new_from_icon_name (GIMP_ICON_CLOSE,
                                             GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (button), gtk_image);
   gtk_widget_show (gtk_image);
