@@ -24,19 +24,15 @@
 #include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
-#include "libgimpconfig/gimpconfig.h"
 
 #include "core/core-types.h"
 
-#include "config/gimpcoreconfig.h"
-
 #include "gegl/gimp-babl.h"
 
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpdocumentlist.h"
 #include "core/gimpimage.h"
-#include "core/gimpimage-color-profile.h"
 #include "core/gimpimage-merge.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimagefile.h"
@@ -49,6 +45,7 @@
 #include "plug-in/gimppluginmanager-file.h"
 #include "plug-in/gimppluginprocedure.h"
 
+#include "file-import.h"
 #include "file-open.h"
 #include "file-remote.h"
 #include "gimp-file.h"
@@ -61,7 +58,7 @@ static void     file_open_sanitize_image       (GimpImage           *image,
 static void     file_open_convert_items        (GimpImage           *dest_image,
                                                 const gchar         *basename,
                                                 GList               *items);
-static GList *  file_open_get_layers           (const GimpImage     *image,
+static GList *  file_open_get_layers           (GimpImage           *image,
                                                 gboolean             merge_visible,
                                                 gint                *n_visible);
 static gboolean file_open_file_proc_is_import  (GimpPlugInProcedure *file_proc);
@@ -139,7 +136,15 @@ file_open_image (Gimp                *gimp,
                                                           file, error);
 
   if (! file_proc)
-    return NULL;
+    {
+      /*  don't bail out on remote files, they might need to be
+       *  downloaded for magic matching
+       */
+      if (g_file_is_native (file))
+        return NULL;
+
+      g_clear_error (error);
+    }
 
   if (! g_file_is_native (file) &&
       ! file_remote_mount_file (gimp, file, progress, &my_error))
@@ -152,7 +157,7 @@ file_open_image (Gimp                *gimp,
       return NULL;
     }
 
-  if (! file_proc->handles_uri)
+  if (! file_proc || ! file_proc->handles_uri)
     {
       path = g_file_get_path (file);
 
@@ -171,6 +176,22 @@ file_open_image (Gimp                *gimp,
               return NULL;
             }
 
+          /*  if we don't have a file proc yet, try again on the local
+           *  file
+           */
+          if (! file_proc)
+            file_proc = gimp_plug_in_manager_file_procedure_find (gimp->plug_in_manager,
+                                                                  GIMP_FILE_PROCEDURE_GROUP_OPEN,
+                                                                  local_file, error);
+
+          if (! file_proc)
+            {
+              g_file_delete (local_file, NULL, NULL);
+              g_object_unref (local_file);
+
+              return NULL;
+            }
+
           path = g_file_get_path (local_file);
         }
     }
@@ -183,6 +204,9 @@ file_open_image (Gimp                *gimp,
   if (! entered_uri)
     entered_uri = g_strdup (path);
 
+  if (progress)
+    g_object_add_weak_pointer (G_OBJECT (progress), (gpointer) &progress);
+
   return_vals =
     gimp_pdb_execute_procedure_by_name (gimp->pdb,
                                         context, progress, error,
@@ -192,6 +216,9 @@ file_open_image (Gimp                *gimp,
                                         G_TYPE_STRING,   entered_uri,
                                         G_TYPE_NONE);
 
+  if (progress)
+    g_object_remove_weak_pointer (G_OBJECT (progress), (gpointer) &progress);
+
   g_free (path);
   g_free (entered_uri);
 
@@ -248,17 +275,11 @@ file_open_image (Gimp                *gimp,
     {
       gimp_image_undo_disable (image);
 
-      gimp_image_import_color_profile (image, context, progress,
-                                       run_mode == GIMP_RUN_INTERACTIVE ?
-                                       TRUE : FALSE);
-
       if (file_open_file_proc_is_import (file_proc))
         {
-          /* Remember the import source */
-          gimp_image_set_imported_file (image, file);
-
-          /* We shall treat this file as an Untitled file */
-          gimp_image_set_file (image, NULL);
+          file_import_image (image, context, file,
+                             run_mode == GIMP_RUN_INTERACTIVE,
+                             progress);
         }
 
       /* Enables undo again */
@@ -634,7 +655,7 @@ file_open_layers (Gimp                *gimp,
       else
         {
           g_set_error_literal (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
-			       _("Image doesn't contain any layers"));
+                               _("Image doesn't contain any layers"));
           *status = GIMP_PDB_EXECUTION_ERROR;
         }
 
@@ -727,6 +748,14 @@ file_open_sanitize_image (GimpImage *image,
    */
   gimp_image_clean_all (image);
 
+  /* Make sure the projection is completely constructed from valid
+   * layers, this is needed in case something triggers projection or
+   * image preview creation before all layers are loaded, see bug #767663.
+   */
+  gimp_image_invalidate (image, 0, 0,
+                         gimp_image_get_width  (image),
+                         gimp_image_get_height (image));
+
   /* Make sure all image states are up-to-date */
   gimp_image_flush (image);
 }
@@ -761,9 +790,9 @@ file_open_convert_items (GimpImage   *dest_image,
 }
 
 static GList *
-file_open_get_layers (const GimpImage *image,
-                      gboolean         merge_visible,
-                      gint            *n_visible)
+file_open_get_layers (GimpImage *image,
+                      gboolean   merge_visible,
+                      gint      *n_visible)
 {
   GList *iter   = NULL;
   GList *layers = NULL;
@@ -795,5 +824,5 @@ file_open_file_proc_is_import (GimpPlugInProcedure *file_proc)
 {
   return !(file_proc &&
            file_proc->mime_type &&
-           strcmp (file_proc->mime_type, "image/xcf") == 0);
+           strcmp (file_proc->mime_type, "image/x-xcf") == 0);
 }
