@@ -68,13 +68,15 @@
 #include "sanity.h"
 #include "signals.h"
 #include "unique.h"
-#include "units.h"
 #include "version.h"
 
 #ifdef G_OS_WIN32
 /* To get PROCESS_DEP_* defined we need _WIN32_WINNT at 0x0601. We still
  * use the API optionally only if present, though.
  */
+#ifdef _WIN32_WINNT
+#undef _WIN32_WINNT
+#endif
 #define _WIN32_WINNT 0x0601
 #include <windows.h>
 #include <conio.h>
@@ -321,8 +323,9 @@ main (int    argc,
     typedef BOOL (WINAPI *t_SetDllDirectoryA) (LPCSTR lpPathName);
     t_SetDllDirectoryA p_SetDllDirectoryA;
 
-    p_SetDllDirectoryA = GetProcAddress (GetModuleHandle ("kernel32.dll"),
-					 "SetDllDirectoryA");
+    p_SetDllDirectoryA =
+      (t_SetDllDirectoryA) GetProcAddress (GetModuleHandle ("kernel32.dll"),
+                                           "SetDllDirectoryA");
     if (p_SetDllDirectoryA)
       (*p_SetDllDirectoryA) ("");
   }
@@ -396,8 +399,9 @@ main (int    argc,
     typedef BOOL (WINAPI *t_SetProcessDEPPolicy) (DWORD dwFlags);
     t_SetProcessDEPPolicy p_SetProcessDEPPolicy;
 
-    p_SetProcessDEPPolicy = GetProcAddress (GetModuleHandle ("kernel32.dll"),
-					    "SetProcessDEPPolicy");
+    p_SetProcessDEPPolicy =
+      (t_SetProcessDEPPolicy) GetProcAddress (GetModuleHandle ("kernel32.dll"),
+                                              "SetProcessDEPPolicy");
     if (p_SetProcessDEPPolicy)
       (*p_SetProcessDEPPolicy) (PROCESS_DEP_ENABLE|PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION);
   }
@@ -408,8 +412,9 @@ main (int    argc,
     typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID) (PCWSTR lpPathName);
     t_SetCurrentProcessExplicitAppUserModelID p_SetCurrentProcessExplicitAppUserModelID;
 
-    p_SetCurrentProcessExplicitAppUserModelID = GetProcAddress (GetModuleHandle ("shell32.dll"),
-                                                                "SetCurrentProcessExplicitAppUserModelID");
+    p_SetCurrentProcessExplicitAppUserModelID =
+      (t_SetCurrentProcessExplicitAppUserModelID) GetProcAddress (GetModuleHandle ("shell32.dll"),
+                                                                  "SetCurrentProcessExplicitAppUserModelID");
     if (p_SetCurrentProcessExplicitAppUserModelID)
       (*p_SetCurrentProcessExplicitAppUserModelID) (L"gimp.GimpApplication");
   }
@@ -519,8 +524,8 @@ main (int    argc,
   if (! new_instance && gimp_unique_open (filenames, as_new))
     {
       if (be_verbose)
-	g_print ("%s\n",
-		 _("Another GIMP instance is already running."));
+        g_print ("%s\n",
+                 _("Another GIMP instance is already running."));
 
       if (batch_commands)
         gimp_unique_batch_run (batch_interpreter, batch_commands);
@@ -710,12 +715,11 @@ gimp_option_dump_gimprc (const gchar  *option_name,
       Gimp     *gimp;
       gboolean  success;
 
-      gimp = g_object_new (GIMP_TYPE_GIMP, NULL);
-
-      units_init (gimp);
       babl_init ();
+      gimp = g_object_new (GIMP_TYPE_GIMP, NULL);
+      gimp_load_config (gimp, NULL, NULL);
 
-      success = gimp_config_dump (format);
+      success = gimp_config_dump (G_OBJECT (gimp), format);
 
       g_object_unref (gimp);
 
@@ -735,7 +739,9 @@ gimp_option_dump_pdb_procedures_deprecated (const gchar  *option_name,
   GList *deprecated_procs;
   GList *iter;
 
+  babl_init ();
   gimp = g_object_new (GIMP_TYPE_GIMP, NULL);
+  gimp_load_config (gimp, NULL, NULL);
 
   /* Make sure to turn on compatibility mode so deprecated procedures
    * are included
