@@ -22,6 +22,7 @@
 #include <pango/pango.h>
 #include <pango/pangoft2.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <lcms2.h>
 #include <gexiv2/gexiv2.h>
 #include <gegl.h>
 
@@ -39,6 +40,7 @@ static gchar * sanity_check_pango             (void);
 static gchar * sanity_check_fontconfig        (void);
 static gchar * sanity_check_freetype          (void);
 static gchar * sanity_check_gdk_pixbuf        (void);
+static gchar * sanity_check_lcms              (void);
 static gchar * sanity_check_gexiv2            (void);
 static gchar * sanity_check_babl              (void);
 static gchar * sanity_check_gegl              (void);
@@ -71,6 +73,9 @@ sanity_check (void)
   if (! abort_message)
     abort_message = sanity_check_gdk_pixbuf ();
 
+  if (! abort_message)
+    abort_message = sanity_check_lcms ();
+
   if (! abort_message)
     abort_message = sanity_check_gexiv2 ();
 
@@ -351,14 +356,62 @@ sanity_check_gdk_pixbuf (void)
   return NULL;
 }
 
+static gchar *
+sanity_check_lcms (void)
+{
+#define LCMS_REQUIRED_MAJOR 2
+#define LCMS_REQUIRED_MINOR 7
+
+  gint lcms_version = cmsGetEncodedCMMversion ();
+
+  if (LCMS_VERSION > lcms_version)
+    {
+      return g_strdup_printf
+        ("Liblcms2 version mismatch!\n\n"
+         "GIMP was compiled against LittleCMS version %d.%d, but the\n"
+         "LittleCMS version found at runtime is only %d.%d.\n\n"
+         "Somehow you or your software packager managed\n"
+         "to install a LittleCMS that is older than what GIMP was\n"
+         "built against.\n\n"
+         "Please make sure that the installed LittleCMS version\n"
+         "is at least %d.%d and that headers and library match.",
+         LCMS_VERSION / 1000, LCMS_VERSION % 100 / 10,
+         lcms_version / 1000, lcms_version % 100 / 10,
+         LCMS_VERSION / 1000, LCMS_VERSION % 100 / 10);
+    }
+
+  if (lcms_version < (LCMS_REQUIRED_MAJOR * 1000 +
+                      LCMS_REQUIRED_MINOR * 10))
+    {
+      const gint lcms_major_version = lcms_version / 1000;
+      const gint lcms_minor_version = lcms_version % 100 / 10;
+
+      return g_strdup_printf
+        ("Liblcms2 version too old!\n\n"
+         "GIMP requires LittleCMS version %d.%d or later.\n"
+         "Installed LittleCMS version is %d.%d.\n\n"
+         "Somehow you or your software packager managed\n"
+         "to install GIMP with an older LittleCMS version.\n\n"
+         "Please upgrade to LittleCMS version %d.%d or later.",
+         LCMS_REQUIRED_MAJOR, LCMS_REQUIRED_MINOR,
+         lcms_major_version, lcms_minor_version,
+         LCMS_REQUIRED_MAJOR, LCMS_REQUIRED_MINOR);
+    }
+
+#undef LCMS_REQUIRED_MAJOR
+#undef LCMS_REQUIRED_MINOR
+
+  return NULL;
+}
+
 static gchar *
 sanity_check_gexiv2 (void)
 {
 #ifdef GEXIV2_MAJOR_VERSION
 
 #define GEXIV2_REQUIRED_MAJOR 0
-#define GEXIV2_REQUIRED_MINOR 7
-#define GEXIV2_REQUIRED_MICRO 0
+#define GEXIV2_REQUIRED_MINOR 10
+#define GEXIV2_REQUIRED_MICRO 3
 
   gint gexiv2_version = gexiv2_get_version ();
 
@@ -400,7 +453,7 @@ sanity_check_babl (void)
 
 #define BABL_REQUIRED_MAJOR 0
 #define BABL_REQUIRED_MINOR 1
-#define BABL_REQUIRED_MICRO 14
+#define BABL_REQUIRED_MICRO 24
 
   babl_get_version (&babl_major_version,
                     &babl_minor_version,
@@ -438,7 +491,7 @@ sanity_check_gegl (void)
 
 #define GEGL_REQUIRED_MAJOR 0
 #define GEGL_REQUIRED_MINOR 3
-#define GEGL_REQUIRED_MICRO 6
+#define GEGL_REQUIRED_MICRO 14
 
   gegl_get_version (&gegl_major_version,
                     &gegl_minor_version,
@@ -483,10 +536,10 @@ sanity_check_gegl_ops (void)
     "gegl:color",
     "gegl:color-enhance",
     "gegl:color-exchange",
-    "gegl:color-reduction",
     "gegl:color-rotate",
     "gegl:color-temperature",
     "gegl:color-to-alpha",
+    "gegl:component-extract",
     "gegl:convolution-matrix",
     "gegl:copy-buffer",
     "gegl:crop",
@@ -496,24 +549,30 @@ sanity_check_gegl_ops (void)
     "gegl:diffraction-patterns",
     "gegl:displace",
     "gegl:distance-transform",
+    "gegl:dither",
     "gegl:dropshadow",
     "gegl:edge",
     "gegl:edge-laplace",
+    "gegl:edge-neon",
     "gegl:edge-sobel",
     "gegl:emboss",
     "gegl:engrave",
     "gegl:exposure",
+    "gegl:fattal02",
     "gegl:fractal-trace",
     "gegl:gaussian-blur",
     "gegl:gaussian-blur-selective",
+    "gegl:gegl",
     "gegl:grid",
     "gegl:high-pass",
     "gegl:illusion",
+    "gegl:image-gradient",
     "gegl:introspect",
     "gegl:invert-gamma",
     "gegl:invert-linear",
     "gegl:lens-distortion",
     "gegl:lens-flare",
+    "gegl:mantiuk06",
     "gegl:map-absolute",
     "gegl:map-relative",
     "gegl:matting-global",
@@ -542,6 +601,8 @@ sanity_check_gegl_ops (void)
     "gegl:pixelize",
     "gegl:polar-coordinates",
     "gegl:red-eye-removal",
+    "gegl:reinhard05",
+    "gegl:rgb-clip",
     "gegl:ripple",
     "gegl:saturation",
     "gegl:scale-ratio",
@@ -551,9 +612,12 @@ sanity_check_gegl_ops (void)
     "gegl:simplex-noise",
     "gegl:shift",
     "gegl:sinus",
+    "gegl:slic",
+    "gegl:snn-mean",
     "gegl:softglow",
     "gegl:stretch-contrast",
     "gegl:stretch-contrast-hsv",
+    "gegl:stress",
     "gegl:supernova",
     "gegl:threshold",
     "gegl:tile",
@@ -568,6 +632,8 @@ sanity_check_gegl_ops (void)
     "gegl:video-degradation",
     "gegl:vignette",
     "gegl:warp",
+    "gegl:waterpixels",
+    "gegl:wavelet-blur",
     "gegl:waves",
     "gegl:whirl-pinch",
     "gegl:write-buffer"
