@@ -41,6 +41,7 @@
 #include "core/gimpcontext.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-color-profile.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
 #include "core/gimpitemtree.h"
@@ -87,11 +88,9 @@ static gboolean   gimp_text_layer_rename         (GimpItem          *item,
 static void       gimp_text_layer_convert_type   (GimpDrawable      *drawable,
                                                   GimpImage         *dest_image,
                                                   const Babl        *new_format,
-                                                  GimpImageBaseType  new_base_type,
-                                                  GimpPrecision      new_precision,
-                                                  gint               layer_dither_type,
-                                                  gint               mask_dither_type,
-                                                  gboolean           convert_profile,
+                                                  GimpColorProfile  *dest_profile,
+                                                  GeglDitherMethod   layer_dither_type,
+                                                  GeglDitherMethod   mask_dither_type,
                                                   gboolean           push_undo,
                                                   GimpProgress      *progress);
 static void       gimp_text_layer_set_buffer     (GimpDrawable      *drawable,
@@ -319,26 +318,24 @@ static void
 gimp_text_layer_convert_type (GimpDrawable      *drawable,
                               GimpImage         *dest_image,
                               const Babl        *new_format,
-                              GimpImageBaseType  new_base_type,
-                              GimpPrecision      new_precision,
-                              gint               layer_dither_type,
-                              gint               mask_dither_type,
-                              gboolean           convert_profile,
+                              GimpColorProfile  *dest_profile,
+                              GeglDitherMethod   layer_dither_type,
+                              GeglDitherMethod   mask_dither_type,
                               gboolean           push_undo,
                               GimpProgress      *progress)
 {
   GimpTextLayer *layer = GIMP_TEXT_LAYER (drawable);
   GimpImage     *image = gimp_item_get_image (GIMP_ITEM (layer));
 
-  if (! layer->text || layer->modified || layer_dither_type != 0)
+  if (! layer->text   ||
+      layer->modified ||
+      layer_dither_type != GEGL_DITHER_NONE)
     {
       GIMP_DRAWABLE_CLASS (parent_class)->convert_type (drawable, dest_image,
                                                         new_format,
-                                                        new_base_type,
-                                                        new_precision,
+                                                        dest_profile,
                                                         layer_dither_type,
                                                         mask_dither_type,
-                                                        convert_profile,
                                                         push_undo,
                                                         progress);
     }
@@ -673,7 +670,9 @@ gimp_text_layer_render (GimpTextLayer *layer)
           if (! unused_eek)
             unused_eek = gimp_context_new (image->gimp, "eek", NULL);
 
-          gimp_item_resize (GIMP_ITEM (mask), unused_eek, width, height, 0, 0);
+          gimp_item_resize (GIMP_ITEM (mask),
+                            unused_eek, GIMP_FILL_TRANSPARENT,
+                            width, height, 0, 0);
         }
     }
 
@@ -693,8 +692,11 @@ gimp_text_layer_render (GimpTextLayer *layer)
           g_free (tmp);
         }
 
-      if (! name)
-        name = g_strdup (_("Empty Text Layer"));
+      if (! name || ! name[0])
+        {
+          g_free (name);
+          name = g_strdup (_("Empty Text Layer"));
+        }
 
       if (gimp_item_is_attached (item))
         {
@@ -722,14 +724,16 @@ static void
 gimp_text_layer_render_layout (GimpTextLayer  *layer,
                                GimpTextLayout *layout)
 {
-  GimpDrawable    *drawable = GIMP_DRAWABLE (layer);
-  GimpItem        *item     = GIMP_ITEM (layer);
-  GeglBuffer      *buffer;
-  cairo_t         *cr;
-  cairo_surface_t *surface;
-  gint             width;
-  gint             height;
-  cairo_status_t   status;
+  GimpDrawable       *drawable = GIMP_DRAWABLE (layer);
+  GimpItem           *item     = GIMP_ITEM (layer);
+  GimpImage          *image    = gimp_item_get_image (item);
+  GeglBuffer         *buffer;
+  GimpColorTransform *transform;
+  cairo_t            *cr;
+  cairo_surface_t    *surface;
+  gint                width;
+  gint                height;
+  cairo_status_t      status;
 
   g_return_if_fail (gimp_drawable_has_alpha (drawable));
 
@@ -758,8 +762,21 @@ gimp_text_layer_render_layout (GimpTextLayer  *layer,
 
   buffer = gimp_cairo_surface_create_buffer (surface);
 
-  gegl_buffer_copy (buffer, NULL, GEGL_ABYSS_NONE,
-                    gimp_drawable_get_buffer (drawable), NULL);
+  transform = gimp_image_get_color_transform_from_srgb_u8 (image);
+
+  if (transform)
+    {
+      gimp_color_transform_process_buffer (transform,
+                                           buffer,
+                                           NULL,
+                                           gimp_drawable_get_buffer (drawable),
+                                           NULL);
+    }
+  else
+    {
+      gegl_buffer_copy (buffer, NULL, GEGL_ABYSS_NONE,
+                        gimp_drawable_get_buffer (drawable), NULL);
+    }
 
   g_object_unref (buffer);
   cairo_surface_destroy (surface);
