@@ -31,15 +31,15 @@
 
 #include "tools-types.h"
 
-#include "gegl/gimp-gegl-config.h"
+#include "operations/gimp-operation-config.h"
 
 #include "core/gimp-utils.h"
 #include "core/gimpdrawable.h"
 #include "core/gimpdrawable-blend.h"
+#include "core/gimpdrawablefilter.h"
 #include "core/gimperror.h"
 #include "core/gimpgradient.h"
 #include "core/gimpimage.h"
-#include "core/gimpimagemap.h"
 #include "core/gimpprogress.h"
 #include "core/gimpprojection.h"
 
@@ -143,9 +143,9 @@ static void   gimp_blend_tool_set_gradient        (GimpBlendTool         *blend_
 
 static gboolean gimp_blend_tool_is_shapeburst     (GimpBlendTool         *blend_tool);
 
-static void   gimp_blend_tool_create_image_map    (GimpBlendTool         *blend_tool,
+static void   gimp_blend_tool_create_filter       (GimpBlendTool         *blend_tool,
                                                    GimpDrawable          *drawable);
-static void   gimp_blend_tool_image_map_flush     (GimpImageMap          *image_map,
+static void   gimp_blend_tool_filter_flush        (GimpDrawableFilter    *filter,
                                                    GimpTool              *tool);
 
 
@@ -171,7 +171,7 @@ gimp_blend_tool_register (GimpToolRegisterCallback  callback,
                 _("Blend Tool: Fill selected area with a color gradient"),
                 N_("Blen_d"), "L",
                 NULL, GIMP_HELP_TOOL_BLEND,
-                GIMP_STOCK_TOOL_BLEND,
+                GIMP_ICON_TOOL_BLEND,
                 data);
 }
 
@@ -204,6 +204,12 @@ gimp_blend_tool_init (GimpBlendTool *blend_tool)
   GimpTool *tool = GIMP_TOOL (blend_tool);
 
   gimp_tool_control_set_scroll_lock     (tool->control, TRUE);
+  gimp_tool_control_set_preserve        (tool->control, FALSE);
+  gimp_tool_control_set_dirty_mask      (tool->control,
+                                         GIMP_DIRTY_IMAGE           |
+                                         GIMP_DIRTY_IMAGE_STRUCTURE |
+                                         GIMP_DIRTY_DRAWABLE        |
+                                         GIMP_DIRTY_ACTIVE_DRAWABLE);
   gimp_tool_control_set_wants_click     (tool->control, TRUE);
   gimp_tool_control_set_precision       (tool->control,
                                          GIMP_CURSOR_PRECISION_SUBPIXEL);
@@ -242,14 +248,14 @@ gimp_blend_tool_initialize (GimpTool     *tool,
   if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
-			   _("Cannot modify the pixels of layer groups."));
+                           _("Cannot modify the pixels of layer groups."));
       return FALSE;
     }
 
   if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
-			   _("The active layer's pixels are locked."));
+                           _("The active layer's pixels are locked."));
       return FALSE;
     }
 
@@ -359,7 +365,7 @@ gimp_blend_tool_button_press (GimpTool            *tool,
       blend_tool->grabbed_point != POINT_INIT_MODE)
     {
       gimp_blend_tool_update_graph (blend_tool);
-      gimp_image_map_apply (blend_tool->image_map, NULL);
+      gimp_drawable_filter_apply (blend_tool->filter, NULL);
     }
 
   gimp_tool_control_activate (tool->control);
@@ -462,7 +468,7 @@ gimp_blend_tool_motion (GimpTool         *tool,
   gimp_blend_tool_update_items (blend_tool);
 
   gimp_blend_tool_update_graph (blend_tool);
-  gimp_image_map_apply (blend_tool->image_map, NULL);
+  gimp_drawable_filter_apply (blend_tool->filter, NULL);
 }
 
 static void
@@ -547,7 +553,7 @@ gimp_blend_tool_active_modifier_key (GimpTool        *tool,
       gimp_blend_tool_update_items (blend_tool);
 
       gimp_blend_tool_update_graph (blend_tool);
-      gimp_image_map_apply (blend_tool->image_map, NULL);
+      gimp_drawable_filter_apply (blend_tool->filter, NULL);
     }
   else if (key == GDK_MOD1_MASK)
     {
@@ -600,14 +606,15 @@ gimp_blend_tool_options_notify (GimpTool         *tool,
     {
       gimp_blend_tool_set_gradient (blend_tool, context->gradient);
 
-      if (blend_tool->image_map)
-        gimp_image_map_apply (blend_tool->image_map, NULL);
+      if (blend_tool->filter)
+        gimp_drawable_filter_apply (blend_tool->filter, NULL);
     }
   else if (blend_tool->render_node &&
            gegl_node_find_property (blend_tool->render_node, pspec->name))
     {
       /* Sync any property changes on the config object that match the op */
       GValue value = G_VALUE_INIT;
+
       g_value_init (&value, pspec->value_type);
 
       g_object_get_property (G_OBJECT (options), pspec->name, &value);
@@ -623,17 +630,22 @@ gimp_blend_tool_options_notify (GimpTool         *tool,
           gimp_blend_tool_update_graph (blend_tool);
         }
 
-      gimp_image_map_apply (blend_tool->image_map, NULL);
+      gimp_drawable_filter_apply (blend_tool->filter, NULL);
     }
-  else if (blend_tool->image_map &&
-           (! strcmp (pspec->name, "opacity") ||
-            ! strcmp (pspec->name, "paint-mode")))
+  else if (blend_tool->filter &&
+           ! strcmp (pspec->name, "opacity"))
     {
-      gimp_image_map_set_mode (blend_tool->image_map,
-                               gimp_context_get_opacity (context),
-                               gimp_context_get_paint_mode (context));
-
-      gimp_image_map_apply (blend_tool->image_map, NULL);
+      gimp_drawable_filter_set_opacity (blend_tool->filter,
+                                        gimp_context_get_opacity (context));
+    }
+  else if (blend_tool->filter &&
+           ! strcmp (pspec->name, "paint-mode"))
+    {
+      gimp_drawable_filter_set_mode (blend_tool->filter,
+                                     gimp_context_get_paint_mode (context),
+                                     GIMP_LAYER_COLOR_SPACE_AUTO,
+                                     GIMP_LAYER_COLOR_SPACE_AUTO,
+                                     GIMP_LAYER_COMPOSITE_AUTO);
     }
 }
 
@@ -867,10 +879,11 @@ gimp_blend_tool_start (GimpBlendTool *blend_tool,
   tool->display  = display;
   tool->drawable = drawable;
 
-  gimp_blend_tool_create_image_map (blend_tool, drawable);
+  gimp_blend_tool_create_filter (blend_tool, drawable);
 
   /* Initially sync all of the properties */
-  gimp_gegl_config_sync_node (GIMP_OBJECT (options), blend_tool->render_node);
+  gimp_operation_config_sync_node (GIMP_OBJECT (options),
+                                   blend_tool->render_node);
 
   /* Connect signal handlers for the gradient */
   gimp_blend_tool_set_gradient (blend_tool, context->gradient);
@@ -906,11 +919,15 @@ gimp_blend_tool_halt (GimpBlendTool *blend_tool)
       blend_tool->dist_buffer = NULL;
     }
 
-  if (blend_tool->image_map)
+  if (blend_tool->filter)
     {
-      gimp_image_map_abort (blend_tool->image_map);
-      g_object_unref (blend_tool->image_map);
-      blend_tool->image_map = NULL;
+      gimp_tool_control_push_preserve (tool->control, TRUE);
+
+      gimp_drawable_filter_abort (blend_tool->filter);
+      g_object_unref (blend_tool->filter);
+      blend_tool->filter = NULL;
+
+      gimp_tool_control_pop_preserve (tool->control);
 
       gimp_image_flush (gimp_display_get_image (tool->display));
     }
@@ -927,11 +944,16 @@ gimp_blend_tool_commit (GimpBlendTool *blend_tool)
 {
   GimpTool *tool = GIMP_TOOL (blend_tool);
 
-  if (blend_tool->image_map)
+  if (blend_tool->filter)
     {
-      gimp_image_map_commit (blend_tool->image_map, GIMP_PROGRESS (tool), FALSE);
-      g_object_unref (blend_tool->image_map);
-      blend_tool->image_map = NULL;
+      gimp_tool_control_push_preserve (tool->control, TRUE);
+
+      gimp_drawable_filter_commit (blend_tool->filter,
+                                   GIMP_PROGRESS (tool), FALSE);
+      g_object_unref (blend_tool->filter);
+      blend_tool->filter = NULL;
+
+      gimp_tool_control_pop_preserve (tool->control);
 
       gimp_image_flush (gimp_display_get_image (tool->display));
     }
@@ -977,11 +999,8 @@ gimp_blend_tool_precalc_shapeburst (GimpBlendTool *blend_tool)
                                   &x, &y, &width, &height))
     return;
 
-  gimp_progress_start (GIMP_PROGRESS (blend_tool), FALSE,
-                       _("Calculating distance map"));
-
   blend_tool->dist_buffer =
-    gimp_drawable_blend_shapeburst_distmap (tool->drawable, TRUE,
+    gimp_drawable_blend_shapeburst_distmap (tool->drawable, FALSE,
                                             GEGL_RECTANGLE (x, y, width, height),
                                             GIMP_PROGRESS (blend_tool));
 
@@ -1098,7 +1117,7 @@ gimp_blend_tool_update_graph (GimpBlendTool *blend_tool)
 static void
 gimp_blend_tool_gradient_dirty (GimpBlendTool *blend_tool)
 {
-  if (! blend_tool->image_map)
+  if (! blend_tool->filter)
     return;
 
   /* Set a property on the node. Otherwise it will cache and refuse to update */
@@ -1106,8 +1125,8 @@ gimp_blend_tool_gradient_dirty (GimpBlendTool *blend_tool)
                  "gradient", blend_tool->gradient,
                  NULL);
 
-  /* Update the image_map */
-  gimp_image_map_apply (blend_tool->image_map, NULL);
+  /* Update the filter */
+  gimp_drawable_filter_apply (blend_tool->filter, NULL);
 }
 
 static void
@@ -1169,8 +1188,8 @@ gimp_blend_tool_is_shapeburst (GimpBlendTool *blend_tool)
 /* image map stuff */
 
 static void
-gimp_blend_tool_create_image_map (GimpBlendTool *blend_tool,
-                                  GimpDrawable  *drawable)
+gimp_blend_tool_create_filter (GimpBlendTool *blend_tool,
+                               GimpDrawable  *drawable)
 {
   GimpBlendOptions *options = GIMP_BLEND_TOOL_GET_OPTIONS (blend_tool);
   GimpContext      *context = GIMP_CONTEXT (options);
@@ -1178,26 +1197,29 @@ gimp_blend_tool_create_image_map (GimpBlendTool *blend_tool,
   if (! blend_tool->graph)
     gimp_blend_tool_create_graph (blend_tool);
 
-  blend_tool->image_map = gimp_image_map_new (drawable,
-                                              C_("undo-type", "Blend"),
-                                              blend_tool->graph,
-                                              GIMP_STOCK_TOOL_BLEND);
+  blend_tool->filter = gimp_drawable_filter_new (drawable,
+                                                 C_("undo-type", "Blend"),
+                                                 blend_tool->graph,
+                                                 GIMP_ICON_TOOL_BLEND);
 
-  gimp_image_map_set_region (blend_tool->image_map,
-                             GIMP_IMAGE_MAP_REGION_DRAWABLE);
+  gimp_drawable_filter_set_region (blend_tool->filter,
+                                   GIMP_FILTER_REGION_DRAWABLE);
+  gimp_drawable_filter_set_opacity (blend_tool->filter,
+                                    gimp_context_get_opacity (context));
+  gimp_drawable_filter_set_mode (blend_tool->filter,
+                                 gimp_context_get_paint_mode (context),
+                                 GIMP_LAYER_COLOR_SPACE_AUTO,
+                                 GIMP_LAYER_COLOR_SPACE_AUTO,
+                                 GIMP_LAYER_COMPOSITE_AUTO);
 
-  g_signal_connect (blend_tool->image_map, "flush",
-                    G_CALLBACK (gimp_blend_tool_image_map_flush),
+  g_signal_connect (blend_tool->filter, "flush",
+                    G_CALLBACK (gimp_blend_tool_filter_flush),
                     blend_tool);
-
-  gimp_image_map_set_mode (blend_tool->image_map,
-                           gimp_context_get_opacity (context),
-                           gimp_context_get_paint_mode (context));
 }
 
 static void
-gimp_blend_tool_image_map_flush (GimpImageMap *image_map,
-                                 GimpTool     *tool)
+gimp_blend_tool_filter_flush (GimpDrawableFilter *filter,
+                              GimpTool           *tool)
 {
   GimpImage *image = gimp_display_get_image (tool->display);
 
