@@ -62,73 +62,75 @@
 
 /*  local function prototypes  */
 
-static void       gimp_curves_tool_constructed    (GObject              *object);
+static void       gimp_curves_tool_constructed     (GObject              *object);
 
-static gboolean   gimp_curves_tool_initialize     (GimpTool             *tool,
-                                                   GimpDisplay          *display,
-                                                   GError              **error);
-static void       gimp_curves_tool_button_release (GimpTool             *tool,
-                                                   const GimpCoords     *coords,
-                                                   guint32               time,
-                                                   GdkModifierType       state,
-                                                   GimpButtonReleaseType release_type,
-                                                   GimpDisplay          *display);
-static gboolean   gimp_curves_tool_key_press      (GimpTool             *tool,
-                                                   GdkEventKey          *kevent,
-                                                   GimpDisplay          *display);
-static void       gimp_curves_tool_oper_update    (GimpTool             *tool,
-                                                   const GimpCoords     *coords,
-                                                   GdkModifierType       state,
-                                                   gboolean              proximity,
-                                                   GimpDisplay          *display);
+static gboolean   gimp_curves_tool_initialize      (GimpTool             *tool,
+                                                    GimpDisplay          *display,
+                                                    GError              **error);
+static void       gimp_curves_tool_button_release  (GimpTool             *tool,
+                                                    const GimpCoords     *coords,
+                                                    guint32               time,
+                                                    GdkModifierType       state,
+                                                    GimpButtonReleaseType release_type,
+                                                    GimpDisplay          *display);
+static gboolean   gimp_curves_tool_key_press       (GimpTool             *tool,
+                                                    GdkEventKey          *kevent,
+                                                    GimpDisplay          *display);
+static void       gimp_curves_tool_oper_update     (GimpTool             *tool,
+                                                    const GimpCoords     *coords,
+                                                    GdkModifierType       state,
+                                                    gboolean              proximity,
+                                                    GimpDisplay          *display);
 
-static void       gimp_curves_tool_color_picked   (GimpColorTool        *color_tool,
-                                                   GimpColorPickState    pick_state,
-                                                   gdouble               x,
-                                                   gdouble               y,
-                                                   const Babl           *sample_format,
-                                                   gpointer              pixel,
-                                                   const GimpRGB        *color);
+static void       gimp_curves_tool_color_picked    (GimpColorTool        *color_tool,
+                                                    GimpColorPickState    pick_state,
+                                                    gdouble               x,
+                                                    gdouble               y,
+                                                    const Babl           *sample_format,
+                                                    gpointer              pixel,
+                                                    const GimpRGB        *color);
 
-static gchar    * gimp_curves_tool_get_operation  (GimpImageMapTool     *im_tool,
-                                                   gchar               **title,
-                                                   gchar               **description,
-                                                   gchar               **undo_desc,
-                                                   gchar               **icon_name,
-                                                   gchar               **help_id);
-static void       gimp_curves_tool_dialog         (GimpImageMapTool     *im_tool);
-static void       gimp_curves_tool_reset          (GimpImageMapTool     *im_tool);
-static gboolean   gimp_curves_tool_settings_import(GimpImageMapTool     *im_tool,
-                                                   GInputStream         *input,
-                                                   GError              **error);
-static gboolean   gimp_curves_tool_settings_export(GimpImageMapTool     *im_tool,
-                                                   GOutputStream        *output,
-                                                   GError              **error);
+static gchar    * gimp_curves_tool_get_operation   (GimpFilterTool       *filter_tool,
+                                                    gchar               **title,
+                                                    gchar               **description,
+                                                    gchar               **undo_desc,
+                                                    gchar               **icon_name,
+                                                    gchar               **help_id);
+static void       gimp_curves_tool_dialog          (GimpFilterTool       *filter_tool);
+static void       gimp_curves_tool_reset           (GimpFilterTool       *filter_tool);
+static gboolean   gimp_curves_tool_settings_import (GimpFilterTool       *filter_tool,
+                                                    GInputStream         *input,
+                                                    GError              **error);
+static gboolean   gimp_curves_tool_settings_export (GimpFilterTool       *filter_tool,
+                                                    GOutputStream        *output,
+                                                    GError              **error);
 
-static void       gimp_curves_tool_export_setup   (GimpSettingsBox      *settings_box,
-                                                   GtkFileChooserDialog *dialog,
-                                                   gboolean              export,
-                                                   GimpCurvesTool       *tool);
-static void       gimp_curves_tool_update_channel (GimpCurvesTool       *tool);
-static void       gimp_curves_tool_config_notify  (GObject              *object,
-                                                   GParamSpec           *pspec,
-                                                   GimpCurvesTool       *tool);
+static void       gimp_curves_tool_export_setup    (GimpSettingsBox      *settings_box,
+                                                    GtkFileChooserDialog *dialog,
+                                                    gboolean              export,
+                                                    GimpCurvesTool       *tool);
+static void       gimp_curves_tool_update_channel  (GimpCurvesTool       *tool);
+static void       gimp_curves_tool_config_notify   (GObject              *object,
+                                                    GParamSpec           *pspec,
+                                                    GimpCurvesTool       *tool);
 
-static void       curves_channel_callback         (GtkWidget            *widget,
-                                                   GimpCurvesTool       *tool);
-static void       curves_channel_reset_callback   (GtkWidget            *widget,
-                                                   GimpCurvesTool       *tool);
+static void       curves_channel_callback          (GtkWidget            *widget,
+                                                    GimpCurvesTool       *tool);
+static void       curves_channel_reset_callback    (GtkWidget            *widget,
+                                                    GimpCurvesTool       *tool);
 
-static gboolean   curves_menu_sensitivity         (gint                  value,
-                                                   gpointer              data);
+static gboolean   curves_menu_sensitivity          (gint                  value,
+                                                    gpointer              data);
 
-static void       curves_curve_type_callback      (GtkWidget            *widget,
-                                                   GimpCurvesTool       *tool);
+static void       curves_curve_type_callback       (GtkWidget            *widget,
+                                                    GimpCurvesTool       *tool);
 
-static const GimpRGB * curves_get_channel_color   (GimpHistogramChannel  channel);
+static gboolean   curves_get_channel_color         (GtkWidget            *widget,
+                                                    GimpHistogramChannel  channel,
+                                                    GimpRGB              *color);
 
 
-G_DEFINE_TYPE (GimpCurvesTool, gimp_curves_tool, GIMP_TYPE_IMAGE_MAP_TOOL)
+G_DEFINE_TYPE (GimpCurvesTool, gimp_curves_tool, GIMP_TYPE_FILTER_TOOL)
 
 #define parent_class gimp_curves_tool_parent_class
 
@@ -148,7 +150,7 @@ gimp_curves_tool_register (GimpToolRegisterCallback  callback,
                 _("Curves Tool: Adjust color curves"),
                 N_("_Curves..."), NULL,
                 NULL, GIMP_HELP_TOOL_CURVES,
-                GIMP_STOCK_TOOL_CURVES,
+                GIMP_ICON_TOOL_CURVES,
                 data);
 }
 
@@ -158,29 +160,29 @@ gimp_curves_tool_register (GimpToolRegisterCallback  callback,
 static void
 gimp_curves_tool_class_init (GimpCurvesToolClass *klass)
 {
-  GObjectClass          *object_class     = G_OBJECT_CLASS (klass);
-  GimpToolClass         *tool_class       = GIMP_TOOL_CLASS (klass);
-  GimpColorToolClass    *color_tool_class = GIMP_COLOR_TOOL_CLASS (klass);
-  GimpImageMapToolClass *im_tool_class    = GIMP_IMAGE_MAP_TOOL_CLASS (klass);
+  GObjectClass        *object_class      = G_OBJECT_CLASS (klass);
+  GimpToolClass       *tool_class        = GIMP_TOOL_CLASS (klass);
+  GimpColorToolClass  *color_tool_class  = GIMP_COLOR_TOOL_CLASS (klass);
+  GimpFilterToolClass *filter_tool_class = GIMP_FILTER_TOOL_CLASS (klass);
 
-  object_class->constructed          = gimp_curves_tool_constructed;
+  object_class->constructed              = gimp_curves_tool_constructed;
 
-  tool_class->initialize             = gimp_curves_tool_initialize;
-  tool_class->button_release         = gimp_curves_tool_button_release;
-  tool_class->key_press              = gimp_curves_tool_key_press;
-  tool_class->oper_update            = gimp_curves_tool_oper_update;
+  tool_class->initialize                 = gimp_curves_tool_initialize;
+  tool_class->button_release             = gimp_curves_tool_button_release;
+  tool_class->key_press                  = gimp_curves_tool_key_press;
+  tool_class->oper_update                = gimp_curves_tool_oper_update;
 
-  color_tool_class->picked           = gimp_curves_tool_color_picked;
+  color_tool_class->picked               = gimp_curves_tool_color_picked;
 
-  im_tool_class->settings_name       = "curves";
-  im_tool_class->import_dialog_title = _("Import Curves");
-  im_tool_class->export_dialog_title = _("Export Curves");
+  filter_tool_class->settings_name       = "curves";
+  filter_tool_class->import_dialog_title = _("Import Curves");
+  filter_tool_class->export_dialog_title = _("Export Curves");
 
-  im_tool_class->get_operation       = gimp_curves_tool_get_operation;
-  im_tool_class->dialog              = gimp_curves_tool_dialog;
-  im_tool_class->reset               = gimp_curves_tool_reset;
-  im_tool_class->settings_import     = gimp_curves_tool_settings_import;
-  im_tool_class->settings_export     = gimp_curves_tool_settings_export;
+  filter_tool_class->get_operation       = gimp_curves_tool_get_operation;
+  filter_tool_class->dialog              = gimp_curves_tool_dialog;
+  filter_tool_class->reset               = gimp_curves_tool_reset;
+  filter_tool_class->settings_import     = gimp_curves_tool_settings_import;
+  filter_tool_class->settings_export     = gimp_curves_tool_settings_export;
 }
 
 static void
@@ -197,7 +199,7 @@ gimp_curves_tool_constructed (GObject *object)
 {
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_signal_connect_object (GIMP_IMAGE_MAP_TOOL (object)->config, "notify",
+  g_signal_connect_object (GIMP_FILTER_TOOL (object)->config, "notify",
                            G_CALLBACK (gimp_curves_tool_config_notify),
                            object, 0);
 
@@ -222,10 +224,7 @@ gimp_curves_tool_initialize (GimpTool     *tool,
       return FALSE;
     }
 
-  gimp_int_combo_box_set_sensitivity (GIMP_INT_COMBO_BOX (c_tool->channel_menu),
-                                      curves_menu_sensitivity, drawable, NULL);
-
-  histogram = gimp_histogram_new (TRUE);
+  histogram = gimp_histogram_new (FALSE);
   gimp_drawable_calculate_histogram (drawable, histogram);
   gimp_histogram_view_set_background (GIMP_HISTOGRAM_VIEW (c_tool->graph),
                                       histogram);
@@ -253,9 +252,9 @@ gimp_curves_tool_button_release (GimpTool              *tool,
                                  GimpButtonReleaseType  release_type,
                                  GimpDisplay           *display)
 {
-  GimpCurvesTool   *c_tool  = GIMP_CURVES_TOOL (tool);
-  GimpImageMapTool *im_tool = GIMP_IMAGE_MAP_TOOL (tool);
-  GimpCurvesConfig *config  = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpCurvesTool   *c_tool      = GIMP_CURVES_TOOL (tool);
+  GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (tool);
+  GimpCurvesConfig *config      = GIMP_CURVES_CONFIG (filter_tool->config);
 
   if (state & gimp_get_extend_selection_mask ())
     {
@@ -324,42 +323,48 @@ gimp_curves_tool_oper_update (GimpTool         *tool,
                               gboolean          proximity,
                               GimpDisplay      *display)
 {
-  GimpColorPickMode  mode;
-  gchar             *status      = NULL;
-  GdkModifierType    extend_mask = gimp_get_extend_selection_mask ();
-  GdkModifierType    toggle_mask = gimp_get_toggle_behavior_mask ();
-
-  GIMP_TOOL_CLASS (parent_class)->oper_update (tool, coords, state, proximity,
-                                               display);
-
-  gimp_tool_pop_status (tool, display);
-
-  if (state & extend_mask)
+  if (gimp_filter_tool_on_guide (GIMP_FILTER_TOOL (tool),
+                                 coords, display))
     {
-      mode   = GIMP_COLOR_PICK_MODE_PALETTE;
-      status = g_strdup (_("Click to add a control point"));
-    }
-  else if (state & toggle_mask)
-    {
-      mode   = GIMP_COLOR_PICK_MODE_PALETTE;
-      status = g_strdup (_("Click to add control points to all channels"));
+      GIMP_TOOL_CLASS (parent_class)->oper_update (tool, coords, state, proximity,
+                                                   display);
     }
   else
     {
-      mode   = GIMP_COLOR_PICK_MODE_NONE;
-      status = gimp_suggest_modifiers (_("Click to locate on curve"),
-                                       (extend_mask | toggle_mask) & ~state,
-                                       _("%s: add control point"),
-                                       _("%s: add control points to all channels"),
-                                       NULL);
+      GimpColorPickMode  mode;
+      gchar             *status      = NULL;
+      GdkModifierType    extend_mask = gimp_get_extend_selection_mask ();
+      GdkModifierType    toggle_mask = gimp_get_toggle_behavior_mask ();
+
+      gimp_tool_pop_status (tool, display);
+
+      if (state & extend_mask)
+        {
+          mode   = GIMP_COLOR_PICK_MODE_PALETTE;
+          status = g_strdup (_("Click to add a control point"));
+        }
+      else if (state & toggle_mask)
+        {
+          mode   = GIMP_COLOR_PICK_MODE_PALETTE;
+          status = g_strdup (_("Click to add control points to all channels"));
+        }
+      else
+        {
+          mode   = GIMP_COLOR_PICK_MODE_NONE;
+          status = gimp_suggest_modifiers (_("Click to locate on curve"),
+                                           (extend_mask | toggle_mask) & ~state,
+                                           _("%s: add control point"),
+                                           _("%s: add control points to all channels"),
+                                           NULL);
+        }
+
+      GIMP_COLOR_TOOL (tool)->pick_mode = mode;
+
+      if (proximity)
+        gimp_tool_push_status (tool, display, "%s", status);
+
+      g_free (status);
     }
-
-  GIMP_COLOR_TOOL (tool)->pick_mode = mode;
-
-  if (proximity)
-    gimp_tool_push_status (tool, display, "%s", status);
-
-  g_free (status);
 }
 
 static void
@@ -371,12 +376,12 @@ gimp_curves_tool_color_picked (GimpColorTool      *color_tool,
                                gpointer            pixel,
                                const GimpRGB      *color)
 {
-  GimpCurvesTool   *tool    = GIMP_CURVES_TOOL (color_tool);
-  GimpImageMapTool *im_tool = GIMP_IMAGE_MAP_TOOL (color_tool);
-  GimpCurvesConfig *config  = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpCurvesTool   *tool        = GIMP_CURVES_TOOL (color_tool);
+  GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (color_tool);
+  GimpCurvesConfig *config      = GIMP_CURVES_CONFIG (filter_tool->config);
   GimpDrawable     *drawable;
 
-  drawable = GIMP_IMAGE_MAP_TOOL (tool)->drawable;
+  drawable = GIMP_FILTER_TOOL (tool)->drawable;
 
   tool->picked_color[GIMP_HISTOGRAM_RED]   = color->r;
   tool->picked_color[GIMP_HISTOGRAM_GREEN] = color->g;
@@ -395,12 +400,12 @@ gimp_curves_tool_color_picked (GimpColorTool      *color_tool,
 }
 
 static gchar *
-gimp_curves_tool_get_operation (GimpImageMapTool  *im_tool,
-                                gchar            **title,
-                                gchar            **description,
-                                gchar            **undo_desc,
-                                gchar            **icon_name,
-                                gchar            **help_id)
+gimp_curves_tool_get_operation (GimpFilterTool  *filter_tool,
+                                gchar          **title,
+                                gchar          **description,
+                                gchar          **undo_desc,
+                                gchar          **icon_name,
+                                gchar          **help_id)
 {
   *description = g_strdup (_("Adjust Color Curves"));
 
@@ -413,11 +418,11 @@ gimp_curves_tool_get_operation (GimpImageMapTool  *im_tool,
 /*******************/
 
 static void
-gimp_curves_tool_dialog (GimpImageMapTool *im_tool)
+gimp_curves_tool_dialog (GimpFilterTool *filter_tool)
 {
-  GimpCurvesTool   *tool         = GIMP_CURVES_TOOL (im_tool);
-  GimpToolOptions  *tool_options = GIMP_TOOL_GET_OPTIONS (im_tool);
-  GimpCurvesConfig *config       = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpCurvesTool   *tool         = GIMP_CURVES_TOOL (filter_tool);
+  GimpToolOptions  *tool_options = GIMP_TOOL_GET_OPTIONS (filter_tool);
+  GimpCurvesConfig *config       = GIMP_CURVES_CONFIG (filter_tool->config);
   GtkListStore     *store;
   GtkWidget        *main_vbox;
   GtkWidget        *frame_vbox;
@@ -432,11 +437,11 @@ gimp_curves_tool_dialog (GimpImageMapTool *im_tool)
   GtkWidget        *bar;
   GtkWidget        *combo;
 
-  g_signal_connect (im_tool->settings_box, "file-dialog-setup",
+  g_signal_connect (filter_tool->settings_box, "file-dialog-setup",
                     G_CALLBACK (gimp_curves_tool_export_setup),
-                    im_tool);
+                    filter_tool);
 
-  main_vbox = gimp_image_map_tool_dialog_get_vbox (im_tool);
+  main_vbox = gimp_filter_tool_dialog_get_vbox (filter_tool);
 
   /*  The combo box for selecting channels  */
   main_frame = gimp_frame_new (NULL);
@@ -465,6 +470,8 @@ gimp_curves_tool_dialog (GimpImageMapTool *im_tool)
                                  config->channel);
   gimp_enum_combo_box_set_icon_prefix (GIMP_ENUM_COMBO_BOX (tool->channel_menu),
                                        "gimp-channel");
+  gimp_int_combo_box_set_sensitivity (GIMP_INT_COMBO_BOX (tool->channel_menu),
+                                      curves_menu_sensitivity, filter_tool, NULL);
   gtk_box_pack_start (GTK_BOX (hbox), tool->channel_menu, FALSE, FALSE, 0);
   gtk_widget_show (tool->channel_menu);
 
@@ -535,8 +542,10 @@ gimp_curves_tool_dialog (GimpImageMapTool *im_tool)
   gtk_container_add (GTK_CONTAINER (frame), tool->graph);
   gtk_widget_show (tool->graph);
 
-  gimp_histogram_options_connect_view (GIMP_HISTOGRAM_OPTIONS (tool_options),
-                                       GIMP_HISTOGRAM_VIEW (tool->graph));
+  g_object_bind_property (G_OBJECT (tool_options), "histogram-scale",
+                          G_OBJECT (tool->graph),  "histogram-scale",
+                          G_BINDING_SYNC_CREATE |
+                          G_BINDING_BIDIRECTIONAL);
 
   /*  The bottom color bar  */
   hbox2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -588,13 +597,13 @@ gimp_curves_tool_dialog (GimpImageMapTool *im_tool)
 }
 
 static void
-gimp_curves_tool_reset (GimpImageMapTool *im_tool)
+gimp_curves_tool_reset (GimpFilterTool *filter_tool)
 {
-  GimpCurvesConfig     *config = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpCurvesConfig     *config = GIMP_CURVES_CONFIG (filter_tool->config);
   GimpCurvesConfig     *default_config;
   GimpHistogramChannel  channel;
 
-  default_config = GIMP_CURVES_CONFIG (im_tool->default_config);
+  default_config = GIMP_CURVES_CONFIG (filter_tool->default_config);
 
   for (channel = GIMP_HISTOGRAM_VALUE;
        channel <= GIMP_HISTOGRAM_ALPHA;
@@ -624,11 +633,11 @@ gimp_curves_tool_reset (GimpImageMapTool *im_tool)
 }
 
 static gboolean
-gimp_curves_tool_settings_import (GimpImageMapTool  *im_tool,
-                                  GInputStream      *input,
-                                  GError           **error)
+gimp_curves_tool_settings_import (GimpFilterTool  *filter_tool,
+                                  GInputStream    *input,
+                                  GError         **error)
 {
-  GimpCurvesConfig *config = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpCurvesConfig *config = GIMP_CURVES_CONFIG (filter_tool->config);
   gchar             header[64];
   gsize             bytes_read;
 
@@ -645,25 +654,25 @@ gimp_curves_tool_settings_import (GimpImageMapTool  *im_tool,
   if (g_str_has_prefix (header, "# GIMP Curves File\n"))
     return gimp_curves_config_load_cruft (config, input, error);
 
-  return GIMP_IMAGE_MAP_TOOL_CLASS (parent_class)->settings_import (im_tool,
-                                                                    input,
-                                                                    error);
+  return GIMP_FILTER_TOOL_CLASS (parent_class)->settings_import (filter_tool,
+                                                                 input,
+                                                                 error);
 }
 
 static gboolean
-gimp_curves_tool_settings_export (GimpImageMapTool  *im_tool,
-                                  GOutputStream     *output,
-                                  GError           **error)
+gimp_curves_tool_settings_export (GimpFilterTool  *filter_tool,
+                                  GOutputStream   *output,
+                                  GError         **error)
 {
-  GimpCurvesTool   *tool   = GIMP_CURVES_TOOL (im_tool);
-  GimpCurvesConfig *config = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpCurvesTool   *tool   = GIMP_CURVES_TOOL (filter_tool);
+  GimpCurvesConfig *config = GIMP_CURVES_CONFIG (filter_tool->config);
 
   if (tool->export_old_format)
     return gimp_curves_config_save_cruft (config, output, error);
 
-  return GIMP_IMAGE_MAP_TOOL_CLASS (parent_class)->settings_export (im_tool,
-                                                                    output,
-                                                                    error);
+  return GIMP_FILTER_TOOL_CLASS (parent_class)->settings_export (filter_tool,
+                                                                 output,
+                                                                 error);
 }
 
 static void
@@ -691,9 +700,9 @@ gimp_curves_tool_export_setup (GimpSettingsBox      *settings_box,
 static void
 gimp_curves_tool_update_channel (GimpCurvesTool *tool)
 {
-  GimpImageMapTool     *im_tool = GIMP_IMAGE_MAP_TOOL (tool);
-  GimpCurvesConfig     *config  = GIMP_CURVES_CONFIG (im_tool->config);
-  GimpCurve            *curve   = config->curve[config->channel];
+  GimpFilterTool       *filter_tool = GIMP_FILTER_TOOL (tool);
+  GimpCurvesConfig     *config      = GIMP_CURVES_CONFIG (filter_tool->config);
+  GimpCurve            *curve       = config->curve[config->channel];
   GimpHistogramChannel  channel;
 
   gimp_int_combo_box_set_active (GIMP_INT_COMBO_BOX (tool->channel_menu),
@@ -708,6 +717,7 @@ gimp_curves_tool_update_channel (GimpCurvesTool *tool)
     case GIMP_HISTOGRAM_VALUE:
     case GIMP_HISTOGRAM_ALPHA:
     case GIMP_HISTOGRAM_RGB:
+    case GIMP_HISTOGRAM_LUMINANCE:
       gimp_curve_get_uchar (curve, sizeof (r), r);
 
       gimp_color_bar_set_buffers (GIMP_COLOR_BAR (tool->xrange),
@@ -743,16 +753,21 @@ gimp_curves_tool_update_channel (GimpCurvesTool *tool)
        channel <= GIMP_HISTOGRAM_ALPHA;
        channel++)
     {
+      GimpRGB  curve_color;
+      gboolean has_color;
+
+      has_color = curves_get_channel_color (tool->graph, channel, &curve_color);
+
       if (channel == config->channel)
         {
           gimp_curve_view_set_curve (GIMP_CURVE_VIEW (tool->graph), curve,
-                                     curves_get_channel_color (channel));
+                                     has_color ? &curve_color : NULL);
         }
       else
         {
           gimp_curve_view_add_background (GIMP_CURVE_VIEW (tool->graph),
                                           config->curve[channel],
-                                          curves_get_channel_color (channel));
+                                          has_color ? &curve_color : NULL);
         }
     }
 
@@ -786,8 +801,8 @@ static void
 curves_channel_callback (GtkWidget      *widget,
                          GimpCurvesTool *tool)
 {
-  GimpImageMapTool *im_tool = GIMP_IMAGE_MAP_TOOL (tool);
-  GimpCurvesConfig *config  = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (tool);
+  GimpCurvesConfig *config      = GIMP_CURVES_CONFIG (filter_tool->config);
   gint              value;
 
   if (gimp_int_combo_box_get_active (GIMP_INT_COMBO_BOX (widget), &value) &&
@@ -803,8 +818,8 @@ static void
 curves_channel_reset_callback (GtkWidget      *widget,
                                GimpCurvesTool *tool)
 {
-  GimpImageMapTool *im_tool = GIMP_IMAGE_MAP_TOOL (tool);
-  GimpCurvesConfig *config  = GIMP_CURVES_CONFIG (im_tool->config);
+  GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (tool);
+  GimpCurvesConfig *config      = GIMP_CURVES_CONFIG (filter_tool->config);
 
   gimp_curve_reset (config->curve[config->channel], FALSE);
 }
@@ -813,9 +828,12 @@ static gboolean
 curves_menu_sensitivity (gint      value,
                          gpointer  data)
 {
-  GimpDrawable         *drawable = GIMP_DRAWABLE (data);
+  GimpDrawable         *drawable = GIMP_FILTER_TOOL (data)->drawable;
   GimpHistogramChannel  channel  = value;
 
+  if (!drawable)
+    return FALSE;
+
   switch (channel)
     {
     case GIMP_HISTOGRAM_VALUE:
@@ -831,6 +849,9 @@ curves_menu_sensitivity (gint      value,
 
     case GIMP_HISTOGRAM_RGB:
       return FALSE;
+
+    case GIMP_HISTOGRAM_LUMINANCE:
+      return FALSE;
     }
 
   return FALSE;
@@ -844,17 +865,19 @@ curves_curve_type_callback (GtkWidget      *widget,
 
   if (gimp_int_combo_box_get_active (GIMP_INT_COMBO_BOX (widget), &value))
     {
-      GimpImageMapTool *im_tool    = GIMP_IMAGE_MAP_TOOL (tool);
-      GimpCurvesConfig *config     = GIMP_CURVES_CONFIG (im_tool->config);
-      GimpCurveType     curve_type = value;
+      GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (tool);
+      GimpCurvesConfig *config      = GIMP_CURVES_CONFIG (filter_tool->config);
+      GimpCurveType     curve_type  = value;
 
       if (config->curve[config->channel]->curve_type != curve_type)
         gimp_curve_set_curve_type (config->curve[config->channel], curve_type);
     }
 }
 
-static const GimpRGB *
-curves_get_channel_color (GimpHistogramChannel channel)
+static gboolean
+curves_get_channel_color (GtkWidget            *widget,
+                          GimpHistogramChannel  channel,
+                          GimpRGB              *color)
 {
   static const GimpRGB channel_colors[GIMP_HISTOGRAM_RGB] =
   {
@@ -866,7 +889,20 @@ curves_get_channel_color (GimpHistogramChannel channel)
   };
 
   if (channel == GIMP_HISTOGRAM_VALUE)
-    return NULL;
+    return FALSE;
 
-  return &channel_colors[channel];
+  if (channel == GIMP_HISTOGRAM_ALPHA)
+    {
+      GtkStyle *style = gtk_widget_get_style (widget);
+
+      gimp_rgba_set (color,
+                     style->text_aa[GTK_STATE_NORMAL].red / 65535.0,
+                     style->text_aa[GTK_STATE_NORMAL].green / 65535.0,
+                     style->text_aa[GTK_STATE_NORMAL].blue / 65535.0,
+                     1.0);
+      return TRUE;
+    }
+
+  *color = channel_colors[channel];
+  return TRUE;
 }
