@@ -32,7 +32,7 @@
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimppropwidgets.h"
 
-#include "gimpcoloroptions.h"
+#include "gimpfilteroptions.h"
 #include "gimpgegltool.h"
 
 #include "gimp-intl.h"
@@ -49,10 +49,10 @@ enum
 
 /*  local function prototypes  */
 
-static void   gimp_gegl_tool_dialog            (GimpImageMapTool *im_tool);
+static void   gimp_gegl_tool_dialog            (GimpFilterTool *filter_tool);
 
-static void   gimp_gegl_tool_operation_changed (GtkWidget        *widget,
-                                                GimpGeglTool     *tool);
+static void   gimp_gegl_tool_operation_changed (GtkWidget      *widget,
+                                                GimpGeglTool   *tool);
 
 
 G_DEFINE_TYPE (GimpGeglTool, gimp_gegl_tool, GIMP_TYPE_OPERATION_TOOL)
@@ -65,7 +65,7 @@ gimp_gegl_tool_register (GimpToolRegisterCallback  callback,
                          gpointer                  data)
 {
   (* callback) (GIMP_TYPE_GEGL_TOOL,
-                GIMP_TYPE_COLOR_OPTIONS,
+                GIMP_TYPE_FILTER_OPTIONS,
                 gimp_color_options_gui,
                 0,
                 "gimp-gegl-tool",
@@ -73,16 +73,16 @@ gimp_gegl_tool_register (GimpToolRegisterCallback  callback,
                 _("GEGL Tool: Use an arbitrary GEGL operation"),
                 N_("_GEGL Operation..."), NULL,
                 NULL, GIMP_HELP_TOOL_GEGL,
-                GIMP_STOCK_GEGL,
+                GIMP_ICON_GEGL,
                 data);
 }
 
 static void
 gimp_gegl_tool_class_init (GimpGeglToolClass *klass)
 {
-  GimpImageMapToolClass *im_tool_class = GIMP_IMAGE_MAP_TOOL_CLASS (klass);
+  GimpFilterToolClass *filter_tool_class = GIMP_FILTER_TOOL_CLASS (klass);
 
-  im_tool_class->dialog = gimp_gegl_tool_dialog;
+  filter_tool_class->dialog = gimp_gegl_tool_dialog;
 }
 
 static void
@@ -124,10 +124,10 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:color",
     "gegl:color-enhance",
     "gegl:color-exchange",
-    "gegl:color-reduction",
     "gegl:color-rotate",
     "gegl:color-temperature",
     "gegl:color-to-alpha",
+    "gegl:component-extract",
     "gegl:convolution-matrix",
     "gegl:cubism",
     "gegl:deinterlace",
@@ -135,23 +135,29 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:diffraction-patterns",
     "gegl:displace",
     "gegl:distance-transform",
+    "gegl:dither",
     "gegl:dropshadow",
     "gegl:edge",
     "gegl:edge-laplace",
+    "gegl:edge-neon",
     "gegl:edge-sobel",
     "gegl:emboss",
     "gegl:engrave",
     "gegl:exposure",
+    "gegl:fattal02",
     "gegl:fractal-trace",
     "gegl:gaussian-blur",
     "gegl:gaussian-blur-selective",
+    "gegl:gegl",
     "gegl:grid",
     "gegl:high-pass",
     "gegl:illusion",
+    "gegl:image-gradient",
     "gegl:invert-linear",
     "gegl:invert-gamma",
     "gegl:lens-distortion",
     "gegl:lens-flare",
+    "gegl:mantiuk06",
     "gegl:maze",
     "gegl:mirrors",
     "gegl:mono-mixer",
@@ -176,15 +182,20 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:plasma",
     "gegl:polar-coordinates",
     "gegl:red-eye-removal",
+    "gegl:reinhard05",
+    "gegl:rgb-clip",
     "gegl:ripple",
     "gegl:saturation",
     "gegl:sepia",
     "gegl:shift",
     "gegl:simplex-noise",
     "gegl:sinus",
+    "gegl:slic",
+    "gegl:snn-mean",
     "gegl:softglow",
     "gegl:stretch-contrast",
     "gegl:stretch-contrast-hsv",
+    "gegl:stress",
     "gegl:supernova",
     "gegl:texturize-canvas",
     "gegl:tile-glass",
@@ -195,6 +206,8 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:value-propagate",
     "gegl:video-degradation",
     "gegl:vignette",
+    "gegl:waterpixels",
+    "gegl:wavelet-blur",
     "gegl:waves",
     "gegl:whirl-pinch",
     "gegl:wind",
@@ -202,6 +215,7 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     /* these ops are blacklisted for other reasons */
     "gegl:contrast-curve",
     "gegl:convert-format", /* pointless */
+    "gegl:ditto", /* pointless */
     "gegl:fill-path",
     "gegl:gray", /* we use gimp's op */
     "gegl:hstack", /* pointless */
@@ -209,15 +223,20 @@ gimp_gegl_tool_operation_blacklisted (const gchar *name,
     "gegl:layer", /* we use gimp's ops */
     "gegl:lcms-from-profile", /* not usable here */
     "gegl:linear-gradient", /* we use the blend tool */
+    "gegl:map-absolute", /* pointless */
+    "gegl:map-relative", /* pointless */
     "gegl:matting-global", /* used in the foreground select tool */
+    "gegl:matting-levin", /* used in the foreground select tool */
     "gegl:opacity", /* poinless */
     "gegl:path",
     "gegl:posterize", /* we use gimp's op */
     "gegl:radial-gradient", /* we use the blend tool */
+    "gegl:rectangle", /* pointless */
     "gegl:seamless-clone", /* used in the seamless clone tool */
     "gegl:text", /* we use gimp's text rendering */
     "gegl:threshold", /* we use gimp's op */
     "gegl:tile", /* pointless */
+    "gegl:unpremul", /* pointless */
     "gegl:vector-stroke",
   };
 
@@ -325,10 +344,10 @@ gimp_get_geglopclasses (void)
 /*****************/
 
 static void
-gimp_gegl_tool_dialog (GimpImageMapTool *image_map_tool)
+gimp_gegl_tool_dialog (GimpFilterTool *filter_tool)
 {
-  GimpGeglTool      *tool   = GIMP_GEGL_TOOL (image_map_tool);
-  GimpOperationTool *o_tool = GIMP_OPERATION_TOOL (image_map_tool);
+  GimpGeglTool      *tool   = GIMP_GEGL_TOOL (filter_tool);
+  GimpOperationTool *o_tool = GIMP_OPERATION_TOOL (filter_tool);
   GtkListStore      *store;
   GtkCellRenderer   *cell;
   GtkWidget         *main_vbox;
@@ -337,9 +356,9 @@ gimp_gegl_tool_dialog (GimpImageMapTool *image_map_tool)
   GList             *opclasses;
   GList             *iter;
 
-  GIMP_IMAGE_MAP_TOOL_CLASS (parent_class)->dialog (image_map_tool);
+  GIMP_FILTER_TOOL_CLASS (parent_class)->dialog (filter_tool);
 
-  main_vbox = gimp_image_map_tool_dialog_get_vbox (image_map_tool);
+  main_vbox = gimp_filter_tool_dialog_get_vbox (filter_tool);
 
   /*  The operation combo box  */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
@@ -361,7 +380,7 @@ gimp_gegl_tool_dialog (GimpImageMapTool *image_map_tool)
       gchar              *label;
 
       if (g_str_has_prefix (opclass->name, "gegl:"))
-        icon_name = GIMP_STOCK_GEGL;
+        icon_name = GIMP_ICON_GEGL;
 
       if (g_str_has_prefix (op_name, "gegl:"))
         op_name += strlen ("gegl:");
@@ -410,7 +429,7 @@ gimp_gegl_tool_dialog (GimpImageMapTool *image_map_tool)
 
   tool->description_label = gtk_label_new ("");
   gtk_label_set_line_wrap (GTK_LABEL (tool->description_label), TRUE);
-  gtk_misc_set_alignment (GTK_MISC (tool->description_label), 0.0, 0.0);
+  gtk_label_set_xalign (GTK_LABEL (tool->description_label), 0.0);
   gtk_box_pack_start (GTK_BOX (main_vbox), tool->description_label,
                       FALSE, FALSE, 0);
   gtk_box_reorder_child (GTK_BOX (main_vbox), tool->description_label, 1);
@@ -465,7 +484,7 @@ gimp_gegl_tool_operation_changed (GtkWidget    *widget,
                                          _("GEGL Operation"),
                                          _("GEGL Operation"),
                                          NULL,
-                                         GIMP_STOCK_GEGL,
+                                         GIMP_ICON_GEGL,
                                          GIMP_HELP_TOOL_GEGL);
       g_free (operation);
     }
