@@ -29,7 +29,6 @@
 
 #include "tools-types.h"
 
-#include "gegl/gimp-gegl-config.h"
 #include "gegl/gimp-gegl-utils.h"
 
 #include "core/gimpchannel.h"
@@ -38,7 +37,6 @@
 #include "core/gimpdrawable.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
-#include "core/gimpimagemap.h"
 #include "core/gimplist.h"
 #include "core/gimpparamspecs-duplicate.h"
 #include "core/gimppickable.h"
@@ -52,7 +50,7 @@
 #include "display/gimpdisplay.h"
 #include "display/gimptoolgui.h"
 
-#include "gimpcoloroptions.h"
+#include "gimpfilteroptions.h"
 #include "gimpoperationtool.h"
 
 #include "gimp-intl.h"
@@ -62,8 +60,10 @@ typedef struct _AuxInput AuxInput;
 
 struct _AuxInput
 {
-  GeglNode  *node;
-  GtkWidget *box;
+  GimpOperationTool *tool;
+  gchar             *pad;
+  GeglNode          *node;
+  GtkWidget         *box;
 };
 
 
@@ -78,15 +78,15 @@ static void        gimp_operation_tool_control         (GimpTool          *tool,
                                                         GimpToolAction     action,
                                                         GimpDisplay       *display);
 
-static gchar     * gimp_operation_tool_get_operation   (GimpImageMapTool  *im_tool,
+static gchar     * gimp_operation_tool_get_operation   (GimpFilterTool    *filter_tool,
                                                         gchar            **title,
                                                         gchar            **description,
                                                         gchar            **undo_desc,
                                                         gchar            **icon_name,
                                                         gchar            **help_id);
-static void        gimp_operation_tool_dialog          (GimpImageMapTool  *im_tool);
-static void        gimp_operation_tool_reset           (GimpImageMapTool  *im_tool);
-static GtkWidget * gimp_operation_tool_get_settings_ui (GimpImageMapTool  *im_tool,
+static void        gimp_operation_tool_dialog          (GimpFilterTool    *filter_tool);
+static void        gimp_operation_tool_reset           (GimpFilterTool    *filter_tool);
+static GtkWidget * gimp_operation_tool_get_settings_ui (GimpFilterTool    *filter_tool,
                                                         GimpContainer     *settings,
                                                         GFile             *settings_file,
                                                         const gchar       *import_dialog_title,
@@ -94,7 +94,7 @@ static GtkWidget * gimp_operation_tool_get_settings_ui (GimpImageMapTool  *im_to
                                                         const gchar       *file_dialog_help_id,
                                                         GFile             *default_folder,
                                                         GtkWidget        **settings_box);
-static void        gimp_operation_tool_color_picked    (GimpImageMapTool  *im_tool,
+static void        gimp_operation_tool_color_picked    (GimpFilterTool    *filter_tool,
                                                         gpointer           identifier,
                                                         gdouble            x,
                                                         gdouble            y,
@@ -113,7 +113,7 @@ static void        gimp_operation_tool_aux_input_free  (AuxInput          *input
 
 
 G_DEFINE_TYPE (GimpOperationTool, gimp_operation_tool,
-               GIMP_TYPE_IMAGE_MAP_TOOL)
+               GIMP_TYPE_FILTER_TOOL)
 
 #define parent_class gimp_operation_tool_parent_class
 
@@ -123,7 +123,7 @@ gimp_operation_tool_register (GimpToolRegisterCallback  callback,
                               gpointer                  data)
 {
   (* callback) (GIMP_TYPE_OPERATION_TOOL,
-                GIMP_TYPE_COLOR_OPTIONS,
+                GIMP_TYPE_FILTER_OPTIONS,
                 gimp_color_options_gui,
                 GIMP_CONTEXT_PROP_MASK_FOREGROUND |
                 GIMP_CONTEXT_PROP_MASK_BACKGROUND,
@@ -132,33 +132,33 @@ gimp_operation_tool_register (GimpToolRegisterCallback  callback,
                 _("Operation Tool: Use an arbitrary GEGL operation"),
                 N_("_GEGL Operation..."), NULL,
                 NULL, GIMP_HELP_TOOL_GEGL,
-                GIMP_STOCK_GEGL,
+                GIMP_ICON_GEGL,
                 data);
 }
 
 static void
 gimp_operation_tool_class_init (GimpOperationToolClass *klass)
 {
-  GObjectClass          *object_class  = G_OBJECT_CLASS (klass);
-  GimpToolClass         *tool_class    = GIMP_TOOL_CLASS (klass);
-  GimpImageMapToolClass *im_tool_class = GIMP_IMAGE_MAP_TOOL_CLASS (klass);
+  GObjectClass        *object_class      = G_OBJECT_CLASS (klass);
+  GimpToolClass       *tool_class        = GIMP_TOOL_CLASS (klass);
+  GimpFilterToolClass *filter_tool_class = GIMP_FILTER_TOOL_CLASS (klass);
 
-  object_class->finalize         = gimp_operation_tool_finalize;
+  object_class->finalize             = gimp_operation_tool_finalize;
 
-  tool_class->initialize         = gimp_operation_tool_initialize;
-  tool_class->control            = gimp_operation_tool_control;
+  tool_class->initialize             = gimp_operation_tool_initialize;
+  tool_class->control                = gimp_operation_tool_control;
 
-  im_tool_class->get_operation   = gimp_operation_tool_get_operation;
-  im_tool_class->dialog          = gimp_operation_tool_dialog;
-  im_tool_class->reset           = gimp_operation_tool_reset;
-  im_tool_class->get_settings_ui = gimp_operation_tool_get_settings_ui;
-  im_tool_class->color_picked    = gimp_operation_tool_color_picked;
+  filter_tool_class->get_operation   = gimp_operation_tool_get_operation;
+  filter_tool_class->dialog          = gimp_operation_tool_dialog;
+  filter_tool_class->reset           = gimp_operation_tool_reset;
+  filter_tool_class->get_settings_ui = gimp_operation_tool_get_settings_ui;
+  filter_tool_class->color_picked    = gimp_operation_tool_color_picked;
 }
 
 static void
 gimp_operation_tool_init (GimpOperationTool *tool)
 {
-  GIMP_IMAGE_MAP_TOOL_GET_CLASS (tool)->settings_name = NULL; /* XXX hack */
+  GIMP_FILTER_TOOL_GET_CLASS (tool)->settings_name = NULL; /* XXX hack */
 }
 
 static void
@@ -216,12 +216,12 @@ gimp_operation_tool_initialize (GimpTool     *tool,
 {
   if (GIMP_TOOL_CLASS (parent_class)->initialize (tool, display, error))
     {
-      GimpImageMapTool  *im_tool  = GIMP_IMAGE_MAP_TOOL (tool);
-      GimpOperationTool *op_tool  = GIMP_OPERATION_TOOL (tool);
-      GimpImage         *image    = gimp_display_get_image (display);
-      GimpDrawable      *drawable = gimp_image_get_active_drawable (image);
+      GimpFilterTool    *filter_tool = GIMP_FILTER_TOOL (tool);
+      GimpOperationTool *op_tool     = GIMP_OPERATION_TOOL (tool);
+      GimpImage         *image       = gimp_display_get_image (display);
+      GimpDrawable      *drawable    = gimp_image_get_active_drawable (image);
 
-      if (im_tool->config)
+      if (filter_tool->config)
         gimp_operation_tool_sync_op (op_tool, drawable);
 
       return TRUE;
@@ -256,14 +256,14 @@ gimp_operation_tool_control (GimpTool       *tool,
 }
 
 static gchar *
-gimp_operation_tool_get_operation (GimpImageMapTool  *im_tool,
-                                   gchar            **title,
-                                   gchar            **description,
-                                   gchar            **undo_desc,
-                                   gchar            **icon_name,
-                                   gchar            **help_id)
+gimp_operation_tool_get_operation (GimpFilterTool  *filter_tool,
+                                   gchar          **title,
+                                   gchar          **description,
+                                   gchar          **undo_desc,
+                                   gchar          **icon_name,
+                                   gchar          **help_id)
 {
-  GimpOperationTool *tool = GIMP_OPERATION_TOOL (im_tool);
+  GimpOperationTool *tool = GIMP_OPERATION_TOOL (filter_tool);
 
   *title       = g_strdup (tool->title);
   *description = g_strdup (tool->description);
@@ -275,13 +275,13 @@ gimp_operation_tool_get_operation (GimpImageMapTool  *im_tool,
 }
 
 static void
-gimp_operation_tool_dialog (GimpImageMapTool *im_tool)
+gimp_operation_tool_dialog (GimpFilterTool *filter_tool)
 {
-  GimpOperationTool *tool = GIMP_OPERATION_TOOL (im_tool);
+  GimpOperationTool *tool = GIMP_OPERATION_TOOL (filter_tool);
   GtkWidget         *main_vbox;
   GList             *list;
 
-  main_vbox = gimp_image_map_tool_dialog_get_vbox (im_tool);
+  main_vbox = gimp_filter_tool_dialog_get_vbox (filter_tool);
 
   /*  The options vbox  */
   tool->options_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
@@ -301,40 +301,40 @@ gimp_operation_tool_dialog (GimpImageMapTool *im_tool)
   if (tool->options_gui)
     {
       gtk_box_pack_start (GTK_BOX (tool->options_box), tool->options_gui,
-                          FALSE, FALSE, 0);
+                          TRUE, TRUE, 0);
       gtk_widget_show (tool->options_gui);
     }
 }
 
 static void
-gimp_operation_tool_reset (GimpImageMapTool *im_tool)
+gimp_operation_tool_reset (GimpFilterTool *filter_tool)
 {
-  GimpOperationTool *tool = GIMP_OPERATION_TOOL (im_tool);
+  GimpOperationTool *tool = GIMP_OPERATION_TOOL (filter_tool);
 
-  GIMP_IMAGE_MAP_TOOL_CLASS (parent_class)->reset (im_tool);
+  GIMP_FILTER_TOOL_CLASS (parent_class)->reset (filter_tool);
 
-  if (im_tool->config && GIMP_TOOL (tool)->drawable)
+  if (filter_tool->config && GIMP_TOOL (tool)->drawable)
     gimp_operation_tool_sync_op (tool, GIMP_TOOL (tool)->drawable);
 }
 
 static GtkWidget *
-gimp_operation_tool_get_settings_ui (GimpImageMapTool  *im_tool,
-                                     GimpContainer     *settings,
-                                     GFile             *settings_file,
-                                     const gchar       *import_dialog_title,
-                                     const gchar       *export_dialog_title,
-                                     const gchar       *file_dialog_help_id,
-                                     GFile             *default_folder,
-                                     GtkWidget        **settings_box)
+gimp_operation_tool_get_settings_ui (GimpFilterTool  *filter_tool,
+                                     GimpContainer   *settings,
+                                     GFile           *settings_file,
+                                     const gchar     *import_dialog_title,
+                                     const gchar     *export_dialog_title,
+                                     const gchar     *file_dialog_help_id,
+                                     GFile           *default_folder,
+                                     GtkWidget      **settings_box)
 {
-  GimpOperationTool *tool = GIMP_OPERATION_TOOL (im_tool);
+  GimpOperationTool *tool = GIMP_OPERATION_TOOL (filter_tool);
   GtkWidget         *widget;
   gchar             *basename;
   GFile             *file;
   gchar             *import_title;
   gchar             *export_title;
 
-  basename = g_strconcat (G_OBJECT_TYPE_NAME (im_tool->config),
+  basename = g_strconcat (G_OBJECT_TYPE_NAME (filter_tool->config),
                           ".settings", NULL);
   file = gimp_directory_file ("filters", basename, NULL);
   g_free (basename);
@@ -343,14 +343,14 @@ gimp_operation_tool_get_settings_ui (GimpImageMapTool  *im_tool,
   export_title = g_strdup_printf (_("Export '%s' Settings"), tool->title);
 
   widget =
-    GIMP_IMAGE_MAP_TOOL_CLASS (parent_class)->get_settings_ui (im_tool,
-                                                               settings,
-                                                               file,
-                                                               import_title,
-                                                               export_title,
-                                                               "help-foo",
-                                                               NULL, /* sic */
-                                                               settings_box);
+    GIMP_FILTER_TOOL_CLASS (parent_class)->get_settings_ui (filter_tool,
+                                                            settings,
+                                                            file,
+                                                            import_title,
+                                                            export_title,
+                                                            file_dialog_help_id,
+                                                            NULL, /* sic */
+                                                            settings_box);
 
   g_free (import_title);
   g_free (export_title);
@@ -361,27 +361,27 @@ gimp_operation_tool_get_settings_ui (GimpImageMapTool  *im_tool,
 }
 
 static void
-gimp_operation_tool_color_picked (GimpImageMapTool  *im_tool,
-                                  gpointer           identifier,
-                                  gdouble            x,
-                                  gdouble            y,
-                                  const Babl        *sample_format,
-                                  const GimpRGB     *color)
+gimp_operation_tool_color_picked (GimpFilterTool  *filter_tool,
+                                  gpointer         identifier,
+                                  gdouble          x,
+                                  gdouble          y,
+                                  const Babl      *sample_format,
+                                  const GimpRGB   *color)
 {
-  GimpOperationTool  *tool = GIMP_OPERATION_TOOL (im_tool);
+  GimpOperationTool  *tool = GIMP_OPERATION_TOOL (filter_tool);
   gchar             **pspecs;
 
   pspecs = g_strsplit (identifier, ":", 2);
 
   if (pspecs[1])
     {
-      GimpImageMapOptions *options      = GIMP_IMAGE_MAP_TOOL_GET_OPTIONS (tool);
-      GimpDrawable        *drawable     = GIMP_TOOL (im_tool)->drawable;
-      GObjectClass        *object_class = G_OBJECT_GET_CLASS (im_tool->config);
-      GParamSpec          *pspec_x;
-      GParamSpec          *pspec_y;
-      gint                 width        = 1;
-      gint                 height       = 1;
+      GimpFilterOptions *options      = GIMP_FILTER_TOOL_GET_OPTIONS (tool);
+      GimpDrawable      *drawable     = GIMP_TOOL (filter_tool)->drawable;
+      GObjectClass      *object_class = G_OBJECT_GET_CLASS (filter_tool->config);
+      GParamSpec        *pspec_x;
+      GParamSpec        *pspec_y;
+      gint               width        = 1;
+      gint               height       = 1;
 
       if (drawable)
         {
@@ -394,7 +394,7 @@ gimp_operation_tool_color_picked (GimpImageMapTool  *im_tool,
 
           switch (options->region)
             {
-            case GIMP_IMAGE_MAP_REGION_SELECTION:
+            case GIMP_FILTER_REGION_SELECTION:
               if (gimp_item_mask_intersect (GIMP_ITEM (drawable),
                                             &off_x, &off_y, &width, &height))
                 {
@@ -403,7 +403,7 @@ gimp_operation_tool_color_picked (GimpImageMapTool  *im_tool,
                 }
               break;
 
-            case GIMP_IMAGE_MAP_REGION_DRAWABLE:
+            case GIMP_FILTER_REGION_DRAWABLE:
               width  = gimp_item_get_width  (GIMP_ITEM (drawable));
               height = gimp_item_get_height (GIMP_ITEM (drawable));
               break;
@@ -439,7 +439,7 @@ gimp_operation_tool_color_picked (GimpImageMapTool  *im_tool,
               g_param_value_validate (pspec_x, &value_x);
               g_param_value_validate (pspec_y, &value_y);
 
-              g_object_set (im_tool->config,
+              g_object_set (filter_tool->config,
                             pspecs[0], g_value_get_int (&value_x),
                             pspecs[1], g_value_get_int (&value_y),
                             NULL);
@@ -452,7 +452,7 @@ gimp_operation_tool_color_picked (GimpImageMapTool  *im_tool,
               g_param_value_validate (pspec_x, &value_x);
               g_param_value_validate (pspec_y, &value_y);
 
-              g_object_set (im_tool->config,
+              g_object_set (filter_tool->config,
                             pspecs[0], g_value_get_double (&value_x),
                             pspecs[1], g_value_get_double (&value_y),
                             NULL);
@@ -469,7 +469,7 @@ gimp_operation_tool_color_picked (GimpImageMapTool  *im_tool,
     }
   else
     {
-      g_object_set (im_tool->config,
+      g_object_set (filter_tool->config,
                     pspecs[0], color,
                     NULL);
     }
@@ -481,8 +481,8 @@ static void
 gimp_operation_tool_sync_op (GimpOperationTool *op_tool,
                              GimpDrawable      *drawable)
 {
-  GimpImageMapTool *im_tool = GIMP_IMAGE_MAP_TOOL (op_tool);
-  GimpToolOptions  *options = GIMP_TOOL_GET_OPTIONS (op_tool);
+  GimpFilterTool   *filter_tool = GIMP_FILTER_TOOL (op_tool);
+  GimpToolOptions  *options     = GIMP_TOOL_GET_OPTIONS (op_tool);
   GParamSpec      **pspecs;
   guint             n_pspecs;
   gint              bounds_x;
@@ -495,7 +495,7 @@ gimp_operation_tool_sync_op (GimpOperationTool *op_tool,
                             &bounds_x, &bounds_y,
                             &bounds_width, &bounds_height);
 
-  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (im_tool->config),
+  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (filter_tool->config),
                                            &n_pspecs);
 
   for (i = 0; i < n_pspecs; i++)
@@ -509,22 +509,22 @@ gimp_operation_tool_sync_op (GimpOperationTool *op_tool,
           if (HAS_KEY (pspec, "unit", "pixel-coordinate") &&
               HAS_KEY (pspec, "axis", "x"))
             {
-              g_object_set (im_tool->config, pspec->name, 0, NULL);
+              g_object_set (filter_tool->config, pspec->name, 0, NULL);
             }
           else if (HAS_KEY (pspec, "unit", "pixel-coordinate") &&
                    HAS_KEY (pspec, "axis", "y"))
             {
-              g_object_set (im_tool->config, pspec->name, 0, NULL);
+              g_object_set (filter_tool->config, pspec->name, 0, NULL);
             }
           else if (HAS_KEY (pspec, "unit", "pixel-distance") &&
                    HAS_KEY (pspec, "axis", "x"))
             {
-              g_object_set (im_tool->config, pspec->name, bounds_width, NULL);
+              g_object_set (filter_tool->config, pspec->name, bounds_width, NULL);
             }
           else if (HAS_KEY (pspec, "unit", "pixel-distance") &&
                    HAS_KEY (pspec, "axis", "y"))
             {
-              g_object_set (im_tool->config, pspec->name, bounds_height, NULL);
+              g_object_set (filter_tool->config, pspec->name, bounds_height, NULL);
             }
         }
       else if (HAS_KEY (pspec, "role", "color-primary"))
@@ -532,14 +532,14 @@ gimp_operation_tool_sync_op (GimpOperationTool *op_tool,
           GimpRGB color;
 
           gimp_context_get_foreground (GIMP_CONTEXT (options), &color);
-          g_object_set (im_tool->config, pspec->name, &color, NULL);
+          g_object_set (filter_tool->config, pspec->name, &color, NULL);
         }
       else if (HAS_KEY (pspec, "role", "color-secondary"))
         {
           GimpRGB color;
 
           gimp_context_get_background (GIMP_CONTEXT (options), &color);
-          g_object_set (im_tool->config, pspec->name, &color, NULL);
+          g_object_set (filter_tool->config, pspec->name, &color, NULL);
         }
     }
 
@@ -552,9 +552,15 @@ gimp_operation_tool_sync_op (GimpOperationTool *op_tool,
 static void
 gimp_operation_tool_aux_input_notify (GimpBufferSourceBox *box,
                                       const GParamSpec    *pspec,
-                                      GimpOperationTool   *tool)
+                                      AuxInput            *input)
 {
-  gimp_image_map_tool_preview (GIMP_IMAGE_MAP_TOOL (tool));
+  /* emit "notify" so GimpFilterTool will update its preview
+   *
+   * FIXME: this is a bad hack that should go away once GimpImageMap
+   * and GimpFilterTool are refactored to be more filter-ish.
+   */
+  g_signal_emit_by_name (GIMP_FILTER_TOOL (input->tool)->config,
+                         "notify", NULL);
 }
 
 static AuxInput *
@@ -566,6 +572,8 @@ gimp_operation_tool_aux_input_new (GimpOperationTool *tool,
   AuxInput    *input = g_slice_new (AuxInput);
   GimpContext *context;
 
+  input->tool = tool;
+  input->pad  = g_strdup (input_pad);
   input->node = gegl_node_new_child (NULL,
                                      "operation", "gegl:buffer-source",
                                      NULL);
@@ -579,10 +587,10 @@ gimp_operation_tool_aux_input_new (GimpOperationTool *tool,
 
   g_signal_connect (input->box, "notify::pickable",
                     G_CALLBACK (gimp_operation_tool_aux_input_notify),
-                    tool);
+                    input);
   g_signal_connect (input->box, "notify::enabled",
                     G_CALLBACK (gimp_operation_tool_aux_input_notify),
-                    tool);
+                    input);
 
   return input;
 }
@@ -598,6 +606,7 @@ gimp_operation_tool_aux_input_clear (AuxInput *input)
 static void
 gimp_operation_tool_aux_input_free (AuxInput *input)
 {
+  g_free (input->pad);
   g_object_unref (input->node);
   gtk_widget_destroy (input->box);
 
@@ -616,14 +625,14 @@ gimp_operation_tool_set_operation (GimpOperationTool *tool,
                                    const gchar       *icon_name,
                                    const gchar       *help_id)
 {
-  GimpImageMapTool *im_tool;
-  GtkSizeGroup     *size_group = NULL;
-  gint              aux;
+  GimpFilterTool *filter_tool;
+  GtkSizeGroup   *size_group = NULL;
+  gint            aux;
 
   g_return_if_fail (GIMP_IS_OPERATION_TOOL (tool));
   g_return_if_fail (operation != NULL);
 
-  im_tool = GIMP_IMAGE_MAP_TOOL (tool);
+  filter_tool = GIMP_FILTER_TOOL (tool);
 
   if (tool->operation)
     g_free (tool->operation);
@@ -654,21 +663,21 @@ gimp_operation_tool_set_operation (GimpOperationTool *tool,
                     (GDestroyNotify) gimp_operation_tool_aux_input_free);
   tool->aux_inputs = NULL;
 
-  gimp_image_map_tool_get_operation (im_tool);
+  gimp_filter_tool_get_operation (filter_tool);
 
   if (undo_desc)
-    GIMP_IMAGE_MAP_TOOL_GET_CLASS (tool)->settings_name = "yes"; /* XXX hack */
+    GIMP_FILTER_TOOL_GET_CLASS (tool)->settings_name = "yes"; /* XXX hack */
   else
-    GIMP_IMAGE_MAP_TOOL_GET_CLASS (tool)->settings_name = NULL; /* XXX hack */
+    GIMP_FILTER_TOOL_GET_CLASS (tool)->settings_name = NULL; /* XXX hack */
 
   if (tool->options_gui)
     {
       gtk_widget_destroy (tool->options_gui);
       tool->options_gui = NULL;
 
-      if (im_tool->active_picker)
+      if (filter_tool->active_picker)
         {
-          im_tool->active_picker = NULL;
+          filter_tool->active_picker = NULL;
           gimp_color_tool_disable (GIMP_COLOR_TOOL (tool));
         }
     }
@@ -681,15 +690,17 @@ gimp_operation_tool_set_operation (GimpOperationTool *tool,
       if (aux == 1)
         {
           g_snprintf (pad,   sizeof (pad),   "aux");
+          /* don't translate "Aux" */
           g_snprintf (label, sizeof (label), _("Aux Input"));
         }
       else
         {
           g_snprintf (pad,   sizeof (pad),   "aux%d", aux);
+          /* don't translate "Aux" */
           g_snprintf (label, sizeof (label), _("Aux%d Input"), aux);
         }
 
-      if (gegl_node_has_pad (im_tool->operation, pad))
+      if (gegl_node_has_pad (filter_tool->operation, pad))
         {
           AuxInput  *input;
           GtkWidget *toggle;
@@ -698,7 +709,7 @@ gimp_operation_tool_set_operation (GimpOperationTool *tool,
             size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
           input = gimp_operation_tool_aux_input_new (tool,
-                                                     im_tool->operation, pad,
+                                                     filter_tool->operation, pad,
                                                      label);
 
           tool->aux_inputs = g_list_append (tool->aux_inputs, input);
@@ -722,13 +733,27 @@ gimp_operation_tool_set_operation (GimpOperationTool *tool,
   if (size_group)
     g_object_unref (size_group);
 
-  if (im_tool->config)
+  if (filter_tool->config)
     {
+      GeglRectangle *area = NULL;
+      GeglRectangle  tmp  = { 0, };
+
+      if (GIMP_TOOL (tool)->drawable)
+        {
+          GimpDrawable *drawable = GIMP_TOOL (tool)->drawable;
+
+          tmp.width  = gimp_item_get_width  (GIMP_ITEM (drawable));
+          tmp.height = gimp_item_get_height (GIMP_ITEM (drawable));
+
+          area = &tmp;
+        }
+
       tool->options_gui =
-        gimp_prop_gui_new (G_OBJECT (im_tool->config),
-                           G_TYPE_FROM_INSTANCE (im_tool->config), 0,
+        gimp_prop_gui_new (G_OBJECT (filter_tool->config),
+                           G_TYPE_FROM_INSTANCE (filter_tool->config), 0,
+                           area,
                            GIMP_CONTEXT (GIMP_TOOL_GET_OPTIONS (tool)),
-                           (GimpCreatePickerFunc) gimp_image_map_tool_add_color_picker,
+                           (GimpCreatePickerFunc) gimp_filter_tool_add_color_picker,
                            tool);
 
       if (tool->options_box)
@@ -740,8 +765,5 @@ gimp_operation_tool_set_operation (GimpOperationTool *tool,
     }
 
   if (GIMP_TOOL (tool)->drawable)
-    {
-      gimp_operation_tool_sync_op (tool, GIMP_TOOL (tool)->drawable);
-      gimp_image_map_tool_preview (im_tool);
-    }
+    gimp_operation_tool_sync_op (tool, GIMP_TOOL (tool)->drawable);
 }
