@@ -37,7 +37,7 @@
 #include "core/gimpimage-pick-layer.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
-#include "core/gimplayer-floating-sel.h"
+#include "core/gimplayer-floating-selection.h"
 #include "core/gimpmarshal.h"
 #include "core/gimptoolinfo.h"
 #include "core/gimpundostack.h"
@@ -51,6 +51,7 @@
 #include "vectors/gimpvectors-warp.h"
 
 #include "widgets/gimpdialogfactory.h"
+#include "widgets/gimpdockcontainer.h"
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimptextbuffer.h"
@@ -146,8 +147,8 @@ static void      gimp_text_tool_text_notify     (GimpText          *text,
 static void      gimp_text_tool_text_changed    (GimpText          *text,
                                                  GimpTextTool      *text_tool);
 
-static gboolean  gimp_text_tool_apply           (GimpTextTool      *text_tool,
-                                                 gboolean           push_undo);
+static void      gimp_text_tool_apply_list      (GimpTextTool      *text_tool,
+                                                 GList             *pspecs);
 
 static void      gimp_text_tool_create_layer    (GimpTextTool      *text_tool,
                                                  GimpText          *text);
@@ -192,7 +193,7 @@ gimp_text_tool_register (GimpToolRegisterCallback  callback,
                 _("Text Tool: Create or edit text layers"),
                 N_("Te_xt"), "T",
                 NULL, GIMP_HELP_TOOL_TEXT,
-                GIMP_STOCK_TOOL_TEXT,
+                GIMP_ICON_TOOL_TEXT,
                 data);
 }
 
@@ -317,6 +318,37 @@ gimp_text_tool_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gimp_text_tool_remove_empty_text_layer (GimpTextTool *text_tool)
+{
+  GimpTextLayer *text_layer = text_tool->layer;
+
+  if (text_layer && text_layer->auto_rename)
+    {
+      GimpText *text = gimp_text_layer_get_text (text_layer);
+
+      if (text && text->box_mode == GIMP_TEXT_BOX_DYNAMIC &&
+          (! text->text || text->text[0] == '\0') &&
+          (! text->markup || text->markup[0] == '\0'))
+        {
+          GimpImage *image = gimp_item_get_image (GIMP_ITEM (text_layer));
+
+          if (text_tool->image == image)
+            g_signal_handlers_block_by_func (image,
+                                             gimp_text_tool_layer_changed,
+                                             text_tool);
+
+          gimp_image_remove_layer (image, GIMP_LAYER (text_layer), TRUE, NULL);
+          gimp_image_flush (image);
+
+          if (text_tool->image == image)
+            g_signal_handlers_unblock_by_func (image,
+                                               gimp_text_tool_layer_changed,
+                                               text_tool);
+        }
+    }
+}
+
 static void
 gimp_text_tool_control (GimpTool       *tool,
                         GimpToolAction  action,
@@ -366,7 +398,14 @@ gimp_text_tool_button_press (GimpTool            *tool,
     {
       gimp_tool_control_activate (tool->control);
 
-      gimp_text_tool_reset_im_context (text_tool);
+      /* clicking anywhere while a preedit is going on aborts the
+       * preedit, this is ugly but at least leaves everything in
+       * a consistent state
+       */
+      if (text_tool->preedit_active)
+        gimp_text_tool_abort_im_context (text_tool);
+      else
+        gimp_text_tool_reset_im_context (text_tool);
 
       text_tool->selecting = FALSE;
 
@@ -730,11 +769,14 @@ gimp_text_tool_get_popup (GimpTool         *tool,
     {
       if (! text_tool->ui_manager)
         {
+          GimpDisplayShell  *shell = gimp_display_get_shell (tool->display);
+          GimpImageWindow   *image_window;
           GimpDialogFactory *dialog_factory;
           GtkWidget         *im_menu;
           GList             *children;
 
-          dialog_factory = gimp_dialog_factory_get_singleton ();
+          image_window   = gimp_display_shell_get_window (shell);
+          dialog_factory = gimp_dock_container_get_dialog_factory (GIMP_DOCK_CONTAINER (image_window));
 
           text_tool->ui_manager =
             gimp_menu_factory_manager_new (gimp_dialog_factory_get_menu_factory (dialog_factory),
@@ -784,7 +826,7 @@ gimp_text_tool_draw (GimpDrawTool *draw_tool)
       ! text_tool->layer ||
       ! text_tool->layer->text)
     {
-      gimp_text_tool_editor_update_im_rect (text_tool);
+      gimp_text_tool_editor_update_im_cursor (text_tool);
 
       return;
     }
@@ -821,7 +863,7 @@ gimp_text_tool_draw (GimpDrawTool *draw_tool)
       gimp_canvas_item_set_highlight (item, TRUE);
     }
 
-  gimp_text_tool_editor_update_im_rect (text_tool);
+  gimp_text_tool_editor_update_im_cursor (text_tool);
 }
 
 static void
@@ -1085,6 +1127,8 @@ gimp_text_tool_connect (GimpTextTool  *text_tool,
                                               gimp_text_tool_layer_notify,
                                               text_tool);
 
+      gimp_text_tool_remove_empty_text_layer (text_tool);
+
       text_tool->layer = layer;
 
       if (layer)
@@ -1130,17 +1174,59 @@ gimp_text_tool_proxy_notify (GimpText         *text,
   if ((pspec->flags & G_PARAM_READWRITE) == G_PARAM_READWRITE &&
       pspec->owner_type == GIMP_TYPE_TEXT)
     {
-      gimp_text_tool_block_drawing (text_tool);
+      if (text_tool->preedit_active)
+        {
+          /* if there is a preedit going on, don't queue pending
+           * changes to be idle-applied with undo; instead, flush the
+           * pending queue (happens only when preedit starts), and
+           * apply the changes to text_tool->text directly. Preedit
+           * will *always* end by removing the preedit string, and if
+           * the preedit was committed, it will insert the resulting
+           * text, which will not trigger this if() any more.
+           */
 
-      text_tool->pending = g_list_append (text_tool->pending, (gpointer) pspec);
+          GList *list = NULL;
 
-      if (text_tool->idle_id)
-        g_source_remove (text_tool->idle_id);
+         /* if there are pending changes, apply them before applying
+           * preedit stuff directly (bypassing undo)
+           */
+          if (text_tool->pending)
+            {
+              gimp_text_tool_block_drawing (text_tool);
+              gimp_text_tool_apply (text_tool, TRUE);
+            }
 
-      text_tool->idle_id =
-        g_idle_add_full (G_PRIORITY_LOW,
-                         gimp_text_tool_apply_idle, text_tool,
-                         NULL);
+          gimp_text_tool_block_drawing (text_tool);
+
+          list = g_list_append (list, (gpointer) pspec);
+          gimp_text_tool_apply_list (text_tool, list);
+          g_list_free (list);
+
+          gimp_text_tool_frame_item (text_tool);
+
+          gimp_image_flush (gimp_item_get_image (GIMP_ITEM (text_tool->layer)));
+
+          gimp_text_tool_unblock_drawing (text_tool);
+        }
+      else
+        {
+          /* else queue the property change for normal processing,
+           * including undo
+           */
+
+          gimp_text_tool_block_drawing (text_tool);
+
+          text_tool->pending = g_list_append (text_tool->pending,
+                                              (gpointer) pspec);
+
+          if (text_tool->idle_id)
+            g_source_remove (text_tool->idle_id);
+
+          text_tool->idle_id =
+            g_idle_add_full (G_PRIORITY_LOW,
+                             gimp_text_tool_apply_idle, text_tool,
+                             NULL);
+        }
     }
 }
 
@@ -1151,6 +1237,10 @@ gimp_text_tool_text_notify (GimpText         *text,
 {
   g_return_if_fail (text == text_tool->text);
 
+  /* an undo cancels all preedit operations */
+  if (text_tool->preedit_active)
+    gimp_text_tool_abort_im_context (text_tool);
+
   gimp_text_tool_block_drawing (text_tool);
 
   if ((pspec->flags & G_PARAM_READWRITE) == G_PARAM_READWRITE)
@@ -1212,95 +1302,13 @@ gimp_text_tool_text_changed (GimpText     *text,
   gimp_text_tool_unblock_drawing (text_tool);
 }
 
-static gboolean
-gimp_text_tool_apply (GimpTextTool *text_tool,
-                      gboolean      push_undo)
+static void
+gimp_text_tool_apply_list (GimpTextTool *text_tool,
+                           GList        *pspecs)
 {
-  const GParamSpec *pspec = NULL;
-  GimpImage        *image;
-  GimpTextLayer    *layer;
-  GObject          *src;
-  GObject          *dest;
-  GList            *list;
-  gboolean          undo_group = FALSE;
-
-  if (text_tool->idle_id)
-    {
-      g_source_remove (text_tool->idle_id);
-      text_tool->idle_id = 0;
-    }
-
-  g_return_val_if_fail (text_tool->text != NULL, FALSE);
-  g_return_val_if_fail (text_tool->layer != NULL, FALSE);
-
-  layer = text_tool->layer;
-  image = gimp_item_get_image (GIMP_ITEM (layer));
-
-  g_return_val_if_fail (layer->text == text_tool->text, FALSE);
-
-  /*  Walk over the list of changes and figure out if we are changing
-   *  a single property or need to push a full text undo.
-   */
-  for (list = text_tool->pending;
-       list && list->next && list->next->data == list->data;
-       list = list->next)
-    /* do nothing */;
-
-  if (g_list_length (list) == 1)
-    pspec = list->data;
-
-  /*  If we are changing a single property, we don't need to push
-   *  an undo if all of the following is true:
-   *   - the redo stack is empty
-   *   - the last item on the undo stack is a text undo
-   *   - the last undo changed the same text property on the same layer
-   *   - the last undo happened less than TEXT_UNDO_TIMEOUT seconds ago
-   */
-  if (pspec)
-    {
-      GimpUndo *undo = gimp_image_undo_can_compress (image, GIMP_TYPE_TEXT_UNDO,
-                                                     GIMP_UNDO_TEXT_LAYER);
-
-      if (undo && GIMP_ITEM_UNDO (undo)->item == GIMP_ITEM (layer))
-        {
-          GimpTextUndo *text_undo = GIMP_TEXT_UNDO (undo);
-
-          if (text_undo->pspec == pspec)
-            {
-              if (gimp_undo_get_age (undo) < TEXT_UNDO_TIMEOUT)
-                {
-                  GimpTool    *tool = GIMP_TOOL (text_tool);
-                  GimpContext *context;
-
-                  context = GIMP_CONTEXT (gimp_tool_get_options (tool));
-
-                  push_undo = FALSE;
-                  gimp_undo_reset_age (undo);
-                  gimp_undo_refresh_preview (undo, context);
-                }
-            }
-        }
-    }
-
-  if (push_undo)
-    {
-      if (layer->modified)
-        {
-          undo_group = TRUE;
-          gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_TEXT, NULL);
-
-          gimp_image_undo_push_text_layer_modified (image, NULL, layer);
-
-          /*  see comment in gimp_text_layer_set()  */
-          gimp_image_undo_push_drawable_mod (image, NULL,
-                                             GIMP_DRAWABLE (layer), TRUE);
-        }
-
-      gimp_image_undo_push_text_layer (image, NULL, layer, pspec);
-    }
-
-  src  = G_OBJECT (text_tool->proxy);
-  dest = G_OBJECT (text_tool->text);
+  GObject *src  = G_OBJECT (text_tool->proxy);
+  GObject *dest = G_OBJECT (text_tool->text);
+  GList   *list;
 
   g_signal_handlers_block_by_func (dest,
                                    gimp_text_tool_text_notify,
@@ -1311,9 +1319,10 @@ gimp_text_tool_apply (GimpTextTool *text_tool,
 
   g_object_freeze_notify (dest);
 
-  for (; list; list = g_list_next (list))
+  for (list = pspecs; list; list = g_list_next (list))
     {
-      GValue value = G_VALUE_INIT;
+      const GParamSpec *pspec;
+      GValue            value = G_VALUE_INIT;
 
       /*  look ahead and compress changes  */
       if (list->next && list->next->data == list->data)
@@ -1329,9 +1338,6 @@ gimp_text_tool_apply (GimpTextTool *text_tool,
       g_value_unset (&value);
     }
 
-  g_list_free (text_tool->pending);
-  text_tool->pending = NULL;
-
   g_object_thaw_notify (dest);
 
   g_signal_handlers_unblock_by_func (dest,
@@ -1340,22 +1346,6 @@ gimp_text_tool_apply (GimpTextTool *text_tool,
   g_signal_handlers_unblock_by_func (dest,
                                      gimp_text_tool_text_changed,
                                      text_tool);
-
-  if (push_undo)
-    {
-      g_object_set (layer, "modified", FALSE, NULL);
-
-      if (undo_group)
-        gimp_image_undo_group_end (image);
-    }
-
-  gimp_text_tool_frame_item (text_tool);
-
-  gimp_image_flush (image);
-
-  gimp_text_tool_unblock_drawing (text_tool);
-
-  return FALSE;
 }
 
 static void
@@ -1529,14 +1519,14 @@ gimp_text_tool_confirm_dialog (GimpTextTool *text_tool)
                                      GIMP_CONTEXT (gimp_tool_get_options (tool)),
                                      _("Confirm Text Editing"),
                                      "gimp-text-tool-confirm",
-                                     GIMP_STOCK_TEXT_LAYER,
+                                     GIMP_ICON_LAYER_TEXT_LAYER,
                                      _("Confirm Text Editing"),
                                      GTK_WIDGET (shell),
                                      gimp_standard_help_func, NULL,
 
                                      _("Create _New Layer"), RESPONSE_NEW,
-                                     GTK_STOCK_CANCEL,       GTK_RESPONSE_CANCEL,
-                                     GTK_STOCK_EDIT,         GTK_RESPONSE_ACCEPT,
+                                     _("_Cancel"),           GTK_RESPONSE_CANCEL,
+                                     _("_Edit"),             GTK_RESPONSE_ACCEPT,
 
                                      NULL);
 
@@ -1565,7 +1555,7 @@ gimp_text_tool_confirm_dialog (GimpTextTool *text_tool)
                            "\n\n"
                            "You can edit the layer or create a new "
                            "text layer from its text attributes."));
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
@@ -1798,6 +1788,113 @@ gimp_text_tool_ensure_layout (GimpTextTool *text_tool)
   return text_tool->layout != NULL;
 }
 
+gboolean
+gimp_text_tool_apply (GimpTextTool *text_tool,
+                      gboolean      push_undo)
+{
+  const GParamSpec *pspec = NULL;
+  GimpImage        *image;
+  GimpTextLayer    *layer;
+  GList            *list;
+  gboolean          undo_group = FALSE;
+
+  if (text_tool->idle_id)
+    {
+      g_source_remove (text_tool->idle_id);
+      text_tool->idle_id = 0;
+    }
+
+  g_return_val_if_fail (text_tool->text != NULL, FALSE);
+  g_return_val_if_fail (text_tool->layer != NULL, FALSE);
+
+  layer = text_tool->layer;
+  image = gimp_item_get_image (GIMP_ITEM (layer));
+
+  g_return_val_if_fail (layer->text == text_tool->text, FALSE);
+
+  /*  Walk over the list of changes and figure out if we are changing
+   *  a single property or need to push a full text undo.
+   */
+  for (list = text_tool->pending;
+       list && list->next && list->next->data == list->data;
+       list = list->next)
+    /* do nothing */;
+
+  if (g_list_length (list) == 1)
+    pspec = list->data;
+
+  /*  If we are changing a single property, we don't need to push
+   *  an undo if all of the following is true:
+   *   - the redo stack is empty
+   *   - the last item on the undo stack is a text undo
+   *   - the last undo changed the same text property on the same layer
+   *   - the last undo happened less than TEXT_UNDO_TIMEOUT seconds ago
+   */
+  if (pspec)
+    {
+      GimpUndo *undo = gimp_image_undo_can_compress (image, GIMP_TYPE_TEXT_UNDO,
+                                                     GIMP_UNDO_TEXT_LAYER);
+
+      if (undo && GIMP_ITEM_UNDO (undo)->item == GIMP_ITEM (layer))
+        {
+          GimpTextUndo *text_undo = GIMP_TEXT_UNDO (undo);
+
+          if (text_undo->pspec == pspec)
+            {
+              if (gimp_undo_get_age (undo) < TEXT_UNDO_TIMEOUT)
+                {
+                  GimpTool    *tool = GIMP_TOOL (text_tool);
+                  GimpContext *context;
+
+                  context = GIMP_CONTEXT (gimp_tool_get_options (tool));
+
+                  push_undo = FALSE;
+                  gimp_undo_reset_age (undo);
+                  gimp_undo_refresh_preview (undo, context);
+                }
+            }
+        }
+    }
+
+  if (push_undo)
+    {
+      if (layer->modified)
+        {
+          undo_group = TRUE;
+          gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_TEXT, NULL);
+
+          gimp_image_undo_push_text_layer_modified (image, NULL, layer);
+
+          /*  see comment in gimp_text_layer_set()  */
+          gimp_image_undo_push_drawable_mod (image, NULL,
+                                             GIMP_DRAWABLE (layer), TRUE);
+        }
+
+      gimp_image_undo_push_text_layer (image, NULL, layer, pspec);
+    }
+
+  gimp_text_tool_apply_list (text_tool, list);
+
+  g_list_free (text_tool->pending);
+  text_tool->pending = NULL;
+
+  if (push_undo)
+    {
+      g_object_set (layer, "modified", FALSE, NULL);
+
+      if (undo_group)
+        gimp_image_undo_group_end (image);
+    }
+
+  gimp_text_tool_frame_item (text_tool);
+
+  gimp_image_flush (image);
+
+  gimp_text_tool_unblock_drawing (text_tool);
+
+  return FALSE;
+}
+
 void
 gimp_text_tool_set_layer (GimpTextTool *text_tool,
                           GimpLayer    *layer)
