@@ -22,6 +22,7 @@
 
 #include "libgimpmath/gimpmath.h"
 #include "libgimpcolor/gimpcolor.h"
+#include "libgimpconfig/gimpconfig.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
@@ -47,23 +48,27 @@ enum
 
 /*  local function prototypes  */
 
-static void       gimp_color_frame_finalize      (GObject        *object);
-static void       gimp_color_frame_get_property  (GObject        *object,
-                                                  guint           property_id,
-                                                  GValue         *value,
-                                                  GParamSpec     *pspec);
-static void       gimp_color_frame_set_property  (GObject        *object,
-                                                  guint           property_id,
-                                                  const GValue   *value,
-                                                  GParamSpec     *pspec);
+static void       gimp_color_frame_dispose           (GObject        *object);
+static void       gimp_color_frame_finalize          (GObject        *object);
+static void       gimp_color_frame_get_property      (GObject        *object,
+                                                      guint           property_id,
+                                                      GValue         *value,
+                                                      GParamSpec     *pspec);
+static void       gimp_color_frame_set_property      (GObject        *object,
+                                                      guint           property_id,
+                                                      const GValue   *value,
+                                                      GParamSpec     *pspec);
 
-static void       gimp_color_frame_style_updated (GtkWidget      *widget);
-static gboolean   gimp_color_frame_draw          (GtkWidget      *widget,
-                                                  cairo_t        *cr);
+static void       gimp_color_frame_style_updated     (GtkWidget      *widget);
+static gboolean   gimp_color_frame_draw              (GtkWidget      *widget,
+                                                      cairo_t        *cr);
 
-static void       gimp_color_frame_menu_callback (GtkWidget      *widget,
-                                                  GimpColorFrame *frame);
-static void       gimp_color_frame_update        (GimpColorFrame *frame);
+static void       gimp_color_frame_menu_callback     (GtkWidget      *widget,
+                                                      GimpColorFrame *frame);
+static void       gimp_color_frame_update            (GimpColorFrame *frame);
+
+static void       gimp_color_frame_create_transform  (GimpColorFrame *frame);
+static void       gimp_color_frame_destroy_transform (GimpColorFrame *frame);
 
 
 G_DEFINE_TYPE (GimpColorFrame, gimp_color_frame, GIMP_TYPE_FRAME)
@@ -77,6 +82,7 @@ gimp_color_frame_class_init (GimpColorFrameClass *klass)
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  object_class->dispose       = gimp_color_frame_dispose;
   object_class->finalize      = gimp_color_frame_finalize;
   object_class->get_property  = gimp_color_frame_get_property;
   object_class->set_property  = gimp_color_frame_set_property;
@@ -164,20 +170,30 @@ gimp_color_frame_init (GimpColorFrame *frame)
       gtk_widget_show (hbox);
 
       frame->name_labels[i] = gtk_label_new (" ");
-      gtk_misc_set_alignment (GTK_MISC (frame->name_labels[i]), 0.0, 0.5);
+      gtk_label_set_xalign (GTK_LABEL (frame->name_labels[i]), 0.0);
       gtk_box_pack_start (GTK_BOX (hbox), frame->name_labels[i],
                           FALSE, FALSE, 0);
       gtk_widget_show (frame->name_labels[i]);
 
       frame->value_labels[i] = gtk_label_new (" ");
       gtk_label_set_selectable (GTK_LABEL (frame->value_labels[i]), TRUE);
-      gtk_misc_set_alignment (GTK_MISC (frame->value_labels[i]), 1.0, 0.5);
+      gtk_label_set_xalign (GTK_LABEL (frame->value_labels[i]), 1.0);
       gtk_box_pack_end (GTK_BOX (hbox), frame->value_labels[i],
                         FALSE, FALSE, 0);
       gtk_widget_show (frame->value_labels[i]);
     }
 }
 
+static void
+gimp_color_frame_dispose (GObject *object)
+{
+  GimpColorFrame *frame = GIMP_COLOR_FRAME (object);
+
+  gimp_color_frame_set_color_config (frame, NULL);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 gimp_color_frame_finalize (GObject *object)
 {
@@ -479,6 +495,41 @@ gimp_color_frame_set_invalid (GimpColorFrame *frame)
   gimp_color_frame_update (frame);
 }
 
+void
+gimp_color_frame_set_color_config (GimpColorFrame  *frame,
+                                   GimpColorConfig *config)
+{
+  g_return_if_fail (GIMP_IS_COLOR_FRAME (frame));
+  g_return_if_fail (config == NULL || GIMP_IS_COLOR_CONFIG (config));
+
+  if (config != frame->config)
+    {
+      if (frame->config)
+        {
+          g_signal_handlers_disconnect_by_func (frame->config,
+                                                gimp_color_frame_destroy_transform,
+                                                frame);
+          g_object_unref (frame->config);
+
+          gimp_color_frame_destroy_transform (frame);
+        }
+
+      frame->config = config;
+
+      if (frame->config)
+        {
+          g_object_ref (frame->config);
+
+          g_signal_connect_swapped (frame->config, "notify",
+                                    G_CALLBACK (gimp_color_frame_destroy_transform),
+                                    frame);
+        }
+
+      gimp_color_area_set_color_config (GIMP_COLOR_AREA (frame->color_area),
+                                        config);
+    }
+}
+
 
 /*  private functions  */
 
@@ -616,7 +667,8 @@ gimp_color_frame_update (GimpColorFrame *frame)
       }
       break;
 
-    case GIMP_COLOR_FRAME_MODE_RGB:
+    case GIMP_COLOR_FRAME_MODE_RGB_PERCENT:
+    case GIMP_COLOR_FRAME_MODE_RGB_U8:
       names[0] = _("Red:");
       names[1] = _("Green:");
       names[2] = _("Blue:");
@@ -624,23 +676,30 @@ gimp_color_frame_update (GimpColorFrame *frame)
       if (has_alpha)
         names[3] = _("Alpha:");
 
-      if (frame->sample_valid)
-        {
-          values = g_new0 (gchar *, 6);
-
-          values[0] = g_strdup_printf ("%d %%", ROUND (frame->color.r * 100.0));
-          values[1] = g_strdup_printf ("%d %%", ROUND (frame->color.g * 100.0));
-          values[2] = g_strdup_printf ("%d %%", ROUND (frame->color.b * 100.0));
-          values[3] = g_strdup_printf ("%d %%", ROUND (frame->color.a * 100.0));
-        }
-
       names[4] = _("Hex:");
 
       if (frame->sample_valid)
         {
-          guchar r, g, b;
+          guchar r, g, b, a;
 
-          gimp_rgb_get_uchar (&frame->color, &r, &g, &b);
+          values = g_new0 (gchar *, 6);
+
+          gimp_rgba_get_uchar (&frame->color, &r, &g, &b, &a);
+
+          if (frame->frame_mode == GIMP_COLOR_FRAME_MODE_RGB_PERCENT)
+            {
+              values[0] = g_strdup_printf ("%.01f %%", frame->color.r * 100.0);
+              values[1] = g_strdup_printf ("%.01f %%", frame->color.g * 100.0);
+              values[2] = g_strdup_printf ("%.01f %%", frame->color.b * 100.0);
+              values[3] = g_strdup_printf ("%.01f %%", frame->color.a * 100.0);
+            }
+          else
+            {
+              values[0] = g_strdup_printf ("%d", r);
+              values[1] = g_strdup_printf ("%d", g);
+              values[2] = g_strdup_printf ("%d", b);
+              values[3] = g_strdup_printf ("%d", a);
+            }
 
           values[4] = g_strdup_printf ("%.2x%.2x%.2x", r, g, b);
         }
@@ -663,10 +722,10 @@ gimp_color_frame_update (GimpColorFrame *frame)
 
           values = g_new0 (gchar *, 5);
 
-          values[0] = g_strdup_printf ("%d \302\260", ROUND (hsv.h * 360.0));
-          values[1] = g_strdup_printf ("%d %%",       ROUND (hsv.s * 100.0));
-          values[2] = g_strdup_printf ("%d %%",       ROUND (hsv.v * 100.0));
-          values[3] = g_strdup_printf ("%d %%",       ROUND (hsv.a * 100.0));
+          values[0] = g_strdup_printf ("%.01f \302\260", hsv.h * 360.0);
+          values[1] = g_strdup_printf ("%.01f %%",       hsv.s * 100.0);
+          values[2] = g_strdup_printf ("%.01f %%",       hsv.v * 100.0);
+          values[3] = g_strdup_printf ("%.01f %%",       hsv.a * 100.0);
         }
       break;
 
@@ -683,16 +742,44 @@ gimp_color_frame_update (GimpColorFrame *frame)
         {
           GimpCMYK cmyk;
 
-          gimp_rgb_to_cmyk (&frame->color, 1.0, &cmyk);
+          if (! frame->transform)
+            gimp_color_frame_create_transform (frame);
+
+          if (frame->transform)
+            {
+              gdouble rgb_values[3];
+              gdouble cmyk_values[4];
+
+              rgb_values[0] = frame->color.r;
+              rgb_values[1] = frame->color.g;
+              rgb_values[2] = frame->color.b;
+
+              gimp_color_transform_process_pixels (frame->transform,
+                                                   babl_format ("R'G'B' double"),
+                                                   rgb_values,
+                                                   babl_format ("CMYK double"),
+                                                   cmyk_values,
+                                                   1);
+
+              cmyk.c = cmyk_values[0] / 100.0;
+              cmyk.m = cmyk_values[1] / 100.0;
+              cmyk.y = cmyk_values[2] / 100.0;
+              cmyk.k = cmyk_values[3] / 100.0;
+            }
+          else
+            {
+              gimp_rgb_to_cmyk (&frame->color, 1.0, &cmyk);
+            }
+
           cmyk.a = frame->color.a;
 
           values = g_new0 (gchar *, 6);
 
-          values[0] = g_strdup_printf ("%d %%", ROUND (cmyk.c * 100.0));
-          values[1] = g_strdup_printf ("%d %%", ROUND (cmyk.m * 100.0));
-          values[2] = g_strdup_printf ("%d %%", ROUND (cmyk.y * 100.0));
-          values[3] = g_strdup_printf ("%d %%", ROUND (cmyk.k * 100.0));
-          values[4] = g_strdup_printf ("%d %%", ROUND (cmyk.a * 100.0));
+          values[0] = g_strdup_printf ("%.01f %%", cmyk.c * 100.0);
+          values[1] = g_strdup_printf ("%.01f %%", cmyk.m * 100.0);
+          values[2] = g_strdup_printf ("%.01f %%", cmyk.y * 100.0);
+          values[3] = g_strdup_printf ("%.01f %%", cmyk.k * 100.0);
+          values[4] = g_strdup_printf ("%.01f %%", cmyk.a * 100.0);
         }
       break;
     }
@@ -717,3 +804,44 @@ gimp_color_frame_update (GimpColorFrame *frame)
 
   g_strfreev (values);
 }
+
+static void
+gimp_color_frame_create_transform (GimpColorFrame *frame)
+{
+  if (frame->config)
+    {
+      GimpColorProfile *cmyk_profile;
+
+      cmyk_profile = gimp_color_config_get_cmyk_color_profile (frame->config,
+                                                               NULL);
+
+      if (cmyk_profile)
+        {
+          static GimpColorProfile *rgb_profile = NULL;
+
+          if (G_UNLIKELY (! rgb_profile))
+            rgb_profile = gimp_color_profile_new_rgb_srgb ();
+
+          frame->transform =
+            gimp_color_transform_new (rgb_profile,
+                                      babl_format ("R'G'B' double"),
+                                      cmyk_profile,
+                                      babl_format ("CMYK double"),
+                                      GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL,
+                                      GIMP_COLOR_TRANSFORM_FLAGS_NOOPTIMIZE |
+                                      GIMP_COLOR_TRANSFORM_FLAGS_BLACK_POINT_COMPENSATION);
+        }
+    }
+}
+
+static void
+gimp_color_frame_destroy_transform (GimpColorFrame *frame)
+{
+  if (frame->transform)
+    {
+      g_object_unref (frame->transform);
+      frame->transform = NULL;
+    }
+
+  gimp_color_frame_update (frame);
+}
