@@ -279,9 +279,11 @@ gimp_dockbook_init (GimpDockbook *dockbook)
                                   GTK_PACK_END);
   gtk_widget_show (dockbook->p->menu_button);
 
-  image = gtk_image_new_from_icon_name (GIMP_STOCK_MENU_LEFT, GTK_ICON_SIZE_MENU);
+  image = gtk_image_new_from_icon_name (GIMP_ICON_MENU_LEFT,
+                                        GTK_ICON_SIZE_MENU);
   gtk_image_set_pixel_size (GTK_IMAGE (image), 12);
-  gtk_image_set_from_icon_name (GTK_IMAGE (image), GIMP_STOCK_MENU_LEFT, GTK_ICON_SIZE_MENU);
+  gtk_image_set_from_icon_name (GTK_IMAGE (image), GIMP_ICON_MENU_LEFT,
+                                GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (dockbook->p->menu_button), image);
   gtk_widget_show (image);
 
@@ -500,7 +502,7 @@ gimp_dockbook_show_menu (GimpDockbook *dockbook)
 
       if (! child_menu_widget)
         {
-          g_warning ("%s: UI manager '%s' has now widget at path '%s'",
+          g_warning ("%s: UI manager '%s' has no widget at path '%s'",
                      G_STRFUNC, dialog_ui_manager->name, dialog_ui_path);
           return FALSE;
         }
@@ -541,6 +543,8 @@ gimp_dockbook_show_menu (GimpDockbook *dockbook)
 
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (parent_menu_widget),
                                        image);
+        gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (parent_menu_widget),
+                                                   TRUE);
         gtk_widget_show (image);
       }
 
@@ -859,9 +863,9 @@ gimp_dockbook_new (GimpMenuFactory *menu_factory)
   dockbook = g_object_new (GIMP_TYPE_DOCKBOOK, NULL);
 
   dockbook->p->ui_manager = gimp_menu_factory_manager_new (menu_factory,
-                                                        "<Dockable>",
-                                                        dockbook,
-                                                        FALSE);
+                                                           "<Dockable>",
+                                                           dockbook,
+                                                           FALSE);
 
   gimp_help_connect (GTK_WIDGET (dockbook), gimp_dockbook_help_func,
                      GIMP_HELP_DOCK, dockbook);
