@@ -30,6 +30,9 @@
 
 #include "widgets-types.h"
 
+#include "config/gimpcoreconfig.h"
+
+#include "core/gimp.h"
 #include "core/gimpcontext.h"
 #include "core/gimpmarshal.h"
 
@@ -61,37 +64,40 @@ typedef enum
 } FgBgTarget;
 
 
-static void     gimp_fg_bg_editor_dispose         (GObject        *object);
-static void     gimp_fg_bg_editor_set_property    (GObject        *object,
-                                                   guint           property_id,
-                                                   const GValue   *value,
-                                                   GParamSpec     *pspec);
-static void     gimp_fg_bg_editor_get_property    (GObject        *object,
-                                                   guint           property_id,
-                                                   GValue         *value,
-                                                   GParamSpec     *pspec);
+static void     gimp_fg_bg_editor_dispose           (GObject          *object);
+static void     gimp_fg_bg_editor_set_property      (GObject          *object,
+                                                     guint             property_id,
+                                                     const GValue     *value,
+                                                     GParamSpec       *pspec);
+static void     gimp_fg_bg_editor_get_property      (GObject          *object,
+                                                     guint             property_id,
+                                                     GValue           *value,
+                                                     GParamSpec       *pspec);
 
-static void     gimp_fg_bg_editor_style_updated   (GtkWidget      *widget);
-static gboolean gimp_fg_bg_editor_draw            (GtkWidget      *widget,
-                                                   cairo_t        *cr);
-static gboolean gimp_fg_bg_editor_button_press    (GtkWidget      *widget,
-                                                   GdkEventButton *bevent);
-static gboolean gimp_fg_bg_editor_button_release  (GtkWidget      *widget,
-                                                   GdkEventButton *bevent);
-static gboolean gimp_fg_bg_editor_drag_motion     (GtkWidget      *widget,
-                                                   GdkDragContext *context,
-                                                   gint            x,
-                                                   gint            y,
-                                                   guint           time);
+static void     gimp_fg_bg_editor_style_updated     (GtkWidget        *widget);
+static gboolean gimp_fg_bg_editor_draw              (GtkWidget        *widget,
+                                                     cairo_t          *cr);
+static gboolean gimp_fg_bg_editor_button_press      (GtkWidget        *widget,
+                                                     GdkEventButton   *bevent);
+static gboolean gimp_fg_bg_editor_button_release    (GtkWidget        *widget,
+                                                     GdkEventButton   *bevent);
+static gboolean gimp_fg_bg_editor_drag_motion       (GtkWidget        *widget,
+                                                     GdkDragContext   *context,
+                                                     gint              x,
+                                                     gint              y,
+                                                     guint             time);
 
-static void     gimp_fg_bg_editor_drag_color      (GtkWidget      *widget,
-                                                   GimpRGB        *color,
-                                                   gpointer        data);
-static void     gimp_fg_bg_editor_drop_color      (GtkWidget      *widget,
-                                                   gint            x,
-                                                   gint            y,
-                                                   const GimpRGB  *color,
-                                                   gpointer        data);
+static void     gimp_fg_bg_editor_drag_color        (GtkWidget        *widget,
+                                                     GimpRGB          *color,
+                                                     gpointer          data);
+static void     gimp_fg_bg_editor_drop_color        (GtkWidget        *widget,
+                                                     gint              x,
+                                                     gint              y,
+                                                     const GimpRGB    *color,
+                                                     gpointer          data);
+
+static void     gimp_fg_bg_editor_create_transform  (GimpFgBgEditor   *editor);
+static void     gimp_fg_bg_editor_destroy_transform (GimpFgBgEditor   *editor);
 
 
 G_DEFINE_TYPE (GimpFgBgEditor, gimp_fg_bg_editor, GTK_TYPE_DRAWING_AREA)
@@ -144,7 +150,6 @@ gimp_fg_bg_editor_class_init (GimpFgBgEditorClass *klass)
 static void
 gimp_fg_bg_editor_init (GimpFgBgEditor *editor)
 {
-  editor->context      = NULL;
   editor->active_color = GIMP_ACTIVE_COLOR_FOREGROUND;
 
   gtk_widget_add_events (GTK_WIDGET (editor),
@@ -155,6 +160,10 @@ gimp_fg_bg_editor_init (GimpFgBgEditor *editor)
                              gimp_fg_bg_editor_drag_color, NULL);
   gimp_dnd_color_dest_add (GTK_WIDGET (editor),
                            gimp_fg_bg_editor_drop_color, NULL);
+
+  gimp_widget_track_monitor (GTK_WIDGET (editor),
+                             G_CALLBACK (gimp_fg_bg_editor_destroy_transform),
+                             NULL);
 }
 
 static void
@@ -275,7 +284,7 @@ gimp_fg_bg_editor_draw (GtkWidget *widget,
   /*  draw the default colors pixbuf  */
   if (! editor->default_icon)
     editor->default_icon = gimp_widget_load_icon (widget,
-                                                  GIMP_STOCK_DEFAULT_COLORS, 12);
+                                                  GIMP_ICON_COLORS_DEFAULT, 12);
 
   default_w = gdk_pixbuf_get_width  (editor->default_icon);
   default_h = gdk_pixbuf_get_height (editor->default_icon);
@@ -294,7 +303,7 @@ gimp_fg_bg_editor_draw (GtkWidget *widget,
   /*  draw the swap colors pixbuf  */
   if (! editor->swap_icon)
     editor->swap_icon = gimp_widget_load_icon (widget,
-                                               GIMP_STOCK_SWAP_COLORS, 12);
+                                               GIMP_ICON_COLORS_SWAP, 12);
 
   swap_w = gdk_pixbuf_get_width  (editor->swap_icon);
   swap_h = gdk_pixbuf_get_height (editor->swap_icon);
@@ -321,11 +330,23 @@ gimp_fg_bg_editor_draw (GtkWidget *widget,
   editor->rect_height = rect_h;
 
 
+  if (! editor->transform)
+    gimp_fg_bg_editor_create_transform (editor);
+
   /*  draw the background area  */
 
   if (editor->context)
     {
       gimp_context_get_background (editor->context, &color);
+
+      if (editor->transform)
+        gimp_color_transform_process_pixels (editor->transform,
+                                             babl_format ("R'G'B'A double"),
+                                             &color,
+                                             babl_format ("R'G'B'A double"),
+                                             &color,
+                                             1);
+
       gimp_cairo_set_source_rgb (cr, &color);
 
       cairo_rectangle (cr,
@@ -352,6 +373,15 @@ gimp_fg_bg_editor_draw (GtkWidget *widget,
   if (editor->context)
     {
       gimp_context_get_foreground (editor->context, &color);
+
+      if (editor->transform)
+        gimp_color_transform_process_pixels (editor->transform,
+                                             babl_format ("R'G'B'A double"),
+                                             &color,
+                                             babl_format ("R'G'B'A double"),
+                                             &color,
+                                             1);
+
       gimp_cairo_set_source_rgb (cr, &color);
 
       cairo_rectangle (cr,
@@ -532,33 +562,46 @@ gimp_fg_bg_editor_set_context (GimpFgBgEditor *editor,
   g_return_if_fail (GIMP_IS_FG_BG_EDITOR (editor));
   g_return_if_fail (context == NULL || GIMP_IS_CONTEXT (context));
 
-  if (context == editor->context)
-    return;
-
-  if (editor->context)
+  if (context != editor->context)
     {
-      g_signal_handlers_disconnect_by_func (editor->context,
-                                            gtk_widget_queue_draw,
-                                            editor);
-      g_object_unref (editor->context);
-      editor->context = NULL;
+      if (editor->context)
+        {
+          g_signal_handlers_disconnect_by_func (editor->context,
+                                                gtk_widget_queue_draw,
+                                                editor);
+          g_object_unref (editor->context);
+
+          g_signal_handlers_disconnect_by_func (editor->color_config,
+                                                gimp_fg_bg_editor_destroy_transform,
+                                                editor);
+          g_object_unref (editor->color_config);
+          editor->color_config = NULL;
+        }
+
+      editor->context = context;
+
+      if (context)
+        {
+          g_object_ref (context);
+
+          g_signal_connect_swapped (context, "foreground-changed",
+                                    G_CALLBACK (gtk_widget_queue_draw),
+                                    editor);
+          g_signal_connect_swapped (context, "background-changed",
+                                    G_CALLBACK (gtk_widget_queue_draw),
+                                    editor);
+
+          editor->color_config = g_object_ref (context->gimp->config->color_management);
+
+          g_signal_connect_swapped (editor->color_config, "notify",
+                                    G_CALLBACK (gimp_fg_bg_editor_destroy_transform),
+                                    editor);
+        }
+
+      gimp_fg_bg_editor_destroy_transform (editor);
+
+      g_object_notify (G_OBJECT (editor), "context");
     }
-
-  editor->context = context;
-
-  if (context)
-    {
-      g_object_ref (context);
-
-      g_signal_connect_swapped (context, "foreground-changed",
-                                G_CALLBACK (gtk_widget_queue_draw),
-                                editor);
-      g_signal_connect_swapped (context, "background-changed",
-                                G_CALLBACK (gtk_widget_queue_draw),
-                                editor);
-    }
-
-  g_object_notify (G_OBJECT (editor), "context");
 }
 
 void
@@ -623,3 +666,34 @@ gimp_fg_bg_editor_drop_color (GtkWidget     *widget,
         }
     }
 }
+
+static void
+gimp_fg_bg_editor_create_transform (GimpFgBgEditor *editor)
+{
+  if (editor->color_config)
+    {
+      static GimpColorProfile *profile = NULL;
+
+      if (G_UNLIKELY (! profile))
+        profile = gimp_color_profile_new_rgb_srgb ();
+
+      editor->transform =
+        gimp_widget_get_color_transform (GTK_WIDGET (editor),
+                                         editor->color_config,
+                                         profile,
+                                         babl_format ("R'G'B'A double"),
+                                         babl_format ("R'G'B'A double"));
+    }
+}
+
+static void
+gimp_fg_bg_editor_destroy_transform (GimpFgBgEditor *editor)
+{
+  if (editor->transform)
+    {
+      g_object_unref (editor->transform);
+      editor->transform = NULL;
+    }
+
+  gtk_widget_queue_draw (GTK_WIDGET (editor));
+}
