@@ -57,8 +57,8 @@
 #define GIMP_HELP_EDIT_COPY_VISIBLE               "gimp-edit-copy-visible"
 #define GIMP_HELP_EDIT_PASTE                      "gimp-edit-paste"
 #define GIMP_HELP_EDIT_PASTE_INTO                 "gimp-edit-paste-into"
-#define GIMP_HELP_EDIT_PASTE_AS_NEW               "gimp-edit-paste-as-new"
 #define GIMP_HELP_EDIT_PASTE_AS_NEW_LAYER         "gimp-edit-paste-as-new-layer"
+#define GIMP_HELP_EDIT_PASTE_AS_NEW_IMAGE         "gimp-edit-paste-as-new-image"
 #define GIMP_HELP_EDIT_CLEAR                      "gimp-edit-clear"
 #define GIMP_HELP_EDIT_FILL_FG                    "gimp-edit-fill-fg"
 #define GIMP_HELP_EDIT_FILL_BG                    "gimp-edit-fill-bg"
@@ -98,6 +98,7 @@
 #define GIMP_HELP_VIEW_ROTATE_270                 "gimp-view-rotate-270"
 #define GIMP_HELP_VIEW_ROTATE_345                 "gimp-view-rotate-345"
 #define GIMP_HELP_VIEW_ROTATE_OTHER               "gimp-view-rotate-other"
+#define GIMP_HELP_VIEW_COLOR_MANAGEMENT           "gimp-view-color-management"
 #define GIMP_HELP_VIEW_SHOW_SELECTION             "gimp-view-show-selection"
 #define GIMP_HELP_VIEW_SHOW_LAYER_BOUNDARY        "gimp-view-show-layer-boundary"
 #define GIMP_HELP_VIEW_SHOW_GUIDES                "gimp-view-show-guides"
@@ -126,6 +127,7 @@
 
 
 #define GIMP_HELP_IMAGE_DIALOG                    "gimp-image-dialog"
+#define GIMP_HELP_IMAGE_DUPLICATE                 "gimp-image-duplicate"
 #define GIMP_HELP_IMAGE_CONVERT_RGB               "gimp-image-convert-rgb"
 #define GIMP_HELP_IMAGE_CONVERT_GRAYSCALE         "gimp-image-convert-grayscale"
 #define GIMP_HELP_IMAGE_CONVERT_INDEXED           "gimp-image-convert-indexed"
@@ -149,12 +151,13 @@
 #define GIMP_HELP_IMAGE_SCALE                     "gimp-image-scale"
 #define GIMP_HELP_IMAGE_SCALE_WARNING             "gimp-image-scale-warning"
 #define GIMP_HELP_IMAGE_CROP                      "gimp-image-crop"
-#define GIMP_HELP_IMAGE_DUPLICATE                 "gimp-image-duplicate"
 #define GIMP_HELP_IMAGE_MERGE_LAYERS              "gimp-image-merge-layers"
 #define GIMP_HELP_IMAGE_FLATTEN                   "gimp-image-flatten"
+#define GIMP_HELP_IMAGE_COLOR_MANAGEMENT_ENABLED  "gimp-image-color-management-enabled"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_ASSIGN      "gimp-image-color-profile-assign"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_CONVERT     "gimp-image-color-profile-convert"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_DISCARD     "gimp-image-color-profile-discard"
+#define GIMP_HELP_IMAGE_COLOR_PROFILE_SAVE        "gimp-image-color-profile-save"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_IMPORT      "gimp-image-color-profile-import"
 #define GIMP_HELP_IMAGE_GRID                      "gimp-image-grid"
 #define GIMP_HELP_IMAGE_PROPERTIES                "gimp-image-properties"
@@ -188,6 +191,7 @@
 #define GIMP_HELP_LAYER_ERODE                     "gimp-layer-erode"
 #define GIMP_HELP_LAYER_VISIBLE                   "gimp-layer-visible"
 #define GIMP_HELP_LAYER_LINKED                    "gimp-layer-linked"
+#define GIMP_HELP_LAYER_COLOR_TAG                 "gimp-layer-color-tag"
 #define GIMP_HELP_LAYER_OPACITY                   "gimp-layer-opacity"
 #define GIMP_HELP_LAYER_MODE                      "gimp-layer-mode"
 #define GIMP_HELP_LAYER_LOCK_ALPHA                "gimp-layer-lock-alpha"
@@ -229,6 +233,9 @@
 #define GIMP_HELP_CHANNEL_LOWER_TO_BOTTOM         "gimp-channel-lower-to-bottom"
 #define GIMP_HELP_CHANNEL_DUPLICATE               "gimp-channel-duplicate"
 #define GIMP_HELP_CHANNEL_DELETE                  "gimp-channel-delete"
+#define GIMP_HELP_CHANNEL_VISIBLE                 "gimp-channel-visible"
+#define GIMP_HELP_CHANNEL_LINKED                  "gimp-channel-linked"
+#define GIMP_HELP_CHANNEL_COLOR_TAG               "gimp-channel-color-tag"
 #define GIMP_HELP_CHANNEL_LOCK_PIXELS             "gimp-channel-lock-pixels"
 #define GIMP_HELP_CHANNEL_LOCK_POSITION           "gimp-channel-lock-position"
 #define GIMP_HELP_CHANNEL_SELECTION_REPLACE       "gimp-channel-selection-replace"
@@ -253,6 +260,7 @@
 #define GIMP_HELP_PATH_MERGE_VISIBLE              "gimp-path-merge-visible"
 #define GIMP_HELP_PATH_VISIBLE                    "gimp-path-visible"
 #define GIMP_HELP_PATH_LINKED                     "gimp-path-linked"
+#define GIMP_HELP_PATH_COLOR_TAG                  "gimp-path-color-tag"
 #define GIMP_HELP_PATH_LOCK_STROKES               "gimp-path-lock-strokes"
 #define GIMP_HELP_PATH_LOCK_POSITION              "gimp-path-lock-position"
 #define GIMP_HELP_PATH_SELECTION_REPLACE          "gimp-path-selection-replace"
@@ -284,7 +292,6 @@
 #define GIMP_HELP_TOOL_CROP                       "gimp-tool-crop"
 #define GIMP_HELP_TOOL_CURVES                     "gimp-tool-curves"
 #define GIMP_HELP_TOOL_CURVES                     "gimp-tool-curves"
-#define GIMP_HELP_TOOL_DESATURATE                 "gimp-tool-desaturate"
 #define GIMP_HELP_TOOL_DODGE_BURN                 "gimp-tool-dodge-burn"
 #define GIMP_HELP_TOOL_ELLIPSE_SELECT             "gimp-tool-ellipse-select"
 #define GIMP_HELP_TOOL_ERASER                     "gimp-tool-eraser"
@@ -308,7 +315,6 @@
 #define GIMP_HELP_TOOL_PENCIL                     "gimp-tool-pencil"
 #define GIMP_HELP_TOOL_PERSPECTIVE                "gimp-tool-perspective"
 #define GIMP_HELP_TOOL_PERSPECTIVE_CLONE          "gimp-tool-perspective-clone"
-#define GIMP_HELP_TOOL_POSTERIZE                  "gimp-tool-posterize"
 #define GIMP_HELP_TOOL_RECT_SELECT                "gimp-tool-rect-select"
 #define GIMP_HELP_TOOL_ROTATE                     "gimp-tool-rotate"
 #define GIMP_HELP_TOOL_SCALE                      "gimp-tool-scale"
@@ -326,6 +332,105 @@
 #define GIMP_HELP_FILTER_RESHOW                   "gimp-filter-reshow"
 #define GIMP_HELP_FILTER_RESET_ALL                "gimp-filter-reset-all"
 
+#define GIMP_HELP_FILTER_ALIEN_MAP                "gimp-filter-alien-map"
+#define GIMP_HELP_FILTER_ANTIALIAS                "gimp-filter-antialias"
+#define GIMP_HELP_FILTER_APPLY_CANVAS             "gimp-filter-apply-canvas"
+#define GIMP_HELP_FILTER_APPLY_LENS               "gimp-filter-apply-lens"
+#define GIMP_HELP_FILTER_BUMP_MAP                 "gimp-filter-bump-map"
+#define GIMP_HELP_FILTER_C2G                      "gimp-filter-c2g"
+#define GIMP_HELP_FILTER_CARTOON                  "gimp-filter-cartoon"
+#define GIMP_HELP_FILTER_CHANNEL_MIXER            "gimp-filter-channel-mixer"
+#define GIMP_HELP_FILTER_CHECKERBOARD             "gimp-filter-checkerboard"
+#define GIMP_HELP_FILTER_COLOR_ENHANCE            "gimp-filter-color-enhance"
+#define GIMP_HELP_FILTER_COLOR_EXCHANGE           "gimp-filter-color-exchange"
+#define GIMP_HELP_FILTER_COLOR_ROTATE             "gimp-filter-color-rotate"
+#define GIMP_HELP_FILTER_COLOR_TEMPERATURE        "gimp-filter-color-temperature"
+#define GIMP_HELP_FILTER_COLOR_TO_ALPHA           "gimp-filter-color-to-alpha"
+#define GIMP_HELP_FILTER_COMPONENT_EXTRACT        "gimp-filter-component-extract"
+#define GIMP_HELP_FILTER_CONVOLUTION_MATRIX       "gimp-filter-convolution-matrix"
+#define GIMP_HELP_FILTER_CUBISM                   "gimp-filter-cubism"
+#define GIMP_HELP_FILTER_DEINTERLACE              "gimp-filter-deinterlace"
+#define GIMP_HELP_FILTER_DESATURATE               "gimp-filter-desaturate"
+#define GIMP_HELP_FILTER_DIFFERENCE_OF_GAUSSIANS  "gimp-filter-difference-of-gaussians"
+#define GIMP_HELP_FILTER_DIFFRACTION_PATTERNS     "gimp-filter-diffraction-patterns"
+#define GIMP_HELP_FILTER_DISPLACE                 "gimp-filter-displace"
+#define GIMP_HELP_FILTER_DISTANCE_MAP             "gimp-filter-distance-map"
+#define GIMP_HELP_FILTER_DITHER                   "gimp-filter-dither"
+#define GIMP_HELP_FILTER_DROPSHADOW               "gimp-filter-dropshadow"
+#define GIMP_HELP_FILTER_EDGE                     "gimp-filter-edge"
+#define GIMP_HELP_FILTER_EDGE_LAPLACE             "gimp-filter-edge-laplace"
+#define GIMP_HELP_FILTER_EDGE_NEON                "gimp-filter-edge-neon"
+#define GIMP_HELP_FILTER_EDGE_SOBEL               "gimp-filter-edge-sobel"
+#define GIMP_HELP_FILTER_EMBOSS                   "gimp-filter-emboss"
+#define GIMP_HELP_FILTER_ENGRAVE                  "gimp-filter-engrave"
+#define GIMP_HELP_FILTER_EXPOSURE                 "gimp-filter-exposure"
+#define GIMP_HELP_FILTER_FATTAL_2002              "gimp-filter-fattal-2002"
+#define GIMP_HELP_FILTER_FRACTAL_TRACE            "gimp-filter-fractal-trace"
+#define GIMP_HELP_FILTER_GAUSSIAN_BLUR            "gimp-filter-gaussian-blur"
+#define GIMP_HELP_FILTER_GAUSSIAN_BLUR_SELECTIVE  "gimp-filter-gaussian-blur-selective"
+#define GIMP_HELP_FILTER_GEGL_GRAPH               "gimp-filter-gegl"
+#define GIMP_HELP_FILTER_GAUSSIAN_GRID            "gimp-filter-grid"
+#define GIMP_HELP_FILTER_HIGH_PASS                "gimp-filter-high-pass"
+#define GIMP_HELP_FILTER_ILLUSION                 "gimp-filter-illusion"
+#define GIMP_HELP_FILTER_IMAGE_GRADIENT           "gimp-filter-image-gradient"
+#define GIMP_HELP_FILTER_KALEIDOSCOPE             "gimp-filter-kaleidoscope"
+#define GIMP_HELP_FILTER_LENS_DISTORTION          "gimp-filter-lens-distortion"
+#define GIMP_HELP_FILTER_LENS_FLARE               "gimp-filter-lens-flare"
+#define GIMP_HELP_FILTER_MANTIUK_2006             "gimp-filter-manituk-2006"
+#define GIMP_HELP_FILTER_MAZE                     "gimp-filter-maze"
+#define GIMP_HELP_FILTER_MONO_MIXER               "gimp-filter-mono-mixer"
+#define GIMP_HELP_FILTER_MOSAIC                   "gimp-filter-mosaic"
+#define GIMP_HELP_FILTER_MOTION_BLUR_CIRCULAR     "gimp-filter-motion-blur-circular"
+#define GIMP_HELP_FILTER_MOTION_BLUR_LINEAR       "gimp-filter-motion-blur-linear"
+#define GIMP_HELP_FILTER_MOTION_BLUR_ZOOM         "gimp-filter-motion-blur-zoom"
+#define GIMP_HELP_FILTER_NOISE_CELL               "gimp-filter-noise-cell"
+#define GIMP_HELP_FILTER_NOISE_CIE_LCH            "gimp-filter-noise-cie-lch"
+#define GIMP_HELP_FILTER_NOISE_HSV                "gimp-filter-noise-hsv"
+#define GIMP_HELP_FILTER_NOISE_HURL               "gimp-filter-noise-hurl"
+#define GIMP_HELP_FILTER_NOISE_PERLIN             "gimp-filter-noise-perlin"
+#define GIMP_HELP_FILTER_NOISE_PICK               "gimp-filter-noise-pick"
+#define GIMP_HELP_FILTER_NOISE_RGB                "gimp-filter-noise-rgb"
+#define GIMP_HELP_FILTER_NOISE_REDUCTION          "gimp-filter-noise-reduction"
+#define GIMP_HELP_FILTER_NOISE_SIMPLEX            "gimp-filter-noise-simplex"
+#define GIMP_HELP_FILTER_NOISE_SLUR               "gimp-filter-noise-slur"
+#define GIMP_HELP_FILTER_NOISE_SOLID              "gimp-filter-noise-solid"
+#define GIMP_HELP_FILTER_NOISE_SPREAD             "gimp-filter-noise-spread"
+#define GIMP_HELP_FILTER_OILIFY                   "gimp-filter-oilify"
+#define GIMP_HELP_FILTER_PANORAMA_PROJECTION      "gimp-filter-panorama-projection"
+#define GIMP_HELP_FILTER_PHOTOCOPY                "gimp-filter-photocopy"
+#define GIMP_HELP_FILTER_PIXELIZE                 "gimp-filter-pixelize"
+#define GIMP_HELP_FILTER_PLASMA                   "gimp-filter-plasma"
+#define GIMP_HELP_FILTER_POLAR_COORDINATES        "gimp-filter-polar-coordinates"
+#define GIMP_HELP_FILTER_POSTERIZE                "gimp-filter-posterize"
+#define GIMP_HELP_FILTER_RED_EYE_REMOVAL          "gimp-filter-red-eye-removal"
+#define GIMP_HELP_FILTER_REINHARD_2005            "gimp-filter-reinhard-2005"
+#define GIMP_HELP_FILTER_RGB_CLIP                 "gimp-filter-rgb-clip"
+#define GIMP_HELP_FILTER_RIPPLE                   "gimp-filter-ripple"
+#define GIMP_HELP_FILTER_SATURATION               "gimp-filter-saturation"
+#define GIMP_HELP_FILTER_SEMI_FLATTEN             "gimp-filter-semi-flatten"
+#define GIMP_HELP_FILTER_SEPIA                    "gimp-filter-sepia"
+#define GIMP_HELP_FILTER_SHIFT                    "gimp-filter-shift"
+#define GIMP_HELP_FILTER_SINUS                    "gimp-filter-sinus"
+#define GIMP_HELP_FILTER_SLIC                     "gimp-filter-slic"
+#define GIMP_HELP_FILTER_SNN_MEAN                 "gimp-filter-snn-mean"
+#define GIMP_HELP_FILTER_SOFTGLOW                 "gimp-filter-softglow"
+#define GIMP_HELP_FILTER_STRETCH_CONTRAST         "gimp-filter-stretch-contrast"
+#define GIMP_HELP_FILTER_STRETCH_CONTRAST_HSV     "gimp-filter-stretch-contrast-hsv"
+#define GIMP_HELP_FILTER_STRESS                   "gimp-filter-stress"
+#define GIMP_HELP_FILTER_SUPERNOVA                "gimp-filter-supernova"
+#define GIMP_HELP_FILTER_THRESHOLD_ALPHA          "gimp-filter-threshold-alpha"
+#define GIMP_HELP_FILTER_TILE_GLASS               "gimp-filter-tile-glass"
+#define GIMP_HELP_FILTER_TILE_PAPER               "gimp-filter-tile-paper"
+#define GIMP_HELP_FILTER_TILE_SEAMLESS            "gimp-filter-tile-seamless"
+#define GIMP_HELP_FILTER_UNSHARP_MASK             "gimp-filter-unsharp-mask"
+#define GIMP_HELP_FILTER_VALUE_PROPAGATE          "gimp-filter-value-propagate"
+#define GIMP_HELP_FILTER_VIDEO_DEGRADATION        "gimp-filter-video-degradation"
+#define GIMP_HELP_FILTER_VIGNETTE                 "gimp-filter-vignette"
+#define GIMP_HELP_FILTER_WATERPIXELS              "gimp-filter-waterpixels"
+#define GIMP_HELP_FILTER_WAVES                    "gimp-filter-waves"
+#define GIMP_HELP_FILTER_WHIRL_PINCH              "gimp-filter-whirl-pinch"
+#define GIMP_HELP_FILTER_WIND                     "gimp-filter-wind"
+
 #define GIMP_HELP_TOOLBOX                         "gimp-toolbox"
 #define GIMP_HELP_TOOLBOX_COLOR_AREA              "gimp-toolbox-color-area"
 #define GIMP_HELP_TOOLBOX_IMAGE_AREA              "gimp-toolbox-image-area"
@@ -349,6 +454,7 @@
 #define GIMP_HELP_DYNAMICS_EDITOR_DIALOG          "gimp-dynamics-editor-dialog"
 
 #define GIMP_HELP_TOOL_PRESET_EDITOR_DIALOG       "gimp-tool-preset-editor-dialog"
+#define GIMP_HELP_TOOL_PRESET_EDITOR_EDIT_ACTIVE  "gimp-tool-preset-editor-edit-active"
 
 #define GIMP_HELP_DYNAMICS_DIALOG                 "gimp-dynamics-dialog"
 #define GIMP_HELP_DYNAMICS_EDIT                   "gimp-dynamics-edit"
@@ -439,7 +545,8 @@
 #define GIMP_HELP_BUFFER_COPY                     "gimp-buffer-copy"
 #define GIMP_HELP_BUFFER_PASTE                    "gimp-buffer-paste"
 #define GIMP_HELP_BUFFER_PASTE_INTO               "gimp-buffer-paste-into"
-#define GIMP_HELP_BUFFER_PASTE_AS_NEW             "gimp-buffer-paste-as-new"
+#define GIMP_HELP_BUFFER_PASTE_AS_NEW_LAYER       "gimp-buffer-paste-as-new-layer"
+#define GIMP_HELP_BUFFER_PASTE_AS_NEW_IMAGE       "gimp-buffer-paste-as-new-image"
 #define GIMP_HELP_BUFFER_DELETE                   "gimp-buffer-delete"
 
 #define GIMP_HELP_TOOL_PRESET_DIALOG              "gimp-tool-preset-dialog"
@@ -448,6 +555,7 @@
 #define GIMP_HELP_TOOL_PRESET_DUPLICATE           "gimp-tool-preset-duplicate"
 #define GIMP_HELP_TOOL_PRESET_COPY_LOCATION       "gimp-tool-preset-copy-location"
 #define GIMP_HELP_TOOL_PRESET_SHOW_IN_FILE_MANAGER "gimp-tool-preset-show-in-file-manager"
+#define GIMP_HELP_TOOL_PRESET_SAVE                "gimp-tool-preset-save"
 #define GIMP_HELP_TOOL_PRESET_RESTORE             "gimp-tool-preset-restore"
 #define GIMP_HELP_TOOL_PRESET_DELETE              "gimp-tool-preset-delete"
 #define GIMP_HELP_TOOL_PRESET_REFRESH             "gimp-tool-preset-refresh"
@@ -488,6 +596,7 @@
 #define GIMP_HELP_PREFS_ICON_THEME                "gimp-prefs-icon-theme"
 #define GIMP_HELP_PREFS_TOOL_OPTIONS              "gimp-prefs-tool-options"
 #define GIMP_HELP_PREFS_TOOLBOX                   "gimp-prefs-toolbox"
+#define GIMP_HELP_PREFS_DIALOG_DEFAULTS           "gimp-prefs-dialog-defaults"
 #define GIMP_HELP_PREFS_INPUT_DEVICES             "gimp-prefs-input-devices"
 #define GIMP_HELP_PREFS_INPUT_CONTROLLERS         "gimp-prefs-input-controllers"
 #define GIMP_HELP_PREFS_IMAGE_WINDOW              "gimp-prefs-image-window"
@@ -594,7 +703,5 @@
 #define GIMP_HELP_CONTROLLER_MOUSE                "gimp-controller-mouse"
 #define GIMP_HELP_CONTROLLER_WHEEL                "gimp-controller-wheel"
 
-#define GIMP_HELP_CONFIG_USE_GEGL                 "gimp-config-use-gegl"
-
 
 #endif /* __GIMP_HELP_IDS_H__ */
