@@ -59,6 +59,7 @@ GtkWidget *
 _gimp_prop_gui_new_generic (GObject              *config,
                             GParamSpec          **param_specs,
                             guint                 n_param_specs,
+                            GeglRectangle        *area,
                             GimpContext          *context,
                             GimpCreatePickerFunc  create_picker_func,
                             gpointer              picker_creator)
@@ -100,11 +101,13 @@ _gimp_prop_gui_new_generic (GObject              *config,
 
           i++;
 
-          widget_x = gimp_prop_widget_new_from_pspec (config, pspec, context,
+          widget_x = gimp_prop_widget_new_from_pspec (config, pspec,
+                                                      area, context,
                                                       create_picker_func,
                                                       picker_creator,
                                                       &label_x);
-          widget_y = gimp_prop_widget_new_from_pspec (config, next_pspec, context,
+          widget_y = gimp_prop_widget_new_from_pspec (config, next_pspec,
+                                                      area, context,
                                                       create_picker_func,
                                                       picker_creator,
                                                       &label_y);
@@ -164,7 +167,7 @@ _gimp_prop_gui_new_generic (GObject              *config,
 
               button = create_picker_func (picker_creator,
                                            pspec_name,
-                                           GIMP_STOCK_CURSOR,
+                                           GIMP_ICON_CURSOR,
                                            _("Pick coordinates from the image"));
               gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
               gtk_widget_show (button);
@@ -179,7 +182,8 @@ _gimp_prop_gui_new_generic (GObject              *config,
           const gchar *label;
           gboolean     expand = FALSE;
 
-          widget = gimp_prop_widget_new_from_pspec (config, pspec, context,
+          widget = gimp_prop_widget_new_from_pspec (config, pspec,
+                                                    area, context,
                                                     create_picker_func,
                                                     picker_creator,
                                                     &label);
@@ -192,7 +196,7 @@ _gimp_prop_gui_new_generic (GObject              *config,
               GtkWidget *l;
 
               l = gtk_label_new_with_mnemonic (label);
-              gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.5);
+              gtk_label_set_xalign (GTK_LABEL (l), 0.0);
               gtk_widget_show (l);
 
               if (GTK_IS_SCROLLED_WINDOW (widget))
@@ -393,6 +397,7 @@ GtkWidget *
 _gimp_prop_gui_new_color_rotate (GObject              *config,
                                  GParamSpec          **param_specs,
                                  guint                 n_param_specs,
+                                 GeglRectangle        *area,
                                  GimpContext          *context,
                                  GimpCreatePickerFunc  create_picker_func,
                                  gpointer              picker_creator)
@@ -441,9 +446,8 @@ _gimp_prop_gui_new_color_rotate (GObject              *config,
 
   box = _gimp_prop_gui_new_generic (config,
                                     param_specs + 6, 2,
-                                    context,
-                                    create_picker_func,
-                                    picker_creator);
+                                    area, context,
+                                    create_picker_func, picker_creator);
   gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);
   gtk_widget_show (box);
 
@@ -456,33 +460,134 @@ _gimp_prop_gui_new_color_rotate (GObject              *config,
   return main_vbox;
 }
 
+static const gchar *
+convolution_matrix_prop_name (gint x,
+                              gint y)
+{
+  static const gchar * const prop_names[5][5] = {
+    {"a1", "a2", "a3", "a4", "a5"},
+    {"b1", "b2", "b3", "b4", "b5"},
+    {"c1", "c2", "c3", "c4", "c5"},
+    {"d1", "d2", "d3", "d4", "d5"},
+    {"e1", "e2", "e3", "e4", "e5"}};
+
+  return prop_names[x][y];
+}
+
+static void
+convolution_matrix_rotate_flip (GtkWidget *button,
+                                GObject   *config)
+{
+  gint rotate = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button),
+                                                    "convolution-matrix-rotate"));
+  gint flip   = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button),
+                                                    "convolution-matrix-flip"));
+  gint x, y;
+
+  while (rotate--)
+    {
+      for (y = 0; y < 2; y++)
+        {
+          for (x = y; x < 4 - y; x++)
+            {
+              gint    i;
+              gdouble temp;
+
+              g_object_get (config,
+                            convolution_matrix_prop_name (x, y), &temp,
+                            NULL);
+
+              for (i = 0; i < 4; i++)
+                {
+                  gint    next_x, next_y;
+                  gdouble val;
+
+                  next_x = 4 - y;
+                  next_y = x;
+
+                  if  (i < 3)
+                    {
+                      g_object_get (config,
+                                    convolution_matrix_prop_name (next_x, next_y), &val,
+                                    NULL);
+                    }
+                  else
+                    {
+                      val = temp;
+                    }
+
+                  g_object_set (config,
+                                convolution_matrix_prop_name (x, y), val,
+                                NULL);
+
+                  x = next_x;
+                  y = next_y;
+                }
+            }
+        }
+    }
+
+  while (flip--)
+    {
+      for (y = 0; y < 5; y++)
+        {
+          for (x = 0; x < 2; x++)
+            {
+              gdouble val1, val2;
+
+              g_object_get (config,
+                            convolution_matrix_prop_name (x, y), &val1,
+                            NULL);
+              g_object_get (config,
+                            convolution_matrix_prop_name (4 - x, y), &val2,
+                            NULL);
+
+              g_object_set (config,
+                            convolution_matrix_prop_name (x, y), val2,
+                            NULL);
+              g_object_set (config,
+                            convolution_matrix_prop_name (4 - x, y), val1,
+                            NULL);
+            }
+        }
+    }
+}
+
 GtkWidget *
 _gimp_prop_gui_new_convolution_matrix (GObject              *config,
                                        GParamSpec          **param_specs,
                                        guint                 n_param_specs,
+                                       GeglRectangle        *area,
                                        GimpContext          *context,
                                        GimpCreatePickerFunc  create_picker_func,
                                        gpointer              picker_creator)
 {
   GtkWidget   *main_vbox;
+  GtkWidget   *vbox;
   GtkWidget   *table;
   GtkWidget   *hbox;
   GtkWidget   *scale;
-  GtkWidget   *vbox;
+  GtkWidget   *vbox2;
   const gchar *label;
-  gint          x, y;
+  gint         x, y;
 
   g_return_val_if_fail (G_IS_OBJECT (config), NULL);
   g_return_val_if_fail (param_specs != NULL, NULL);
   g_return_val_if_fail (n_param_specs > 0, NULL);
   g_return_val_if_fail (GIMP_IS_CONTEXT (context), NULL);
 
-  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
+
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+  gtk_box_pack_start (GTK_BOX (main_vbox), vbox, FALSE, FALSE, 0);
+  gtk_widget_show (vbox);
+
+  /* matrix */
 
   table = gtk_table_new (5, 5, TRUE);
   gtk_table_set_row_spacings (GTK_TABLE (table), 2);
   gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_box_pack_start (GTK_BOX (main_vbox), table, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
   gtk_widget_show (table);
 
   for (y = 0; y < 5; y++)
@@ -490,12 +595,9 @@ _gimp_prop_gui_new_convolution_matrix (GObject              *config,
       for (x = 0; x < 5; x++)
         {
           GtkWidget *spin;
-          gchar      prop_name[3] = { 0, };
 
-          prop_name[0] = "abcde"[x];
-          prop_name[1] = "12345"[y];
-
-          spin = gimp_prop_spin_button_new (config, prop_name,
+          spin = gimp_prop_spin_button_new (config,
+                                            convolution_matrix_prop_name (x, y),
                                             1.0, 10.0, 2);
           gtk_entry_set_width_chars (GTK_ENTRY (spin), 8);
           gtk_table_attach (GTK_TABLE (table), spin,
@@ -506,42 +608,120 @@ _gimp_prop_gui_new_convolution_matrix (GObject              *config,
         }
     }
 
+  /* rotate / flip buttons */
+  {
+    typedef struct
+    {
+      const gchar *tooltip;
+      const gchar *icon_name;
+      gint         rotate;
+      gint         flip;
+    } ButtonInfo;
+
+    gint             i;
+    const ButtonInfo buttons[] = {
+      {
+        .tooltip   = _("Rotate matrix 90° counter-clockwise"),
+        .icon_name = GIMP_ICON_OBJECT_ROTATE_270,
+        .rotate    = 1,
+        .flip      = 0
+      },
+      {
+        .tooltip   = _("Rotate matrix 90° clockwise"),
+        .icon_name = GIMP_ICON_OBJECT_ROTATE_90,
+        .rotate    = 3,
+        .flip      = 0
+      },
+      {
+        .tooltip   = _("Flip matrix horizontally"),
+        .icon_name = GIMP_ICON_OBJECT_FLIP_HORIZONTAL,
+        .rotate    = 0,
+        .flip      = 1
+      },
+      {
+        .tooltip   = _("Flip matrix vertically"),
+        .icon_name = GIMP_ICON_OBJECT_FLIP_VERTICAL,
+        .rotate    = 2,
+        .flip      = 1
+      }};
+
+    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    gtk_widget_show (hbox);
+
+    for (i = 0; i < G_N_ELEMENTS (buttons); i++)
+      {
+        const ButtonInfo *info = &buttons[i];
+        GtkWidget        *button;
+        GtkWidget        *image;
+
+        button = gtk_button_new ();
+        gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
+        gtk_widget_set_tooltip_text (button, info->tooltip);
+        gtk_widget_set_can_focus (button, FALSE);
+        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+        gtk_widget_show (button);
+
+        image = gtk_image_new_from_icon_name (info->icon_name,
+                                              GTK_ICON_SIZE_BUTTON);
+        gtk_container_add (GTK_CONTAINER (button), image);
+        gtk_widget_show (image);
+
+        g_object_set_data (G_OBJECT (button),
+                           "convolution-matrix-rotate",
+                           GINT_TO_POINTER (info->rotate));
+        g_object_set_data (G_OBJECT (button),
+                           "convolution-matrix-flip",
+                           GINT_TO_POINTER (info->flip));
+
+        g_signal_connect (button, "clicked",
+                          G_CALLBACK (convolution_matrix_rotate_flip),
+                          config);
+      }
+  }
+
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+  gtk_box_pack_start (GTK_BOX (main_vbox), vbox, FALSE, FALSE, 0);
+  gtk_widget_show (vbox);
+
+  /* divisor / offset spin scales */
+
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
   gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   scale = gimp_prop_widget_new (config, "divisor",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (hbox), scale, TRUE, TRUE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "offset",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (hbox), scale, TRUE, TRUE, 0);
   gtk_widget_show (scale);
 
+  /* rest of the properties */
+
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
   gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  vbox = _gimp_prop_gui_new_generic (config,
-                                     param_specs + 27, 4,
-                                     context,
-                                     create_picker_func,
-                                     picker_creator);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
+  vbox2 = _gimp_prop_gui_new_generic (config,
+                                      param_specs + 27, 4,
+                                      area, context,
+                                      create_picker_func, picker_creator);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
+  gtk_widget_show (vbox2);
 
-  vbox = _gimp_prop_gui_new_generic (config,
-                                     param_specs + 31,
-                                     n_param_specs - 31,
-                                     context,
-                                     create_picker_func,
-                                     picker_creator);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
+  vbox2 = _gimp_prop_gui_new_generic (config,
+                                      param_specs + 31,
+                                      n_param_specs - 31,
+                                      area, context,
+                                      create_picker_func, picker_creator);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
+  gtk_widget_show (vbox2);
 
   return main_vbox;
 }
@@ -550,6 +730,7 @@ GtkWidget *
 _gimp_prop_gui_new_channel_mixer (GObject              *config,
                                   GParamSpec          **param_specs,
                                   guint                 n_param_specs,
+                                  GeglRectangle        *area,
                                   GimpContext          *context,
                                   GimpCreatePickerFunc  create_picker_func,
                                   gpointer              picker_creator)
@@ -578,17 +759,17 @@ _gimp_prop_gui_new_channel_mixer (GObject              *config,
   gtk_widget_show (vbox);
 
   scale = gimp_prop_widget_new (config, "rr-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "rg-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "rb-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
@@ -602,17 +783,17 @@ _gimp_prop_gui_new_channel_mixer (GObject              *config,
   gtk_widget_show (vbox);
 
   scale = gimp_prop_widget_new (config, "gr-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "gg-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "gb-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
@@ -626,23 +807,23 @@ _gimp_prop_gui_new_channel_mixer (GObject              *config,
   gtk_widget_show (vbox);
 
   scale = gimp_prop_widget_new (config, "br-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "bg-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
   scale = gimp_prop_widget_new (config, "bb-gain",
-                                context, NULL, NULL, &label);
+                                area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);
   gtk_widget_show (scale);
 
 
   checkbox = gimp_prop_widget_new (config, "preserve-luminosity",
-                                   context, NULL, NULL, &label);
+                                   area, context, NULL, NULL, &label);
   gtk_box_pack_start (GTK_BOX (main_vbox), checkbox, FALSE, FALSE, 0);
   gtk_widget_show (checkbox);
 
@@ -654,6 +835,7 @@ GtkWidget *
 _gimp_prop_gui_new_diffraction_patterns (GObject              *config,
                                          GParamSpec          **param_specs,
                                          guint                 n_param_specs,
+                                         GeglRectangle        *area,
                                          GimpContext          *context,
                                          GimpCreatePickerFunc  create_picker_func,
                                          gpointer              picker_creator)
@@ -670,9 +852,8 @@ _gimp_prop_gui_new_diffraction_patterns (GObject              *config,
 
   vbox = _gimp_prop_gui_new_generic (config,
                                      param_specs + 0, 3,
-                                     context,
-                                     create_picker_func,
-                                     picker_creator);
+                                     area, context,
+                                     create_picker_func, picker_creator);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox,
                             gtk_label_new (_("Frequencies")));
@@ -680,9 +861,8 @@ _gimp_prop_gui_new_diffraction_patterns (GObject              *config,
 
   vbox = _gimp_prop_gui_new_generic (config,
                                      param_specs + 3, 3,
-                                     context,
-                                     create_picker_func,
-                                     picker_creator);
+                                     area, context,
+                                     create_picker_func, picker_creator);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox,
                             gtk_label_new (_("Contours")));
@@ -690,9 +870,8 @@ _gimp_prop_gui_new_diffraction_patterns (GObject              *config,
 
   vbox = _gimp_prop_gui_new_generic (config,
                                      param_specs + 6, 3,
-                                     context,
-                                     create_picker_func,
-                                     picker_creator);
+                                     area, context,
+                                     create_picker_func, picker_creator);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox,
                             gtk_label_new (_("Sharp Edges")));
@@ -700,9 +879,8 @@ _gimp_prop_gui_new_diffraction_patterns (GObject              *config,
 
   vbox = _gimp_prop_gui_new_generic (config,
                                      param_specs + 9, 3,
-                                     context,
-                                     create_picker_func,
-                                     picker_creator);
+                                     area, context,
+                                     create_picker_func, picker_creator);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox,
                             gtk_label_new (_("Other Options")));
