@@ -40,6 +40,7 @@
 #include "core/gimpcontext.h"
 
 #include "gimpcolorpanel.h"
+#include "gimpmessagebox.h"
 #include "gimpspinscale.h"
 #include "gimppropgui.h"
 #include "gimppropgui-constructors.h"
@@ -56,13 +57,17 @@ static GtkWidget * gimp_prop_kelvin_presets_new      (GObject       *config,
                                                       const gchar   *property_name);
 static void        gimp_prop_widget_new_seed_clicked (GtkButton     *button,
                                                       GtkAdjustment *adj);
-
+static gboolean    gimp_prop_string_to_boolean       (GBinding      *binding,
+                                                      const GValue  *from_value,
+                                                      GValue        *to_value,
+                                                      gpointer       user_data);
 
 /*  public functions  */
 
 GtkWidget *
 gimp_prop_widget_new (GObject              *config,
                       const gchar          *property_name,
+                      GeglRectangle        *area,
                       GimpContext          *context,
                       GimpCreatePickerFunc  create_picker_func,
                       gpointer              picker_creator,
@@ -75,7 +80,7 @@ gimp_prop_widget_new (GObject              *config,
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (config),
                                         property_name);
 
-  return gimp_prop_widget_new_from_pspec (config, pspec, context,
+  return gimp_prop_widget_new_from_pspec (config, pspec, area, context,
                                           create_picker_func, picker_creator,
                                           label);
 }
@@ -83,6 +88,7 @@ gimp_prop_widget_new (GObject              *config,
 GtkWidget *
 gimp_prop_widget_new_from_pspec (GObject               *config,
                                  GParamSpec            *pspec,
+                                 GeglRectangle         *area,
                                  GimpContext           *context,
                                  GimpCreatePickerFunc   create_picker_func,
                                  gpointer               picker_creator,
@@ -157,6 +163,7 @@ gimp_prop_widget_new_from_pspec (GObject               *config,
         {
           gdouble value;
 
+          /* Get the min and max for the given property. */
           _gimp_prop_widgets_get_numeric_values (config, pspec,
                                                  &value, &lower, &upper,
                                                  G_STRFUNC);
@@ -224,9 +231,36 @@ gimp_prop_widget_new_from_pspec (GObject               *config,
 
           widget = hbox;
         }
+      else if (area)
+        {
+          if (HAS_KEY (pspec, "unit", "pixel-coordinate") ||
+              HAS_KEY (pspec, "unit", "pixel-distance"))
+            {
+              if (HAS_KEY (pspec, "axis", "x"))
+                {
+                  g_printerr ("XXX setting width %d on %s\n",
+                              area->width, pspec->name);
+
+                  gimp_spin_scale_set_scale_limits (GIMP_SPIN_SCALE (widget),
+                                                    area->x,
+                                                    area->x + area->width);
+                }
+              else if (HAS_KEY (pspec, "axis","y"))
+                {
+                  g_printerr ("XXX setting height %d on %s\n",
+                              area->height, pspec->name);
+
+                  gimp_spin_scale_set_scale_limits (GIMP_SPIN_SCALE (widget),
+                                                    area->y,
+                                                    area->y + area->height);
+                }
+            }
+        }
     }
   else if (G_IS_PARAM_SPEC_STRING (pspec))
     {
+      *label = g_param_spec_get_nick (pspec);
+
       if (GIMP_IS_PARAM_SPEC_CONFIG_PATH (pspec))
         {
           widget =
@@ -244,17 +278,39 @@ gimp_prop_widget_new_from_pspec (GObject               *config,
           g_object_unref (buffer);
 
           widget = gtk_scrolled_window_new (NULL, NULL);
+          gtk_widget_set_size_request (widget, -1, 150);
           gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget),
                                                GTK_SHADOW_IN);
           gtk_container_add (GTK_CONTAINER (widget), view);
           gtk_widget_show (view);
         }
+      else if (HAS_KEY (pspec, "error", "true"))
+        {
+          GtkWidget *l;
+
+          widget = gimp_message_box_new (GIMP_ICON_WILBER_EEK);
+          gimp_message_box_set_primary_text (GIMP_MESSAGE_BOX (widget), "%s",
+                                             *label);
+          gimp_message_box_set_text (GIMP_MESSAGE_BOX (widget), "%s", "");
+
+          l = GIMP_MESSAGE_BOX (widget)->label[1];
+
+          g_object_bind_property (config, pspec->name,
+                                  l,  "label",
+                                  G_BINDING_SYNC_CREATE);
+          g_object_bind_property_full (config, pspec->name,
+                                       widget, "visible",
+                                       G_BINDING_SYNC_CREATE,
+                                       gimp_prop_string_to_boolean,
+                                       NULL,
+                                       NULL, NULL);
+          *label = NULL;
+        }
       else
         {
           widget = gimp_prop_entry_new (config, pspec->name, -1);
         }
 
-      *label = g_param_spec_get_nick (pspec);
     }
   else if (G_IS_PARAM_SPEC_BOOLEAN (pspec))
     {
@@ -286,7 +342,7 @@ gimp_prop_widget_new_from_pspec (GObject               *config,
         {
           button = create_picker_func (picker_creator,
                                        pspec->name,
-                                       GIMP_STOCK_COLOR_PICKER_GRAY,
+                                       GIMP_ICON_COLOR_PICKER_GRAY,
                                        _("Pick color from the image"));
           gtk_box_pack_start (GTK_BOX (widget), button, FALSE, FALSE, 0);
           gtk_widget_show (button);
@@ -307,6 +363,7 @@ gimp_prop_widget_new_from_pspec (GObject               *config,
 typedef GtkWidget * (* GimpPropGuiNewFunc) (GObject              *config,
                                             GParamSpec          **param_specs,
                                             guint                 n_param_specs,
+                                            GeglRectangle        *area,
                                             GimpContext          *context,
                                             GimpCreatePickerFunc  create_picker_func,
                                             gpointer              picker_creator);
@@ -316,6 +373,7 @@ static const struct
   const gchar        *config_type;
   GimpPropGuiNewFunc  gui_new_func;
 }
+
 gui_new_funcs[] =
 {
   { "GimpGegl-gegl-color-rotate-config",
@@ -335,6 +393,7 @@ GtkWidget *
 gimp_prop_gui_new (GObject              *config,
                    GType                 owner_type,
                    GParamFlags           flags,
+                   GeglRectangle        *area,
                    GimpContext          *context,
                    GimpCreatePickerFunc  create_picker_func,
                    gpointer              picker_creator)
@@ -385,6 +444,7 @@ gimp_prop_gui_new (GObject              *config,
 
               gui = gui_new_funcs[i].gui_new_func (config,
                                                    param_specs, n_param_specs,
+                                                   area,
                                                    context,
                                                    create_picker_func,
                                                    picker_creator);
@@ -488,7 +548,7 @@ gimp_prop_kelvin_presets_new (GObject     *config,
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 
   gtk_button_set_image (GTK_BUTTON (button),
-                        gtk_image_new_from_icon_name (GIMP_STOCK_MENU_LEFT,
+                        gtk_image_new_from_icon_name (GIMP_ICON_MENU_LEFT,
                                                       GTK_ICON_SIZE_MENU));
 
   menu = gtk_menu_new ();
@@ -534,3 +594,16 @@ gimp_prop_widget_new_seed_clicked (GtkButton     *button,
 
   gtk_adjustment_set_value (adj, value);
 }
+
+static gboolean
+gimp_prop_string_to_boolean (GBinding     *binding,
+                             const GValue *from_value,
+                             GValue       *to_value,
+                             gpointer      user_data)
+{
+  const gchar *string = g_value_get_string (from_value);
+
+  g_value_set_boolean (to_value, string && *string);
+
+  return TRUE;
+}
