@@ -2,7 +2,7 @@
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * gimpsamplepointeditor.c
- * Copyright (C) 2005 Michael Natterer <mitch@gimp.org>
+ * Copyright (C) 2005-2016 Michael Natterer <mitch@gimp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 
 #include "widgets-types.h"
 
+#include "config/gimpcoreconfig.h"
+
 #include "core/gimp.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-pick-color.h"
@@ -41,6 +43,9 @@
 #include "gimp-intl.h"
 
 
+#define N_POINTS 4
+
+
 enum
 {
   PROP_0,
@@ -135,7 +140,7 @@ gimp_sample_point_editor_init (GimpSamplePointEditor *editor)
   gtk_box_pack_start (GTK_BOX (editor), editor->table, FALSE, FALSE, 0);
   gtk_widget_show (editor->table);
 
-  for (i = 0; i < 4; i++)
+  for (i = 0; i < N_POINTS; i++)
     {
       GtkWidget *frame;
 
@@ -248,6 +253,8 @@ gimp_sample_point_editor_set_image (GimpImageEditor *image_editor,
                                     GimpImage       *image)
 {
   GimpSamplePointEditor *editor = GIMP_SAMPLE_POINT_EDITOR (image_editor);
+  GimpColorConfig       *config = NULL;
+  gint                   i;
 
   if (image_editor->image)
     {
@@ -283,6 +290,14 @@ gimp_sample_point_editor_set_image (GimpImageEditor *image_editor,
       g_signal_connect (gimp_image_get_projection (image), "update",
                         G_CALLBACK (gimp_sample_point_editor_proj_update),
                         editor);
+
+      config = image->gimp->config->color_management;
+    }
+
+  for (i = 0; i < N_POINTS; i++)
+    {
+      gimp_color_frame_set_color_config (GIMP_COLOR_FRAME (editor->color_frames[i]),
+                                         config);
     }
 
   gimp_sample_point_editor_points_changed (editor);
@@ -317,7 +332,7 @@ gimp_sample_point_editor_set_sample_merged (GimpSamplePointEditor *editor,
 
       editor->sample_merged = sample_merged;
 
-      for (i = 0; i < 4; i++)
+      for (i = 0; i < N_POINTS; i++)
         editor->dirty[i] = TRUE;
 
       gimp_sample_point_editor_dirty (editor, -1);
@@ -359,7 +374,7 @@ gimp_sample_point_editor_point_moved (GimpImage             *image,
 {
   gint i = g_list_index (gimp_image_get_sample_points (image), sample_point);
 
-  if (i < 4)
+  if (i < N_POINTS)
     gimp_sample_point_editor_dirty (editor, i);
 }
 
@@ -380,7 +395,7 @@ gimp_sample_point_editor_proj_update (GimpImage             *image,
 
   sample_points = gimp_image_get_sample_points (image_editor->image);
 
-  n_points = MIN (4, g_list_length (sample_points));
+  n_points = MIN (N_POINTS, g_list_length (sample_points));
 
   for (i = 0, list = sample_points;
        i < n_points;
@@ -411,7 +426,7 @@ gimp_sample_point_editor_points_changed (GimpSamplePointEditor *editor)
   if (image_editor->image)
     {
       sample_points = gimp_image_get_sample_points (image_editor->image);
-      n_points = MIN (4, g_list_length (sample_points));
+      n_points = MIN (N_POINTS, g_list_length (sample_points));
     }
 
   for (i = 0; i < n_points; i++)
@@ -420,7 +435,7 @@ gimp_sample_point_editor_points_changed (GimpSamplePointEditor *editor)
       editor->dirty[i] = TRUE;
     }
 
-  for (i = n_points; i < 4; i++)
+  for (i = n_points; i < N_POINTS; i++)
     {
       gtk_widget_set_sensitive (editor->color_frames[i], FALSE);
       gimp_color_frame_set_invalid (GIMP_COLOR_FRAME (editor->color_frames[i]));
@@ -462,7 +477,7 @@ gimp_sample_point_editor_update (GimpSamplePointEditor *editor)
 
   sample_points = gimp_image_get_sample_points (image_editor->image);
 
-  n_points = MIN (4, g_list_length (sample_points));
+  n_points = MIN (N_POINTS, g_list_length (sample_points));
 
   for (i = 0, list = sample_points;
        i < n_points;
