@@ -138,7 +138,7 @@ gimp_save_dialog_new (Gimp *gimp)
                        "title",                 _("Save Image"),
                        "role",                  "gimp-file-save",
                        "help-id",               GIMP_HELP_FILE_SAVE,
-                       "stock-id",              GTK_STOCK_SAVE,
+                       "ok-button-label",       _("_Save"),
 
                        "automatic-label",       _("By Extension"),
                        "automatic-help-id",     GIMP_HELP_FILE_SAVE_BY_EXTENSION,
@@ -345,7 +345,7 @@ gimp_save_dialog_add_compat_toggle (GimpSaveDialog *dialog)
 
   /* Additional information explaining what this mode does. */
   dialog->compat_info = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (dialog->compat_info), 0.0, 0.0);
+  gtk_label_set_xalign (GTK_LABEL (dialog->compat_info), 0.0);
   gimp_label_set_attributes (GTK_LABEL (dialog->compat_info),
                              PANGO_ATTR_STYLE, PANGO_STYLE_ITALIC,
                              -1);
