@@ -47,7 +47,7 @@
 #include "core/gimpimage-sample-points.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimpitemstack.h"
-#include "core/gimplayer-floating-sel.h"
+#include "core/gimplayer-floating-selection.h"
 #include "core/gimplayer-new.h"
 #include "core/gimplayermask.h"
 #include "core/gimpparasitelist.h"
@@ -321,7 +321,9 @@ xcf_load_image (Gimp     *gimp,
                                        "gimp-metadata");
   if (parasite)
     {
-      GimpImagePrivate *private = GIMP_IMAGE_GET_PRIVATE (image);
+      GimpImagePrivate *private    = GIMP_IMAGE_GET_PRIVATE (image);
+      const gchar      *xmp_data   = gimp_parasite_data (parasite);
+      gint              xmp_length = gimp_parasite_data_size (parasite);
 
       if (has_metadata)
         {
@@ -329,6 +331,14 @@ xcf_load_image (Gimp     *gimp,
                       "has both 'gimp-image-metadata' and 'gimp-metadata' "
                       "parasites, dropping old 'gimp-metadata'\n");
         }
+      else if (xmp_length < 14 ||
+               strncmp (xmp_data, "GIMP_XMP_1", 10) != 0)
+        {
+          gimp_message (gimp, G_OBJECT (info->progress),
+                        GIMP_MESSAGE_WARNING,
+                        _("Corrupt 'gimp-metadata' parasite discovered.\n"
+                          "XMP data could not be migrated."));
+        }
       else
         {
           GimpMetadata *metadata = gimp_image_get_metadata (image);
@@ -340,8 +350,8 @@ xcf_load_image (Gimp     *gimp,
             metadata = gimp_metadata_new ();
 
           if (! gimp_metadata_set_from_xmp (metadata,
-                                            gimp_parasite_data (parasite),
-                                            gimp_parasite_data_size (parasite),
+                                            (const guint8 *) xmp_data + 10,
+                                            xmp_length - 10,
                                             &my_error))
             {
               gimp_message (gimp, G_OBJECT (info->progress),
@@ -518,7 +528,8 @@ xcf_load_image (Gimp     *gimp,
   if (info->active_channel)
     gimp_image_set_active_channel (image, info->active_channel);
 
-  gimp_image_set_file (image, info->file);
+  if (info->file)
+    gimp_image_set_file (image, info->file);
 
   if (info->tattoo_state > 0)
     gimp_image_set_tattoo_state (image, info->tattoo_state);
@@ -679,7 +690,7 @@ xcf_load_image_props (XcfInfo   *info,
                 gimp_message (info->gimp, G_OBJECT (info->progress),
                               GIMP_MESSAGE_ERROR,
                               "Unknown compression type: %d",
-			      (gint) compression);
+                              (gint) compression);
                 return FALSE;
               }
 
@@ -1008,6 +1019,15 @@ xcf_load_layer_props (XcfInfo    *info,
           }
           break;
 
+        case PROP_COLOR_TAG:
+          {
+            GimpColorTag color_tag;
+
+            info->cp += xcf_read_int32 (info->input, (guint32 *) &color_tag, 1);
+            gimp_item_set_color_tag (GIMP_ITEM (*layer), color_tag, FALSE);
+          }
+          break;
+
         case PROP_LOCK_CONTENT:
           {
             gboolean lock_content;
@@ -1073,10 +1093,46 @@ xcf_load_layer_props (XcfInfo    *info,
 
             info->cp += xcf_read_int32 (info->input, &mode, 1);
 
-            if (mode == GIMP_OVERLAY_MODE)
-              mode = GIMP_SOFTLIGHT_MODE;
+            if (mode == GIMP_LAYER_MODE_OVERLAY_LEGACY)
+              mode = GIMP_LAYER_MODE_SOFTLIGHT_LEGACY;
 
-            gimp_layer_set_mode (*layer, (GimpLayerModeEffects) mode, FALSE);
+            gimp_layer_set_mode (*layer, (GimpLayerMode) mode, FALSE);
+          }
+          break;
+
+        case PROP_BLEND_SPACE:
+          {
+            guint32 blend_space;
+
+            info->cp += xcf_read_int32 (info->input, &blend_space, 1);
+
+            gimp_layer_set_blend_space (*layer,
+                                        (GimpLayerColorSpace) blend_space,
+                                        FALSE);
+          }
+          break;
+
+        case PROP_COMPOSITE_SPACE:
+          {
+            guint32 composite_space;
+
+            info->cp += xcf_read_int32 (info->input, &composite_space, 1);
+
+            gimp_layer_set_composite_space (*layer,
+                                            (GimpLayerColorSpace) composite_space,
+                                            FALSE);
+          }
+          break;
+
+        case PROP_COMPOSITE_MODE:
+          {
+            guint32 composite_mode;
+
+            info->cp += xcf_read_int32 (info->input, &composite_mode, 1);
+
+            gimp_layer_set_composite_mode (*layer,
+                                           (GimpLayerCompositeMode) composite_mode,
+                                           FALSE);
           }
           break;
 
@@ -1131,6 +1187,18 @@ xcf_load_layer_props (XcfInfo    *info,
         case PROP_GROUP_ITEM:
           {
             GimpLayer *group;
+            gboolean   is_active_layer;
+
+            /* We're going to delete *layer, Don't leave its pointers
+             * in @info.  After that, we'll restore them back with the
+             * new pointer. See bug #767873.
+             */
+            is_active_layer = (*layer == info->active_layer);
+            if (is_active_layer)
+              info->active_layer = NULL;
+
+            if (*layer == info->floating_sel)
+              info->floating_sel = NULL;
 
             group = gimp_group_layer_new (image);
 
@@ -1140,6 +1208,13 @@ xcf_load_layer_props (XcfInfo    *info,
             g_object_ref_sink (*layer);
             g_object_unref (*layer);
             *layer = group;
+
+            if (is_active_layer)
+              info->active_layer = *layer;
+
+            /* Don't restore info->floating_sel because group layers
+             * can't be floating selections
+             */
           }
           break;
 
@@ -1210,6 +1285,12 @@ xcf_load_channel_props (XcfInfo      *info,
           {
             GimpChannel *mask;
 
+            /* We're going to delete *channel, Don't leave its pointer
+             * in @info. See bug #767873.
+             */
+            if (*channel == info->active_channel)
+              info->active_channel = NULL;
+
             mask =
               gimp_selection_new (image,
                                   gimp_item_get_width  (GIMP_ITEM (*channel)),
@@ -1224,6 +1305,10 @@ xcf_load_channel_props (XcfInfo      *info,
             *channel = mask;
             (*channel)->boundary_known = FALSE;
             (*channel)->bounds_known   = FALSE;
+
+            /* Don't restore info->active_channel because the
+             * selection can't be the active channel
+             */
           }
           break;
 
@@ -1255,6 +1340,15 @@ xcf_load_channel_props (XcfInfo      *info,
           }
           break;
 
+        case PROP_COLOR_TAG:
+          {
+            GimpColorTag color_tag;
+
+            info->cp += xcf_read_int32 (info->input, (guint32 *) &color_tag, 1);
+            gimp_item_set_color_tag (GIMP_ITEM (*channel), color_tag, FALSE);
+          }
+          break;
+
         case PROP_LINKED:
           {
             gboolean linked;
@@ -1467,7 +1561,7 @@ xcf_load_layer (XcfInfo    *info,
 
   /* create a new layer */
   layer = gimp_layer_new (image, width, height,
-                          format, name, 255, GIMP_NORMAL_MODE);
+                          format, name, 255, GIMP_LAYER_MODE_NORMAL_LEGACY);
   g_free (name);
   if (! layer)
     return NULL;
