@@ -500,6 +500,8 @@ xcf_save_layer_props (XcfInfo    *info,
                                   gimp_item_get_visible (GIMP_ITEM (layer))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LINKED, error,
                                   gimp_item_get_linked (GIMP_ITEM (layer))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_COLOR_TAG, error,
+                                  gimp_item_get_color_tag (GIMP_ITEM (layer))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_CONTENT, error,
                                   gimp_item_get_lock_content (GIMP_ITEM (layer))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_ALPHA, error,
@@ -532,6 +534,12 @@ xcf_save_layer_props (XcfInfo    *info,
                                   offset_x, offset_y));
   xcf_check_error (xcf_save_prop (info, image, PROP_MODE, error,
                                   gimp_layer_get_mode (layer)));
+  xcf_check_error (xcf_save_prop (info, image, PROP_BLEND_SPACE, error,
+                                  gimp_layer_get_blend_space (layer)));
+  xcf_check_error (xcf_save_prop (info, image, PROP_COMPOSITE_SPACE, error,
+                                  gimp_layer_get_composite_space (layer)));
+  xcf_check_error (xcf_save_prop (info, image, PROP_COMPOSITE_MODE, error,
+                                  gimp_layer_get_composite_mode (layer)));
   xcf_check_error (xcf_save_prop (info, image, PROP_TATTOO, error,
                                   gimp_item_get_tattoo (GIMP_ITEM (layer))));
 
@@ -596,6 +604,8 @@ xcf_save_channel_props (XcfInfo      *info,
                                   gimp_item_get_visible (GIMP_ITEM (channel))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LINKED, error,
                                   gimp_item_get_linked (GIMP_ITEM (channel))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_COLOR_TAG, error,
+                                  gimp_item_get_color_tag (GIMP_ITEM (channel))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_CONTENT, error,
                                   gimp_item_get_lock_content (GIMP_ITEM (channel))));
   xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_POSITION, error,
@@ -722,8 +732,8 @@ xcf_save_prop (XcfInfo    *info,
         mode = va_arg (args, gint32);
         size = 4;
 
-        if (mode == GIMP_OVERLAY_MODE)
-          mode = GIMP_SOFTLIGHT_MODE;
+        if (mode == GIMP_LAYER_MODE_OVERLAY_LEGACY)
+          mode = GIMP_LAYER_MODE_SOFTLIGHT_LEGACY;
 
         xcf_write_prop_type_check_error (info, prop_type);
         xcf_write_int32_check_error (info, &size, 1);
@@ -731,6 +741,45 @@ xcf_save_prop (XcfInfo    *info,
       }
       break;
 
+    case PROP_BLEND_SPACE:
+      {
+        gint32 blend_space;
+
+        blend_space = va_arg (args, gint32);
+        size = 4;
+
+        xcf_write_prop_type_check_error (info, prop_type);
+        xcf_write_int32_check_error (info, &size, 1);
+        xcf_write_int32_check_error (info, (guint32 *) &blend_space, 1);
+      }
+      break;
+
+    case PROP_COMPOSITE_SPACE:
+      {
+        gint32 composite_space;
+
+        composite_space = va_arg (args, gint32);
+        size = 4;
+
+        xcf_write_prop_type_check_error (info, prop_type);
+        xcf_write_int32_check_error (info, &size, 1);
+        xcf_write_int32_check_error (info, (guint32 *) &composite_space, 1);
+      }
+      break;
+
+    case PROP_COMPOSITE_MODE:
+      {
+        gint32 composite_mode;
+
+        composite_mode = va_arg (args, gint32);
+        size = 4;
+
+        xcf_write_prop_type_check_error (info, prop_type);
+        xcf_write_int32_check_error (info, &size, 1);
+        xcf_write_int32_check_error (info, (guint32 *) &composite_mode, 1);
+      }
+      break;
+
     case PROP_VISIBLE:
       {
         guint32 visible;
@@ -757,6 +806,19 @@ xcf_save_prop (XcfInfo    *info,
       }
       break;
 
+    case PROP_COLOR_TAG:
+      {
+        guint32 color_tag;
+
+        color_tag = va_arg (args, gint32);
+        size = 4;
+
+        xcf_write_prop_type_check_error (info, prop_type);
+        xcf_write_int32_check_error (info, &size, 1);
+        xcf_write_int32_check_error (info, &color_tag, 1);
+      }
+      break;
+
     case PROP_LOCK_CONTENT:
       {
         guint32 lock_content;
@@ -890,12 +952,19 @@ xcf_save_prop (XcfInfo    *info,
 
     case PROP_GUIDES:
       {
+        GList *iter;
         GList *guides;
         gint   n_guides;
 
         guides = va_arg (args, GList *);
         n_guides = g_list_length (guides);
 
+        for (iter = guides; iter; iter = g_list_next (iter))
+          {
+            /* Do not save custom guides. */
+            if (gimp_guide_is_custom (GIMP_GUIDE (iter->data)))
+              n_guides--;
+          }
         size = n_guides * (4 + 1);
 
         xcf_write_prop_type_check_error (info, prop_type);
@@ -907,6 +976,9 @@ xcf_save_prop (XcfInfo    *info,
             gint32     position = gimp_guide_get_position (guide);
             gint8      orientation;
 
+            if (gimp_guide_is_custom (guide))
+              continue;
+
             switch (gimp_guide_get_orientation (guide))
               {
               case GIMP_ORIENTATION_HORIZONTAL:
@@ -1941,7 +2013,7 @@ xcf_save_vectors (XcfInfo    *info,
       tattoo        = gimp_item_get_tattoo (GIMP_ITEM (vectors));
       parasites     = gimp_item_get_parasites (GIMP_ITEM (vectors));
       num_parasites = gimp_parasite_list_persistent_length (parasites);
-      num_strokes   = g_list_length (vectors->strokes);
+      num_strokes   = g_queue_get_length (vectors->strokes);
 
       xcf_write_string_check_error (info, (gchar **) &name, 1);
       xcf_write_int32_check_error  (info, &tattoo,          1);
@@ -1952,7 +2024,7 @@ xcf_save_vectors (XcfInfo    *info,
 
       xcf_check_error (xcf_save_parasite_list (info, parasites, error));
 
-      for (stroke_list = g_list_first (vectors->strokes);
+      for (stroke_list = g_list_first (vectors->strokes->head);
            stroke_list;
            stroke_list = g_list_next (stroke_list))
         {
