@@ -55,6 +55,22 @@
 #define TYPE_GRAYA_DBL      (FLOAT_SH(1)|COLORSPACE_SH(PT_GRAY)|EXTRA_SH(1)|CHANNELS_SH(1)|BYTES_SH(0))
 #endif
 
+#ifndef TYPE_CMYKA_DBL
+#define TYPE_CMYKA_DBL      (FLOAT_SH(1)|COLORSPACE_SH(PT_CMYK)|EXTRA_SH(1)|CHANNELS_SH(4)|BYTES_SH(0))
+#endif
+
+#ifndef TYPE_CMYKA_HALF_FLT
+#define TYPE_CMYKA_HALF_FLT (FLOAT_SH(1)|COLORSPACE_SH(PT_CMYK)|EXTRA_SH(1)|CHANNELS_SH(4)|BYTES_SH(2))
+#endif
+
+#ifndef TYPE_CMYKA_FLT
+#define TYPE_CMYKA_FLT      (FLOAT_SH(1)|COLORSPACE_SH(PT_CMYK)|EXTRA_SH(1)|CHANNELS_SH(4)|BYTES_SH(4))
+#endif
+
+#ifndef TYPE_CMYKA_16
+#define TYPE_CMYKA_16       (COLORSPACE_SH(PT_CMYK)|EXTRA_SH(1)|CHANNELS_SH(4)|BYTES_SH(2))
+#endif
+
 
 /**
  * SECTION: gimpcolorprofile
@@ -184,9 +200,10 @@ gimp_color_profile_new_from_file (GFile   *file,
 
   if (path)
     {
-      GMappedFile  *mapped;
+      GMappedFile *mapped;
 
       mapped = g_mapped_file_new (path, FALSE, error);
+      g_free (path);
 
       if (! mapped)
         return NULL;
@@ -357,6 +374,37 @@ gimp_color_profile_new_from_lcms_profile (gpointer   lcms_profile,
   return NULL;
 }
 
+/**
+ * gimp_color_profile_save_to_file:
+ * profile: a #GimpColorProfile
+ * @file:   a #GFile
+ * @error:  return location for #GError
+ *
+ * This function saves @profile to @file as ICC profile.
+ *
+ * Return value: %TRUE on success, %FALSE if an error occured.
+ *
+ * Since: 2.10
+ **/
+gboolean
+gimp_color_profile_save_to_file (GimpColorProfile  *profile,
+                                 GFile             *file,
+                                 GError           **error)
+{
+  g_return_val_if_fail (GIMP_IS_COLOR_PROFILE (profile), FALSE);
+  g_return_val_if_fail (G_IS_FILE (file), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_file_replace_contents (file,
+                                  (const gchar *) profile->priv->data,
+                                  profile->priv->length,
+                                  NULL, FALSE,
+                                  G_FILE_CREATE_NONE,
+                                  NULL,
+                                  NULL,
+                                  error);
+}
+
 /**
  * gimp_color_profile_get_icc_profile:
  * @profile: a #GimpColorProfile
@@ -521,7 +569,7 @@ gimp_color_profile_get_copyright (GimpColorProfile *profile)
  * string that can be used to label the profile in a user interface.
  *
  * Unlike gimp_color_profile_get_description(), this function always
- * returns a string (as a fallback, it returns "(unnamed profile)".
+ * returns a string (as a fallback, it returns "(unnamed profile)").
  *
  * Return value: the @profile's label. The returned value belongs to
  *               @profile and must not be modified or freed.
@@ -840,7 +888,7 @@ gimp_color_profile_new_from_color_profile (GimpColorProfile *profile,
       curve = cmsBuildGamma (NULL, 1.00);
 
       gimp_color_profile_set_tag (target_profile, cmsSigProfileDescriptionTag,
-                                  "linear variant generated by GIMP");
+                                  "linear TRC variant generated by GIMP");
     }
   else
     {
@@ -851,7 +899,7 @@ gimp_color_profile_new_from_color_profile (GimpColorProfile *profile,
       curve = cmsBuildParametricToneCurve (NULL, 4, srgb_parameters);
 
       gimp_color_profile_set_tag (target_profile, cmsSigProfileDescriptionTag,
-                                  "sRGB gamma variant generated by GIMP");
+                                  "sRGB TRC variant generated by GIMP");
     }
 
   if (gimp_color_profile_is_rgb (profile))
@@ -921,7 +969,7 @@ gimp_color_profile_new_from_color_profile (GimpColorProfile *profile,
 }
 
 /**
- * gimp_color_profile_new_srgb_gamma_from_color_profile:
+ * gimp_color_profile_new_srgb_trc_from_color_profile:
  * @profile: a #GimpColorProfile
  *
  * This function creates a new RGB #GimpColorProfile with a sRGB gamma
@@ -933,7 +981,7 @@ gimp_color_profile_new_from_color_profile (GimpColorProfile *profile,
  * Since: 2.10
  **/
 GimpColorProfile *
-gimp_color_profile_new_srgb_gamma_from_color_profile (GimpColorProfile *profile)
+gimp_color_profile_new_srgb_trc_from_color_profile (GimpColorProfile *profile)
 {
   g_return_val_if_fail (GIMP_IS_COLOR_PROFILE (profile), NULL);
 
@@ -941,7 +989,7 @@ gimp_color_profile_new_srgb_gamma_from_color_profile (GimpColorProfile *profile)
 }
 
 /**
- * gimp_color_profile_new_linear_gamma_from_color_profile:
+ * gimp_color_profile_new_linear_from_color_profile:
  * @profile: a #GimpColorProfile
  *
  * This function creates a new RGB #GimpColorProfile with a linear TRC
@@ -953,7 +1001,7 @@ gimp_color_profile_new_srgb_gamma_from_color_profile (GimpColorProfile *profile)
  * Since: 2.10
  **/
 GimpColorProfile *
-gimp_color_profile_new_linear_gamma_from_color_profile (GimpColorProfile *profile)
+gimp_color_profile_new_linear_from_color_profile (GimpColorProfile *profile)
 {
   g_return_val_if_fail (GIMP_IS_COLOR_PROFILE (profile), NULL);
 
@@ -1102,11 +1150,11 @@ gimp_color_profile_new_rgb_srgb_linear_internal (void)
   cmsFreeToneCurve (curve[0]);
 
   gimp_color_profile_set_tag (profile, cmsSigProfileDescriptionTag,
-                              "GIMP built-in Linear RGB");
+                              "GIMP built-in Linear sRGB");
   gimp_color_profile_set_tag (profile, cmsSigDeviceMfgDescTag,
                               "GIMP");
   gimp_color_profile_set_tag (profile, cmsSigDeviceModelDescTag,
-                              "Linear RGB");
+                              "Linear sRGB");
   gimp_color_profile_set_tag (profile, cmsSigCopyrightTag,
                               "Public Domain");
 
@@ -1225,7 +1273,7 @@ gimp_color_profile_new_rgb_adobe (void)
 }
 
 static cmsHPROFILE *
-gimp_color_profile_new_gray_srgb_internal (void)
+gimp_color_profile_new_d65_gray_srgb_trc_internal (void)
 {
   cmsHPROFILE profile;
 
@@ -1243,11 +1291,11 @@ gimp_color_profile_new_gray_srgb_internal (void)
   cmsFreeToneCurve (curve);
 
   gimp_color_profile_set_tag (profile, cmsSigProfileDescriptionTag,
-                              "GIMP built-in sRGB-Gamma Grayscale");
+                              "GIMP built-in D65 Grayscale with sRGB TRC");
   gimp_color_profile_set_tag (profile, cmsSigDeviceMfgDescTag,
                               "GIMP");
   gimp_color_profile_set_tag (profile, cmsSigDeviceModelDescTag,
-                              "sRGB-Gamma Grayscale");
+                              "D65 Grayscale with sRGB TRC");
   gimp_color_profile_set_tag (profile, cmsSigCopyrightTag,
                               "Public Domain");
 
@@ -1255,7 +1303,7 @@ gimp_color_profile_new_gray_srgb_internal (void)
 }
 
 /**
- * gimp_color_profile_new_gray_srgb
+ * gimp_color_profile_new_d65_gray_srgb_trc
  *
  * This function creates a grayscale #GimpColorProfile with an
  * sRGB TRC. See gimp_color_profile_new_srgb().
@@ -1265,7 +1313,7 @@ gimp_color_profile_new_gray_srgb_internal (void)
  * Since: 2.10
  **/
 GimpColorProfile *
-gimp_color_profile_new_gray_srgb (void)
+gimp_color_profile_new_d65_gray_srgb_trc (void)
 {
   static GimpColorProfile *profile = NULL;
 
@@ -1274,7 +1322,7 @@ gimp_color_profile_new_gray_srgb (void)
 
   if (G_UNLIKELY (profile == NULL))
     {
-      cmsHPROFILE lcms_profile = gimp_color_profile_new_gray_srgb_internal ();
+      cmsHPROFILE lcms_profile = gimp_color_profile_new_d65_gray_srgb_trc_internal ();
 
       profile = gimp_color_profile_new_from_lcms_profile (lcms_profile, NULL);
 
@@ -1287,7 +1335,7 @@ gimp_color_profile_new_gray_srgb (void)
 }
 
 static cmsHPROFILE
-gimp_color_profile_new_gray_srgb_linear_internal (void)
+gimp_color_profile_new_d65_gray_linear_internal (void)
 {
   cmsHPROFILE profile;
 
@@ -1301,11 +1349,11 @@ gimp_color_profile_new_gray_srgb_linear_internal (void)
   cmsFreeToneCurve (curve);
 
   gimp_color_profile_set_tag (profile, cmsSigProfileDescriptionTag,
-                              "GIMP built-in Linear Grayscale");
+                              "GIMP built-in D65 Linear Grayscale");
   gimp_color_profile_set_tag (profile, cmsSigDeviceMfgDescTag,
                               "GIMP");
   gimp_color_profile_set_tag (profile, cmsSigDeviceModelDescTag,
-                              "Linear Grayscale");
+                              "D65 Linear Grayscale");
   gimp_color_profile_set_tag (profile, cmsSigCopyrightTag,
                               "Public Domain");
 
@@ -1313,7 +1361,7 @@ gimp_color_profile_new_gray_srgb_linear_internal (void)
 }
 
 /**
- * gimp_color_profile_new_gray_srgb_linear_gray:
+ * gimp_color_profile_new_d65_gray_srgb_gray:
  *
  * This function creates a profile for babl_model("Y"). Please
  * somebody write someting smarter here.
@@ -1323,7 +1371,7 @@ gimp_color_profile_new_gray_srgb_linear_internal (void)
  * Since: 2.10
  **/
 GimpColorProfile *
-gimp_color_profile_new_gray_srgb_linear (void)
+gimp_color_profile_new_d65_gray_linear (void)
 {
   static GimpColorProfile *profile = NULL;
 
@@ -1332,7 +1380,73 @@ gimp_color_profile_new_gray_srgb_linear (void)
 
   if (G_UNLIKELY (profile == NULL))
     {
-      cmsHPROFILE lcms_profile = gimp_color_profile_new_gray_srgb_linear_internal ();
+      cmsHPROFILE lcms_profile = gimp_color_profile_new_d65_gray_linear_internal ();
+
+      profile = gimp_color_profile_new_from_lcms_profile (lcms_profile, NULL);
+
+      cmsCloseProfile (lcms_profile);
+    }
+
+  data = gimp_color_profile_get_icc_profile (profile, &length);
+
+  return gimp_color_profile_new_from_icc_profile (data, length, NULL);
+}
+
+static cmsHPROFILE *
+gimp_color_profile_new_d50_gray_lab_trc_internal (void)
+{
+  cmsHPROFILE profile;
+
+  /* white point is D50 from the ICC profile illuminant specs */
+  cmsCIExyY whitepoint = {0.345702915, 0.358538597, 1.0};
+
+  cmsFloat64Number lab_parameters[5] =
+    { 3.0, 1.0 / 1.16,  0.16 / 1.16, 2700.0 / 24389.0, 0.08000  };
+
+  cmsToneCurve *curve = cmsBuildParametricToneCurve (NULL, 4,
+                                                     lab_parameters);
+
+  profile = cmsCreateGrayProfile (&whitepoint, curve);
+
+  cmsFreeToneCurve (curve);
+
+  gimp_color_profile_set_tag (profile, cmsSigProfileDescriptionTag,
+                              "GIMP built-in D50 Grayscale with LAB L TRC");
+  gimp_color_profile_set_tag (profile, cmsSigDeviceMfgDescTag,
+                              "GIMP");
+  gimp_color_profile_set_tag (profile, cmsSigDeviceModelDescTag,
+                              "D50 Grayscale with LAB L TRC");
+  gimp_color_profile_set_tag (profile, cmsSigCopyrightTag,
+                              "Public Domain");
+
+  return profile;
+}
+
+
+/**
+ * gimp_color_profile_new_d50_gray_lab_trc
+ *
+ * This function creates a grayscale #GimpColorProfile with the
+ * D50 ICC profile illuminant as the profile white point and the
+ * LAB companding curve as the TRC.
+ *
+ * Return value: a gray profile with the D50 ICC profile illuminant
+ * as the profile white point and the LAB companding curve as the TRC.
+ * as the TRC.
+ *
+ * Since: 2.10
+ **/
+GimpColorProfile *
+gimp_color_profile_new_d50_gray_lab_trc (void)
+{
+  static GimpColorProfile *profile = NULL;
+
+  const guint8 *data;
+  gsize         length;
+
+  if (G_UNLIKELY (profile == NULL))
+    {
+      cmsHPROFILE lcms_profile = gimp_color_profile_new_d50_gray_lab_trc_internal ();
 
       profile = gimp_color_profile_new_from_lcms_profile (lcms_profile, NULL);
 
@@ -1355,10 +1469,10 @@ gimp_color_profile_new_gray_srgb_linear (void)
  * @format, unless lcms doesn't support @format.
  *
  * Note that this function currently only supports RGB, RGBA, R'G'B',
- * R'G'B'A and the cairo-RGB24 and cairo-ARGB32 formats.
+ * R'G'B'A, Y, YA, Y', Y'A and the cairo-RGB24 and cairo-ARGB32 formats.
  *
  * Return value: the #Babl format to be used instead of @format, or %NULL
- *               is the passed @format is not supported at all.
+ *               if the passed @format is not supported at all.
  *
  * Since: 2.10
  **/
@@ -1370,8 +1484,10 @@ gimp_color_profile_get_format (const Babl *format,
   const Babl *type;
   const Babl *model;
   gboolean    has_alpha;
-  gboolean    gray;
-  gboolean    linear;
+  gboolean    rgb    = FALSE;
+  gboolean    gray   = FALSE;
+  gboolean    cmyk   = FALSE;
+  gboolean    linear = FALSE;
 
   g_return_val_if_fail (format != NULL, NULL);
   g_return_val_if_fail (lcms_format != NULL, NULL);
@@ -1382,9 +1498,13 @@ gimp_color_profile_get_format (const Babl *format,
 
   if (format == babl_format ("cairo-RGB24"))
     {
-      *lcms_format = TYPE_RGB_8;
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+      *lcms_format = TYPE_BGRA_8;
+#else
+      *lcms_format = TYPE_ARGB_8;
+#endif
 
-      return babl_format ("R'G'B' u8");
+      return format;
     }
   else if (format == babl_format ("cairo-ARGB32"))
     {
@@ -1392,29 +1512,70 @@ gimp_color_profile_get_format (const Babl *format,
 
       return babl_format ("R'G'B'A u8");
     }
-  else if (model == babl_model ("RGB") ||
-           model == babl_model ("RGBA"))
+  else if (model == babl_model ("RGB")  ||
+           model == babl_model ("RGBA") ||
+           model == babl_model ("RaGaBaA"))
     {
-      gray   = FALSE;
+      rgb    = TRUE;
       linear = TRUE;
     }
-  else if (model == babl_model ("R'G'B'") ||
-           model == babl_model ("R'G'B'A"))
+  else if (model == babl_model ("R'G'B'")  ||
+           model == babl_model ("R'G'B'A") ||
+           model == babl_model ("R'aG'aB'aA"))
     {
-      gray   = FALSE;
-      linear = FALSE;
+      rgb = TRUE;
     }
-  else if (model == babl_model ("Y") ||
-           model == babl_model ("YA"))
+  else if (model == babl_model ("Y")  ||
+           model == babl_model ("YA") ||
+           model == babl_model ("YaA"))
     {
       gray   = TRUE;
       linear = TRUE;
     }
-  else if (model == babl_model ("Y'") ||
-           model == babl_model ("Y'A"))
+  else if (model == babl_model ("Y'")  ||
+           model == babl_model ("Y'A") ||
+           model == babl_model ("Y'aA"))
     {
-      gray   = TRUE;
-      linear = FALSE;
+      gray = TRUE;
+    }
+  else if (model == babl_model ("CMYK"))
+#if 0
+    /* FIXME missing from babl */
+           || model == babl_model ("CMYKA"))
+#endif
+    {
+      cmyk = TRUE;
+    }
+  else if (model == babl_model ("CIE Lab") ||
+           model == babl_model ("CIE Lab alpha"))
+    {
+      if (has_alpha)
+        {
+          *lcms_format = TYPE_RGBA_FLT;
+
+          return babl_format ("RGBA float");
+        }
+      else
+        {
+          *lcms_format = TYPE_RGB_FLT;
+
+          return babl_format ("RGB float");
+        }
+    }
+  else if (babl_format_is_palette (format))
+    {
+      if (has_alpha)
+        {
+          *lcms_format = TYPE_RGBA_8;
+
+          return babl_format ("R'G'B'A u8");
+        }
+      else
+        {
+          *lcms_format = TYPE_RGB_8;
+
+          return babl_format ("R'G'B' u8");
+        }
     }
   else
     {
@@ -1435,17 +1596,21 @@ gimp_color_profile_get_format (const Babl *format,
     {
       if (has_alpha)
         {
-          if (gray)
-            *lcms_format = TYPE_GRAYA_8;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGBA_8;
+          else if (gray)
+            *lcms_format = TYPE_GRAYA_8;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYKA_8;
         }
       else
         {
-          if (gray)
-            *lcms_format = TYPE_GRAY_8;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGB_8;
+          else if (gray)
+            *lcms_format = TYPE_GRAY_8;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYK_8;
         }
 
       output_format = format;
@@ -1454,17 +1619,21 @@ gimp_color_profile_get_format (const Babl *format,
     {
       if (has_alpha)
         {
-          if (gray)
-            *lcms_format = TYPE_GRAYA_16;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGBA_16;
+          else if (gray)
+            *lcms_format = TYPE_GRAYA_16;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYKA_16;
         }
       else
         {
-          if (gray)
-            *lcms_format = TYPE_GRAY_16;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGB_16;
+          else if (gray)
+            *lcms_format = TYPE_GRAY_16;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYK_16;
         }
 
       output_format = format;
@@ -1473,17 +1642,21 @@ gimp_color_profile_get_format (const Babl *format,
     {
       if (has_alpha)
         {
-          if (gray)
-            *lcms_format = TYPE_GRAYA_HALF_FLT;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGBA_HALF_FLT;
+          else if (gray)
+            *lcms_format = TYPE_GRAYA_HALF_FLT;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYKA_HALF_FLT;
         }
       else
         {
-          if (gray)
-            *lcms_format = TYPE_GRAY_HALF_FLT;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGB_HALF_FLT;
+          else if (gray)
+            *lcms_format = TYPE_GRAY_HALF_FLT;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYK_HALF_FLT;
         }
 
       output_format = format;
@@ -1492,17 +1665,21 @@ gimp_color_profile_get_format (const Babl *format,
     {
       if (has_alpha)
         {
-          if (gray)
-            *lcms_format = TYPE_GRAYA_FLT;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGBA_FLT;
+          else if (gray)
+            *lcms_format = TYPE_GRAYA_FLT;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYKA_FLT;
         }
       else
         {
-          if (gray)
-            *lcms_format = TYPE_GRAY_FLT;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGB_FLT;
+          else if (gray)
+            *lcms_format = TYPE_GRAY_FLT;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYK_FLT;
         }
 
       output_format = format;
@@ -1511,17 +1688,21 @@ gimp_color_profile_get_format (const Babl *format,
     {
       if (has_alpha)
         {
-          if (gray)
-            *lcms_format = TYPE_GRAYA_DBL;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGBA_DBL;
+          else if (gray)
+            *lcms_format = TYPE_GRAYA_DBL;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYKA_DBL;
         }
       else
         {
-          if (gray)
-            *lcms_format = TYPE_GRAY_DBL;
-          else
+          if (rgb)
             *lcms_format = TYPE_RGB_DBL;
+          else if (gray)
+            *lcms_format = TYPE_GRAY_DBL;
+          else if (cmyk)
+            *lcms_format = TYPE_CMYK_DBL;
         }
 
       output_format = format;
@@ -1529,7 +1710,7 @@ gimp_color_profile_get_format (const Babl *format,
 
   if (*lcms_format == 0)
     {
-      g_printerr ("%s: layer format %s not supported, "
+      g_printerr ("%s: format %s not supported, "
                   "falling back to float\n",
                   G_STRFUNC, babl_get_name (format));
 
