@@ -24,13 +24,15 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
-#include "libgimpcolor/gimpcolor.h"
 #include "libgimpbase/gimpbase.h"
+#include "libgimpcolor/gimpcolor.h"
+#include "libgimpconfig/gimpconfig.h"
 
 #include "gimpwidgetstypes.h"
 
 #include "gimpcairo-utils.h"
 #include "gimpcolorarea.h"
+#include "gimpwidgetsutils.h"
 
 
 /**
@@ -67,64 +69,72 @@ typedef struct _GimpColorAreaPrivate GimpColorAreaPrivate;
 
 struct _GimpColorAreaPrivate
 {
-  guchar            *buf;
-  guint              width;
-  guint              height;
-  guint              rowstride;
+  GimpColorConfig    *config;
+  GimpColorTransform *transform;
 
-  GimpColorAreaType  type;
-  GimpRGB            color;
-  guint              draw_border  : 1;
-  guint              needs_render : 1;
+  guchar             *buf;
+  guint               width;
+  guint               height;
+  guint               rowstride;
+
+  GimpColorAreaType   type;
+  GimpRGB             color;
+  guint               draw_border  : 1;
+  guint               needs_render : 1;
 };
 
-#define GET_PRIVATE(obj) G_TYPE_INSTANCE_GET_PRIVATE (obj, \
-                                                      GIMP_TYPE_COLOR_AREA, \
-                                                      GimpColorAreaPrivate)
+#define GET_PRIVATE(obj) \
+        G_TYPE_INSTANCE_GET_PRIVATE (obj, \
+                                     GIMP_TYPE_COLOR_AREA, \
+                                     GimpColorAreaPrivate)
 
 
-static void      gimp_color_area_get_property        (GObject            *object,
-                                                      guint               property_id,
-                                                      GValue             *value,
-                                                      GParamSpec         *pspec);
-static void      gimp_color_area_set_property        (GObject            *object,
-                                                      guint               property_id,
-                                                      const GValue       *value,
-                                                      GParamSpec         *pspec);
-static void      gimp_color_area_finalize            (GObject            *object);
+static void      gimp_color_area_dispose             (GObject           *object);
+static void      gimp_color_area_finalize            (GObject           *object);
+static void      gimp_color_area_get_property        (GObject           *object,
+                                                      guint              property_id,
+                                                      GValue            *value,
+                                                      GParamSpec        *pspec);
+static void      gimp_color_area_set_property        (GObject           *object,
+                                                      guint              property_id,
+                                                      const GValue      *value,
+                                                      GParamSpec        *pspec);
 
-static void      gimp_color_area_size_allocate       (GtkWidget          *widget,
-                                                      GtkAllocation      *allocation);
-static void      gimp_color_area_state_flags_changed (GtkWidget          *widget,
-                                                      GtkStateFlags       previous_state);
-static gboolean  gimp_color_area_draw                (GtkWidget          *widget,
-                                                      cairo_t            *cr);
-static void      gimp_color_area_render_buf          (GtkWidget          *widget,
-                                                      gboolean            insensitive,
-                                                      GimpColorAreaType   type,
-                                                      guchar             *buf,
-                                                      guint               width,
-                                                      guint               height,
-                                                      guint               rowstride,
-                                                      GimpRGB            *color);
-static void      gimp_color_area_render              (GimpColorArea      *area);
+static void      gimp_color_area_size_allocate       (GtkWidget         *widget,
+                                                      GtkAllocation     *allocation);
+static void      gimp_color_area_state_flags_changed (GtkWidget         *widget,
+                                                      GtkStateFlags      previous_state);
+static gboolean  gimp_color_area_draw                (GtkWidget         *widget,
+                                                      cairo_t           *cr);
+static void      gimp_color_area_render_buf          (GtkWidget         *widget,
+                                                      gboolean           insensitive,
+                                                      GimpColorAreaType  type,
+                                                      guchar            *buf,
+                                                      guint              width,
+                                                      guint              height,
+                                                      guint              rowstride,
+                                                      GimpRGB           *color);
+static void      gimp_color_area_render              (GimpColorArea     *area);
 
-static void      gimp_color_area_drag_begin          (GtkWidget          *widget,
-                                                      GdkDragContext     *context);
-static void      gimp_color_area_drag_end            (GtkWidget          *widget,
-                                                      GdkDragContext     *context);
-static void      gimp_color_area_drag_data_received  (GtkWidget          *widget,
-                                                      GdkDragContext     *context,
-                                                      gint                x,
-                                                      gint                y,
-                                                      GtkSelectionData   *selection_data,
-                                                      guint               info,
-                                                      guint               time);
-static void      gimp_color_area_drag_data_get       (GtkWidget          *widget,
-                                                      GdkDragContext     *context,
-                                                      GtkSelectionData   *selection_data,
-                                                      guint               info,
-                                                      guint               time);
+static void      gimp_color_area_drag_begin          (GtkWidget         *widget,
+                                                      GdkDragContext    *context);
+static void      gimp_color_area_drag_end            (GtkWidget         *widget,
+                                                      GdkDragContext    *context);
+static void      gimp_color_area_drag_data_received  (GtkWidget         *widget,
+                                                      GdkDragContext    *context,
+                                                      gint               x,
+                                                      gint               y,
+                                                      GtkSelectionData  *selection_data,
+                                                      guint              info,
+                                                      guint              time);
+static void      gimp_color_area_drag_data_get       (GtkWidget         *widget,
+                                                      GdkDragContext    *context,
+                                                      GtkSelectionData  *selection_data,
+                                                      guint              info,
+                                                      guint              time);
+
+static void      gimp_color_area_create_transform    (GimpColorArea     *area);
+static void      gimp_color_area_destroy_transform   (GimpColorArea     *area);
 
 
 G_DEFINE_TYPE (GimpColorArea, gimp_color_area, GTK_TYPE_DRAWING_AREA)
@@ -152,9 +162,10 @@ gimp_color_area_class_init (GimpColorAreaClass *klass)
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
+  object_class->dispose             = gimp_color_area_dispose;
+  object_class->finalize            = gimp_color_area_finalize;
   object_class->get_property        = gimp_color_area_get_property;
   object_class->set_property        = gimp_color_area_set_property;
-  object_class->finalize            = gimp_color_area_finalize;
 
   widget_class->size_allocate       = gimp_color_area_size_allocate;
   widget_class->state_flags_changed = gimp_color_area_state_flags_changed;
@@ -226,13 +237,13 @@ gimp_color_area_class_init (GimpColorAreaClass *klass)
 static void
 gimp_color_area_init (GimpColorArea *area)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (area);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (area);
 
-  private->buf         = NULL;
-  private->width       = 0;
-  private->height      = 0;
-  private->rowstride   = 0;
-  private->draw_border = FALSE;
+  priv->buf         = NULL;
+  priv->width       = 0;
+  priv->height      = 0;
+  priv->rowstride   = 0;
+  priv->draw_border = FALSE;
 
   gtk_drag_dest_set (GTK_WIDGET (area),
                      GTK_DEST_DEFAULT_HIGHLIGHT |
@@ -240,17 +251,31 @@ gimp_color_area_init (GimpColorArea *area)
                      GTK_DEST_DEFAULT_DROP,
                      &target, 1,
                      GDK_ACTION_COPY);
+
+  gimp_widget_track_monitor (GTK_WIDGET (area),
+                             G_CALLBACK (gimp_color_area_destroy_transform),
+                             NULL);
+}
+
+static void
+gimp_color_area_dispose (GObject *object)
+{
+  GimpColorArea *area = GIMP_COLOR_AREA (object);
+
+  gimp_color_area_set_color_config (area, NULL);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 gimp_color_area_finalize (GObject *object)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (object);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (object);
 
-  if (private->buf)
+  if (priv->buf)
     {
-      g_free (private->buf);
-      private->buf = NULL;
+      g_free (priv->buf);
+      priv->buf = NULL;
     }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -262,20 +287,20 @@ gimp_color_area_get_property (GObject    *object,
                               GValue     *value,
                               GParamSpec *pspec)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (object);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_COLOR:
-      g_value_set_boxed (value, &private->color);
+      g_value_set_boxed (value, &priv->color);
       break;
 
     case PROP_TYPE:
-      g_value_set_enum (value, private->type);
+      g_value_set_enum (value, priv->type);
       break;
 
     case PROP_DRAW_BORDER:
-      g_value_set_boolean (value, private->draw_border);
+      g_value_set_boolean (value, priv->draw_border);
       break;
 
     default:
@@ -328,23 +353,22 @@ static void
 gimp_color_area_size_allocate (GtkWidget     *widget,
                                GtkAllocation *allocation)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (widget);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (widget);
 
-  if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
-    GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
 
-  if (allocation->width  != private->width ||
-      allocation->height != private->height)
+  if (allocation->width  != priv->width ||
+      allocation->height != priv->height)
     {
-      private->width  = allocation->width;
-      private->height = allocation->height;
+      priv->width  = allocation->width;
+      priv->height = allocation->height;
 
-      private->rowstride = private->width * 4 + 4;
+      priv->rowstride = priv->width * 4 + 4;
 
-      g_free (private->buf);
-      private->buf = g_new (guchar, private->rowstride * private->height);
+      g_free (priv->buf);
+      priv->buf = g_new (guchar, priv->rowstride * priv->height);
 
-      private->needs_render = TRUE;
+      priv->needs_render = TRUE;
     }
 }
 
@@ -352,12 +376,12 @@ static void
 gimp_color_area_state_flags_changed (GtkWidget     *widget,
                                      GtkStateFlags  previous_state)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (widget);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (widget);
 
   if ((gtk_widget_get_state_flags (widget) & GTK_STATE_FLAG_INSENSITIVE) !=
       (previous_state & GTK_STATE_FLAG_INSENSITIVE))
     {
-      private->needs_render = TRUE;
+      priv->needs_render = TRUE;
     }
 
   if (GTK_WIDGET_CLASS (parent_class)->state_flags_changed)
@@ -370,27 +394,61 @@ gimp_color_area_draw (GtkWidget *widget,
                       cairo_t   *cr)
 {
   GimpColorArea        *area    = GIMP_COLOR_AREA (widget);
-  GimpColorAreaPrivate *private = GET_PRIVATE (widget);
+  GimpColorAreaPrivate *priv    = GET_PRIVATE (area);
   GtkStyleContext      *context = gtk_widget_get_style_context (widget);
   cairo_surface_t      *buffer;
 
-  if (! private->buf || ! gtk_widget_is_drawable (widget))
+  if (! priv->buf)
     return FALSE;
 
-  if (private->needs_render)
+  if (priv->needs_render)
     gimp_color_area_render (area);
 
-  buffer = cairo_image_surface_create_for_data (private->buf,
-                                                CAIRO_FORMAT_RGB24,
-                                                private->width,
-                                                private->height,
-                                                private->rowstride);
+  if (! priv->transform)
+    gimp_color_area_create_transform (area);
+
+  if (priv->transform)
+    {
+      const Babl *format = babl_format ("cairo-RGB24");
+      guchar     *buf    = g_new (guchar, priv->rowstride * priv->height);
+      guchar     *src    = priv->buf;
+      guchar     *dest   = buf;
+      gint        i;
+
+      for (i = 0; i < priv->height; i++)
+        {
+          gimp_color_transform_process_pixels (priv->transform,
+                                               format, src,
+                                               format, dest,
+                                               priv->width);
+
+          src  += priv->rowstride;
+          dest += priv->rowstride;
+        }
+
+      buffer = cairo_image_surface_create_for_data (buf,
+                                                    CAIRO_FORMAT_RGB24,
+                                                    priv->width,
+                                                    priv->height,
+                                                    priv->rowstride);
+      cairo_surface_set_user_data (buffer, NULL,
+                                   buf, (cairo_destroy_func_t) g_free);
+    }
+  else
+    {
+      buffer = cairo_image_surface_create_for_data (priv->buf,
+                                                    CAIRO_FORMAT_RGB24,
+                                                    priv->width,
+                                                    priv->height,
+                                                    priv->rowstride);
+    }
+
   cairo_set_source_surface (cr, buffer, 0.0, 0.0);
   cairo_surface_destroy (buffer);
 
   cairo_paint (cr);
 
-  if (private->draw_border)
+  if (priv->draw_border)
     {
       GdkRGBA color;
 
@@ -400,7 +458,7 @@ gimp_color_area_draw (GtkWidget *widget,
                                    &color);
       gdk_cairo_set_source_rgba (cr, &color);
 
-      cairo_rectangle (cr, 0.5, 0.5, private->width - 1, private->height - 1);
+      cairo_rectangle (cr, 0.5, 0.5, priv->width - 1, priv->height - 1);
 
       cairo_stroke (cr);
     }
@@ -444,19 +502,19 @@ void
 gimp_color_area_set_color (GimpColorArea *area,
                            const GimpRGB *color)
 {
-  GimpColorAreaPrivate *private;
+  GimpColorAreaPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_AREA (area));
   g_return_if_fail (color != NULL);
 
-  private = GET_PRIVATE (area);
+  priv = GET_PRIVATE (area);
 
-  if (gimp_rgba_distance (&private->color, color) < 0.000001)
+  if (gimp_rgba_distance (&priv->color, color) < 0.000001)
     return;
 
-  private->color = *color;
+  priv->color = *color;
 
-  private->needs_render = TRUE;
+  priv->needs_render = TRUE;
   gtk_widget_queue_draw (GTK_WIDGET (area));
 
   g_object_notify (G_OBJECT (area), "color");
@@ -475,14 +533,14 @@ void
 gimp_color_area_get_color (GimpColorArea *area,
                            GimpRGB       *color)
 {
-  GimpColorAreaPrivate *private;
+  GimpColorAreaPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_AREA (area));
   g_return_if_fail (color != NULL);
 
-  private = GET_PRIVATE (area);
+  priv = GET_PRIVATE (area);
 
-  *color = private->color;
+  *color = priv->color;
 }
 
 /**
@@ -497,13 +555,13 @@ gimp_color_area_get_color (GimpColorArea *area,
 gboolean
 gimp_color_area_has_alpha (GimpColorArea *area)
 {
-  GimpColorAreaPrivate *private;
+  GimpColorAreaPrivate *priv;
 
   g_return_val_if_fail (GIMP_IS_COLOR_AREA (area), FALSE);
 
-  private = GET_PRIVATE (area);
+  priv = GET_PRIVATE (area);
 
-  return private->type != GIMP_COLOR_AREA_FLAT;
+  return priv->type != GIMP_COLOR_AREA_FLAT;
 }
 
 /**
@@ -519,17 +577,17 @@ void
 gimp_color_area_set_type (GimpColorArea     *area,
                           GimpColorAreaType  type)
 {
-  GimpColorAreaPrivate *private;
+  GimpColorAreaPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_AREA (area));
 
-  private = GET_PRIVATE (area);
+  priv = GET_PRIVATE (area);
 
-  if (private->type != type)
+  if (priv->type != type)
     {
-      private->type = type;
+      priv->type = type;
 
-      private->needs_render = TRUE;
+      priv->needs_render = TRUE;
       gtk_widget_queue_draw (GTK_WIDGET (area));
 
       g_object_notify (G_OBJECT (area), "type");
@@ -549,17 +607,17 @@ void
 gimp_color_area_set_draw_border (GimpColorArea *area,
                                  gboolean       draw_border)
 {
-  GimpColorAreaPrivate *private;
+  GimpColorAreaPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_AREA (area));
 
-  private = GET_PRIVATE (area);
+  priv = GET_PRIVATE (area);
 
   draw_border = draw_border ? TRUE : FALSE;
 
-  if (private->draw_border != draw_border)
+  if (priv->draw_border != draw_border)
     {
-      private->draw_border = draw_border;
+      priv->draw_border = draw_border;
 
       gtk_widget_queue_draw (GTK_WIDGET (area));
 
@@ -567,6 +625,54 @@ gimp_color_area_set_draw_border (GimpColorArea *area,
     }
 }
 
+/**
+ * gimp_color_area_set_color_config:
+ * @area:   a #GimpColorArea widget.
+ * @config: a #GimpColorConfig object.
+ *
+ * Sets the color management configuration to use with this color area.
+ *
+ * Since: 2.10
+ */
+void
+gimp_color_area_set_color_config (GimpColorArea   *area,
+                                  GimpColorConfig *config)
+{
+  GimpColorAreaPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_COLOR_AREA (area));
+  g_return_if_fail (config == NULL || GIMP_IS_COLOR_CONFIG (config));
+
+  priv = GET_PRIVATE (area);
+
+  if (config != priv->config)
+    {
+      if (priv->config)
+        {
+          g_signal_handlers_disconnect_by_func (priv->config,
+                                                gimp_color_area_destroy_transform,
+                                                area);
+          g_object_unref (priv->config);
+
+          gimp_color_area_destroy_transform (area);
+        }
+
+      priv->config = config;
+
+      if (priv->config)
+        {
+          g_object_ref (priv->config);
+
+          g_signal_connect_swapped (priv->config, "notify",
+                                    G_CALLBACK (gimp_color_area_destroy_transform),
+                                    area);
+        }
+    }
+}
+
+
+/*  private functions  */
+
 static void
 gimp_color_area_render_buf (GtkWidget         *widget,
                             gboolean           insensitive,
@@ -721,26 +827,26 @@ gimp_color_area_render_buf (GtkWidget         *widget,
 static void
 gimp_color_area_render (GimpColorArea *area)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (area);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (area);
 
-  if (! private->buf)
+  if (! priv->buf)
     return;
 
   gimp_color_area_render_buf (GTK_WIDGET (area),
                               ! gtk_widget_is_sensitive (GTK_WIDGET (area)),
-                              private->type,
-                              private->buf,
-                              private->width, private->height, private->rowstride,
-                              &private->color);
+                              priv->type,
+                              priv->buf,
+                              priv->width, priv->height, priv->rowstride,
+                              &priv->color);
 
-  private->needs_render = FALSE;
+  priv->needs_render = FALSE;
 }
 
 static void
 gimp_color_area_drag_begin (GtkWidget      *widget,
                             GdkDragContext *context)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (widget);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (widget);
   GimpRGB               color;
   GtkWidget            *window;
   GtkWidget            *frame;
@@ -758,7 +864,7 @@ gimp_color_area_drag_begin (GtkWidget      *widget,
 
   gimp_color_area_get_color (GIMP_COLOR_AREA (widget), &color);
 
-  color_area = gimp_color_area_new (&color, private->type, 0);
+  color_area = gimp_color_area_new (&color, priv->type, 0);
 
   gtk_widget_set_size_request (color_area,
                                DRAG_PREVIEW_SIZE, DRAG_PREVIEW_SIZE);
@@ -821,19 +927,55 @@ gimp_color_area_drag_data_get (GtkWidget        *widget,
                                guint             info,
                                guint             time)
 {
-  GimpColorAreaPrivate *private = GET_PRIVATE (widget);
+  GimpColorAreaPrivate *priv = GET_PRIVATE (widget);
   guint16               vals[4];
 
-  vals[0] = private->color.r * 0xffff;
-  vals[1] = private->color.g * 0xffff;
-  vals[2] = private->color.b * 0xffff;
+  vals[0] = priv->color.r * 0xffff;
+  vals[1] = priv->color.g * 0xffff;
+  vals[2] = priv->color.b * 0xffff;
 
-  if (private->type == GIMP_COLOR_AREA_FLAT)
+  if (priv->type == GIMP_COLOR_AREA_FLAT)
     vals[3] = 0xffff;
   else
-    vals[3] = private->color.a * 0xffff;
+    vals[3] = priv->color.a * 0xffff;
 
   gtk_selection_data_set (selection_data,
                           gdk_atom_intern ("application/x-color", FALSE),
                           16, (guchar *) vals, 8);
 }
+
+static void
+gimp_color_area_create_transform (GimpColorArea *area)
+{
+  GimpColorAreaPrivate *priv = GET_PRIVATE (area);
+
+  if (priv->config)
+    {
+      static GimpColorProfile *profile = NULL;
+
+      const Babl *format = babl_format ("cairo-RGB24");
+
+      if (G_UNLIKELY (! profile))
+        profile = gimp_color_profile_new_rgb_srgb ();
+
+      priv->transform = gimp_widget_get_color_transform (GTK_WIDGET (area),
+                                                         priv->config,
+                                                         profile,
+                                                         format,
+                                                         format);
+    }
+}
+
+static void
+gimp_color_area_destroy_transform (GimpColorArea *area)
+{
+  GimpColorAreaPrivate *priv = GET_PRIVATE (area);
+
+  if (priv->transform)
+    {
+      g_object_unref (priv->transform);
+      priv->transform = NULL;
+    }
+
+  gtk_widget_queue_draw (GTK_WIDGET (area));
+}
