@@ -27,6 +27,7 @@
 #include <gtk/gtk.h>
 
 #include "libgimpcolor/gimpcolor.h"
+#include "libgimpconfig/gimpconfig.h"
 
 #include "gimpwidgetstypes.h"
 
@@ -246,7 +247,7 @@ gimp_color_button_class_init (GimpColorButtonClass *klass)
                                    g_param_spec_string ("title", NULL, NULL,
                                                         NULL,
                                                         GIMP_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY));
+                                                        G_PARAM_CONSTRUCT));
   /**
    * GimpColorButton:color:
    *
@@ -321,26 +322,26 @@ static void
 gimp_color_button_init (GimpColorButton      *button,
                         GimpColorButtonClass *klass)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (button);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (button);
   GtkActionGroup         *group;
   gint                    i;
 
-  private->title  = NULL;
-  private->dialog = NULL;
+  priv->title  = NULL;
+  priv->dialog = NULL;
 
-  private->color_area = g_object_new (GIMP_TYPE_COLOR_AREA,
-                                      "drag-mask", GDK_BUTTON1_MASK,
-                                      NULL);
+  priv->color_area = g_object_new (GIMP_TYPE_COLOR_AREA,
+                                   "drag-mask", GDK_BUTTON1_MASK,
+                                   NULL);
 
-  g_signal_connect (private->color_area, "color-changed",
+  g_signal_connect (priv->color_area, "color-changed",
                     G_CALLBACK (gimp_color_button_area_changed),
                     button);
 
-  gtk_container_add (GTK_CONTAINER (button), private->color_area);
-  gtk_widget_show (private->color_area);
+  gtk_container_add (GTK_CONTAINER (button), priv->color_area);
+  gtk_widget_show (priv->color_area);
 
   /* right-click opens a popup */
-  private->ui_manager = gtk_ui_manager_new ();
+  priv->ui_manager = gtk_ui_manager_new ();
 
   group = gtk_action_group_new ("color-button");
 
@@ -368,11 +369,11 @@ gimp_color_button_init (GimpColorButton      *button,
       g_object_unref (action);
     }
 
-  gtk_ui_manager_insert_action_group (private->ui_manager, group, -1);
+  gtk_ui_manager_insert_action_group (priv->ui_manager, group, -1);
   g_object_unref (group);
 
   gtk_ui_manager_add_ui_from_string
-    (private->ui_manager,
+    (priv->ui_manager,
      "<ui>\n"
      "  <popup action=\"color-button-popup\">\n"
      "    <menuitem action=\"" GIMP_COLOR_BUTTON_COLOR_FG "\" />\n"
@@ -388,12 +389,12 @@ gimp_color_button_init (GimpColorButton      *button,
 static void
 gimp_color_button_finalize (GObject *object)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (object);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (object);
 
-  if (private->title)
+  if (priv->title)
     {
-      g_free (private->title);
-      private->title = NULL;
+      g_free (priv->title);
+      priv->title = NULL;
     }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -402,24 +403,24 @@ gimp_color_button_finalize (GObject *object)
 static void
 gimp_color_button_dispose (GObject *object)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (object);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (object);
 
-  if (private->dialog)
+  if (priv->dialog)
     {
-      gtk_widget_destroy (private->dialog);
-      private->dialog = NULL;
+      gtk_widget_destroy (priv->dialog);
+      priv->dialog = NULL;
     }
 
-  if (private->color_area)
+  if (priv->color_area)
     {
-      gtk_widget_destroy (private->color_area);
-      private->color_area = NULL;
+      gtk_widget_destroy (priv->color_area);
+      priv->color_area = NULL;
     }
 
-  if (private->ui_manager)
+  if (priv->ui_manager)
     {
-      g_object_unref (private->ui_manager);
-      private->ui_manager = NULL;
+      g_object_unref (priv->ui_manager);
+      priv->ui_manager = NULL;
     }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -431,24 +432,24 @@ gimp_color_button_get_property (GObject    *object,
                                 GValue     *value,
                                 GParamSpec *pspec)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (object);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_TITLE:
-      g_value_set_string (value, private->title);
+      g_value_set_string (value, priv->title);
       break;
 
     case PROP_COLOR:
-      g_object_get_property (G_OBJECT (private->color_area), "color", value);
+      g_object_get_property (G_OBJECT (priv->color_area), "color", value);
       break;
 
     case PROP_TYPE:
-      g_object_get_property (G_OBJECT (private->color_area), "type", value);
+      g_object_get_property (G_OBJECT (priv->color_area), "type", value);
       break;
 
     case PROP_UPDATE:
-      g_value_set_boolean (value, private->continuous_update);
+      g_value_set_boolean (value, priv->continuous_update);
       break;
 
     default:
@@ -463,23 +464,22 @@ gimp_color_button_set_property (GObject      *object,
                                 const GValue *value,
                                 GParamSpec   *pspec)
 {
-  GimpColorButton        *button  = GIMP_COLOR_BUTTON (object);
-  GimpColorButtonPrivate *private = GET_PRIVATE (object);
+  GimpColorButton        *button = GIMP_COLOR_BUTTON (object);
+  GimpColorButtonPrivate *priv   = GET_PRIVATE (object);
   gint                    other;
 
   switch (property_id)
     {
     case PROP_TITLE:
-      g_free (private->title);
-      private->title = g_value_dup_string (value);
+      gimp_color_button_set_title (button, g_value_get_string (value));
       break;
 
     case PROP_COLOR:
-      g_object_set_property (G_OBJECT (private->color_area), "color", value);
+      g_object_set_property (G_OBJECT (priv->color_area), "color", value);
       break;
 
     case PROP_TYPE:
-      g_object_set_property (G_OBJECT (private->color_area), "type", value);
+      g_object_set_property (G_OBJECT (priv->color_area), "type", value);
       break;
 
     case PROP_UPDATE:
@@ -487,14 +487,14 @@ gimp_color_button_set_property (GObject      *object,
       break;
 
     case PROP_AREA_WIDTH:
-      gtk_widget_get_size_request (private->color_area, NULL, &other);
-      gtk_widget_set_size_request (private->color_area,
+      gtk_widget_get_size_request (priv->color_area, NULL, &other);
+      gtk_widget_set_size_request (priv->color_area,
                                    g_value_get_int (value), other);
       break;
 
     case PROP_AREA_HEIGHT:
-      gtk_widget_get_size_request (private->color_area, &other, NULL);
-      gtk_widget_set_size_request (private->color_area,
+      gtk_widget_get_size_request (priv->color_area, &other, NULL);
+      gtk_widget_set_size_request (priv->color_area,
                                    other, g_value_get_int (value));
       break;
 
@@ -508,11 +508,11 @@ static gboolean
 gimp_color_button_button_press (GtkWidget      *widget,
                                 GdkEventButton *bevent)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (widget);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (widget);
 
   if (gdk_event_triggers_context_menu ((GdkEvent *) bevent))
     {
-      GtkWidget *menu = gtk_ui_manager_get_widget (private->ui_manager,
+      GtkWidget *menu = gtk_ui_manager_get_widget (priv->ui_manager,
                                                    "/color-button-popup");
 
       gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (widget));
@@ -529,10 +529,10 @@ static void
 gimp_color_button_state_flags_changed (GtkWidget     *widget,
                                        GtkStateFlags  previous_state)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (widget);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (widget);
 
-  if (! gtk_widget_is_sensitive (widget) && private->dialog)
-    gtk_widget_hide (private->dialog);
+  if (! gtk_widget_is_sensitive (widget) && priv->dialog)
+    gtk_widget_hide (priv->dialog);
 
   if (GTK_WIDGET_CLASS (parent_class)->state_flags_changed)
     GTK_WIDGET_CLASS (parent_class)->state_flags_changed (widget,
@@ -543,24 +543,22 @@ static void
 gimp_color_button_clicked (GtkButton *button)
 {
   GimpColorButton        *color_button = GIMP_COLOR_BUTTON (button);
-  GimpColorButtonPrivate *private      = GET_PRIVATE (button);
+  GimpColorButtonPrivate *priv      = GET_PRIVATE (button);
+  GtkWidget              *selection;
+  GimpRGB                 color;
 
-  if (! private->dialog)
+  if (! priv->dialog)
     {
       GtkWidget *dialog;
-      GtkWidget *selection;
-      GimpRGB    color;
 
-      gimp_color_button_get_color (color_button, &color);
-
-      dialog = private->dialog =
-        gimp_dialog_new (private->title, COLOR_SELECTION_KEY,
+      dialog = priv->dialog =
+        gimp_dialog_new (priv->title, COLOR_SELECTION_KEY,
                          gtk_widget_get_toplevel (GTK_WIDGET (button)), 0,
                          gimp_color_button_help_func, NULL,
 
-                         GIMP_STOCK_RESET, RESPONSE_RESET,
-                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                         GTK_STOCK_OK,     GTK_RESPONSE_OK,
+                         _("_Reset"),  RESPONSE_RESET,
+                         _("_Cancel"), GTK_RESPONSE_CANCEL,
+                         _("_OK"),     GTK_RESPONSE_OK,
 
                          NULL);
 
@@ -575,15 +573,12 @@ gimp_color_button_clicked (GtkButton *button)
                         color_button);
       g_signal_connect (dialog, "destroy",
                         G_CALLBACK (gtk_widget_destroyed),
-                        &private->dialog);
+                        &priv->dialog);
 
       selection = gimp_color_selection_new ();
       gtk_container_set_border_width (GTK_CONTAINER (selection), 6);
       gimp_color_selection_set_show_alpha (GIMP_COLOR_SELECTION (selection),
                                            gimp_color_button_has_alpha (color_button));
-      gimp_color_selection_set_color (GIMP_COLOR_SELECTION (selection), &color);
-      gimp_color_selection_set_old_color (GIMP_COLOR_SELECTION (selection),
-                                          &color);
       gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
                           selection, TRUE, TRUE, 0);
       gtk_widget_show (selection);
@@ -592,11 +587,30 @@ gimp_color_button_clicked (GtkButton *button)
                         G_CALLBACK (gimp_color_button_selection_changed),
                         button);
 
-      g_object_set_data (G_OBJECT (private->dialog), COLOR_SELECTION_KEY,
+      g_object_set_data (G_OBJECT (priv->dialog), COLOR_SELECTION_KEY,
                          selection);
     }
+  else
+    {
+      selection = g_object_get_data (G_OBJECT (priv->dialog),
+                                     COLOR_SELECTION_KEY);
+    }
 
-  gtk_window_present (GTK_WINDOW (private->dialog));
+  gimp_color_button_get_color (color_button, &color);
+
+  g_signal_handlers_block_by_func (selection,
+                                   gimp_color_button_selection_changed,
+                                   button);
+
+  gimp_color_selection_set_color (GIMP_COLOR_SELECTION (selection), &color);
+  gimp_color_selection_set_old_color (GIMP_COLOR_SELECTION (selection),
+                                      &color);
+
+  g_signal_handlers_unblock_by_func (selection,
+                                     gimp_color_button_selection_changed,
+                                     button);
+
+  gtk_window_present (GTK_WINDOW (priv->dialog));
 }
 
 static GType
@@ -653,24 +667,24 @@ gimp_color_button_new (const gchar       *title,
  *
  * Sets the @button dialog's title.
  *
- * Since: GIMP 3.0
+ * Since: GIMP 2.10
  **/
 void
 gimp_color_button_set_title (GimpColorButton *button,
                              const gchar     *title)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_BUTTON (button));
   g_return_if_fail (title != NULL);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  g_free (private->title);
-  private->title = g_strdup (title);
+  g_free (priv->title);
+  priv->title = g_strdup (title);
 
-  if (private->dialog)
-    gtk_window_set_title (GTK_WINDOW (private->dialog), title);
+  if (priv->dialog)
+    gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
 
   g_object_notify (G_OBJECT (button), "title");
 }
@@ -681,18 +695,18 @@ gimp_color_button_set_title (GimpColorButton *button,
  *
  * Returns: The @button dialog's title.
  *
- * Since: GIMP 3.0
+ * Since: GIMP 2.10
  **/
 const gchar *
 gimp_color_button_get_title (GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_val_if_fail (GIMP_IS_COLOR_BUTTON (button), NULL);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  return private->title;
+  return priv->title;
 }
 
 /**
@@ -706,14 +720,14 @@ void
 gimp_color_button_set_color (GimpColorButton *button,
                              const GimpRGB   *color)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_BUTTON (button));
   g_return_if_fail (color != NULL);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  gimp_color_area_set_color (GIMP_COLOR_AREA (private->color_area), color);
+  gimp_color_area_set_color (GIMP_COLOR_AREA (priv->color_area), color);
 
   g_object_notify (G_OBJECT (button), "color");
 }
@@ -729,14 +743,14 @@ void
 gimp_color_button_get_color (GimpColorButton *button,
                              GimpRGB         *color)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_BUTTON (button));
   g_return_if_fail (color != NULL);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  gimp_color_area_get_color (GIMP_COLOR_AREA (private->color_area), color);
+  gimp_color_area_get_color (GIMP_COLOR_AREA (priv->color_area), color);
 }
 
 /**
@@ -751,13 +765,13 @@ gimp_color_button_get_color (GimpColorButton *button,
 gboolean
 gimp_color_button_has_alpha (GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_val_if_fail (GIMP_IS_COLOR_BUTTON (button), FALSE);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  return gimp_color_area_has_alpha (GIMP_COLOR_AREA (private->color_area));
+  return gimp_color_area_has_alpha (GIMP_COLOR_AREA (priv->color_area));
 }
 
 /**
@@ -771,13 +785,13 @@ void
 gimp_color_button_set_type (GimpColorButton   *button,
                             GimpColorAreaType  type)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_BUTTON (button));
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  gimp_color_area_set_type (GIMP_COLOR_AREA (private->color_area), type);
+  gimp_color_area_set_type (GIMP_COLOR_AREA (priv->color_area), type);
 
   g_object_notify (G_OBJECT (button), "type");
 }
@@ -793,13 +807,13 @@ gimp_color_button_set_type (GimpColorButton   *button,
 gboolean
 gimp_color_button_get_update (GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_val_if_fail (GIMP_IS_COLOR_BUTTON (button), FALSE);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  return private->continuous_update;
+  return priv->continuous_update;
 }
 
 /**
@@ -815,25 +829,25 @@ void
 gimp_color_button_set_update (GimpColorButton *button,
                               gboolean         continuous)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_BUTTON (button));
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  if (continuous != private->continuous_update)
+  if (continuous != priv->continuous_update)
     {
-      private->continuous_update = continuous ? TRUE : FALSE;
+      priv->continuous_update = continuous ? TRUE : FALSE;
 
-      if (private->dialog)
+      if (priv->dialog)
         {
           GimpColorSelection *selection;
           GimpRGB             color;
 
-          selection = g_object_get_data (G_OBJECT (private->dialog),
+          selection = g_object_get_data (G_OBJECT (priv->dialog),
                                          COLOR_SELECTION_KEY);
 
-          if (private->continuous_update)
+          if (priv->continuous_update)
             {
               gimp_color_selection_get_color (selection, &color);
               gimp_color_button_set_color (button, &color);
@@ -849,26 +863,53 @@ gimp_color_button_set_update (GimpColorButton *button,
     }
 }
 
+/**
+ * gimp_color_button_set_color_config:
+ * @button: a #GimpColorButton widget.
+ * @config: a #GimpColorConfig object.
+ *
+ * Sets the color management configuration to use with this color button's
+ * #GimpColorArea.
+ *
+ * Since: 2.10
+ */
+void
+gimp_color_button_set_color_config (GimpColorButton *button,
+                                    GimpColorConfig *config)
+{
+  GimpColorButtonPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_COLOR_BUTTON (button));
+  g_return_if_fail (config == NULL || GIMP_IS_COLOR_CONFIG (config));
+
+  priv = GET_PRIVATE (button);
+
+  gimp_color_area_set_color_config (GIMP_COLOR_AREA (priv->color_area),
+                                    config);
+}
+
 /**
  * gimp_color_button_get_ui_manager:
  * @button: a #GimpColorButton.
  *
  * Returns: The @button's #GtkUIManager.
  *
+ * Since: GIMP 2.10
  * Since: GIMP 3.0
  **/
 GtkUIManager *
 gimp_color_button_get_ui_manager (GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private;
+  GimpColorButtonPrivate *priv;
 
   g_return_val_if_fail (GIMP_IS_COLOR_BUTTON (button), NULL);
 
-  private = GET_PRIVATE (button);
+  priv = GET_PRIVATE (button);
 
-  return private->ui_manager;
+  return priv->ui_manager;
 }
 
+
 /*  private functions  */
 
 static void
@@ -876,7 +917,7 @@ gimp_color_button_dialog_response (GtkWidget       *dialog,
                                    gint             response_id,
                                    GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (button);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (button);
   GtkWidget              *selection;
   GimpRGB                 color;
 
@@ -889,7 +930,7 @@ gimp_color_button_dialog_response (GtkWidget       *dialog,
       break;
 
     case GTK_RESPONSE_OK:
-      if (! private->continuous_update)
+      if (! priv->continuous_update)
         {
           gimp_color_selection_get_color (GIMP_COLOR_SELECTION (selection),
                                           &color);
@@ -900,7 +941,7 @@ gimp_color_button_dialog_response (GtkWidget       *dialog,
       break;
 
     default:
-      if (private->continuous_update)
+      if (priv->continuous_update)
         {
           gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (selection),
                                               &color);
@@ -952,14 +993,14 @@ static void
 gimp_color_button_area_changed (GtkWidget       *color_area,
                                 GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (button);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (button);
 
-  if (private->dialog)
+  if (priv->dialog)
     {
       GimpColorSelection *selection;
       GimpRGB             color;
 
-      selection = g_object_get_data (G_OBJECT (private->dialog),
+      selection = g_object_get_data (G_OBJECT (priv->dialog),
                                      COLOR_SELECTION_KEY);
 
       gimp_color_button_get_color (button, &color);
@@ -982,21 +1023,21 @@ static void
 gimp_color_button_selection_changed (GtkWidget       *selection,
                                      GimpColorButton *button)
 {
-  GimpColorButtonPrivate *private = GET_PRIVATE (button);
+  GimpColorButtonPrivate *priv = GET_PRIVATE (button);
 
-  if (private->continuous_update)
+  if (priv->continuous_update)
     {
       GimpRGB color;
 
       gimp_color_selection_get_color (GIMP_COLOR_SELECTION (selection), &color);
 
-      g_signal_handlers_block_by_func (private->color_area,
+      g_signal_handlers_block_by_func (priv->color_area,
                                        gimp_color_button_area_changed,
                                        button);
 
-      gimp_color_area_set_color (GIMP_COLOR_AREA (private->color_area), &color);
+      gimp_color_area_set_color (GIMP_COLOR_AREA (priv->color_area), &color);
 
-      g_signal_handlers_unblock_by_func (private->color_area,
+      g_signal_handlers_unblock_by_func (priv->color_area,
                                          gimp_color_button_area_changed,
                                          button);
 
