@@ -27,13 +27,15 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
-#include "libgimpcolor/gimpcolor.h"
 #include "libgimpbase/gimpbase.h"
+#include "libgimpconfig/gimpconfig.h"
+#include "libgimpcolor/gimpcolor.h"
 
 #include "gimpwidgetstypes.h"
 
 #include "gimpcairo-utils.h"
 #include "gimpcolorscale.h"
+#include "gimpwidgetsutils.h"
 
 
 /**
@@ -56,6 +58,9 @@ typedef struct _GimpColorScalePrivate GimpColorScalePrivate;
 
 struct _GimpColorScalePrivate
 {
+  GimpColorConfig          *config;
+  GimpColorTransform       *transform;
+
   GimpColorSelectorChannel  channel;
   GimpRGB                   rgb;
   GimpHSV                   hsv;
@@ -68,11 +73,13 @@ struct _GimpColorScalePrivate
   gboolean                  needs_render;
 };
 
-#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-                                                       GIMP_TYPE_COLOR_SCALE, \
-                                                       GimpColorScalePrivate))
+#define GET_PRIVATE(obj) \
+        G_TYPE_INSTANCE_GET_PRIVATE (obj, \
+                                     GIMP_TYPE_COLOR_SCALE, \
+                                     GimpColorScalePrivate)
 
 
+static void     gimp_color_scale_dispose             (GObject        *object);
 static void     gimp_color_scale_finalize            (GObject        *object);
 static void     gimp_color_scale_get_property        (GObject        *object,
                                                       guint           property_id,
@@ -91,6 +98,8 @@ static gboolean gimp_color_scale_button_press        (GtkWidget      *widget,
                                                       GdkEventButton *event);
 static gboolean gimp_color_scale_button_release      (GtkWidget      *widget,
                                                       GdkEventButton *event);
+static gboolean gimp_color_scale_scroll              (GtkWidget      *widget,
+                                                      GdkEventScroll *event);
 static gboolean gimp_color_scale_draw                (GtkWidget      *widget,
                                                       cairo_t        *cr);
 
@@ -98,6 +107,9 @@ static void     gimp_color_scale_render              (GimpColorScale *scale);
 static void     gimp_color_scale_render_alpha        (GimpColorScale *scale);
 static void     gimp_color_scale_render_stipple      (GimpColorScale *scale);
 
+static void     gimp_color_scale_create_transform    (GimpColorScale *scale);
+static void     gimp_color_scale_destroy_transform   (GimpColorScale *scale);
+
 
 G_DEFINE_TYPE (GimpColorScale, gimp_color_scale, GTK_TYPE_SCALE)
 
@@ -110,6 +122,7 @@ gimp_color_scale_class_init (GimpColorScaleClass *klass)
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  object_class->dispose              = gimp_color_scale_dispose;
   object_class->finalize             = gimp_color_scale_finalize;
   object_class->get_property         = gimp_color_scale_get_property;
   object_class->set_property         = gimp_color_scale_set_property;
@@ -118,6 +131,7 @@ gimp_color_scale_class_init (GimpColorScaleClass *klass)
   widget_class->state_flags_changed  = gimp_color_scale_state_flags_changed;
   widget_class->button_press_event   = gimp_color_scale_button_press;
   widget_class->button_release_event = gimp_color_scale_button_release;
+  widget_class->scroll_event         = gimp_color_scale_scroll;
   widget_class->draw                 = gimp_color_scale_draw;
 
   /**
@@ -140,36 +154,50 @@ gimp_color_scale_class_init (GimpColorScaleClass *klass)
 static void
 gimp_color_scale_init (GimpColorScale *scale)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (scale);
-  GtkRange              *range   = GTK_RANGE (scale);
+  GimpColorScalePrivate *priv  = GET_PRIVATE (scale);
+  GtkRange              *range = GTK_RANGE (scale);
 
   gtk_range_set_slider_size_fixed (range, TRUE);
   gtk_range_set_flippable (GTK_RANGE (scale), TRUE);
 
   gtk_scale_set_draw_value (GTK_SCALE (scale), FALSE);
 
-  private->channel      = GIMP_COLOR_SELECTOR_VALUE;
-  private->needs_render = TRUE;
+  priv->channel      = GIMP_COLOR_SELECTOR_VALUE;
+  priv->needs_render = TRUE;
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (range),
                                   GTK_ORIENTATION_HORIZONTAL);
 
-  gimp_rgba_set (&private->rgb, 0.0, 0.0, 0.0, 1.0);
-  gimp_rgb_to_hsv (&private->rgb, &private->hsv);
+  gimp_rgba_set (&priv->rgb, 0.0, 0.0, 0.0, 1.0);
+  gimp_rgb_to_hsv (&priv->rgb, &priv->hsv);
+
+  gimp_widget_track_monitor (GTK_WIDGET (scale),
+                             G_CALLBACK (gimp_color_scale_destroy_transform),
+                             NULL);
+}
+
+static void
+gimp_color_scale_dispose (GObject *object)
+{
+  GimpColorScale *scale = GIMP_COLOR_SCALE (object);
+
+  gimp_color_scale_set_color_config (scale, NULL);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 gimp_color_scale_finalize (GObject *object)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (object);
+  GimpColorScalePrivate *priv = GET_PRIVATE (object);
 
-  if (private->buf)
+  if (priv->buf)
     {
-      g_free (private->buf);
-      private->buf       = NULL;
-      private->width     = 0;
-      private->height    = 0;
-      private->rowstride = 0;
+      g_free (priv->buf);
+      priv->buf       = NULL;
+      priv->width     = 0;
+      priv->height    = 0;
+      priv->rowstride = 0;
     }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -181,12 +209,12 @@ gimp_color_scale_get_property (GObject    *object,
                                GValue     *value,
                                GParamSpec *pspec)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (object);
+  GimpColorScalePrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_CHANNEL:
-      g_value_set_enum (value, private->channel);
+      g_value_set_enum (value, priv->channel);
       break;
 
     default:
@@ -219,8 +247,8 @@ static void
 gimp_color_scale_size_allocate (GtkWidget     *widget,
                                 GtkAllocation *allocation)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (widget);
-  GtkRange              *range   = GTK_RANGE (widget);
+  GimpColorScalePrivate *priv  = GET_PRIVATE (widget);
+  GtkRange              *range = GTK_RANGE (widget);
   GdkRectangle           range_rect;
   gint                   focus = 0;
   gint                   trough_border;
@@ -267,17 +295,17 @@ gimp_color_scale_size_allocate (GtkWidget     *widget,
       break;
     }
 
-  if (scale_width != private->width || scale_height != private->height)
+  if (scale_width != priv->width || scale_height != priv->height)
     {
-      private->width  = scale_width;
-      private->height = scale_height;
+      priv->width  = scale_width;
+      priv->height = scale_height;
 
-      private->rowstride = private->width * 4;
+      priv->rowstride = priv->width * 4;
 
-      g_free (private->buf);
-      private->buf = g_new (guchar, private->rowstride * private->height);
+      g_free (priv->buf);
+      priv->buf = g_new (guchar, priv->rowstride * priv->height);
 
-      private->needs_render = TRUE;
+      priv->needs_render = TRUE;
     }
 }
 
@@ -342,12 +370,48 @@ gimp_color_scale_button_release (GtkWidget      *widget,
   return GTK_WIDGET_CLASS (parent_class)->button_release_event (widget, event);
 }
 
+static gboolean
+gimp_color_scale_scroll (GtkWidget      *widget,
+                         GdkEventScroll *event)
+{
+  if (gtk_orientable_get_orientation (GTK_ORIENTABLE (widget)) ==
+      GTK_ORIENTATION_HORIZONTAL)
+    {
+      GdkEventScroll *my_event;
+      gboolean        retval;
+
+      my_event = (GdkEventScroll *) gdk_event_copy ((GdkEvent *) event);
+
+      switch (my_event->direction)
+        {
+        case GDK_SCROLL_UP:
+          my_event->direction = GDK_SCROLL_RIGHT;
+          break;
+
+        case GDK_SCROLL_DOWN:
+          my_event->direction = GDK_SCROLL_LEFT;
+          break;
+
+        default:
+          break;
+        }
+
+      retval = GTK_WIDGET_CLASS (parent_class)->scroll_event (widget, my_event);
+
+      gdk_event_free ((GdkEvent *) my_event);
+
+      return retval;
+    }
+
+  return GTK_WIDGET_CLASS (parent_class)->scroll_event (widget, event);
+}
+
 static gboolean
 gimp_color_scale_draw (GtkWidget *widget,
                        cairo_t   *cr)
 {
   GimpColorScale        *scale     = GIMP_COLOR_SCALE (widget);
-  GimpColorScalePrivate *private   = GET_PRIVATE (widget);
+  GimpColorScalePrivate *priv      = GET_PRIVATE (widget);
   GtkRange              *range     = GTK_RANGE (widget);
   GtkStyleContext       *context   = gtk_widget_get_style_context (widget);
   gboolean               sensitive = gtk_widget_is_sensitive (widget);
@@ -361,7 +425,7 @@ gimp_color_scale_draw (GtkWidget *widget,
   gint                   x, y;
   gint                   w, h;
 
-  if (! private->buf || ! gtk_widget_is_drawable (widget))
+  if (! priv->buf)
     return FALSE;
 
   gtk_style_context_save (context);
@@ -393,14 +457,14 @@ gimp_color_scale_draw (GtkWidget *widget,
 
   slider_size = gtk_range_get_min_slider_size (range) / 2;
 
-  if (private->needs_render)
+  if (priv->needs_render)
     {
       gimp_color_scale_render (scale);
 
       if (! sensitive)
         gimp_color_scale_render_stipple (scale);
 
-      private->needs_render = FALSE;
+      priv->needs_render = FALSE;
     }
 
   gtk_style_context_set_state (context, gtk_widget_get_state_flags (widget));
@@ -408,11 +472,44 @@ gimp_color_scale_draw (GtkWidget *widget,
   gtk_render_background (context, cr, x, y, w, h);
   gtk_render_frame (context, cr, x, y, w, h);
 
-  buffer = cairo_image_surface_create_for_data (private->buf,
-                                                CAIRO_FORMAT_RGB24,
-                                                private->width,
-                                                private->height,
-                                                private->rowstride);
+  if (! priv->transform)
+    gimp_color_scale_create_transform (scale);
+
+  if (priv->transform)
+    {
+      const Babl *format = babl_format ("cairo-RGB24");
+      guchar     *buf    = g_new (guchar, priv->rowstride * priv->height);
+      guchar     *src    = priv->buf;
+      guchar     *dest   = buf;
+      gint        i;
+
+      for (i = 0; i < priv->height; i++)
+        {
+          gimp_color_transform_process_pixels (priv->transform,
+                                               format, src,
+                                               format, dest,
+                                               priv->width);
+
+          src  += priv->rowstride;
+          dest += priv->rowstride;
+        }
+
+      buffer = cairo_image_surface_create_for_data (buf,
+                                                    CAIRO_FORMAT_RGB24,
+                                                    priv->width,
+                                                    priv->height,
+                                                    priv->rowstride);
+      cairo_surface_set_user_data (buffer, NULL,
+                                   buf, (cairo_destroy_func_t) g_free);
+    }
+  else
+    {
+      buffer = cairo_image_surface_create_for_data (priv->buf,
+                                                    CAIRO_FORMAT_RGB24,
+                                                    priv->width,
+                                                    priv->height,
+                                                    priv->rowstride);
+    }
 
   switch (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)))
     {
@@ -550,17 +647,17 @@ void
 gimp_color_scale_set_channel (GimpColorScale           *scale,
                               GimpColorSelectorChannel  channel)
 {
-  GimpColorScalePrivate *private;
+  GimpColorScalePrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_SCALE (scale));
 
-  private = GET_PRIVATE (scale);
+  priv = GET_PRIVATE (scale);
 
-  if (channel != private->channel)
+  if (channel != priv->channel)
     {
-      private->channel = channel;
+      priv->channel = channel;
 
-      private->needs_render = TRUE;
+      priv->needs_render = TRUE;
       gtk_widget_queue_draw (GTK_WIDGET (scale));
 
       g_object_notify (G_OBJECT (scale), "channel");
@@ -580,21 +677,67 @@ gimp_color_scale_set_color (GimpColorScale *scale,
                             const GimpRGB  *rgb,
                             const GimpHSV  *hsv)
 {
-  GimpColorScalePrivate *private;
+  GimpColorScalePrivate *priv;
 
   g_return_if_fail (GIMP_IS_COLOR_SCALE (scale));
   g_return_if_fail (rgb != NULL);
   g_return_if_fail (hsv != NULL);
 
-  private = GET_PRIVATE (scale);
+  priv = GET_PRIVATE (scale);
 
-  private->rgb = *rgb;
-  private->hsv = *hsv;
+  priv->rgb = *rgb;
+  priv->hsv = *hsv;
 
-  private->needs_render = TRUE;
+  priv->needs_render = TRUE;
   gtk_widget_queue_draw (GTK_WIDGET (scale));
 }
 
+/**
+ * gimp_color_scale_set_color_config:
+ * @scale:  a #GimpColorScale widget.
+ * @config: a #GimpColorConfig object.
+ *
+ * Sets the color management configuration to use with this color scale.
+ *
+ * Since: 2.10
+ */
+void
+gimp_color_scale_set_color_config (GimpColorScale  *scale,
+                                   GimpColorConfig *config)
+{
+  GimpColorScalePrivate *priv;
+
+  g_return_if_fail (GIMP_IS_COLOR_SCALE (scale));
+  g_return_if_fail (config == NULL || GIMP_IS_COLOR_CONFIG (config));
+
+  priv = GET_PRIVATE (scale);
+
+  if (config != priv->config)
+    {
+      if (priv->config)
+        {
+          g_signal_handlers_disconnect_by_func (priv->config,
+                                                gimp_color_scale_destroy_transform,
+                                                scale);
+          g_object_unref (priv->config);
+
+          gimp_color_scale_destroy_transform (scale);
+        }
+
+      priv->config = config;
+
+      if (priv->config)
+        {
+          g_object_ref (priv->config);
+
+          g_signal_connect_swapped (priv->config, "notify",
+                                    G_CALLBACK (gimp_color_scale_destroy_transform),
+                                    scale);
+        }
+    }
+}
+
+
 /* as in gtkrange.c */
 static gboolean
 should_invert (GtkRange *range)
@@ -621,8 +764,8 @@ should_invert (GtkRange *range)
 static void
 gimp_color_scale_render (GimpColorScale *scale)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (scale);
-  GtkRange              *range   = GTK_RANGE (scale);
+  GimpColorScalePrivate *priv  = GET_PRIVATE (scale);
+  GtkRange              *range = GTK_RANGE (scale);
   GimpRGB                rgb;
   GimpHSV                hsv;
   guint                  x, y;
@@ -632,19 +775,19 @@ gimp_color_scale_render (GimpColorScale *scale)
   guchar                *buf;
   guchar                *d;
 
-  if ((buf = private->buf) == NULL)
+  if ((buf = priv->buf) == NULL)
     return;
 
-  if (private->channel == GIMP_COLOR_SELECTOR_ALPHA)
+  if (priv->channel == GIMP_COLOR_SELECTOR_ALPHA)
     {
       gimp_color_scale_render_alpha (scale);
       return;
     }
 
-  rgb = private->rgb;
-  hsv = private->hsv;
+  rgb = priv->rgb;
+  hsv = priv->hsv;
 
-  switch (private->channel)
+  switch (priv->channel)
     {
     case GIMP_COLOR_SELECTOR_HUE:        channel_value = &hsv.h; break;
     case GIMP_COLOR_SELECTOR_SATURATION: channel_value = &hsv.s; break;
@@ -655,7 +798,7 @@ gimp_color_scale_render (GimpColorScale *scale)
     case GIMP_COLOR_SELECTOR_ALPHA:      channel_value = &rgb.a; break;
     }
 
-  switch (private->channel)
+  switch (priv->channel)
     {
     case GIMP_COLOR_SELECTOR_HUE:
     case GIMP_COLOR_SELECTOR_SATURATION:
@@ -672,9 +815,9 @@ gimp_color_scale_render (GimpColorScale *scale)
   switch (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)))
     {
     case GTK_ORIENTATION_HORIZONTAL:
-      for (x = 0, d = buf; x < private->width; x++, d += 4)
+      for (x = 0, d = buf; x < priv->width; x++, d += 4)
         {
-          gdouble value = (gdouble) x / (gdouble) (private->width - 1);
+          gdouble value = (gdouble) x / (gdouble) (priv->width - 1);
           guchar  r, g, b;
 
           if (invert)
@@ -690,18 +833,18 @@ gimp_color_scale_render (GimpColorScale *scale)
           GIMP_CAIRO_RGB24_SET_PIXEL (d, r, g, b);
         }
 
-      d = buf + private->rowstride;
-      for (y = 1; y < private->height; y++)
+      d = buf + priv->rowstride;
+      for (y = 1; y < priv->height; y++)
         {
-          memcpy (d, buf, private->rowstride);
-          d += private->rowstride;
+          memcpy (d, buf, priv->rowstride);
+          d += priv->rowstride;
         }
       break;
 
     case GTK_ORIENTATION_VERTICAL:
-      for (y = 0; y < private->height; y++)
+      for (y = 0; y < priv->height; y++)
         {
-          gdouble value = (gdouble) y / (gdouble) (private->height - 1);
+          gdouble value = (gdouble) y / (gdouble) (priv->height - 1);
           guchar  r, g, b;
 
           if (invert)
@@ -714,12 +857,12 @@ gimp_color_scale_render (GimpColorScale *scale)
 
           gimp_rgb_get_uchar (&rgb, &r, &g, &b);
 
-          for (x = 0, d = buf; x < private->width; x++, d += 4)
+          for (x = 0, d = buf; x < priv->width; x++, d += 4)
             {
               GIMP_CAIRO_RGB24_SET_PIXEL (d, r, g, b);
             }
 
-          buf += private->rowstride;
+          buf += priv->rowstride;
         }
       break;
     }
@@ -728,8 +871,8 @@ gimp_color_scale_render (GimpColorScale *scale)
 static void
 gimp_color_scale_render_alpha (GimpColorScale *scale)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (scale);
-  GtkRange              *range   = GTK_RANGE (scale);
+  GimpColorScalePrivate *priv  = GET_PRIVATE (scale);
+  GtkRange              *range = GTK_RANGE (scale);
   GimpRGB                rgb;
   gboolean               invert;
   gdouble                a;
@@ -739,8 +882,8 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
 
   invert = should_invert (range);
 
-  buf = private->buf;
-  rgb = private->rgb;
+  buf = priv->buf;
+  rgb = priv->rgb;
 
   switch (gtk_orientable_get_orientation (GTK_ORIENTABLE (range)))
     {
@@ -751,10 +894,10 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
 
         light = buf;
         /* this won't work correctly for very thin scales */
-        dark  = (private->height > GIMP_CHECK_SIZE_SM ?
-                 buf + GIMP_CHECK_SIZE_SM * private->rowstride : light);
+        dark  = (priv->height > GIMP_CHECK_SIZE_SM ?
+                 buf + GIMP_CHECK_SIZE_SM * priv->rowstride : light);
 
-        for (x = 0, d = light, l = dark; x < private->width; x++)
+        for (x = 0, d = light, l = dark; x < priv->width; x++)
           {
             if ((x % GIMP_CHECK_SIZE_SM) == 0)
               {
@@ -765,7 +908,7 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
                 l = t;
               }
 
-            a = (gdouble) x / (gdouble) (private->width - 1);
+            a = (gdouble) x / (gdouble) (priv->width - 1);
 
             if (invert)
               a = 1.0 - a;
@@ -789,15 +932,15 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
             d += 4;
           }
 
-        for (y = 0, d = buf; y < private->height; y++, d += private->rowstride)
+        for (y = 0, d = buf; y < priv->height; y++, d += priv->rowstride)
           {
             if (y == 0 || y == GIMP_CHECK_SIZE_SM)
               continue;
 
             if ((y / GIMP_CHECK_SIZE_SM) & 1)
-              memcpy (d, dark, private->rowstride);
+              memcpy (d, dark, priv->rowstride);
             else
-              memcpy (d, light, private->rowstride);
+              memcpy (d, light, priv->rowstride);
           }
       }
       break;
@@ -807,9 +950,9 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
         guchar  light[4] = {0xff, 0xff, 0xff, 0xff};
         guchar  dark[4] = {0xff, 0xff, 0xff, 0xff};
 
-        for (y = 0, d = buf; y < private->height; y++, d += private->rowstride)
+        for (y = 0, d = buf; y < priv->height; y++, d += priv->rowstride)
           {
-            a = (gdouble) y / (gdouble) (private->height - 1);
+            a = (gdouble) y / (gdouble) (priv->height - 1);
 
             if (invert)
               a = 1.0 - a;
@@ -830,7 +973,7 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
                                         (GIMP_CHECK_DARK +
                                          (rgb.b - GIMP_CHECK_DARK) * a) * 255.999);
 
-            for (x = 0, l = d; x < private->width; x++, l += 4)
+            for (x = 0, l = d; x < priv->width; x++, l += 4)
               {
                 if (((x / GIMP_CHECK_SIZE_SM) ^ (y / GIMP_CHECK_SIZE_SM)) & 1)
                   {
@@ -860,7 +1003,7 @@ gimp_color_scale_render_alpha (GimpColorScale *scale)
 static void
 gimp_color_scale_render_stipple (GimpColorScale *scale)
 {
-  GimpColorScalePrivate *private = GET_PRIVATE (scale);
+  GimpColorScalePrivate *priv    = GET_PRIVATE (scale);
   GtkWidget             *widget  = GTK_WIDGET (scale);
   GtkStyleContext       *context = gtk_widget_get_style_context (widget);
   GdkRGBA                color;
@@ -869,7 +1012,7 @@ gimp_color_scale_render_stipple (GimpColorScale *scale)
   guchar                 insensitive[4];
   guint                  x, y;
 
-  if ((buf = private->buf) == NULL)
+  if ((buf = priv->buf) == NULL)
     return;
 
   gtk_style_context_get_background_color (context,
@@ -879,11 +1022,11 @@ gimp_color_scale_render_stipple (GimpColorScale *scale)
 
   GIMP_CAIRO_RGB24_SET_PIXEL (insensitive, r, g, b);
 
-  for (y = 0; y < private->height; y++, buf += private->rowstride)
+  for (y = 0; y < priv->height; y++, buf += priv->rowstride)
     {
       guchar *d = buf + 4 * (y % 2);
 
-      for (x = 0; x < private->width - (y % 2); x += 2, d += 8)
+      for (x = 0; x < priv->width - (y % 2); x += 2, d += 8)
         {
           d[0] = insensitive[0];
           d[1] = insensitive[1];
@@ -892,3 +1035,39 @@ gimp_color_scale_render_stipple (GimpColorScale *scale)
         }
     }
 }
+
+static void
+gimp_color_scale_create_transform (GimpColorScale *scale)
+{
+  GimpColorScalePrivate *priv = GET_PRIVATE (scale);
+
+  if (priv->config)
+    {
+      static GimpColorProfile *profile = NULL;
+
+      const Babl *format = babl_format ("cairo-RGB24");
+
+      if (G_UNLIKELY (! profile))
+        profile = gimp_color_profile_new_rgb_srgb ();
+
+      priv->transform = gimp_widget_get_color_transform (GTK_WIDGET (scale),
+                                                         priv->config,
+                                                         profile,
+                                                         format,
+                                                         format);
+    }
+}
+
+static void
+gimp_color_scale_destroy_transform (GimpColorScale *scale)
+{
+  GimpColorScalePrivate *priv = GET_PRIVATE (scale);
+
+  if (priv->transform)
+    {
+      g_object_unref (priv->transform);
+      priv->transform = NULL;
+    }
+
+  gtk_widget_queue_draw (GTK_WIDGET (scale));
+}
