@@ -25,6 +25,7 @@
 
 #include "libgimpbase/gimpbase.h"
 
+#undef GIMP_DISABLE_DEPRECATED
 #include "gimpicons.h"
 
 #include "icons/Color/gimp-icon-pixbufs.c"
@@ -61,7 +62,6 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_ANCHOR,         N_("Anchor"),          0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_CENTER,         N_("C_enter"),         0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_DUPLICATE,      N_("_Duplicate"),      0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_EDIT,           N_("_Edit"),           0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_LINKED,         N_("Linked"),          0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_PASTE_AS_NEW,   N_("Paste as New"),    0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_PASTE_INTO,     N_("Paste Into"),      0, 0, LIBGIMP_DOMAIN },
@@ -92,9 +92,6 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_HCENTER,                  NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_VCENTER,                  NULL,        0, 0, LIBGIMP_DOMAIN },
 
-  { GIMP_STOCK_HFILL,                    NULL,        0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_VFILL,                    NULL,        0, 0, LIBGIMP_DOMAIN },
-
   { GIMP_STOCK_HCHAIN,                   NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_HCHAIN_BROKEN,            NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_VCHAIN,                   NULL,        0, 0, LIBGIMP_DOMAIN },
@@ -123,9 +120,10 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_CHAR_PICKER,              NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_LETTER_SPACING, N_("L_etter Spacing"), 0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_LINE_SPACING,   N_("L_ine Spacing"),   0, 0, LIBGIMP_DOMAIN },
+
   { GIMP_STOCK_TEXT_DIR_LTR,             NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TEXT_DIR_RTL,             NULL,        0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_PRINT_RESOLUTION,         NULL,        0, 0, LIBGIMP_DOMAIN },
+  { GIMP_STOCK_PATTERN,                  NULL,        0, 0, LIBGIMP_DOMAIN },
 
   { GIMP_STOCK_CONVERT_RGB,              NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_CONVERT_GRAYSCALE,        NULL,        0, 0, LIBGIMP_DOMAIN },
@@ -257,15 +255,12 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_TOOL_FOREGROUND_SELECT, N_("_Select"), 0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_FUZZY_SELECT,        NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_HUE_SATURATION,      NULL,        0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_TOOL_HANDLE_TRANSFORM,N_("_Transform"),0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_HEAL,                NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_INK,                 NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_ISCISSORS,           NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_LEVELS,              NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_MEASURE,             NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_MOVE,                NULL,        0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_TOOL_MYPAINT_BRUSH,       NULL,        0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_TOOL_N_POINT_DEFORMATION, NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_PAINTBRUSH,          NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_PATH,                NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_PENCIL,              NULL,        0, 0, LIBGIMP_DOMAIN },
@@ -275,13 +270,10 @@ static const GtkStockItem gimp_stock_items[] =
   { GIMP_STOCK_TOOL_RECT_SELECT,         NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_ROTATE,       N_("_Rotate"),      0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_SCALE,        N_("_Scale"),       0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_TOOL_SEAMLESS_CLONE,      NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_SHEAR,        N_("_Shear"),       0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_SMUDGE,              NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_TEXT,                NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_THRESHOLD,           NULL,        0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_TOOL_UNIFIED_TRANSFORM, N_("_Transform"), 0, 0, LIBGIMP_DOMAIN },
-  { GIMP_STOCK_TOOL_WARP,                NULL,        0, 0, LIBGIMP_DOMAIN },
   { GIMP_STOCK_TOOL_ZOOM,                NULL,        0, 0, LIBGIMP_DOMAIN }
 };
 
@@ -347,47 +339,47 @@ register_bidi_stock_icon (GtkIconFactory *factory,
 }
 
 
-static GFile *icon_theme_path         = NULL;
-static GFile *default_icon_theme_path = NULL;
+static GFile *icon_theme_path     = NULL;
+static GFile *default_search_path = NULL;
 
 
 static void
-gimp_icons_change_icon_theme (GFile *path)
+gimp_icons_change_icon_theme (GFile *new_search_path)
 {
-  if (! default_icon_theme_path)
-    default_icon_theme_path = gimp_data_directory_file ("icons",
-                                                        GIMP_DEFAULT_ICON_THEME,
-                                                        NULL);
+  GFile *old_search_path = g_file_get_parent (icon_theme_path);
 
-  if (! g_file_equal (path, icon_theme_path))
+  if (! default_search_path)
+    default_search_path = gimp_data_directory_file ("icons", NULL);
+
+  if (! g_file_equal (new_search_path, old_search_path))
     {
       GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
 
-      if (g_file_equal (icon_theme_path, default_icon_theme_path))
+      if (g_file_equal (old_search_path, default_search_path))
         {
-          /*  if the current icon theme is the default theme, simply
-           *  prepend the new theme's path
+          /*  if the old icon theme is in the default search path,
+           *  simply prepend the new theme's path
            */
-          gchar *path_str = g_file_get_path (path);
+          gchar *path_str = g_file_get_path (new_search_path);
 
           gtk_icon_theme_prepend_search_path (icon_theme, path_str);
           g_free (path_str);
         }
       else
         {
-          /*  if the current theme is not the default theme, we need
-           *  to deal with the search path's first element
+          /*  if the old theme is not in the default search path,
+           *  we need to deal with the search path's first element
            */
           gchar **paths;
           gint    n_paths;
 
           gtk_icon_theme_get_search_path (icon_theme, &paths, &n_paths);
 
-          if (g_file_equal (path, default_icon_theme_path))
+          if (g_file_equal (new_search_path, default_search_path))
             {
-              /*  when switching to the default theme, remove the
-               *  first search path element, the default theme will
-               *  still be in the search path as fallback
+              /*  when switching to a theme in the default path, remove
+               *  the first search path element, the default search path
+               *  is still in the search path
                */
               gtk_icon_theme_set_search_path (icon_theme,
                                               (const gchar **) paths + 1,
@@ -395,12 +387,12 @@ gimp_icons_change_icon_theme (GFile *path)
             }
           else
             {
-              /*  when switching between two non-default themes, replace
+              /*  when switching between two non-default search paths, replace
                *  the first element of the search path with the new
                *  theme's path
                */
               g_free (paths[0]);
-              paths[0] = g_file_get_path (path);
+              paths[0] = g_file_get_path (new_search_path);
 
               gtk_icon_theme_set_search_path (icon_theme,
                                               (const gchar **) paths, n_paths);
@@ -408,69 +400,115 @@ gimp_icons_change_icon_theme (GFile *path)
 
           g_strfreev (paths);
         }
-
-      g_object_unref (icon_theme_path);
-      icon_theme_path = g_object_ref (path);
     }
+
+  g_object_unref (old_search_path);
+}
+
+static void
+gimp_icons_notify_system_icon_theme (GObject    *settings,
+                                     GParamSpec *param,
+                                     gpointer    unused)
+{
+  GdkScreen *screen = gdk_screen_get_default ();
+  GValue     value  = G_VALUE_INIT;
+
+  g_value_init (&value, G_TYPE_STRING);
+
+  if (gdk_screen_get_setting (screen, "gtk-icon-theme-name", &value))
+    {
+      const gchar *new_system_icon_theme = g_value_get_string (&value);
+      gchar *cur_system_icon_theme = NULL;
+
+      g_object_get (settings,
+                    "gtk-fallback-icon-theme", &cur_system_icon_theme,
+                    NULL);
+      if (g_strcmp0 (cur_system_icon_theme, new_system_icon_theme))
+        {
+          g_object_set (settings,
+                        "gtk-fallback-icon-theme", new_system_icon_theme,
+                        NULL);
+
+          g_object_notify (settings, "gtk-icon-theme-name");
+        }
+
+      g_free (cur_system_icon_theme);
+    }
+
+  g_value_unset (&value);
+}
+
+static gboolean
+gimp_icons_sanity_check (GFile       *path,
+                         const gchar *theme_name)
+{
+  gboolean exists = FALSE;
+  GFile *child = g_file_get_child (path, theme_name);
+
+  if (g_file_query_exists (child, NULL))
+    {
+      GFile *index = g_file_get_child (child, "index.theme");
+
+      if (g_file_query_exists (index, NULL))
+        exists = TRUE;
+      else
+        g_printerr ("%s: Icon theme path has no '%s/index.theme': %s\n",
+                    G_STRFUNC, theme_name, gimp_file_get_utf8_name (path));
+
+      g_object_unref (index);
+    }
+  else
+    g_printerr ("%s: Icon theme path has no '%s' subdirectory: %s\n",
+                G_STRFUNC, theme_name, gimp_file_get_utf8_name (path));
+
+  g_object_unref (child);
+
+  return exists;
 }
 
 void
 gimp_icons_set_icon_theme (GFile *path)
 {
+  gchar *icon_theme_name;
+  GFile *search_path;
+
   g_return_if_fail (path == NULL || G_IS_FILE (path));
 
   if (path)
     path = g_object_ref (path);
   else
-    path = gimp_data_directory_file (gimp_data_directory (),
-                                     "icons", GIMP_DEFAULT_ICON_THEME,
-                                     NULL);
+    path = gimp_data_directory_file ("icons", GIMP_DEFAULT_ICON_THEME, NULL);
 
-  if (! g_file_query_exists (path, NULL))
-    {
-      g_printerr ("%s: Icon theme path does not exist: %s\n",
-                  G_STRFUNC, gimp_file_get_utf8_name (path));
-    }
-  else
-    {
-      GFile *hicolor = g_file_get_child (path, "hicolor");
+  search_path = g_file_get_parent (path);
+  icon_theme_name = g_file_get_basename (path);
 
-      if (! g_file_query_exists (hicolor, NULL))
+  if (gimp_icons_sanity_check (search_path, "hicolor") &&
+      gimp_icons_sanity_check (search_path, icon_theme_name))
+    {
+      if (icon_theme_path)
         {
-          g_printerr ("%s: Icon theme path has no 'hicolor' subdirectory: %s\n",
-                      G_STRFUNC, gimp_file_get_utf8_name (path));
+          /*  this is an icon theme change  */
+          gimp_icons_change_icon_theme (search_path);
+
+          if (! g_file_equal (icon_theme_path, path))
+            {
+              g_object_unref (icon_theme_path);
+              icon_theme_path = g_object_ref (path);
+            }
+
+          g_object_set (gtk_settings_get_for_screen (gdk_screen_get_default ()),
+                        "gtk-icon-theme-name", icon_theme_name,
+                        NULL);
         }
       else
         {
-          GFile *index = g_file_get_child (hicolor, "index.theme");
-
-          if (! g_file_query_exists (index, NULL))
-            {
-              g_printerr ("%s: Icon theme path has no 'hicolor/index.theme': %s\n",
-                          G_STRFUNC, gimp_file_get_utf8_name (path));
-            }
-          else
-            {
-              /*  the path points to what looks like a valid icon theme  */
-
-              if (icon_theme_path)
-                {
-                  /*  this is an icon theme change  */
-                  gimp_icons_change_icon_theme (path);
-                }
-              else
-                {
-                  /*  this is the first call upon initialization  */
-                  icon_theme_path = g_object_ref (path);
-                }
-            }
-
-          g_object_unref (index);
+          /*  this is the first call upon initialization  */
+          icon_theme_path = g_object_ref (path);
         }
-
-      g_object_unref (hicolor);
     }
 
+  g_free (icon_theme_name);
+  g_object_unref (search_path);
   g_object_unref (path);
 }
 
@@ -482,7 +520,7 @@ gimp_icons_set_icon_theme (GFile *path)
  * You don't need to call this function as gimp_ui_init() already does
  * this for you.
  *
- * Deprecated: 2.10: USe gimp_icons_init() instead.
+ * Deprecated: 2.10: Use gimp_icons_init() instead.
  */
 void
 gimp_stock_init (void)
@@ -503,10 +541,13 @@ gimp_icons_init (void)
 {
   static gboolean initialized = FALSE;
 
-  GdkPixbuf *pixbuf;
-  GError    *error = NULL;
-  gchar     *icons_dir;
-  gint       i;
+  GtkSettings *settings;
+  GdkPixbuf   *pixbuf;
+  GError      *error = NULL;
+  gchar       *icons_dir;
+  gchar       *system_icon_theme;
+  gchar       *gimp_icon_theme;
+  gint         i;
 
   if (initialized)
     return;
@@ -545,26 +586,51 @@ gimp_icons_init (void)
    *  the path again and acts as fallback for missing icons in other
    *  themes.
    */
-  if (! default_icon_theme_path)
-    default_icon_theme_path = gimp_data_directory_file ("icons",
-                                                        GIMP_DEFAULT_ICON_THEME,
-                                                        NULL);
+  if (! default_search_path)
+    default_search_path = gimp_data_directory_file ("icons",
+                                                    NULL);
 
-  icons_dir = g_file_get_path (default_icon_theme_path);
+  icons_dir = g_file_get_path (default_search_path);
   gtk_icon_theme_prepend_search_path (gtk_icon_theme_get_default (),
                                       icons_dir);
   g_free (icons_dir);
 
   /*  if an icon theme was chosen before init(), change to it  */
-  if (icon_theme_path &&
-      ! g_file_equal (icon_theme_path, default_icon_theme_path))
+  if (icon_theme_path)
     {
-      icons_dir = g_file_get_path (icon_theme_path);
-      gtk_icon_theme_prepend_search_path (gtk_icon_theme_get_default (),
-                                          icons_dir);
-      g_free (icons_dir);
+      GFile *search_path = g_file_get_parent (icon_theme_path);
+
+      if (!g_file_equal (search_path, default_search_path))
+        {
+          gchar *icon_dir = g_file_get_path (search_path);
+
+          gtk_icon_theme_prepend_search_path (gtk_icon_theme_get_default (),
+                                              icon_dir);
+          g_free (icon_dir);
+        }
+      g_object_unref (search_path);
+
+      gimp_icon_theme = g_file_get_basename (icon_theme_path);
+    }
+  else
+    {
+      gimp_icon_theme = g_strdup (GIMP_DEFAULT_ICON_THEME);
     }
 
+  settings = gtk_settings_get_for_screen (gdk_screen_get_default ());
+
+  g_object_get (settings, "gtk-icon-theme-name", &system_icon_theme, NULL);
+
+  g_object_set (settings,
+                "gtk-fallback-icon-theme", system_icon_theme,
+                "gtk-icon-theme-name", gimp_icon_theme,
+                NULL);
+
+  g_free (gimp_icon_theme);
+  g_free (system_icon_theme);
+
+  g_signal_connect (settings, "notify::gtk-icon-theme-name",
+                    G_CALLBACK (gimp_icons_notify_system_icon_theme), NULL);
   pixbuf = gdk_pixbuf_new_from_resource ("/org/gimp/icons/64/gimp-wilber-eek.png",
                                          &error);
 
