@@ -663,6 +663,12 @@ gimp_ruler_set_position (GimpRuler *ruler,
           (ABS (xdiff) > IMMEDIATE_REDRAW_THRESHOLD ||
            ABS (ydiff) > IMMEDIATE_REDRAW_THRESHOLD))
         {
+          if (priv->pos_redraw_idle_id)
+            {
+              g_source_remove (priv->pos_redraw_idle_id);
+              priv->pos_redraw_idle_id = 0;
+            }
+
           gimp_ruler_queue_pos_redraw (ruler);
         }
       else if (! priv->pos_redraw_idle_id)
@@ -867,7 +873,7 @@ gimp_ruler_size_allocate (GtkWidget     *widget,
   resized = (widget_allocation.width  != allocation->width ||
              widget_allocation.height != allocation->height);
 
-  gtk_widget_set_allocation (widget, allocation);
+  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
 
   if (gtk_widget_get_realized (widget))
     {
@@ -948,6 +954,8 @@ gimp_ruler_style_updated (GtkWidget *widget)
                         "font-scale", &priv->font_scale,
                         NULL);
 
+  priv->backing_store_valid = FALSE;
+
   if (priv->layout)
     {
       g_object_unref (priv->layout);
@@ -1275,15 +1283,16 @@ gimp_ruler_queue_pos_redraw (GimpRuler *ruler)
 
   gtk_widget_get_allocation (GTK_WIDGET(ruler), &allocation);
 
-  gtk_widget_queue_draw_area (GTK_WIDGET(ruler),
+  gtk_widget_queue_draw_area (GTK_WIDGET (ruler),
                               rect.x + allocation.x,
                               rect.y + allocation.y,
                               rect.width,
                               rect.height);
 
-  if (priv->last_pos_rect.width != 0 || priv->last_pos_rect.height != 0)
+  if (priv->last_pos_rect.width  != 0 &&
+      priv->last_pos_rect.height != 0)
     {
-      gtk_widget_queue_draw_area (GTK_WIDGET(ruler),
+      gtk_widget_queue_draw_area (GTK_WIDGET (ruler),
                                   priv->last_pos_rect.x + allocation.x,
                                   priv->last_pos_rect.y + allocation.y,
                                   priv->last_pos_rect.width,
@@ -1338,7 +1347,17 @@ gimp_ruler_draw_pos (GimpRuler *ruler,
       cairo_fill (cr);
     }
 
-  priv->last_pos_rect = pos_rect;
+  if (priv->last_pos_rect.width  != 0 &&
+      priv->last_pos_rect.height != 0)
+    {
+      gdk_rectangle_union (&priv->last_pos_rect,
+                           &pos_rect,
+                           &priv->last_pos_rect);
+    }
+  else
+    {
+      priv->last_pos_rect = pos_rect;
+    }
 }
 
 static void
