@@ -194,7 +194,7 @@ query (void)
                           gih_load_args, gih_load_return_vals);
 
   gimp_plugin_icon_register (LOAD_PROC, GIMP_ICON_TYPE_ICON_NAME,
-                             (const guint8 *) GIMP_STOCK_BRUSH);
+                             (const guint8 *) GIMP_ICON_BRUSH);
   gimp_register_file_handler_mime (LOAD_PROC, "image/x-gimp-gih");
   gimp_register_file_handler_uri (LOAD_PROC);
   gimp_register_magic_load_handler (LOAD_PROC, "gih", "", "");
@@ -217,7 +217,7 @@ query (void)
                           gih_save_args, NULL);
 
   gimp_plugin_icon_register (SAVE_PROC, GIMP_ICON_TYPE_ICON_NAME,
-                             (const guint8 *) GIMP_STOCK_BRUSH);
+                             (const guint8 *) GIMP_ICON_BRUSH);
   gimp_register_file_handler_mime (SAVE_PROC, "image/x-gimp-gih");
   gimp_register_file_handler_uri (SAVE_PROC);
   gimp_register_save_handler (SAVE_PROC, "gih", "");
@@ -618,7 +618,8 @@ gih_load_one_brush (GInputStream  *input,
 
   layer_ID = gimp_layer_new (image_ID, name,
                              bh.width, bh.height,
-                             image_type, 100, GIMP_NORMAL_MODE);
+                             image_type,
+                             100, GIMP_LAYER_MODE_NORMAL_LEGACY);
   g_free (name);
 
   if (layer_ID != -1)
