@@ -73,6 +73,7 @@
 #include "libgimp/gimp.h"
 #include "libgimp/gimpui.h"
 
+#include "psd-util.h"
 #include "psd-save.h"
 
 #include "libgimp/stdplugins-intl.h"
@@ -129,60 +130,60 @@ static PSD_Image_Data PSDImageData;
  */
 
 static void        psd_lmode_layer      (gint32               idLayer,
-					 gchar               *psdMode);
+                                         gchar               *psdMode);
 
 static void        reshuffle_cmap_write (guchar              *mapGimp);
 
 static void        save_header          (FILE                *fd,
-					 gint32               image_id);
+                                         gint32               image_id);
 
 static void        save_color_mode_data (FILE                *fd,
-					 gint32               image_id);
+                                         gint32               image_id);
 
 static void        save_resources       (FILE                *fd,
-					 gint32               image_id);
+                                         gint32               image_id);
 
 static void        save_layer_and_mask  (FILE                *fd,
-					 gint32               image_id);
+                                         gint32               image_id);
 
 static void        save_data            (FILE                *fd,
-					 gint32               image_id);
+                                         gint32               image_id);
 
 static void        xfwrite              (FILE                *fd,
-					 gconstpointer        buf,
-					 glong                len,
-					 const gchar         *why);
+                                         gconstpointer        buf,
+                                         glong                len,
+                                         const gchar         *why);
 
 static void        write_pascalstring   (FILE               *fd,
-					 const gchar        *val,
-					 gint                padding,
-					 const gchar        *why);
+                                         const gchar        *val,
+                                         gint                padding,
+                                         const gchar        *why);
 
 static void        write_string         (FILE               *fd,
-					 const gchar        *val,
-					 const gchar        *why);
+                                         const gchar        *val,
+                                         const gchar        *why);
 
 static void        write_gchar          (FILE               *fd,
-					 guchar              val,
-					 const gchar        *why);
+                                         guchar              val,
+                                         const gchar        *why);
 
 static void        write_gint16         (FILE               *fd,
-					 gint16              val,
-					 const gchar        *why);
+                                         gint16              val,
+                                         const gchar        *why);
 
 static void        write_gint32         (FILE               *fd,
-					 gint32              val,
-					 const gchar        *why);
+                                         gint32              val,
+                                         const gchar        *why);
 
 static void        write_datablock_luni (FILE               *fd,
-					 const gchar        *val,
-					 const gchar        *why);
+                                         const gchar        *val,
+                                         const gchar        *why);
 
 
 static void        write_pixel_data     (FILE               *fd,
-					 gint32              drawableID,
-					 glong              *ChanLenPosition,
-					 gint32              rowlenOffset);
+                                         gint32              drawableID,
+                                         glong              *ChanLenPosition,
+                                         gint32              rowlenOffset);
 
 static gint32      create_merged_image  (gint32              imageID);
 
@@ -196,57 +197,95 @@ psd_lmode_layer (gint32  idLayer,
 {
   switch (gimp_layer_get_mode (idLayer))
     {
-    case GIMP_NORMAL_MODE:
+    case GIMP_LAYER_MODE_NORMAL_LEGACY:
       strcpy (psdMode, "norm");
       break;
-    case GIMP_DARKEN_ONLY_MODE:
+    case GIMP_LAYER_MODE_DARKEN_ONLY:
+    case GIMP_LAYER_MODE_DARKEN_ONLY_LEGACY:
       strcpy (psdMode, "dark");
       break;
-    case GIMP_LIGHTEN_ONLY_MODE:
+    case GIMP_LAYER_MODE_LIGHTEN_ONLY:
+    case GIMP_LAYER_MODE_LIGHTEN_ONLY_LEGACY:
       strcpy (psdMode, "lite");
       break;
-    case GIMP_HUE_MODE:
+    case GIMP_LAYER_MODE_LCH_HUE:
+    case GIMP_LAYER_MODE_HSV_HUE_LEGACY:
       strcpy (psdMode, "hue ");
       break;
-    case GIMP_SATURATION_MODE:
+    case GIMP_LAYER_MODE_LCH_CHROMA:
+    case GIMP_LAYER_MODE_HSV_SATURATION_LEGACY:
       strcpy (psdMode, "sat ");
       break;
-    case GIMP_COLOR_MODE:
+    case GIMP_LAYER_MODE_LCH_COLOR:
+    case GIMP_LAYER_MODE_HSV_COLOR_LEGACY:
       strcpy (psdMode, "colr");
       break;
-    case GIMP_ADDITION_MODE:
+    case GIMP_LAYER_MODE_ADDITION:
+    case GIMP_LAYER_MODE_ADDITION_LEGACY:
       strcpy (psdMode, "lddg");
       break;
-    case GIMP_MULTIPLY_MODE:
+    case GIMP_LAYER_MODE_MULTIPLY:
+    case GIMP_LAYER_MODE_MULTIPLY_LEGACY:
       strcpy (psdMode, "mul ");
       break;
-    case GIMP_SCREEN_MODE:
+    case GIMP_LAYER_MODE_SCREEN:
+    case GIMP_LAYER_MODE_SCREEN_LEGACY:
       strcpy (psdMode, "scrn");
       break;
-    case GIMP_DISSOLVE_MODE:
+    case GIMP_LAYER_MODE_DISSOLVE:
       strcpy (psdMode, "diss");
       break;
-    case GIMP_DIFFERENCE_MODE:
+    case GIMP_LAYER_MODE_DIFFERENCE:
+    case GIMP_LAYER_MODE_DIFFERENCE_LEGACY:
       strcpy (psdMode, "diff");
       break;
-    case GIMP_VALUE_MODE:                  /* ? */
+    case GIMP_LAYER_MODE_LCH_LIGHTNESS:
+    case GIMP_LAYER_MODE_HSV_VALUE_LEGACY:                  /* ? */
       strcpy (psdMode, "lum ");
       break;
-    case GIMP_HARDLIGHT_MODE:
+    case GIMP_LAYER_MODE_HARDLIGHT:
+    case GIMP_LAYER_MODE_HARDLIGHT_LEGACY:
       strcpy (psdMode, "hLit");
       break;
-    case GIMP_OVERLAY_MODE:
-    case GIMP_SOFTLIGHT_MODE:
+    case GIMP_LAYER_MODE_OVERLAY_LEGACY:
+    case GIMP_LAYER_MODE_SOFTLIGHT_LEGACY:
       strcpy (psdMode, "sLit");
       break;
-    case GIMP_NEW_OVERLAY_MODE:
+    case GIMP_LAYER_MODE_OVERLAY:
       strcpy (psdMode, "over");
       break;
+    case GIMP_LAYER_MODE_DODGE:
+    case GIMP_LAYER_MODE_DODGE_LEGACY:
+      strcpy (psdMode, "div");
+      break;
+    case GIMP_LAYER_MODE_EXCLUSION:
+      strcpy (psdMode, "smud");
+      break;
+    case GIMP_LAYER_MODE_BURN:
+    case GIMP_LAYER_MODE_BURN_LEGACY:
+      strcpy (psdMode, "idiv");
+      break;
+    case GIMP_LAYER_MODE_LINEAR_BURN:
+      strcpy (psdMode, "lbrn");
+      break;
+    case GIMP_LAYER_MODE_LINEAR_LIGHT:
+      strcpy (psdMode, "lLit");
+      break;
+    case GIMP_LAYER_MODE_PIN_LIGHT:
+      strcpy (psdMode, "pLit");
+      break;
+    case GIMP_LAYER_MODE_VIVID_LIGHT:
+      strcpy (psdMode, "vLit");
+      break;
+    case GIMP_LAYER_MODE_HARD_MIX:
+      strcpy (psdMode, "hMix");
+      break;
+
     default:
       {
         const gchar *nick = "?";
 
-        gimp_enum_get_value (GIMP_TYPE_LAYER_MODE_EFFECTS,
+        gimp_enum_get_value (GIMP_TYPE_LAYER_MODE,
                              gimp_layer_get_mode (idLayer),
                              NULL, &nick, NULL, NULL);
 
@@ -1074,8 +1113,20 @@ save_layer_and_mask (FILE   *fd,
       write_pascalstring (fd, layerName, 4, "layer name");
       IFDBG printf ("\t\tLayer name: %s\n", layerName);
 
+      /* Additional layer information blocks */
+      /* Unicode layer name */
       write_datablock_luni(fd, layerName, "luni extra data block");
 
+      /* Layer color tag */
+      xfwrite (fd, "8BIMlclr", 8, "sheet color signature");
+      write_gint32 (fd, 8, "sheet color size");
+      write_gint16 (fd,
+                    gimp_to_psd_layer_color_tag(gimp_item_get_color_tag(PSDImageData.lLayers[i])),
+                    "sheet color code");
+      write_gint16 (fd, 0, "sheet color unused value");
+      write_gint16 (fd, 0, "sheet color unused value");
+      write_gint16 (fd, 0, "sheet color unused value");
+
       /* Write real length for: Extra data */
 
       eof_pos = ftell (fd);
@@ -1110,7 +1161,7 @@ save_layer_and_mask (FILE   *fd,
   IFDBG printf ("\t\tTotal layers info section length: %d\n",
                 (int) (eof_pos - LayerInfoPos - sizeof (gint32)));
 
-  /* Write actual size of Layer and mask information secton */
+  /* Write actual size of Layer and mask information section */
 
   fseek (fd, LayerMaskPos, SEEK_SET);
   write_gint32 (fd, eof_pos - LayerMaskPos - sizeof (gint32), "layers & mask information length");
@@ -1210,11 +1261,11 @@ write_pixel_data (FILE   *fd,
       for (y = 0; y < height; y += tile_height)
         {
           int tlen;
-	  gegl_buffer_get (buffer,
+          gegl_buffer_get (buffer,
                            GEGL_RECTANGLE (0, y,
                                            width,
                                            MIN (height - y, tile_height)),
-			   1.0, format, data,
+                           1.0, format, data,
                            GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
           tlen = get_compress_channel_data (&data[chan],
                                              width,
@@ -1281,11 +1332,11 @@ write_pixel_data (FILE   *fd,
           for (y = 0; y < height; y += tile_height)
             {
               int tlen;
-	      gegl_buffer_get (mbuffer,
+              gegl_buffer_get (mbuffer,
                                GEGL_RECTANGLE (0, y,
                                                width,
                                                MIN (height - y, tile_height)),
-			       1.0, mformat, data,
+                               1.0, mformat, data,
                                GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
               tlen = get_compress_channel_data (&data[0],
                                                 width,
@@ -1390,41 +1441,41 @@ create_merged_image (gint32 image_id)
       gint                  width              = gegl_buffer_get_width (buffer);
       gint                  height             = gegl_buffer_get_height (buffer);
       GeglBufferIterator   *iter               = gegl_buffer_iterator_new (buffer, GEGL_RECTANGLE (0, 0, width, height),
-									   0, format,
+                                                                           0, format,
                                                                            GEGL_ACCESS_READ, GEGL_ABYSS_NONE);
 
       while (gegl_buffer_iterator_next (iter))
-	{
-	  guchar *data = iter->data[0];
-	  gint y;
+        {
+          guchar *data = iter->data[0];
+          gint y;
 
-	  for (y = 0; y < iter->roi->height; y++)
-	    {
-	      guchar *d = data;
-	      gint x;
+          for (y = 0; y < iter->roi->height; y++)
+            {
+              guchar *d = data;
+              gint x;
 
-	      for (x = 0; x < iter->roi->width; x++)
-		{
-		  gint32 alpha = d[bpp - 1];
+              for (x = 0; x < iter->roi->width; x++)
+                {
+                  gint32 alpha = d[bpp - 1];
 
-		  if (alpha < 255)
-		    {
-		      gint i;
+                  if (alpha < 255)
+                    {
+                      gint i;
 
-		      transparency_found = TRUE;
+                      transparency_found = TRUE;
 
-		      /* blend against white, photoshop does this. */
-		      for (i = 0; i < bpp - 1; i++)
-			d[i] = ((guint32) d[i] * alpha) / 255 + 255 - alpha;
-		    }
+                      /* blend against white, photoshop does this. */
+                      for (i = 0; i < bpp - 1; i++)
+                        d[i] = ((guint32) d[i] * alpha) / 255 + 255 - alpha;
+                    }
 
-		}
+                }
 
               d += bpp;
-	    }
+            }
 
-	  data += n_components;
-	}
+          data += n_components;
+        }
 
       g_object_unref (buffer);
 
