@@ -35,7 +35,7 @@
 #define MIN_RUN     3
 
 /*  Local function prototypes  */
-static gchar * gimp_layer_mode_effects_name (GimpLayerModeEffects mode);
+static gchar * gimp_layer_mode_effects_name (GimpLayerMode mode);
 
 
 /* Utility function */
@@ -605,17 +605,31 @@ encode_packbits (const gchar *src,
   return g_string_free (dst_str, FALSE);
 }
 
-GimpLayerModeEffects
-psd_to_gimp_blend_mode (const gchar *psd_mode)
+GimpLayerMode
+psd_to_gimp_blend_mode (const gchar            *psd_mode,
+                        GimpLayerCompositeMode *layer_composite)
 {
+  /* FIXME: Do all photoshop modes use the union rule?  We always return
+   * "union", effectively, only returning "auto" for modes that default to
+   * "union", to reduce UI clutter.
+   */
+  if (layer_composite) *layer_composite = GIMP_LAYER_COMPOSITE_SRC_ATOP;
+
   if (g_ascii_strncasecmp (psd_mode, "norm", 4) == 0)           /* Normal (ps3) */
-    return GIMP_NORMAL_MODE;
+    {
+      if (layer_composite) *layer_composite = GIMP_LAYER_COMPOSITE_AUTO;
+      return GIMP_LAYER_MODE_NORMAL_LEGACY;
+    }
+
   if (g_ascii_strncasecmp (psd_mode, "dark", 4) == 0)           /* Darken (ps3) */
-    return GIMP_DARKEN_ONLY_MODE;
+    return GIMP_LAYER_MODE_DARKEN_ONLY;
+
   if (g_ascii_strncasecmp (psd_mode, "lite", 4) == 0)           /* Lighten (ps3) */
-      return GIMP_LIGHTEN_ONLY_MODE;
+      return GIMP_LAYER_MODE_LIGHTEN_ONLY;
+
   if (g_ascii_strncasecmp (psd_mode, "hue ", 4) == 0)           /* Hue (ps3) */
-    return GIMP_HUE_MODE;
+    return GIMP_LAYER_MODE_LCH_HUE;
+
   if (g_ascii_strncasecmp (psd_mode, "sat ", 4) == 0)           /* Saturation (ps3) */
     {
       if (CONVERSION_WARNINGS)
@@ -625,10 +639,12 @@ psd_to_gimp_blend_mode (const gchar *psd_mode)
                      "blend mode: %s. Results will differ.",
                      mode_name);
         }
-      return GIMP_SATURATION_MODE;
+      return GIMP_LAYER_MODE_LCH_CHROMA;
     }
+
   if (g_ascii_strncasecmp (psd_mode, "colr", 4) == 0)           /* Color (ps3) */
-    return GIMP_COLOR_MODE;
+    return GIMP_LAYER_MODE_LCH_COLOR;
+
   if (g_ascii_strncasecmp (psd_mode, "lum ", 4) == 0)           /* Luminosity (ps3) */
     {
       if (CONVERSION_WARNINGS)
@@ -638,98 +654,94 @@ psd_to_gimp_blend_mode (const gchar *psd_mode)
                      "blend mode: %s. Results will differ.",
                      mode_name);
         }
-      return GIMP_VALUE_MODE;
+      return GIMP_LAYER_MODE_LCH_LIGHTNESS;
     }
+
   if (g_ascii_strncasecmp (psd_mode, "mul ", 4) == 0)           /* Multiply (ps3) */
-    return GIMP_MULTIPLY_MODE;
+    return GIMP_LAYER_MODE_MULTIPLY;
+
   if (g_ascii_strncasecmp (psd_mode, "lddg", 4) == 0)           /* Linear Dodge (cs2) */
-    return GIMP_ADDITION_MODE;
+    return GIMP_LAYER_MODE_ADDITION;
+
   if (g_ascii_strncasecmp (psd_mode, "scrn", 4) == 0)           /* Screen (ps3) */
-    return GIMP_SCREEN_MODE;
+    return GIMP_LAYER_MODE_SCREEN;
+
   if (g_ascii_strncasecmp (psd_mode, "diss", 4) == 0)           /* Dissolve (ps3) */
-    return GIMP_DISSOLVE_MODE;
+    {
+      if (layer_composite) *layer_composite = GIMP_LAYER_COMPOSITE_AUTO;
+      return GIMP_LAYER_MODE_DISSOLVE;
+    }
+
   if (g_ascii_strncasecmp (psd_mode, "over", 4) == 0)           /* Overlay (ps3) */
-    return GIMP_NEW_OVERLAY_MODE;
+    return GIMP_LAYER_MODE_OVERLAY;
+
   if (g_ascii_strncasecmp (psd_mode, "hLit", 4) == 0)           /* Hard light (ps3) */
-    return GIMP_HARDLIGHT_MODE;
+    return GIMP_LAYER_MODE_HARDLIGHT;
+
   if (g_ascii_strncasecmp (psd_mode, "sLit", 4) == 0)           /* Soft light (ps3) */
     {
       if (CONVERSION_WARNINGS)
         {
           static gchar  *mode_name = "SOFT LIGHT";
-          g_message ("Gimp uses a different equation to photoshop for "
+          g_message ("GIMP uses a different equation to Photoshop for "
                      "blend mode: %s. Results will differ.",
                      mode_name);
         }
-    return GIMP_SOFTLIGHT_MODE;
+    return GIMP_LAYER_MODE_SOFTLIGHT;
     }
+
   if (g_ascii_strncasecmp (psd_mode, "diff", 4) == 0)           /* Difference (ps3) */
-    return GIMP_DIFFERENCE_MODE;
+    return GIMP_LAYER_MODE_DIFFERENCE;
+
   if (g_ascii_strncasecmp (psd_mode, "smud", 4) == 0)           /* Exclusion (ps6) */
-    {
-      if (CONVERSION_WARNINGS)
-        {
-          static gchar  *mode_name = "EXCLUSION";
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     mode_name);
-        }
-      return GIMP_NORMAL_MODE;
-    }
+      return GIMP_LAYER_MODE_EXCLUSION;
+
   if (g_ascii_strncasecmp (psd_mode, "div ", 4) == 0)           /* Color dodge (ps6) */
-      return GIMP_DODGE_MODE;
+      return GIMP_LAYER_MODE_DODGE;
+
   if (g_ascii_strncasecmp (psd_mode, "idiv", 4) == 0)           /* Color burn (ps6) */
-      return GIMP_BURN_MODE;
+      return GIMP_LAYER_MODE_BURN;
+
   if (g_ascii_strncasecmp (psd_mode, "lbrn", 4) == 0)           /* Linear burn (ps7)*/
-    {
-      if (CONVERSION_WARNINGS)
-        {
-          static gchar  *mode_name = "LINEAR BURN";
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     mode_name);
-        }
-      return GIMP_NORMAL_MODE;
-    }
+      return GIMP_LAYER_MODE_LINEAR_BURN;
+
   if (g_ascii_strncasecmp (psd_mode, "lddg", 4) == 0)           /* Linear dodge (ps7)*/
-    return GIMP_ADDITION_MODE;
+      return GIMP_LAYER_MODE_ADDITION;
+
   if (g_ascii_strncasecmp (psd_mode, "lLit", 4) == 0)           /* Linear light (ps7)*/
-    {
-      if (CONVERSION_WARNINGS)
-        {
-          static gchar  *mode_name = "LINEAR LIGHT";
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     mode_name);
-        }
-      return GIMP_NORMAL_MODE;
-    }
+      return GIMP_LAYER_MODE_LINEAR_LIGHT;
+
   if (g_ascii_strncasecmp (psd_mode, "pLit", 4) == 0)           /* Pin light (ps7)*/
-    {
-      if (CONVERSION_WARNINGS)
-        {
-          static gchar  *mode_name = "PIN LIGHT";
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     mode_name);
-        }
-      return GIMP_NORMAL_MODE;
-    }
+      return GIMP_LAYER_MODE_PIN_LIGHT;
+
   if (g_ascii_strncasecmp (psd_mode, "vLit", 4) == 0)           /* Vivid light (ps7)*/
-    {
-      if (CONVERSION_WARNINGS)
-        {
-          static gchar  *mode_name = "VIVID LIGHT";
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     mode_name);
-        }
-      return GIMP_NORMAL_MODE;
-    }
+    return GIMP_LAYER_MODE_VIVID_LIGHT;
+
   if (g_ascii_strncasecmp (psd_mode, "hMix", 4) == 0)           /* Hard Mix (CS)*/
+      return GIMP_LAYER_MODE_HARD_MIX;
+
+  if (g_ascii_strncasecmp (psd_mode, "dkCl", 4) == 0)           /* Darker Color */
     {
       if (CONVERSION_WARNINGS)
         {
-          static gchar  *mode_name = "HARD MIX";
+          static gchar  *mode_name = "DARKER COLOR";
           g_message ("Unsupported blend mode: %s. Mode reverts to normal",
                      mode_name);
         }
-      return GIMP_NORMAL_MODE;
+      if (layer_composite) *layer_composite = GIMP_LAYER_COMPOSITE_AUTO;
+      return GIMP_LAYER_MODE_NORMAL_LEGACY;
+    }
+
+  if (g_ascii_strncasecmp (psd_mode, "lgCl", 4) == 0)           /* Lighter Color */
+    {
+      if (CONVERSION_WARNINGS)
+        {
+          static gchar  *mode_name = "LIGHTER COLOR";
+          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                     mode_name);
+        }
+      if (layer_composite) *layer_composite = GIMP_LAYER_COMPOSITE_AUTO;
+      return GIMP_LAYER_MODE_NORMAL_LEGACY;
     }
 
   if (CONVERSION_WARNINGS)
@@ -739,128 +751,206 @@ psd_to_gimp_blend_mode (const gchar *psd_mode)
                  mode_name);
       g_free (mode_name);
     }
-  return GIMP_NORMAL_MODE;
+
+  if (layer_composite) *layer_composite = GIMP_LAYER_COMPOSITE_AUTO;
+
+  return GIMP_LAYER_MODE_NORMAL_LEGACY;
 }
 
 gchar *
-gimp_to_psd_blend_mode (GimpLayerModeEffects gimp_layer_mode)
+gimp_to_psd_blend_mode (GimpLayerMode          layer_mode,
+                        GimpLayerCompositeMode layer_composite)
 {
   gchar *psd_mode;
 
-  switch (gimp_layer_mode)
+  if (((layer_mode == GIMP_LAYER_MODE_NORMAL_LEGACY ||
+        layer_mode == GIMP_LAYER_MODE_DISSOLVE) &&
+       (layer_composite != GIMP_LAYER_COMPOSITE_AUTO &&
+        layer_composite != GIMP_LAYER_COMPOSITE_SRC_OVER)) ||
+      layer_composite != GIMP_LAYER_COMPOSITE_SRC_OVER)
     {
-      case GIMP_NORMAL_MODE:
-        psd_mode = g_strndup ("norm", 4);                       /* Normal (ps3) */
-        break;
-      case GIMP_DISSOLVE_MODE:
-        psd_mode = g_strndup ("diss", 4);                       /* Dissolve (ps3) */
-        break;
-      case GIMP_BEHIND_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
-        break;
-      case GIMP_MULTIPLY_MODE:
-        psd_mode = g_strndup ("mul ", 4);                       /* Multiply (ps3) */
-        break;
-      case GIMP_SCREEN_MODE:
-        psd_mode = g_strndup ("scrn", 4);                       /* Screen (ps3) */
-        break;
-      case GIMP_NEW_OVERLAY_MODE:
-        psd_mode = g_strndup ("over", 4);                       /* Overlay (ps3) */
-        break;
-      case GIMP_DIFFERENCE_MODE:
-        psd_mode = g_strndup ("diff", 4);                       /* Difference (ps3) */
-        break;
-      case GIMP_ADDITION_MODE:
-        psd_mode = g_strndup ("lddg", 4);                       /* Linear dodge (ps7)*/
-        break;
-      case GIMP_SUBTRACT_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
-        break;
-      case GIMP_DARKEN_ONLY_MODE:
-        psd_mode = g_strndup ("dark", 4);                       /* Darken (ps3) */
-        break;
-      case GIMP_LIGHTEN_ONLY_MODE:
-        psd_mode = g_strndup ("lite", 4);                       /* Lighten (ps3) */
-        break;
-      case GIMP_HUE_MODE:
-        psd_mode = g_strndup ("hue ", 4);                       /* Hue (ps3) */
-        break;
-      case GIMP_SATURATION_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Gimp uses a different equation to photoshop for "
-                     "blend mode: %s. Results will differ.",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("sat ", 4);                       /* Saturation (ps3) */
-        break;
-      case GIMP_COLOR_MODE:
-        psd_mode = g_strndup ("colr", 4);                       /* Color (ps3) */
-        break;
-      case GIMP_VALUE_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Gimp uses a different equation to photoshop for "
-                     "blend mode: %s. Results will differ.",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("lum ", 4);                       /* Luminosity (ps3) */
-        break;
-      case GIMP_DIVIDE_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
-        break;
-      case GIMP_DODGE_MODE:
-        psd_mode = g_strndup ("div ", 4);                       /* Color Dodge (ps6) */
-        break;
-      case GIMP_BURN_MODE:
-        psd_mode = g_strndup ("idiv", 4);                       /* Color Burn (ps6) */
-        break;
-      case GIMP_HARDLIGHT_MODE:
-        psd_mode = g_strndup ("hLit", 4);                       /* Hard Light (ps3) */
-        break;
-      case GIMP_OVERLAY_MODE:
-      case GIMP_SOFTLIGHT_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-         psd_mode = g_strndup ("sLit", 4);                       /* Soft Light (ps3) */
-        break;
-      case GIMP_GRAIN_EXTRACT_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
-        break;
-      case GIMP_GRAIN_MERGE_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
-        break;
-      case GIMP_COLOR_ERASE_MODE:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
-        break;
+      /* FIXME, and above if() */
+    }
 
-      default:
-        if (CONVERSION_WARNINGS)
-          g_message ("Unsupported blend mode: %s. Mode reverts to normal",
-                     gimp_layer_mode_effects_name (gimp_layer_mode));
-        psd_mode = g_strndup ("norm", 4);
+  switch (layer_mode)
+    {
+    case GIMP_LAYER_MODE_NORMAL_LEGACY:
+      psd_mode = g_strndup ("norm", 4);                       /* Normal (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_DISSOLVE:
+      psd_mode = g_strndup ("diss", 4);                       /* Dissolve (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_BEHIND:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
+      break;
+
+    case GIMP_LAYER_MODE_MULTIPLY:
+    case GIMP_LAYER_MODE_MULTIPLY_LEGACY:
+      psd_mode = g_strndup ("mul ", 4);                       /* Multiply (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_SCREEN:
+    case GIMP_LAYER_MODE_SCREEN_LEGACY:
+      psd_mode = g_strndup ("scrn", 4);                       /* Screen (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_OVERLAY:
+      psd_mode = g_strndup ("over", 4);                       /* Overlay (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_DIFFERENCE:
+    case GIMP_LAYER_MODE_DIFFERENCE_LEGACY:
+      psd_mode = g_strndup ("diff", 4);                       /* Difference (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_ADDITION:
+    case GIMP_LAYER_MODE_ADDITION_LEGACY:
+      psd_mode = g_strndup ("lddg", 4);                       /* Linear dodge (ps7)*/
+      break;
+
+    case GIMP_LAYER_MODE_SUBTRACT_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
+      break;
+
+    case GIMP_LAYER_MODE_DARKEN_ONLY:
+    case GIMP_LAYER_MODE_DARKEN_ONLY_LEGACY:
+      psd_mode = g_strndup ("dark", 4);                       /* Darken (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_LIGHTEN_ONLY:
+    case GIMP_LAYER_MODE_LIGHTEN_ONLY_LEGACY:
+      psd_mode = g_strndup ("lite", 4);                       /* Lighten (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_LCH_HUE:
+    case GIMP_LAYER_MODE_HSV_HUE_LEGACY:
+      psd_mode = g_strndup ("hue ", 4);                       /* Hue (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_LCH_CHROMA:
+    case GIMP_LAYER_MODE_HSV_SATURATION_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("GIMP uses a different equation to Photoshop for "
+                   "blend mode: %s. Results may differ.",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("sat ", 4);                       /* Saturation (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_LCH_COLOR:
+    case GIMP_LAYER_MODE_HSV_COLOR_LEGACY:
+      psd_mode = g_strndup ("colr", 4);                       /* Color (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_LCH_LIGHTNESS:
+    case GIMP_LAYER_MODE_HSV_VALUE_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("GIMP uses a different equation to Photoshop for "
+                   "blend mode: %s. Results may differ.",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("lum ", 4);                       /* Luminosity (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_DIVIDE_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
+      break;
+
+    case GIMP_LAYER_MODE_DODGE:
+    case GIMP_LAYER_MODE_DODGE_LEGACY:
+      psd_mode = g_strndup ("div ", 4);                       /* Color Dodge (ps6) */
+      break;
+
+    case GIMP_LAYER_MODE_EXCLUSION:
+      psd_mode = g_strndup ("smud", 4);                       /* Exclusion (ps6) */
+      break;
+
+    case GIMP_LAYER_MODE_BURN:
+    case GIMP_LAYER_MODE_BURN_LEGACY:
+      psd_mode = g_strndup ("idiv", 4);                       /* Color Burn (ps6) */
+      break;
+
+    case GIMP_LAYER_MODE_LINEAR_BURN:
+      psd_mode = g_strndup ("lbrn", 4);                       /* Linear Burn (ps6) */
+      break;
+
+    case GIMP_LAYER_MODE_HARDLIGHT:
+    case GIMP_LAYER_MODE_HARDLIGHT_LEGACY:
+      psd_mode = g_strndup ("hLit", 4);                       /* Hard Light (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_VIVID_LIGHT:
+      if (CONVERSION_WARNINGS)
+        g_message ("GIMP uses a different equation to Photoshop for "
+                   "blend mode: %s. Results may differ.",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("vLit", 4);                       /* Vivid Light (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_LINEAR_LIGHT:
+      psd_mode = g_strndup ("lLit", 4);                       /* Linear light (ps7)*/
+      break;
+
+    case GIMP_LAYER_MODE_PIN_LIGHT:
+      psd_mode = g_strndup ("pLit", 4);                       /* Pin light (ps7)*/
+      break;
+
+    case GIMP_LAYER_MODE_HARD_MIX:
+      psd_mode = g_strndup ("hMix", 4);                       /* Hard Mix (CS)*/
+      break;
+
+    case GIMP_LAYER_MODE_OVERLAY_LEGACY:
+    case GIMP_LAYER_MODE_SOFTLIGHT_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("sLit", 4);                       /* Soft Light (ps3) */
+      break;
+
+    case GIMP_LAYER_MODE_GRAIN_EXTRACT:
+    case GIMP_LAYER_MODE_GRAIN_EXTRACT_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
+      break;
+
+    case GIMP_LAYER_MODE_GRAIN_MERGE:
+    case GIMP_LAYER_MODE_GRAIN_MERGE_LEGACY:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
+      break;
+
+    case GIMP_LAYER_MODE_COLOR_ERASE:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
+      break;
+
+    default:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported blend mode: %s. Mode reverts to normal",
+                   gimp_layer_mode_effects_name (layer_mode));
+      psd_mode = g_strndup ("norm", 4);
     }
 
   return psd_mode;
 }
 
 static gchar *
-gimp_layer_mode_effects_name (GimpLayerModeEffects mode)
+gimp_layer_mode_effects_name (GimpLayerMode mode)
 {
   static gchar *layer_mode_effects_names[] =
   {
@@ -873,6 +963,7 @@ gimp_layer_mode_effects_name (GimpLayerModeEffects mode)
     "DIFFERENCE",
     "ADD",
     "SUBTRACT",
+    "EXCLUSION"
     "DARKEN",
     "LIGHTEN",
     "HUE",
@@ -882,17 +973,112 @@ gimp_layer_mode_effects_name (GimpLayerModeEffects mode)
     "DIVIDE",
     "DODGE",
     "BURN",
+    "LINEAR BURN"
     "HARD LIGHT",
     "SOFT LIGHT",
+    "VIVID LIGHT",
+    "LINEAR LIGHT",
+    "PIN LIGHT",
+    "HARD MIX"
     "GRAIN EXTRACT",
     "GRAIN MERGE",
     "COLOR ERASE"
   };
   static gchar *err_name = NULL;
-  if (mode >= 0 && mode <= GIMP_COLOR_ERASE_MODE)
+  if (mode >= 0 && mode <= GIMP_LAYER_MODE_COLOR_ERASE)
     return layer_mode_effects_names[mode];
   g_free (err_name);
 
   err_name = g_strdup_printf ("UNKNOWN (%d)", mode);
   return err_name;
 }
+
+GimpColorTag
+psd_to_gimp_layer_color_tag (guint16 layer_color_tag)
+{
+  GimpColorTag colorTag;
+
+  switch (layer_color_tag)
+    {
+    case 1:
+      colorTag = GIMP_COLOR_TAG_RED;
+      break;
+
+    case 2:
+      colorTag = GIMP_COLOR_TAG_ORANGE;
+      break;
+
+    case 3:
+      colorTag = GIMP_COLOR_TAG_YELLOW;
+      break;
+
+    case 4:
+      colorTag = GIMP_COLOR_TAG_GREEN;
+      break;
+
+    case 5:
+      colorTag = GIMP_COLOR_TAG_BLUE;
+      break;
+
+    case 6:
+      colorTag = GIMP_COLOR_TAG_VIOLET;
+      break;
+
+    case 7:
+      colorTag = GIMP_COLOR_TAG_GRAY;
+      break;
+
+    default:
+      if (CONVERSION_WARNINGS)
+        g_message ("Unsupported Photoshop layer color tag: %i. GIMP layer color tag set to none.",
+                       layer_color_tag);
+      colorTag = GIMP_COLOR_TAG_NONE;
+    }
+
+  return colorTag;
+}
+
+guint16
+gimp_to_psd_layer_color_tag (GimpColorTag layer_color_tag)
+{
+  guint16 color_tag;
+
+  switch (layer_color_tag)
+    {
+    case GIMP_COLOR_TAG_RED:
+        color_tag = 1;
+      break;
+
+    case GIMP_COLOR_TAG_ORANGE:
+        color_tag = 2;
+      break;
+
+    case GIMP_COLOR_TAG_YELLOW:
+        color_tag = 3;
+      break;
+
+    case GIMP_COLOR_TAG_GREEN:
+        color_tag = 4;
+      break;
+
+    case GIMP_COLOR_TAG_BLUE:
+        color_tag = 5;
+      break;
+
+    case GIMP_COLOR_TAG_VIOLET:
+        color_tag = 6;
+      break;
+
+    case GIMP_COLOR_TAG_GRAY:
+        color_tag = 7;
+      break;
+
+    default:
+      if (CONVERSION_WARNINGS)
+        g_message ("Photoshop doesn't support GIMP layer color tag: %i. Photoshop layer color tag set to none.",
+                   layer_color_tag);
+        color_tag = 0;
+    }
+
+  return color_tag;
+}
